Package: pander
Authors@R: c(
    person("Gergely", "Daróczi", , "daroczig@rapporter.net", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3149-8537")),
    person("Roman", "Tsegelskyi", , "roman.tsegelskyi@gmail.com", role = c("aut")))
Title: An R 'Pandoc' Writer
Type: Package
Encoding: UTF-8
Description: Contains some functions catching all messages, 'stdout' and other
    useful information while evaluating R code and other helpers to return user
    specified text elements (like: header, paragraph, table, image, lists etc.)
    in 'pandoc' markdown or several type of R objects similarly automatically
    transformed to markdown format. Also capable of exporting/converting (the
    resulting) complex 'pandoc' documents to e.g. HTML, 'PDF', 'docx' or 'odt'. This
    latter reporting feature is supported in brew syntax or with a custom reference
    class with a smarty caching 'backend'.
Version: 0.6.6
Date: 2025-03-01
URL: https://rapporter.github.io/pander/
BugReports: https://github.com/rapporter/pander/issues
License: AGPL-3 | file LICENSE
Depends: R (>= 2.15.0)
Imports: grDevices, graphics, methods, utils, stats, digest, tools,
        Rcpp
Suggests: grid, lattice, ggplot2 (>= 0.9.2), sylly, sylly.en, logger,
        survival, microbenchmark, zoo, nlme, descr, MASS, knitr,
        rmarkdown, tables, reshape, memisc, Epi, randomForest, tseries,
        gtable, rms, forecast, data.table
SystemRequirements: pandoc (https://johnmacfarlane.net/pandoc) for
        exporting markdown files to other formats.
LinkingTo: Rcpp
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-03-01 00:02:44 UTC; daroczig
Author: Gergely Daróczi [aut, cre] (<https://orcid.org/0000-0003-3149-8537>),
  Roman Tsegelskyi [aut]
Maintainer: Gergely Daróczi <daroczig@rapporter.net>
Repository: CRAN
Date/Publication: 2025-03-01 01:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 10:41:45 UTC; unix
Archs: pander.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(pander,"NULL")
S3method(pander,"function")
S3method(pander,Arima)
S3method(pander,CrossTable)
S3method(pander,Date)
S3method(pander,Glm)
S3method(pander,POSIXct)
S3method(pander,POSIXlt)
S3method(pander,anova)
S3method(pander,aov)
S3method(pander,aovlist)
S3method(pander,call)
S3method(pander,cast_df)
S3method(pander,character)
S3method(pander,clogit)
S3method(pander,coxph)
S3method(pander,cph)
S3method(pander,data.frame)
S3method(pander,data.table)
S3method(pander,default)
S3method(pander,density)
S3method(pander,describe)
S3method(pander,ets)
S3method(pander,evals)
S3method(pander,factor)
S3method(pander,formula)
S3method(pander,ftable)
S3method(pander,glm)
S3method(pander,gtable)
S3method(pander,htest)
S3method(pander,image)
S3method(pander,irts)
S3method(pander,list)
S3method(pander,lm)
S3method(pander,lme)
S3method(pander,logical)
S3method(pander,lrm)
S3method(pander,manova)
S3method(pander,matrix)
S3method(pander,microbenchmark)
S3method(pander,name)
S3method(pander,nls)
S3method(pander,numeric)
S3method(pander,ols)
S3method(pander,orm)
S3method(pander,polr)
S3method(pander,prcomp)
S3method(pander,randomForest)
S3method(pander,rapport)
S3method(pander,rlm)
S3method(pander,sessionInfo)
S3method(pander,smooth.spline)
S3method(pander,stat.table)
S3method(pander,summary.aov)
S3method(pander,summary.aovlist)
S3method(pander,summary.glm)
S3method(pander,summary.lm)
S3method(pander,summary.lme)
S3method(pander,summary.manova)
S3method(pander,summary.nls)
S3method(pander,summary.polr)
S3method(pander,summary.prcomp)
S3method(pander,summary.rms)
S3method(pander,summary.survreg)
S3method(pander,summary.table)
S3method(pander,survdiff)
S3method(pander,survfit)
S3method(pander,survreg)
S3method(pander,table)
S3method(pander,tabular)
S3method(pander,ts)
S3method(pander,vector)
S3method(pander,zoo)
export(Pandoc)
export(Pandoc.brew)
export(Pandoc.convert)
export(add.blank.lines)
export(add.significance.stars)
export(cache.off)
export(cache.on)
export(emphasize.cells)
export(emphasize.cols)
export(emphasize.italics.cells)
export(emphasize.italics.cols)
export(emphasize.italics.rows)
export(emphasize.rows)
export(emphasize.strong.cells)
export(emphasize.strong.cols)
export(emphasize.strong.rows)
export(emphasize.verbatim.cells)
export(emphasize.verbatim.cols)
export(emphasize.verbatim.rows)
export(eval.msgs)
export(evals)
export(evalsOptions)
export(has.rownames)
export(openFileInOS)
export(p)
export(pander)
export(panderOptions)
export(pander_return)
export(pandoc.date)
export(pandoc.date.return)
export(pandoc.emphasis)
export(pandoc.emphasis.return)
export(pandoc.footnote)
export(pandoc.footnote.return)
export(pandoc.formula)
export(pandoc.formula.return)
export(pandoc.header)
export(pandoc.header.return)
export(pandoc.horizontal.rule)
export(pandoc.horizontal.rule.return)
export(pandoc.image)
export(pandoc.image.return)
export(pandoc.indent)
export(pandoc.link)
export(pandoc.link.return)
export(pandoc.list)
export(pandoc.list.return)
export(pandoc.p)
export(pandoc.p.return)
export(pandoc.strikeout)
export(pandoc.strikeout.return)
export(pandoc.strong)
export(pandoc.strong.return)
export(pandoc.table)
export(pandoc.table.return)
export(pandoc.title)
export(pandoc.title.return)
export(pandoc.verbatim)
export(pandoc.verbatim.return)
export(path_to_pandoc)
export(redraw.recordedplot)
export(redrawPlot)
export(remove.extra.newlines)
export(repChar)
export(set.alignment)
export(set.caption)
export(splitLine)
export(trim.spaces)
export(wrap)
importFrom(Rcpp,evalCpp)
importFrom(digest,digest)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,recordPlot)
importFrom(grDevices,replayPlot)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,par)
importFrom(methods,getGeneric)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(stats,coef)
importFrom(stats,cycle)
importFrom(stats,end)
importFrom(stats,frequency)
importFrom(stats,ftable)
importFrom(stats,naprint)
importFrom(stats,pchisq)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,vcov)
importFrom(tools,file_path_sans_ext)
importFrom(utils,as.roman)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,tail)
useDynLib(pander)

# [pander: An R Pandoc Writer](https://github.com/Rapporter/pander)

The main aim of the *pander* [R](http://r-project.org) package is to provide a minimal and easy tool for rendering **R objects** into [Pandoc](http://johnmacfarlane.net/pandoc/)'s **markdown**. The package is also capable of exporting/converting complex Pandoc documents (reports) in [various ways](#creating-pandoc-documents). Regarding the difference between `pander` and other packages for exporting R objects to different file formats, please refer to this [section](#difference-from-other-rendering-packages).

Current build and test coverage status: [![](https://travis-ci.org/Rapporter/pander.png?branch=master)](https://travis-ci.org/Rapporter/pander) [![](http://codecov.io/github/Rapporter/pander/coverage.svg?branch=master)](http://codecov.io/github/Rapporter/pander?branch=master).

Some CRAN statistics: [![](http://cranlogs.r-pkg.org/badges/pander)](https://cran.r-project.org/package=pander) [![](http://cranlogs.r-pkg.org/badges/grand-total/pander)](https://cran.r-project.org/package=pander)

# Installation

The stable version [![](http://www.r-pkg.org/badges/version/pander)](https://cran.r-project.org/package=pander) can be installed easily in the `R` console like any other package:

```r
install.packages('pander')
```

On the other hand, I welcome everyone to use the most recent version of the package with quick-fixes, new features and probably new bugs. It's currently hosted on [GitHub](https://github.com/Rapporter/pander). To get the latest development version from [GitHub](https://github.com/Rapporter/pander) of the `devtools` package from [CRAN](https://cran.r-project.org/package=devtools):

```r
devtools::install_github('Rapporter/pander')
```

Or download the [sources](https://github.com/Rapporter/pander/zipball/master) and build manually. If you're running R on Windows, you will need to install [Rtools](https://cran.r-project.org/bin/windows/Rtools/).

## Dependencies

Few cool packages from CRAN are needed for installing and/or using `pander`:

 * [digest](https://cran.r-project.org/package=digest) to compute hashes while caching,
 * [Rcpp](https://cran.r-project.org/package=Rcpp) to compile certain functions used by the package.

And there are also a few optional suggested or supported R packages, such as:

 * [sylly](https://cran.r-project.org/package=sylly) to use hyphenation when splitting large table cells,
 * [lattice](https://cran.r-project.org/package=lattice) and [ggplot2](https://cran.r-project.org/package=ggplot2) for unified plot theme,
 * [futile.logger](https://cran.r-project.org/package=futile.logger) for logging capabilities inside `evals`,
 * [survival](https://cran.r-project.org/package=survival), [microbenchmark](https://cran.r-project.org/package=microbenchmark), [zoo](https://cran.r-project.org/package=zoo), [nlme](https://cran.r-project.org/package=nlme), [descr](https://cran.r-project.org/package=descr), [MASS](https://cran.r-project.org/package=MASS), [tables](https://cran.r-project.org/package=tables), [reshape](https://cran.r-project.org/package=reshape), [memisc](https://cran.r-project.org/package=memisc), [Epi](https://cran.r-project.org/package=Epi), [randomForest](https://cran.r-project.org/package=randomForest), [tseries](https://cran.r-project.org/package=tseries), [rms](https://cran.r-project.org/package=rms) packages include some supported R classes,
 * and *pander* can be also very useful inside of [knitr](https://cran.r-project.org/package=knitr). More information about how to use *pander* and *knitr* can be found specialized vignette, which can be accessed by `vignette('knitr', package = 'pander')` or available online [here](http://rapporter.github.io/pander/knitr.html).

### Pandoc

*pander* heavily builds on [Pandoc](http://johnmacfarlane.net/pandoc), which should be **pre-installed** before trying to convert your reports to [different formats](http://johnmacfarlane.net/pandoc/). Although **main functions work without Pandoc**, e.g. you can transform R objects into markdown or generate a markdown formatted report via [Pandoc.brew](#brew-to-pandoc) or the custom [reference class](#live-report-generation), but installing that great piece of software is suggested if you want to convert markdown to PDF/docx/HTML or other formats.

Starting v0.98.932 [RStudio](http://www.rstudio.com/) comes with a bundled `Pandoc` binary, so one can save the tedious steps of installing Pandoc.

If you do not have RStudio installed, please refer to the [installation process of Pandoc](http://johnmacfarlane.net/pandoc/installing.html), which is quite straightforward on most-popular operating systems: download and run the binary (a few megabytes), and get a full-blown document converter in a few seconds/minutes. On some Linux distributions, it might be a bit more complicated (as repositories tend to provide out-dated versions of Pandoc, so you would need `cabal-install` to [install from sources](http://johnmacfarlane.net/pandoc/installing.html#all-platforms)). Please do not forget to restart your R session to update your `PATH` after installation!

# Helper functions

The package contains numerous helper functions, which render user specified inputs in Pandoc's markdown format or apply some extra formatting on it. All  Pandoc-related functions' names are starting with `pandoc`. For example `pandoc.table` is used for rendering tables in markdown. For a technical documentation, see the HTML help files of the package at [Rdocumentation](http://www.rdocumentation.org/packages/pander).

<a id="primitive-functions"></a>

All `pandoc` functions generally prints to console and do not return anything by default. If you want the opposite, to get markdown in a string, call each function ending in `.return`, for example `pandoc.table.return`. For more details, please see the official documentation in e.g. `?pandoc.strong`.

The full list of currently available primitive Pandoc-related functions are:

  * pandoc.indent
  * pandoc.p
  * pandoc.strong
  * pandoc.emphasis
  * pandoc.strikeout
  * pandoc.verbatim
  * pandoc.link
  * pandoc.image
  * pandoc.date
  * pandoc.formula
  * pandoc.footnote
  * pandoc.horizontal.rule
  * pandoc.header
  * pandoc.title

<a id="lists"></a>
For example there is a helper function rendering R **lists** into markdown:

```r
> l <- list(
+        "First list element",
+        paste0(1:5, '. subelement'),
+        "Second element",
+        list('F', 'B', 'I', c('phone', 'pad', 'talics')))
> pandoc.list(l, 'roman')
```
Which command produces the following output:
```
I. First list element 
    I. 1. subelement 
    II. 2. subelement 
    III. 3. subelement 
    IV. 4. subelement 
    V. 5. subelement  
II. Second element 
    I. F 
    II. B 
    III. I 
        I. phone 
        II. pad 
        III. talics   

<!-- end of list -->

```

<a id="tables"></a>

# Markdown tables

One of the most popular feature in *pander* is `pandoc.table`, rendering most tabular R objects into markdown tables with various options and settings (e.g. [style](#style), [caption](#caption), [cell highlighting](#highlight-cells), [cell alignment](#cell-alignment), [width](#table-and-cell-width)). This section aims to provide quick introduction to most common options, but for more usage/implementation details and examples, please refer to specialized vignette, which can be accessed by `vignette('pandoc_table')` or available online [here](http://rapporter.github.io/pander/pandoc_table.html).

Let's start with a small example:

```r
> pandoc.table(mtcars[1:3, 1:4])
```
Which command produces the following output by default:
```
-------------------------------------------
      &nbsp;         mpg   cyl   disp   hp 
------------------- ----- ----- ------ ----
   **Mazda RX4**     21     6    160   110 

 **Mazda RX4 Wag**   21     6    160   110 

  **Datsun 710**    22.8    4    108    93 
-------------------------------------------

```

<a id="style"></a><a id="styles"></a>

Please note that all below features are also supported by the more concise `pander` [generic S3 method](#generic-pander-method)!

## Formats

All [four Pandoc formats](http://johnmacfarlane.net/pandoc/README.html#tables) are supported by *pander*. From those (*multiline*, *simple*, *grid*, *pipe/rmarkdown*), I'd suggest sticking to the default `multiline` format with the most features, except when using `rmarkdown` v1.0 or jupyter notebook, where `multiline` is not supported (for this end the default table format is `rmarkdown` when `pander` is called inside of a jupyter notebook). Please see a few examples below:

<a id="multiline-table"></a>

The default style is the [`multiline` format](http://johnmacfarlane.net/pandoc/README.html#multiline-tables) (except for calling `pander` inside of a of a jupyter notebook) as most features (e.g. multi-line cells and alignment) are supported:

```rout
> m <- mtcars[1:2, 1:3]
> pandoc.table(m)

--------------------------------------
      &nbsp;         mpg   cyl   disp 
------------------- ----- ----- ------
   **Mazda RX4**     21     6    160  

 **Mazda RX4 Wag**   21     6    160  
--------------------------------------

```

While [`simple` tables](http://johnmacfarlane.net/pandoc/README.html#simple-tables) are much more compact, but do not support line breaks in cells:

```rout
> pandoc.table(m, style = "simple")

      &nbsp;         mpg   cyl   disp 
------------------- ----- ----- ------
   **Mazda RX4**     21     6    160  
 **Mazda RX4 Wag**   21     6    160  

```

My personal favorite, the [`grid` format](http://johnmacfarlane.net/pandoc/README.html#grid-tables) is really handy for [emacs](http://emacswiki.org/emacs/TableMode) users and it does support line breaks inside of cells, but cell alignment is not possible in most parsers:

```rout
> pandoc.table(m, style = "grid")

+---------------------+-------+-------+--------+
|       &nbsp;        |  mpg  |  cyl  |  disp  |
+=====================+=======+=======+========+
|    **Mazda RX4**    |  21   |   6   |  160   |
+---------------------+-------+-------+--------+
|  **Mazda RX4 Wag**  |  21   |   6   |  160   |
+---------------------+-------+-------+--------+

```

And the so called `rmarkdown` or [pipe table format](http://johnmacfarlane.net/pandoc/README.html#pipe-tables) is often used directly with `knitr`, since it was supporters by the first versions of the `markdown` package:

```rout
> pandoc.table(m, style = "rmarkdown")

|       &nbsp;        |  mpg  |  cyl  |  disp  |
|:-------------------:|:-----:|:-----:|:------:|
|    **Mazda RX4**    |  21   |   6   |  160   |
|  **Mazda RX4 Wag**  |  21   |   6   |  160   |

```

But once again, you should simply stick to the default [multiline table format](#multiline-table) in most cases. Otherwise, it's wise to update the default table format via [`panderOptions`](#pander-options).

## Caption

It's really easy to add a **caption** to a table:

```rout
> pandoc.table(m, style = "grid", caption = "Hello caption!")

+---------------------+-------+-------+--------+
|       &nbsp;        |  mpg  |  cyl  |  disp  |
+=====================+=======+=======+========+
|    **Mazda RX4**    |  21   |   6   |  160   |
+---------------------+-------+-------+--------+
|  **Mazda RX4 Wag**  |  21   |   6   |  160   |
+---------------------+-------+-------+--------+

Table: Hello caption!

```

For more convenient and flexible usage, you might be interested in the special `set.caption` helper function. Call the function at any time, and the next table or plot will catch up the provided caption:

```rout
> set.caption("Hello caption!")
> pandoc.table(m)

--------------------------------------
      &nbsp;         mpg   cyl   disp 
------------------- ----- ----- ------
   **Mazda RX4**     21     6    160  

 **Mazda RX4 Wag**   21     6    160  
--------------------------------------

Table: Hello caption!

```

Unless `permanent` option is set for `TRUE` (by default), caption will be set only for next table. To disable permanently set caption, just call `set.caption(NULL)` or call `set.caption` with `permanent` parameter being set to `FALSE`.

<a id="highlight-cells"></a>

## Highlighting cells

One of the fanciest features in *pander* is the ease of highlighting rows, columns or any cells in a table. This is a real markdown feature without custom HTML or LaTeX-only tweaks, so all HTML/PDF/MS Word/OpenOffice etc. formats are supported.

This can be achieved by calling `pandoc.table` directly and passing any (or more) of the following arguments **or** calling the R function with the same names before rendering a table with either the `pander` [generic S3 method](#generic-pander-method) or via `pandoc.table`:

* emphasize.italics.rows
* emphasize.italics.cols
* emphasize.italics.cells
* emphasize.strong.rows
* emphasize.strong.cols
* emphasize.strong.cells
* emphasize.verbatim.rows
* emphasize.verbatim.cols
* emphasize.verbatim.cells

The `emphasize.italics` helpers would turn the affected cells to *italic*, `emphasize.strong` would apply a **bold** style to the cell and `emphasize.verbatim` would apply a `verbatim` style to the cell. A cell can be also *italic*,  **bold** and `verbatim` at the same time.

Those functions and arguments ending in `rows` or `cols` take a vector (like which columns or rows to emphasize in a table), while the `cells` argument take either a vector (for one dimensional "tables") or an array-like data structure with two columns holding row and column indexes of cells to be emphasized -- just like what `which(..., arr.ind = TRUE)` returns. A quick-example:

```rout
> t <- mtcars[1:3, 1:5]
> emphasize.italics.cols(1)
> emphasize.italics.rows(1)
> emphasize.strong.cells(which(t > 20, arr.ind = TRUE))
> pandoc.table(t)

---------------------------------------------------------------
      &nbsp;           mpg      cyl    disp       hp      drat 
------------------- ---------- ----- --------- --------- ------
   **Mazda RX4**     ***21***   *6*  ***160*** ***110*** *3.9* 

 **Mazda RX4 Wag**   ***21***    6    **160**   **110**   3.9  

  **Datsun 710**    ***22.8***   4    **108**   **93**    3.85 
---------------------------------------------------------------

```

For more examples, please see our "[Highlight cells in markdown tables](http://blog.rapporter.net/2013/04/hihglight-cells-in-markdown-tables.html)" blog post.

## Cell alignment

You can specify the alignment of the cells (left, right or center/centre) in a table directly by setting the `justify` parameter:

```rout
> pandoc.table(head(iris[,1:3], 2), justify = c('right', 'center', 'left'))

-------------------------------------------
  Sepal.Length  Sepal.Width  Petal.Length  
-------------- ------------- --------------
           5.1      3.5      1.4           

           4.9       3       1.4           
-------------------------------------------

```

Or pre-define the alignment for (all future) `pandoc.table` or the `pander` [S3 generic method](#generic-pander-method) by a helper function:

```rout
> set.alignment('left', row.names = 'right')
> pandoc.table(mtcars[1:2,  1:5])

--------------------------------------------------
             &nbsp; mpg   cyl   disp   hp   drat  
------------------- ----- ----- ------ ---- ------
      **Mazda RX4** 21    6     160    110  3.9   

  **Mazda RX4 Wag** 21    6     160    110  3.9   
--------------------------------------------------

```

Just like with [captions](#caption), you can also specify the `permanent` option to be `TRUE` to update the default cell alignment for all future tables. And beside using `set.alignment` helper function or passing parameters directly to `pandoc.table`, you may also set the default alignment styles with [`panderOptions`](#pander-options).

What's even more fun, you can specify a function that takes the R object as its argument to compute some unique alignment for your table based on e.g. column values or variable types:

```rout
> panderOptions('table.alignment.default',
+   function(df)
+     ifelse(sapply(df, mean) > 2, 'left', 'right'))
> pandoc.table(head(iris[,1:3], 2))

-------------------------------------------
Sepal.Length   Sepal.Width     Petal.Length
-------------- ------------- --------------
5.1            3.5                      1.4

4.9            3                        1.4
-------------------------------------------

```

## Table and cell width

`pandoc.table` can also deal with the problem of really **wide tables**. Ever had an issue in LaTeX or MS Word when tried to print a correlation matrix of 40 variables? Not a problem any more as you can split up the table with auto-added captions. The `split.table` option defaults to 80 characters:

```rout
> pandoc.table(mtcars[1:2, ], style = "grid", caption = "Hello caption!")

+---------------------+-------+-------+--------+------+--------+-------+
|       &nbsp;        |  mpg  |  cyl  |  disp  |  hp  |  drat  |  wt   |
+=====================+=======+=======+========+======+========+=======+
|    **Mazda RX4**    |  21   |   6   |  160   | 110  |  3.9   | 2.62  |
+---------------------+-------+-------+--------+------+--------+-------+
|  **Mazda RX4 Wag**  |  21   |   6   |  160   | 110  |  3.9   | 2.875 |
+---------------------+-------+-------+--------+------+--------+-------+

Table: Hello caption! (continued below)

+---------------------+--------+------+------+--------+--------+
|       &nbsp;        |  qsec  |  vs  |  am  |  gear  |  carb  |
+=====================+========+======+======+========+========+
|    **Mazda RX4**    | 16.46  |  0   |  1   |   4    |   4    |
+---------------------+--------+------+------+--------+--------+
|  **Mazda RX4 Wag**  | 17.02  |  0   |  1   |   4    |   4    |
+---------------------+--------+------+------+--------+--------+

```

And too wide cells can also be split by line breaks. The maximum number of characters in a cell is specified by `split.cells` parameter (default to 30), can be a single value, vector (values for each column separately) and relative vector (percentages of `split.tables` parameter):

```rout
> df <- data.frame(a = 'Lorem ipsum', b = 'dolor sit', c = 'amet')
> pandoc.table(df, split.cells = 5)

----------------
  a     b    c  
----- ----- ----
Lorem dolor amet
ipsum  sit      
----------------

> pandoc.table(df, split.cells = c(5, 20, 5))

--------------------
  a       b      c  
----- --------- ----
Lorem dolor sit amet
ipsum               
--------------------

> pandoc.table(df, split.cells = c("80%", "10%", "10%"))

----------------------
     a        b    c  
----------- ----- ----
Lorem ipsum dolor amet
             sit      
----------------------

```

If the `sylly` package is installed, `pandoc.table` can even split the cells with hyphening support:

```rout
> pandoc.table(data.frame(baz = 'foobar'), use.hyphening = TRUE, split.cells = 3)

-----
 baz 
-----
foo- 
 bar 
-----

```

## Minor features

Funtionality described in other sections is most notable, but `pander/pandoc.table` also has smaller nifty features that are worth mentioning:

* `plain.ascii` - allows to have the output without `markdown` markup:

```rout
> pandoc.table(mtcars[1:3, 1:4])

-------------------------------------------
      &nbsp;         mpg   cyl   disp   hp 
------------------- ----- ----- ------ ----
   **Mazda RX4**     21     6    160   110 

 **Mazda RX4 Wag**   21     6    160   110 

  **Datsun 710**    22.8    4    108    93 
-------------------------------------------

> pandoc.table(mtcars[1:3, 1:4], plain.ascii = TRUE)

-------------------------------------------
                     mpg   cyl   disp   hp 
------------------- ----- ----- ------ ----
     Mazda RX4       21     6    160   110 

   Mazda RX4 Wag     21     6    160   110 

    Datsun 710      22.8    4    108    93 
-------------------------------------------

```

* `missing` - set a string to replace missing values:

```rout
> m <- mtcars[1:3, 1:5]
> m$mpg <- NA
> pandoc.table(m, missing = '?')

--------------------------------------------------
      &nbsp;         mpg   cyl   disp   hp   drat 
------------------- ----- ----- ------ ---- ------
   **Mazda RX4**      ?     6    160   110   3.9  

 **Mazda RX4 Wag**    ?     6    160   110   3.9  

  **Datsun 710**      ?     4    108    93   3.85 
--------------------------------------------------

```

* `keep.line.breaks` - allows to preserve line breaks inside cells. Not that by default `pandoc.table` automatically omits all line breaks found in each table cell to be able to apply the `table.split` functionality.

```rout
> m <- data.frame(a="foo\nbar", b="pander")
> pandoc.table(m)

--------------
   a      b   
------- ------
foo bar pander
--------------

> pandoc.table(m, keep.line.breaks = TRUE)

----------
 a    b   
--- ------
foo pander
bar       
----------

```

To see all possible options, please check [`?pandoc.table`](http://www.rdocumentation.org/packages/pander/functions/pandoc.table.return)

And please note, that all above mentioned features are also supported by the `pander` [generic S3 method](#generic-pander-method) and defaults can be updated via [`panderOptions`](#pander-options) for permanent settings.

# Generic pander method

`pander` or `pandoc` (call as you wish) can deal with a bunch of R object types as being a pandocized `S3` generic method with a variety of already supported classes:

<a id="supported-r-classes"></a>

```rout
> methods(pander)
 [1] pander.anova*           pander.aov*             pander.aovlist*         pander.Arima*           pander.call*
 [6] pander.cast_df*         pander.character*       pander.clogit*          pander.coxph*           pander.cph*
[11] pander.CrossTable*      pander.data.frame*      pander.Date*            pander.default*         pander.density*
[16] pander.describe*        pander.evals*           pander.factor*          pander.formula*         pander.ftable*
[21] pander.function*        pander.glm*             pander.Glm*             pander.gtable*          pander.htest*
[26] pander.image*           pander.irts*            pander.list*            pander.lm*              pander.lme*
[31] pander.logical*         pander.lrm*             pander.manova*          pander.matrix*          pander.microbenchmark*
[36] pander.mtable*          pander.name*            pander.nls*             pander.NULL*            pander.numeric*
[41] pander.ols*             pander.orm*             pander.polr*            pander.POSIXct*         pander.POSIXlt*
[46] pander.prcomp*          pander.randomForest*    pander.rapport*         pander.rlm*             pander.sessionInfo*
[51] pander.smooth.spline*   pander.stat.table*      pander.summary.aov*     pander.summary.aovlist* pander.summary.glm*
[56] pander.summary.lm*      pander.summary.lme*     pander.summary.manova*  pander.summary.nls*     pander.summary.polr*
[61] pander.summary.prcomp*  pander.summary.rms*     pander.summary.survreg* pander.summary.table*   pander.survdiff*
[66] pander.survfit*         pander.survreg*         pander.table*           pander.tabular*         pander.ts*
[71] pander.zoo*
```

If you think that pander lacks support for any other R class(es), please feel free to open a [ticket](https://github.com/Rapporter/pander/pulls) suggesting a new feature or submit [pull request](https://github.com/Rapporter/pander/issues) and we will be happy to extend the package.

Besides the most basic R object types (vectors, matrices, tables or data frames), list-support might be interesting for you:

```rout
> pander(list(a = 1, b = 2, c = table(mtcars$am), x = list(myname = 1, 2), 56))

```

A nested list can be seen above with a table and all (optional) list names. As a matter of fact, `pander.list` is the default method of `pander` too, when you call it on an unsupported R object class:

```rout
> x <- chisq.test(table(mtcars$am, mtcars$gear))
> class(x) <- "I've never heard of!"
> pander(x)
 **WARNING**^[Chi-squared approximation may be incorrect]

  * **statistic**:

    -----------
     X-squared
    -----------
       20.94
    -----------

  * **parameter**:

    ----
     df
    ----
     2
    ----

  * **p.value**: _2.831e-05_
  * **method**: Pearson's Chi-squared test
  * **data.name**: table(mtcars$am, mtcars$gear)
  * **observed**:

    -------------------
    &nbsp;   3   4   5
    ------- --- --- ---
     **0**  15   4   0

     **1**   0   8   5
    -------------------

  * **expected**:

    -------------------------
    &nbsp;    3     4     5
    ------- ----- ----- -----
     **0**  8.906 7.125 2.969

     **1**  6.094 4.875 2.031
    -------------------------

  * **residuals**:

    ----------------------------
    &nbsp;    3      4      5
    ------- ------ ------ ------
     **0**  2.042  -1.171 -1.723

     **1**  -2.469 1.415  2.083
    ----------------------------

  * **stdres**:

    ----------------------------
    &nbsp;    3      4      5
    ------- ------ ------ ------
     **0**  4.395  -2.323 -2.943

     **1**  -4.395 2.323  2.943
    ----------------------------

<!-- end of list -->

```

So `pander` showed a not known class in an (almost) user-friendly way. And we got some warnings too styled with [Pandoc **footnote**](http://johnmacfarlane.net/pandoc/README.html#footnotes)! If that document is exported to e.g. `HTML` or `pdf`, then the error/warning message could be found on the bottom of the page with a link. *Note*: there were two warnings in the above call - both captured and returned! Well, this is the feature of `Pandoc.brew`, see [below](#brew-to-pandoc).

But the output of different **statistical methods** are tried to be prettyfied. Some the above call normally returns like:

```rout
> pander(chisq.test(table(mtcars$am, mtcars$gear)))

-------------------------------------
 Test statistic   df      P value    
---------------- ---- ---------------
     20.94        2   2.831e-05 * * *
-------------------------------------

Table: Pearson's Chi-squared test: `table(mtcars$am, mtcars$gear)`

 **WARNING**^[Chi-squared approximation may be incorrect]
```

A few other examples on the supported R classes:

```rout
> pander(t.test(extra ~ group, data = sleep))

---------------------------------------------------------
 Test statistic   df    P value   Alternative hypothesis 
---------------- ----- --------- ------------------------
     -1.861      17.78  0.07939         two.sided        
---------------------------------------------------------

Table: Welch Two Sample t-test: `extra` by `group`

> ## Dobson (1990) Page 93: Randomized Controlled Trial (examples from: ?glm)
> counts <- c(18, 17, 15, 20, 10, 20, 25, 13, 12)
> outcome <- gl(3, 1, 9)
> treatment <- gl(3, 3)
> m <- glm(counts ~ outcome + treatment, family = poisson())
> pander(m)

--------------------------------------------------------------
     &nbsp;        Estimate   Std. Error   z value   Pr(>|z|) 
----------------- ---------- ------------ --------- ----------
 **(Intercept)**    3.045       0.1709      17.81   5.427e-71 

  **outcome2**     -0.4543      0.2022     -2.247    0.02465  

  **outcome3**      -0.293      0.1927      -1.52     0.1285  

 **treatment2**   1.338e-15      0.2      6.69e-15      1     

 **treatment3**   1.421e-15      0.2      7.105e-15     1     
--------------------------------------------------------------

Table: Fitting generalized (poisson/log) linear model: counts ~ outcome + treatment

> pander(anova(m))

--------------------------------------------------------
    &nbsp;       Df   Deviance   Resid. Df   Resid. Dev 
--------------- ---- ---------- ----------- ------------
   **NULL**      NA      NA          8         10.58    

  **outcome**    2     5.452         6         5.129    

 **treatment**   2   2.665e-15       4         5.129    
--------------------------------------------------------

Table: Analysis of Deviance Table

> pander(aov(m))

-----------------------------------------------------------
    &nbsp;       Df   Sum Sq    Mean Sq   F value   Pr(>F) 
--------------- ---- --------- --------- --------- --------
  **outcome**    2     92.67     46.33     2.224    0.2242 

 **treatment**   2   8.382e-31 4.191e-31 2.012e-32    1    

 **Residuals**   4     83.33     20.83      NA        NA   
-----------------------------------------------------------

Table: Analysis of Variance Model

> pander(prcomp(USArrests))

-------------------------------------------------
    &nbsp;       PC1     PC2      PC3      PC4   
-------------- ------- -------- -------- --------
  **Murder**   0.0417  -0.04482 0.07989  -0.9949 

 **Assault**   0.9952  -0.05876 -0.06757 0.03894 

 **UrbanPop**  0.04634  0.9769  -0.2005  -0.05817

   **Rape**    0.07516  0.2007   0.9741  0.07233 
-------------------------------------------------

Table: Principal Components Analysis

> pander(density(mtcars$hp))

--------------------------------------------
   &nbsp;      Coordinates   Density values 
------------- ------------- ----------------
  **Min.**       -32.12          5e-06      

 **1st Qu.**      80.69        0.0004068    

 **Median**       193.5         0.001665    

  **Mean**        193.5         0.002214    

 **3rd Qu.**      306.3         0.00409     

  **Max.**        419.1         0.006051    
--------------------------------------------

Table: Kernel density of *mtcars$hp* (bandwidth: 28.04104)

> ## Don't like scientific notation?
> panderOptions('round', 2)
> pander(density(mtcars$hp))

--------------------------------------------
   &nbsp;      Coordinates   Density values 
------------- ------------- ----------------
  **Min.**       -32.12            0        

 **1st Qu.**      80.69            0        

 **Median**       193.5            0        

  **Mean**        193.5            0        

 **3rd Qu.**      306.3            0        

  **Max.**        419.1           0.01      
--------------------------------------------

Table: Kernel density of *mtcars$hp* (bandwidth: 28.04104)

```

And of course tables are formatted (e.g. auto add of line breaks, splitting up tables, hyphenation support or markdown format) based on the user specified [`panderOptions`](#pander-options).

# Creating Pandoc documents

The package is also capable of creating complex Pandoc documents (reports) from **R objects** in multiple ways:

  * create somehow a markdown text file (e.g. with `brew`, `knitr` or any scripts of yours, maybe with `Pandoc.brew` - see just [below](#brew-to-pandoc)) and transform that to other formats (like HTML, odt, PDF, docx etc.) with `Pandoc.convert` - similarly to [`pandoc` function in knitr](http://yihui.name/knitr/demo/pandoc/). Basically this is a wrapper around a [Pandoc](http://johnmacfarlane.net/pandoc/) call, which has not much to do with R actually.

  * users might write some reports with literate programming (similar to `knitr`) in a forked version of [brew](https://cran.r-project.org/package=brew) syntax resulting. This means that the user can include R code chunks in a document, and brewing that results in a pretty Pandoc's markdown document and also in a **bunch of other formats** (like HTML, odt, PDF, docx etc.). The great advantage of this [function](#brew-to-pandoc) is that you do not have to transform your R objects to markdown manually, it's all handled automagically.

    *Example*: this [`README.md`](https://github.com/Rapporter/pander/blob/master/README.md) is cooked with [`Pandoc.brew`](#brew-to-pandoc) based on [`inst/README.brew`](https://github.com/Rapporter/pander/blob/master/inst/README.brew) and also exported to [HTML](http://rapporter.github.com/pander/). Details can be found [below](#brew-to-pandoc) or head directly to [examples](#examples).

<!-- endlist -->

 * and users might create a report in a live R session by adding some R objects and paragraphs to a `Pandoc` reference class object. Details can be found [below](#live-report-generation).

## Brew to Pandoc

The [brew](https://cran.r-project.org/package=brew) package, which is a templating framework for report generation, has not been updated since 2011, but it's still used in bunch of R projects based on its simple design and useful features in literate programming. For a quick overview, please see the following documents if you are not familiar with `brew`:

  * [slides on "Building a reporting sytem with BREW"](http://www.slideshare.net/xavierguardiola/building-a-reporting-sytem-with-brew)
  * [learnr blogpost on brew](http://learnr.wordpress.com/2009/09/09/brew-creating-repetitive-reports/)

**In short**: a `brew` document is a simple text file with some special tags. `Pandoc.brew` uses only two of them (as building on a personalized version of Jeff's really great `brew` function):

  * `<% ... %>` stand for running inline R commands as usual,
  * `<%= ... %>` does pretty much the same but applies `pander` to the returning R object (instead of `cat` like the original `brew` function does). So putting there any R object, it would return in a nice Pandoc's markdown format with all possible error/warning messages etc.

This latter tries to be smart in some ways:

  * A code chunk block (R commands between the tags) can return any number of values at any part of the block.
  * Plots and images are grabbed in the document, rendered to a `png` file and `pander` method would result in a Pandoc markdown formatted image link. This means that the image would be rendered/shown/included in the exported document.
  * All warnings/messages and errors are recorded in the blocks and returned in the document as footnotes or inline messages.
  * All heavy R commands (e.g. those taking more then 0.1 sec to evaluate) are [**cached**](#caching) so re`brew`ing a report would not result in a coffee break.

Besides this, the custom `brew` function can do more and also less compared to the original [`brew` package](https://cran.r-project.org/package=brew). First of all, the internal caching mechanism of `brew` has been removed and rewritten for some extra profits besides improved caching.

For example now multiple R expressions can be passed between the `<%= ... %>` tags, and not only the text results, but **the evaluated R objects** are also (invisibly) returned in a structured list. This can be really useful while post-processing the results of `brew`. Quick example:

```rout
> str(Pandoc.brew(text ='
+   Pi equals to `<%= pi %>`.
+   And here are some random data:
+   `<%= runif(10) %>`
+ '))

Pi equals to _3.142_.
And here are some random data:
_0.6631_, _0.849_, _0.06986_, _0.3343_, _0.5209_, _0.3471_, _0.866_, _0.05548_, _0.8933_ and _0.2121_

List of 2
 $ :List of 4
  ..$ type  : chr "text"
  ..$ text  :List of 2
  .. ..$ raw : chr "Pi equals to _3.142_.\nAnd here are some random data:\n"
  .. ..$ eval: chr "Pi equals to _3.142_.\nAnd here are some random data:\n"
  ..$ chunks:List of 2
  .. ..$ raw : chr "_3.142_"
  .. ..$ eval: chr "_3.142_"
  ..$ msg   :List of 3
  .. ..$ messages: NULL
  .. ..$ warnings: NULL
  .. ..$ errors  : NULL
 $ :List of 2
  ..$ type   : chr "block"
  ..$ robject:List of 6
  .. ..$ src   : chr "runif(10)"
  .. ..$ result: num [1:10] 0.6631 0.849 0.0699 0.3343 0.5209 ...
  .. ..$ output: chr "_0.6631_, _0.849_, _0.06986_, _0.3343_, _0.5209_, _0.3471_, _0.866_, _0.05548_, _0.8933_ and _0.2121_"
  .. ..$ type  : chr "numeric"
  .. ..$ msg   :List of 3
  .. .. ..$ messages: NULL
  .. .. ..$ warnings: NULL
  .. .. ..$ errors  : NULL
  .. ..$ stdout: NULL
  .. ..- attr(*, "class")= chr "evals"
```

This document was generated by `Pandoc.brew` based on [`inst/README.brew`](https://github.com/Rapporter/pander/blob/master/inst/README.brew) so the above examples were generated automatically by running:

```r
Pandoc.brew(system.file('README.brew', package = 'pander'))
```

The output is set to `stdout` by default, which means that the resulting text is written to the R console. But setting the `output` to a text file and running Pandoc on that to create a `HTML`, `odt`, `docx` or other document in one go is also possible. To export a brewed file to other then Pandoc's markdown, please use the `convert` parameter. For example:

```r
text <- paste('# Header',
              '',
              'What a lovely list:\n<%= as.list(runif(10)) %>',
              'A wide table:\n<%= mtcars[1:3, ] %>',
              'And a nice chart:\n\n<%= plot(1:10) %>',
          sep = '\n')
Pandoc.brew(text = text, output = tempfile(), convert = 'html')
Pandoc.brew(text = text, output = tempfile(), convert = 'pdf')
```

So to brew this README with all R chunks automatically converted to html, please run:

```r
Pandoc.brew(system.file('README.brew', package='pander'), output = tempfile(), convert = 'html')
```

### Examples

The package bundles some examples for `Pandoc.brew` to let you check its features pretty fast. These are:

  * [minimal.brew](https://github.com/Rapporter/pander/blob/master/inst/examples/minimal.brew)
  * [short-code-long-report.brew](https://github.com/Rapporter/pander/blob/master/inst/examples/short-code-long-report.brew)
  * [graphs.brew](https://github.com/Rapporter/pander/blob/master/inst/examples/graphs.brew)

To `brew` these examples on your machine, try to run the followings commands:

```r
Pandoc.brew(system.file('examples/minimal.brew', package='pander'))
Pandoc.brew(system.file('examples/minimal.brew', package='pander'), output = tempfile(), convert = 'html')

Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'))
Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'), output = tempfile(), convert = 'html')

Pandoc.brew(system.file('examples/graphs.brew', package='pander'))
Pandoc.brew(system.file('examples/graphs.brew', package='pander'), output = tempfile(), convert = 'html')
```

For easier access, I have uploaded some exported documents of the above examples as well:

  * minimal.brew: [markdown](http://rapporter.github.com/pander/minimal.md) [html](http://rapporter.github.com/pander/minimal.html) [pdf](http://rapporter.github.com/pander/minimal.pdf) [odt](http://rapporter.github.com/pander/minimal.odt) [docx](http://rapporter.github.com/pander/minimal.docx)
  * short-code-long-report.brew: [markdown](http://rapporter.github.com/pander/short-code-long-report.md) [html](http://rapporter.github.com/pander/short-code-long-report.html) [pdf](http://rapporter.github.com/pander/short-code-long-report.pdf) [odt](http://rapporter.github.com/pander/short-code-long-report.odt) [docx](http://rapporter.github.com/pander/short-code-long-report.docx)
  * graphs.brew: [markdown](http://rapporter.github.com/pander/graphs.md) [html](http://rapporter.github.com/pander/graphs.html) [pdf](http://rapporter.github.com/pander/graphs.pdf) [odt](http://rapporter.github.com/pander/graphs.odt) [docx](http://rapporter.github.com/pander/graphs.docx)

Please check out `pdf`, `docx`, `odt` and other formats by changing the above `convert` option on your machine, and do not forget to [give some feedback](https://github.com/Rapporter/pander/issues)!

## Live report generation

`pander` package has a special reference class called `Pandoc` which could collect some blocks in a live R session and export the whole document to Pandoc/PDF/HTML etc. Without any serious further explanations, please check out the below (self-commenting) example:

```r
## Initialize a new Pandoc object
myReport <- Pandoc$new()

## Add author, title and date of document
myReport$author <- 'Gergely Daróczi'
myReport$title  <- 'Demo'

## Or it could be done while initializing
myReport <- Pandoc$new('Gergely Daróczi', 'Demo')

## Add some free text
myReport$add.paragraph('Hello there, this is a really short tutorial!')

## Add maybe a header for later stuff
myReport$add.paragraph('# Showing some raw R objects below')

## Adding a short matrix
myReport$add(matrix(5,5,5))

## Or a table with even
myReport$add.paragraph('Hello table:')
myReport$add(table(mtcars$am, mtcars$gear))

## Or a "large" data frame which barely fits on a page
myReport$add(mtcars)

## And a simple linear model with Anova tables
ml <- with(lm(mpg ~ hp + wt), data = mtcars)
myReport$add(ml)
myReport$add(anova(ml))
myReport$add(aov(ml))

## And do some principal component analysis at last
myReport$add(prcomp(USArrests))

## Sorry, I did not show how Pandoc deals with plots:
myReport$add(plot(1:10))

## Want to see the report? Just print it:
myReport

## Exporting to PDF (default)
myReport$export()

## Or to docx in tempdir():
myReport$format <- 'docx'
myReport$export(tempfile())

## You do not want to see the generated report after generation?
myReport$export(open = FALSE)
```

# Capturing evaluation information with evals

When working on the [rapport package](http://rapport-package.info), I really needed some nifty R function that can evaluate R expression along with capturing errors and warnings. Unfortunately the `evaluate` package had only limited features at that time, as it could not return the raw R object, but only the standard output with messages. So I wrote my own function, and soon some further feature requests arose, like identifying if an R expression results in a plot etc. This section aims to give a quick introduction to the functionality of `evals`, but for more usage/implementation details, please refer to specialized vignette, which can be accessed by `vignette('evals', package='pander')` or available online [here](http://rapporter.github.io/pander/evals.html).

But probably it's easier to explain what `evals` can do with a simple example:

```rout
> evals('1:10')
[[1]]
$src
[1] "1:10"

$result
 [1]  1  2  3  4  5  6  7  8  9 10

$output
[1] " [1]  1  2  3  4  5  6  7  8  9 10"

$type
[1] "integer"

$msg
$msg$messages
NULL

$msg$warnings
NULL

$msg$errors
NULL

$stdout
NULL

attr(,"class")
[1] "evals"
```

So `evals` can evaluate a character vector of R expressions, and it returns a list of captured stuff while running those:

 * `src` holds the R expression,
 * `result` contains the raw R object as is,
 * `output` represents how the R object is printed to the standard output,
 * `type` is the `class` of the returned R object,
 * `msg` is a list of possible messages captured while running the R expression and
 * `stdout` contains if anything was written to the standard output.

Besides capturing this nifty list of important circumstances, `evals` can automatically identify if an R expression is returning anything to a graphical device, and can save the resulting image in a variety of file formats along with some extra options, like applying a custom theme on base, `lattice` or `ggplot2` plots:

```rout
> evals('hist(mtcars$hp)')[[1]]$result
![](plots/plot-1.png)
```

So instead of a captured R object (which would be `NULL` in this situation by the way), we get the path of the image where the plot was saved:

![](https://raw.githubusercontent.com/Rapporter/pander/gh-pages/plots/graphs-1.png)

Well, this is not a standard histogram usually returned by the `hist` function, right? As mentioned before, `evals` have some extra features like applying the user defined theme on various plots automatically. Please see the `graphs.brew` example [above](#examples) for further details, or check the related [global options](#evals-options). If you do not like this feature, simply add `evalsOptions('graph.unify', FALSE)` to your `.Rprofile`.

Further features are described in the [technical docs](http://www.rdocumentation.org/packages/pander/functions/evals), and now I'll only give a brief introduction to another important feature of `evals`.

## Caching

As `pander::evals` is using a **custom caching algorithm** in the means of evaluating R expressions, it might be worthwhile to give a short summary of what is going on in the background when you are running e.g. [`Pandoc.brew`](#brew-to-pandoc), the ["live report generation"](#live-report-generation) engine or `evals` directly:

  * Each passed R chunk is `parse`d to single R expressions.
  * Each parsed expression's **part** (let it be a function, variable, constant etc.) is evaluated (as `name`) separately to a `list`. This list describes the unique structure *and* the content of the passed R expressions. This has some really great benefits (see below).
  * A **hash** is computed of each list element and *cached* too in `pander`'s local environments. This is useful if you are using large data frames, just imagine: the caching algorithm would have to compute the hash for the same data frame each time it's touched! This way the hash is recomputed only if the R object with the given name is changed.
  * The list of such R objects is serialized, then an `SHA-1` hash is computed, which is unique and there is no real risk of collision.
  * If [`evals`](#evals) can find the cached results in an environment of `pander`'s namespace (if `cache.mode` set to `enviroment` - see [below](#pander-options)) or in a file named to the computed hash (if `cache.mode` set to `disk`), then it is returned on the spot. *The objects modified/created by the cached code are also updated.*
  * Otherwise the call is evaluated and the results and the modified R objects of the environment are optionally saved to cache (e.g. if `cache` is active *and* if the `proc.time()` of the evaluation is higher then it is defined in `cache.time` - see details in [evals' options](#evals-options)).

<a id="in-practice"></a>

As `pander` does not cache based on raw sources of chunks and there is no easy way of enabling/disabling caching on a chunk basis, the users have to live with some *great advantages* and some *minor tricky situations* - which latter cannot be solved theoretically in my opinion, but [I'd love to hear your feedback](https://github.com/Rapporter/pander/issues).

The caching hash is computed based on the structure and **content** of the R commands instead of the used variable names or R expressions, so let us make some POC example to show the **greatest asset**:

```r
x <- mtcars$hp
y <- 1e3
evals('sapply(rep(x, y), mean)')
```

It took a while, huh? :)

Let us create some custom functions and variables, which are not identical to the above call:

```r
f <- sapply
g <- rep
h <- mean
X <- mtcars$hp * 1
Y <- 1000
```

And now try to run something like:

```r
evals('f(g(X, Y), h)')
```

Yes, it was returned from cache!

About the **kickback**:

As `pander` (or rather: `evals`) does not really deal with what is written in the provided sources but rather checks what is **inside that**, there might be some tricky situations where you would expect the cache to work, but it would not. Short example: we are computing and saving to a variable something heavy in a chunk (please run these in a clean R session to avoid conflicts):

```r
evals('x <- sapply(rep(mtcars$hp, 1e3), mean)')
```

It is cached, just run again, you will see.

But if you would create `x` in your *global environment* with any value (which has nothing to do with the special environment of the report!) **and** `x` was not defined in the report before this call (**and** you had no `x` value in your global environment before), then the content of `x` would result in a new hash for the cache - so caching would not work. E.g.:

```r
x <- 'foobar'
evals('x <- sapply(rep(mtcars$hp, 1e3), mean)')
```

I really think this is a minor issue (with very special coincidences) which cannot be addressed cleverly - but **could be avoided with some cautions** (e.g. run `Pandoc.brew` in a clean R session like with `Rscript` or [`littler`](http://dirk.eddelbuettel.com/code/littler.html) - if you are really afraid of this issue). And after all: you loose nothing, just the cache would not work for that only line and only once in most of the cases.

Other cases when the hash of a call will not match cached hashes:

  * a number is replaced by a variable holding the number, e.g.: `evals('1:5')` vs. `x <- 1:5;evals('x')`
  * a part of an R object is replaced by a variable holding that, e.g.: `evals('mean(mtcars$hp)')` vs. `x <- mtcars$hp;evals('mean(x)')`

But the e.g. following do work from cache fine:

```
x  <- mtcars$hp
xx <- mtcars$hp*1
evals('mean(x)')
evals('mean(xx)')
```

<a id='pander-options'></a><a id='panderoptions'></a>

# General options

The package comes with a variety of globally adjustable options, which have an effect on the result of your reports. You can query and update these options with the `panderOptions` function:

  * `digits`: numeric (default: `2`) passed to `format`.  Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
  * `decimal.mark`: string (default: `.`) passed to `format`
  * `formula.caption.prefix`: string (default: `Formula: `) passed to `pandoc.formula` to be used as caption prefix. Be sure about what you are doing if changing to other than `Formula: ` or `:`.
  * `big.mark`: string (default: `''`) passed to `format`
  * `round`: numeric (default: `Inf`) passed to `round`. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
  * `keep.trailing.zeros`: boolean (default: `FALSE`) show or remove trailing zeros in numbers (e.g. in numeric vectors or in columns of tables with numeric values)
  * `keep.line.breaks`: boolean (default: `FALSE`) to keep or remove line breaks from cells in a table
  * `missing`: string (default: `NA`) to replace missing values in vectors, tables etc.
  * `date`: string (default: `'%Y/%m/%d %X'`) passed to `format` when printing dates (`POSIXct` or `POSIXt`)
  * `header.style`: `'atx'` or `'setext'` passed to `pandoc.header`
  * `list.style`: `'bullet'` (default), `'ordered'` or `'roman'` passed to `pandoc.list`. Please not that this has no effect on `pander` methods.
  * `table.style`: `'multiline'`, `'grid'` or `'simple'` passed to `pandoc.table`
  * `table.emphasize.rownames`: boolean (default: `TRUE`) if row names should be highlighted
  * `table.split.table`: numeric passed to `pandoc.table` and also affects `pander` methods. This option tells `pander` where to split too wide tables. The default value (`80`) suggests the conventional number of characters used in a line, feel free to change (e.g. to `Inf` to disable this feature) if you are not using a VT100 terminal any more :)
  * `table.split.cells`: numeric (default: 30) passed to `pandoc.table` and also affects pander methods. This option tells pander where to split too wide cells with line breaks. Set `Inf`` to disable.
  * `table.caption.prefix`: string (default: `Table: `) passed to `pandoc.table` to be used as caption prefix. Be sure about what you are doing if changing to other than `Table: ` or `:`.
  * `table.continues`: string (default: `Table continues below`) passed to `pandoc.table` to be used as caption for long (split) without a use defined caption
  * `table.continues.affix`: string (default: `(continued below)`) passed to `pandoc.table` to be used as an affix concatenated to the user defined caption for long (split) tables
  * `table.alignment.default`: string (default: `centre`) that defines the default alignment of cells. Can be `left`, `right` or `centre` that latter can be also spelled as `center`
  * `table.alignment.rownames`: string (default: `centre`) that defines the alignment of rownames in tables. Can be `left`, `right` or `centre` that latter can be also spelled as `center`
  * `use.hyphening`: boolean (default: `FALSE`) if try to use hyphening when splitting large cells according to table.split.cells. Requires `sylly` package.
  * `evals.messages`: boolean (default: `TRUE`) passed to `evals`' `pander` method specifying if messages should be rendered
  * `p.wrap`: a string (default:`'_'`) to wrap vector elements passed to `p` function
  * `p.sep`: a string (default: `', '`) with the main separator passed to `p` function
  * `p.copula`: a string (default: `'and'`) a string with ending separator passed to `p` function
  * `plain.ascii`: boolean (default: FALSE) to define if output should be in plain ascii or not
  * `graph.nomargin`: boolean (default: `TRUE`) if trying to keep plots' margins at minimal
  * `graph.fontfamily`: string (default: `'sans'`) specifying the font family to be used in images. Please note, that using a custom font on Windows requires `grDevices:::windowsFonts` first.
  * `graph.fontcolor`: string (default: `'black'`) specifying the default font color
  * `graph.fontsize`: numeric (default: `12`) specifying the *base* font size in pixels. Main title is rendered with `1.2` and labels with `0.8` multiplier.
  * `graph.grid`: boolean (default: `TRUE`) if a grid should be added to the plot
  * `graph.grid.minor`: boolean (default: `TRUE`) if a miner grid should be also rendered
  * `graph.grid.color`: string (default: `'grey'`) specifying the color of the rendered grid
  * `graph.grid.lty`: string (default: `'dashed'`) specifying the line type of grid
  * `graph.boxes`: boolean (default: `FALSE`) if to render a border around of plot (and e.g. around strip)
  * `graph.legend.position`: string (default: `'right'`) specifying the position of the legend: 'top', 'right', 'bottom' or 'left'
  * `graph.background`: string (default: `'white'`) specifying the plots main background's color
  * `graph.panel.background`: string (default: `'transparent'`) specifying the plot's main panel background. Please *note*, that this option is not supported with `base` graphics.
  * `graph.colors`: character vector of default color palette (defaults to a [colorblind theme](http://jfly.iam.u-tokyo.ac.jp/color/)). Please *note* that this update work with `base` plots by appending the `col` argument to the call if not set.
  * `graph.color.rnd`: boolean (default: `FALSE`) specifying if the palette should be reordered randomly before rendering each plot to get colorful images
  * `graph.axis.angle`: numeric (default: `1`) specifying the angle of axes' labels. The available options are based on `par(les)` and sets if the labels should be:

      *  `1`: parallel to the axis,
      *  `2`: horizontal,
      *  `3`: perpendicular to the axis or
      *  `4`: vertical.

  * `graph.symbol`: numeric (default: `1`) specifying a symbol (see the `pch` parameter of `par`)
  * `knitr.auto.asis`: boolean (default: `TRUE`) if the results of `pander` should be considered as `asis` in `knitr`. Equals to specifying `results='asis'` in the R chunk, so thus there is no need to do so if set to `TRUE`.

<a id='evals-options'></a><a id='evalsoptions'></a>

Besides localization of numeric formats or the styles of tables, lists and plots, there are some technical options as well, which would effect e.g. [caching](#caching) or the format of rendered image files. You can query/update those with the `evalsOptions` function as the main backend of `pander` calls is a custom evaluation function called [`evals`](#evals).

The list of possible options are:

  * `parse`: if `TRUE` the provided `txt` elements would be merged into one string and parsed to logical chunks. This is useful if you would want to get separate results of your code parts - not just the last returned value, but you are passing the whole script in one string. To manually lock lines to each other (e.g. calling a `plot` and on next line adding an `abline` or `text` to it), use a plus char (`+`) at the beginning of each line which should be evaluated with the previous one(s). If set to `FALSE`, [`evals`](#evals) would not try to parse R code, it would get evaluated in separate runs - as provided. Please see the documentation of [`evals`](#evals).
  * `cache`: [caching](#caching) the result of R calls if set to `TRUE`
  * `cache.mode`: cached results could be stored in an `environment` in _current_ R session or let it be permanent on `disk`.
  * `cache.dir`: path to a directory holding cache files if `cache.mode` set to `disk`. Default set to `.cache` in current working directory.
  * `cache.time`: number of seconds to limit caching based on `proc.time`. If set to `0`, all R commands, if set to `Inf`, none is cached (despite the `cache` parameter).
  * `cache.copy.images`: copy images to new file names if an image is returned from the *disk* cache? If set to `FALSE` (default), the cached path would be returned.
  * `classes`: a vector or list of classes which should be returned. If set to `NULL` (by default) all R objects will be returned.
  * `hooks`: list of hooks to be run for given classes in the form of `list(class = fn)`. If you would also specify some parameters of the function, a list should be provided in the form of `list(fn, param1, param2=NULL)` etc. So the hooks would become `list(class1=list(fn, param1, param2=NULL), ...)`. See example of [`evals`](#evals) for more details. A default hook can be specified too by setting the class to `'default'`. This can be handy if you do not want to define separate methods/functions to each possible class, but automatically apply the default hook to all classes not mentioned in the list. You may also specify only one element in the list like: `hooks=list('default' = pander_return)`. Please note, that nor error/warning messages, nor stdout is captured (so: updated) while running hooks!
  * `length`: any R object exceeding the specified length will not be returned. The default value (`Inf`) does not filter out any R objects.
  * `output`: a character vector of required returned values. This might be useful if you are only interested in the `result`, and do not want to save/see e.g. `messages` or `print`ed `output`. See examples of [`evals`](#evals).
  * `graph.unify`: should `evals` try to unify the style of (`base`, `lattice` and `ggplot2`) plots? If set to `TRUE`, some `panderOptions()` would apply. By default this is disabled not to freak out useRs :)
  * `graph.name`: set the file name of saved plots which is `%s` by default. A simple character string might be provided where `%d` would be replaced by the index of the generating `txt` source, `%n` with an incremented integer in `graph.dir` with similar file names and `%t` by some unique random characters. When used in a `brew` file, `%i` is also available which would be replaced by the chunk number.
  * `graph.dir`: path to a directory where to place generated images. If the directory does not exist, [`evals`](#evals) try to create that. Default set to `plots` in current working directory.
  * `graph.output`: set the required file format of saved plots. Currently it could be any of  `grDevices`: `png`, `bmp`, `jpeg`, `jpg`, `tiff`, `svg` or `pdf`. Set to `NA` not to save plots at all and tweak that setting with `capture.plot()` on demand.
  * `width`: width of generated plot in pixels for even vector formats
  * `height`: height of generated plot in pixels for even vector formats
  * `res`: nominal resolution in `ppi`. The height and width of vector images will be calculated based in this.
  * `hi.res`: generate high resolution plots also? If set to `TRUE`, each R code parts resulting an image would be run twice.
  * `hi.res.width`: width of generated high resolution plot in pixels for even vector formats. The `height` and `res` of high resolution image is automatically computed based on the above options to preserve original plot aspect ratio.
  * `graph.env`: save the environments in which plots were generated to distinct files (based on `graph.name`) with `env` extension?
  * `graph.recordplot`: save the plot via `recordPlot` to distinct files (based on `graph.name`) with `recodplot` extension?
  * `graph.RDS` save the raw R object returned (usually with `lattice` or `ggplot2`) while generating the plots to distinct files (based on `graph.name`) with `RDS` extension?
  * `log`: `NULL` or  an optionally passed *logger name* from `futile.logger` to record all info, trace, debug and error messages.

# Difference from other rendering packages

How does `pander` differ from Sweave, brew, knitr, R2HTML and the other tools of literate programming? First of all `pander` can be used as a helper with any other literate programming solution, so **you can call `pander` inside of `knitr` chunks**.

But if you stick with `pander`'s literate programming engine, then there's not much need for calling `ascii`, `xtable`, `Hmisc`, `tables` etc. or even `pander` in the R command chunks to **transform R objects** into markdown, HTML, tex etc. as `Pandoc.brew` automatically results in Pandoc's markdown, which can be converted to almost any text document format. Conversion can be done automatically after calling `pander` reporting functions ([Pander.brew](#brew-to-pandoc) or [Pandoc](#live-report-generation)).

Based on the fact that `pander` transforms R objects into markdown, **no "traditional" R console output is shown** in the resulting document (nor in markdown, nor in exported docs), but **all R objects are transformed to tables, list etc**. Well, there is an option (`show.src`) to show the original R commands before the formatted output, and `pander` calls can be also easily tweaked to return the printed version of the R objects - if you would need that in some strange situation - like writing an R tutorial. But  really think that nor R code, nor raw R results have anything to do with an exported report.

Of course **all warnings, messages and errors are captured** while evaluating R expressions just like `stdout` besides the **raw R objects**. So the resulting report also includes the raw R objects for further edits if needed - which is a very unique feature.

**Graphs and plots are automatically identified** in code chunks and saved to disk in a `png` file linked in the resulting document. This means that if you create a report (e.g. `brew` a text file) and export it to PDF/docx etc. all the plots/images would be there. There are some parameters to specify the resolution of the image and also the type (e.g. `jpg`, `svg` or `pdf`) besides a **wide variety of [theme options](#pander-options)**. About the latter, please check the `graphs.brew` example [above](#examples).

And `pander` uses its built-in (IMHO quite decent) [**caching**](#caching) engine. This means that if the evaluation of some R commands takes too long time (which can be set by option/parameter), then the results are saved in a file and returned from there on next similar R code's evaluation. This caching algorithm tries to be smart, as it not only  checks the passed R sources, but the content of all variables and functions, and saves the hash of those. This is a quite secure way of caching (see details [above](#caching)), but if you would encounter any issues, just switch off the cache. I've not seen any issues for years :)

# ESS

I have created some simple LISP functions which would be handy if you are using the best damn IDE for R. These functions and default key-bindings are [shipped with the package](https://github.com/Rapporter/pander/tree/master/inst/pander.el), feel free to personalize.

As time passed these small functions grew heavier (with my Emacs knowledge) so I ended up with a small library:

## pander-mode

I am currently working on `pander-mode` which is a small *minor-mode* for Emacs. There are a few (but useful) functions with default keybindings:

  * `pander-brew` (`C-c p b`): Run `Pandoc.brew` on current buffer or region (if mark is active), show results in *ess-output* and (optionally) copy results to clipboard while setting working directory to `tempdir()` temporary.
  * `pander-brew-export` (`C-c p B`): Run `Pandoc.brew` on current buffer or region (if mark is active) and export results to specified (auto-complete in minibuffer) format. Also tries to open exported document.
  * `pander-eval` (`C-c p e`): Run `pander` on (automatically evaluated) region *or* current chunk (if marker is not set), show results (of last returned R object) in `*ess-output*` and (optionally) copy those to clipboard while setting working directory to `tempdir()` temporary.

Few options of `pander-mode`: `M-x customize-group pander`

  * `pander-clipboard`: If non-nil then the result of `pander-*` functions would be copied to clipboard.
  * `pander-show-source`: If non-nil then the source of R commands would also show up in generated documents while running 'pander-eval'. This would not affect `brew` functions ATM.

To use this small lib, just type: `M-x pander-mode` on any document. It might be useful to add a hook to `markdown-mode` if you find this useful.

<script type="text/javascript">
    $(document).ready(function() {
        $('#logo').empty();
        $('img[src="https://travis-ci.org/Rapporter/pander.png?branch=master"]').css('border', 'none').css('padding', '0px').parent().parent().css('text-align', 'justify');
        $('img[src="https://coveralls.io/repos/Rapporter/pander/badge.svg?branch=master"]').css('border', 'none').css('padding', '0px').parent().parent().css('text-align', 'justify');
        $('nav').css('height', '100%');
        $("img").unbind("click");
    });
</script>
<a href="https://github.com/Rapporter/pander"><img style="position: fixed; top: -5px; right: -5px; border: 0;" src="https://camo.githubusercontent.com/38ef81f8aca64bb9a64448d0d70f1308ef5341ab/68747470733a2f2f73332e616d617a6f6e6177732e636f6d2f6769746875622f726962626f6e732f666f726b6d655f72696768745f6461726b626c75655f3132313632312e706e67" data-canonical-src="https://s3.amazonaws.com/github/ribbons/forkme_right_darkblue_121621.png"></a> 

.onLoad <- function(libname, pkgname) {
    ## pander settings
    options('pander' = list(
                'digits'                   = 4,
                'decimal.mark'             = '.',
                'formula.caption.prefix'   = 'Formula: ',
                'big.mark'                 = '',
                'round'                    = Inf,
                'keep.trailing.zeros'      = FALSE,
                'keep.line.breaks'         = FALSE,
                'missing'                  = NA,
                'date'                     = '%Y/%m/%d %X',
                'header.style'             = 'atx',
                'list.style'               = 'bullet',
                'table.style'              = ifelse(
                    getOption('jupyter.in_kernel', FALSE),
                    ## The jupyter notebook does not understand the multiline table format
                    'rmarkdown',
                    'multiline'),
                'table.emphasize.rownames' = TRUE,
                'table.split.table'        = 80,
                'table.split.cells'        = 30,
                'table.caption.prefix'     = 'Table: ',
                'table.continues'          = 'Table continues below',
                'table.continues.affix'    = '(continued below)',
                'table.alignment.default'  = 'centre',
                'table.alignment.rownames' = 'centre',
                'use.hyphening'            = FALSE,
                'evals.messages'           = TRUE,
                'p.wrap'                   = '_',
                'p.sep'                    = ', ',
                'p.copula'                 = ' and ',
                'plain.ascii'              = FALSE,
                'graph.nomargin'           = TRUE,
                'graph.fontfamily'         = 'sans',
                'graph.fontcolor'          = 'black',
                'graph.fontsize'           = 12,
                'graph.grid'               = TRUE,
                'graph.grid.minor'         = TRUE,
                'graph.grid.color'         = 'grey',
                'graph.grid.lty'           = 'dashed',
                'graph.boxes'              = FALSE,
                'graph.legend.position'    = 'right',
                'graph.background'         = 'white',
                'graph.panel.background'   = 'transparent',
                'graph.colors'             = c("#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7", "#999999", "#E69F00"), #nolint
                'graph.color.rnd'          = FALSE,
                'graph.axis.angle'         = 1,
                'graph.symbol'             = 1,
                'knitr.auto.asis'          = TRUE,
                'pandoc.binary'            = Sys.which('pandoc')
                ))

    ## evals options
    options('evals' = list(
                'parse'                 = TRUE,
                'cache'                 = TRUE,
                'cache.mode'            = 'environment',
                'cache.dir'             = '.cache',
                'cache.time'            = 0.1,
                'cache.copy.images'     = FALSE,
                'classes'               = NULL,
                'hooks'                 = NULL,
                'length'                = Inf,
                'output'                = 'all',
                'graph.unify'           = FALSE,
                'graph.name'            = '%t',
                'graph.dir'             = 'plots',
                'graph.output'          = 'png',
                'width'                 = 480,
                'height'                = 480,
                'res'                   = 72,
                'hi.res'                = FALSE,
                'hi.res.width'          = 960,
                'graph.env'             = FALSE,
                'graph.recordplot'      = FALSE,
                'graph.RDS'             = FALSE,
                'log'                   = NULL
                ))
}

## general (temporary) storage for pander's stuff
storage           <- new.env()
storage$caption   <- NULL
storage$alignment <- NULL
debug             <- new.env()
debug$nested      <- 0
debug$nestedID    <- 0

## cache storage
cached.results <- new.env() # cache of results from evals
cached.environments <- new.env() # cache of changes to the environment (assignments in particular)

## cache hash storage
hash.cache.obj       <- new.env() # raw R objects of which hash was computed before
hash.cache.hash      <- new.env() # the computed hash of the above R objects
hash.cache.last.used <- new.env() # when was the hash last queried

## masked plots
masked.plots <- new.env()
masked.plots$plot <- masked.plots$barplot <- masked.plots$lines <- masked.plots$pie <- masked.plots$boxplot <- masked.plots$polygon <- masked.plots$points <- masked.plots$legend <- masked.plots$hist <- masked.plots$pairs <- masked.plots$stripchart <- masked.plots$clusplot <- masked.plots$text <- function (...) { #nolint

    mc      <- match.call()
    fn      <- deparse(mc[[1]])
    fn.pkg  <- gsub('.*library/|/help.*', '', help(fn)[1])
    fn.orig <- parse(text = paste0(fn.pkg, '::', fn))[[1]]
    mc      <- match.call(get(fn, envir = .GlobalEnv))

    if (!(!is.null(mc$plot) && !mc$plot)) {

        ## pander options
        fc  <- panderOptions('graph.fontcolor')
        fbs <- panderOptions('graph.fontsize')
        bc  <- panderOptions('graph.background')
        gc  <- panderOptions('graph.grid.color')
        cex <- fbs / 12
        cs <- panderOptions('graph.colors')
        if (panderOptions('graph.color.rnd'))
            cs <- sample(cs)
        cb <- cs[1]

        ## global par update
        if (!fn %in% c('text')) {
            par(
                family   = panderOptions('graph.fontfamily'),
                cex      = cex, cex.axis = cex * 0.8, cex.lab = cex, cex.main = cex * 1.2, cex.sub = cex,
                bg       = bc, # nolint TODO: how could we color only the inner plot area globally? Not like: https://stat.ethz.ch/pipermail/r-help/2003-May/033971.html
                las      = panderOptions('graph.axis.angle'),
                lwd      = 2,
                pch      = panderOptions('graph.symbol'),
                col.axis = fc, col.lab = fc, col.main = fc, col.sub = fc)
        }

        ## remove margins

        if (panderOptions('graph.nomargin') & !fn %in% c('text')) {
            par(mar = c(4.1, 4.3, 2.1, 0.1))
        }

        ## default: grid is added to all plots
        doAddGrid <- TRUE

        ## update colors
        if (is.null(mc$col) & is.null(mc$color))
            mc$col <- cb
        if (fn == 'boxplot')
            mc$border <- 'black'
        if (fn == 'clusplot') {
            mc$col <- NULL
            if (is.null(mc$color))
                mc$color <- TRUE
            if (is.null(mc$shade))
                mc$shade <- TRUE
            if (is.null(mc$labels))
                mc$labels <- 4
            if (is.null(mc$col.p))
                mc$col.p <- 'black'
            if (is.null(mc$col.clus))
                mc$col.clus <- cs
        }

        ## remove boxes
        if (fn %in% c('pairs', 'stripchart')) {
            doAddGrid <- FALSE
            par(fg = fc)
        } else {
            if (panderOptions('graph.boxes'))
                par(fg = gc)
            else
                par(fg = bc)
        }

        if (fn == 'pie')
            mc$col <- cs

    }

    ## call
    mc[[1]] <- fn.orig
    eval(mc, envir = parent.frame())

    ## grid
    if (all(par()$mfrow == 1) & panderOptions('graph.grid') & doAddGrid & !(!is.null(mc$plot) && !mc$plot)) {

        g <- tryCatch(grid(lty = panderOptions('graph.grid.lty'),
                           col = panderOptions('graph.grid.color'),
                           lwd = 0.5),
                      error = function(e) e)
        if (!inherits(g, 'error')) {
            if (panderOptions('graph.grid.minor'))
                g <- tryCatch(add.minor.ticks(2, 2, grid = TRUE), error = function(e) e)
        }
        if (inherits(g, 'error'))
            warning('Applying default formatting to image is somehow compromised (the result could differ from what you specified in `panderOptions`). Hints: printing `lattice`/`ggplot2` is not needed and tweaking `base` plots with `par` might have some side-effects!') #nolint

    }
}


#' Querying/setting pander option
#'
#' To list all \code{pander} options, just run this function without any parameters provided. To query only one value, pass the first parameter. To set that, use the \code{value} parameter too.
#'
#' The following \code{pander} options are available:
#'
#' \itemize{
#'      \item \code{digits}: numeric (default: \code{2}) passed to \code{format}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
#'      \item \code{decimal.mark}: string (default: \code{.}) passed to \code{format}
#'      \item \code{formula.caption.prefix}: string (default: \code{'Formula: '}) passed to \code{\link{pandoc.formula}} to be used as caption prefix. Be sure about what you are doing if changing to other than \code{'Formula: '} or \code{':'}.
#'      \item \code{big.mark}: string (default: '') passed to \code{format}.
#'      \item \code{round}: numeric (default: \code{Inf}) passed to \code{round}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
#'      \item \code{keep.trailing.zeros}: boolean (default: \code{FALSE}) to show or remove trailing zeros in numbers
#'      \item \code{keep.line.breaks}: boolean (default: \code{FALSE}) to keep or remove line breaks from cells in a table
#'      \item \code{missing}: string (default: \code{NA}) to replace missing values in vectors, tables etc.
#'      \item \code{date}: string (default: \code{'\%Y/\%m/\%d \%X'}) passed to \code{format} when printing dates (\code{POSIXct} or \code{POSIXt})
#'      \item \code{header.style}: \code{'atx'} or \code{'setext'} passed to \code{\link{pandoc.header}}
#'      \item \code{list.style}: \code{'bullet'}, \code{'ordered'} or \code{'roman'} passed to \code{\link{pandoc.list}}. Please not that this has no effect on \code{pander} methods.
#'      \item \code{table.style}: \code{'multiline'}, \code{'grid'}, \code{'simple'} or \code{'rmarkdown'} passed to \code{\link{pandoc.table}}
#'      \item \code{table.emphasize.rownames}: boolean (default: \code{TRUE}) if row names should be highlighted
#'      \item \code{table.split.table}: numeric passed to \code{\link{pandoc.table}} and also affects \code{pander} methods. This option tells \code{pander} where to split too wide tables. The default value (\code{80}) suggests the conventional number of characters used in a line, feel free to change (e.g. to \code{Inf} to disable this feature) if you are not using a VT100 terminal any more :)
#'      \item \code{table.split.cells}: numeric or numeric vector (default: \code{30}) passed to \code{\link{pandoc.table}} and also affects \code{pander} methods. This option tells \code{pander} where to split too wide cells with line breaks. Numeric vector specifies values for cells separately. Set \code{Inf} to disable.
#'      \item \code{table.caption.prefix}: string (default: \code{'Table: '}) passed to \code{\link{pandoc.table}} to be used as caption prefix. Be sure about what you are doing if changing to other than \code{'Table: '} or \code{':'}.
#'      \item \code{table.continues}: string (default: \code{'Table continues below'}) passed to \code{\link{pandoc.table}} to be used as caption for long (split) without a use defined caption
#'      \item \code{table.continues.affix}: string (default: \code{'(continued below)'}) passed to \code{\link{pandoc.table}} to be used as an affix concatenated to the user defined caption for long (split) tables
#'      \item \code{table.alignment.default}: string (default: \code{centre}) that defines the default alignment of cells. Can be \code{left}, \code{right} or \code{centre} that latter can be also spelled as \code{center}.
#'      \item \code{table.alignment.rownames}: string (default: \code{centre}) that defines the alignment of rownames in tables. Can be \code{left}, \code{right} or \code{centre} that latter can be also spelled as \code{center}.
#'      \item \code{use.hyphening}: boolean (default: \code{FALSE}) if try to use hyphening when splitting large cells according to table.split.cells. Requires \pkg{sylly}.
#'      \item \code{evals.messages}: boolean (default: \code{TRUE}) passed to \code{evals}' \code{pander} method specifying if messages should be rendered
#'      \item \code{p.wrap}: a string (default: \code{'_'}) to wrap vector elements passed to \code{p} function
#'      \item \code{p.sep}: a string (default: \code{', '}) with the main separator passed to \code{p} function
#'      \item \code{p.copula}: a string (default: \code{' and '}) with ending separator passed to \code{p} function
#'      \item \code{plain.ascii}: boolean (default: \code{FALSE}) to define if output should be in plain ascii or not
#'      \item \code{graph.nomargin}: boolean (default: \code{TRUE}) if trying to keep plots' margins at minimal
#'      \item \code{graph.fontfamily}: string (default: \code{'sans'}) specifying the font family to be used in images. Please note, that using a custom font on Windows requires \code{grDevices:::windowsFonts} first.
#'      \item \code{graph.fontcolor}: string (default: \code{'black'}) specifying the default font color
#'      \item \code{graph.fontsize}: numeric (default: \code{12}) specifying the \emph{base} font size in pixels. Main title is rendered with \code{1.2} and labels with \code{0.8} multiplier.
#'      \item \code{graph.grid}: boolean (default: \code{TRUE}) if a grid should be added to the plot
#'      \item \code{graph.grid.minor}: boolean (default: \code{TRUE}) if a miner grid should be also rendered
#'      \item \code{graph.grid.color}: string (default: \code{'grey'}) specifying the color of the rendered grid
#'      \item \code{graph.grid.lty}: string (default: \code{'dashed'}) specifying the line type of grid
#'      \item \code{graph.boxes}: boolean (default: \code{FALSE}) if to render a border around of plot (and e.g. around strip)
#'      \item \code{graph.legend.position}: string (default: \code{'right'}) specifying the position of the legend: 'top', 'right', 'bottom' or 'left'
#'      \item \code{graph.background}: string (default: \code{'white'}) specifying the plots main background's color
#'      \item \code{graph.panel.background}: string (default: \code{'transparent'}) specifying the plot's main panel background. Please \emph{note}, that this option is not supported with \code{base} graphics.
#'      \item \code{graph.colors}: character vector of default color palette (defaults to a colorblind theme: \url{http://jfly.iam.u-tokyo.ac.jp/color/}). Please \emph{note} that this update work with \code{base} plots by appending the \code{col} argument to the call if not set.
#'      \item \code{graph.color.rnd}: boolean (default: \code{FALSE}) specifying if the palette should be reordered randomly before rendering each plot to get colorful images
#'      \item \code{graph.axis.angle}: numeric (default: \code{1}) specifying the angle of axes' labels. The available options are based on \code{par(les)} and sets if the labels should be:
#'      \itemize{
#'              \item \code{1}: parallel to the axis,
#'              \item \code{2}: horizontal,
#'              \item \code{3}: perpendicular to the axis or
#'              \item \code{4}: vertical.
#'      }
#'      \item \code{graph.symbol}: numeric (default: \code{1}) specifying a symbol (see the \code{pch} parameter of \code{par})
#'      \item \code{knitr.auto.asis}: boolean (default: \code{TRUE}) if the results of \code{pander} should be considered as \code{'asis'} in \code{knitr}. Equals to specifying \code{results='asis'} in the R chunk, so thus there is no need to do so if set to \code{TRUE}.
#'      \item \code{pandoc.binary}: full path of \code{pandoc}'s binary. By default, \code{pandoc} is in the path.
#' }
#' @param o option name (string). See below.
#' @param value value to assign (optional)
#' @export
#' @seealso \code{\link{evalsOptions}}
#' @examples \dontrun{
#' panderOptions()
#' panderOptions('digits')
#' panderOptions('digits', 5)
#' }
panderOptions <- function(o, value) {

    res <- getOption('pander')

    ## just querying
    if (missing(value)) {

        if (missing(o))
            return(res)

        if (o %in% names(res))
            return(res[[o]])

        pandoc.header('Possible `pander` options:', style = 'setext')
        pandoc.list(names(res))
        stop('Wrong option queried.')

    } else {

        if (!o %in% names(res))
            stop(paste('Invalid option name:', o))

        ## fix assigning NULL to a list element
        if (is.null(value)) {
            res[o] <- list(NULL)
        } else {
            res[[o]] <- value
        }

        options('pander' = res)

    }

}


#' Querying/setting evals option
#'
#' To list all \code{evals} options, just run this function without any parameters provided. To query only one value, pass the first parameter. To set that, use the \code{value} parameter too.
#'
#' The following \code{evals} options are available:
#'
#' \itemize{
#'      \item \code{parse}: if \code{TRUE} the provided \code{txt} elements would be merged into one string and parsed to logical chunks. This is useful if you would want to get separate results of your code parts - not just the last returned value, but you are passing the whole script in one string. To manually lock lines to each other (e.g. calling a \code{plot} and on next line adding an \code{abline} or \code{text} to it), use a plus char (\code{+}) at the beginning of each line which should be evaluated with the previous one(s). If set to \code{FALSE}, \code{evals} would not try to parse R code, it would get evaluated in separate runs - as provided. Please see examples of \code{\link{evals}}.
#'      \item \code{cache}: caching the result of R calls if set to \code{TRUE}
#'      \item \code{cache.mode}: cached results could be stored in an \code{environment} in \emph{current} R session or let it be permanent on \code{disk}.
#'      \item \code{cache.dir}: path to a directory holding cache files if \code{cache.mode} set to \code{disk}. Default to \code{.cache} in current working directory.
#'      \item \code{cache.time}: number of seconds to limit caching based on \code{proc.time}. If set to \code{0}, all R commands, if set to \code{Inf}, none is cached (despite the \code{cache} parameter).
#'      \item \code{cache.copy.images}: copy images to new files if an image is returned from cache? If set to \code{FALSE} (default) the "old" path would be returned.
#'      \item \code{classes}: a vector or list of classes which should be returned. If set to \code{NULL} (by default) all R objects will be returned.
#'      \item \code{hooks}: list of hooks to be run for given classes in the form of \code{list(class = fn)}. If you would also specify some parameters of the function, a list should be provided in the form of \code{list(fn, param1, param2=NULL)} etc. So the hooks would become \code{list(class1=list(fn, param1, param2=NULL), ...)}. See examples of \code{\link{evals}}. A default hook can be specified too by setting the class to \code{'default'}. This can be handy if you do not want to define separate methods/functions to each possible class, but automatically apply the default hook to all classes not mentioned in the list. You may also specify only one element in the list like: \code{hooks=list('default' = pander_return)}. Please note, that nor error/warning messages, nor stdout is captured (so: updated) while running hooks!
#'      \item \code{length}: any R object exceeding the specified length will not be returned. The default value (\code{Inf}) does not filter out any R objects.
#'      \item \code{output}: a character vector of required returned values. This might be useful if you are only interested in the \code{result}, and do not want to save/see e.g. \code{messages} or \code{print}ed \code{output}. See examples of \code{\link{evals}}.
#'      \item \code{graph.unify}: boolean (default: \code{FALSE}) that determines if \code{evals} should try to unify the style of (\code{base}, \code{lattice} and \code{ggplot2}) plots? If set to \code{TRUE}, some \code{panderOptions()} would apply.
#'      \item \code{graph.name}: set the file name of saved plots which is \code{\link{tempfile}} by default. A simple character string might be provided where \code{\%d} would be replaced by the index of the generating \code{txt} source, \code{\%n} with an incremented integer in \code{graph.dir} with similar file names and \code{\%t} by some random characters. A function's name to be \code{eval}uated can be passed here too.
#'      \item \code{graph.dir}: path to a directory where to place generated images. If the directory does not exist, \code{\link{evals}} try to create that. Default set to \code{plots} in current working directory.
#'      \item \code{graph.output}: set the required file format of saved plots. Currently it could be any of  \code{grDevices}: \code{png}, \code{bmp}, \code{jpeg}, \code{jpg}, \code{tiff}, \code{svg} or \code{pdf}. Set to \code{NA} not to save plots at all and tweak that setting with \code{capture.plot()} on demand.
#'      \item \code{width}: width of generated plot in pixels for even vector formats
#'      \item \code{height}: height of generated plot in pixels for even vector formats
#'      \item \code{res}: nominal resolution in \code{ppi}. The height and width of vector images will be calculated based in this.
#'      \item \code{hi.res}: generate high resolution plots also? If set to \code{TRUE}, each R code parts resulting an image would be run twice.
#'      \item \code{hi.res.width}: width of generated high resolution plot in pixels for even vector formats. The \code{height} and \code{res} of high resolution image is automatically computed based on the above options to preserve original plot aspect ratio.
#'      \item \code{graph.env}: save the environments in which plots were generated to distinct files (based on \code{graph.name}) with \code{env} extension?
#'      \item \code{graph.recordplot}: save the plot via \code{recordPlot} to distinct files (based on \code{graph.name}) with \code{recodplot} extension?
#'      \item \code{graph.RDS}: save the raw R object returned (usually with \code{lattice} or \code{ggplot2}) while generating the plots to distinct files (based on \code{graph.name}) with \code{RDS} extension?
#'      \item \code{log}: \code{NULL} or  an optionally passed \emph{logger name} from \pkg{futile.logger} to record all info, trace, debug and error messages.
#' }
#' @param o option name (string). See below.
#' @param value value to assign (optional)
#' @export
#' @seealso \code{\link{evals}} \code{\link{panderOptions}}
#' @examples
#' evalsOptions()
#' evalsOptions('cache')
#' evalsOptions('cache', FALSE)
evalsOptions <- function(o, value) {

    res <- getOption('evals')

    ## just querying
    if (missing(value)) {

        if (missing(o))
            return(res)

        if (o %in% names(res))
            return(res[[o]])

        pandoc.header('Possible `evals` options:', style = 'setext')
        pandoc.list(names(res))
        stop('Wrong option queried.')

    } else {

        if (!o %in% names(res))
            stop(paste('Invalid option name:', o))

        ## fix assigning NULL to a list element
        if (is.null(value)) {
            res[o] <- list(NULL)
        } else {
            res[[o]] <- value
        }

        options('evals' = res)

    }

}

library(testthat)
library(pander)
context('has.rownames')

test_that('has.rownames: TRUE', {
    expect_true(has.rownames(mtcars))
    expect_true(has.rownames(mtcars[1:2, ]))
    expect_true(has.rownames(mtcars[1, ]))
    expect_true(has.rownames(table(mtcars$am, mtcars$gear)))
    expect_true(has.rownames(table(mtcars$am, mtcars$gear) + 0.1))
    expect_true(has.rownames(summary(lm(mtcars$hp~1))$coeff))
})

test_that('has.rownames: FALSE', {
    expect_false(has.rownames(mtcars$am))
    expect_false(has.rownames(mtcars[1:2, 5]))
    expect_false(has.rownames(summary(mtcars$am)))
    expect_false(has.rownames(table(mtcars$am)))
    expect_false(has.rownames(table(mtcars$am) + 0.1))
})

context('splitLine')

test_that('splitLine leaves non-string arguments and empty string unchanged', {
  expect_equal(1, splitLine(1))
  expect_equal(1:10, splitLine(1:10))
  expect_equal(c('string 1', 'string 2', 'string 3'), splitLine(c('string 1', 'string 2', 'string 3')))
  expect_equal(mtcars, splitLine(mtcars))
  expect_equal(summary(lm(mtcars$hp~1)), splitLine(summary(lm(mtcars$hp~1))))
  expect_equal(splitLine('', 10), '')
  expect_equal(splitLine('', 0), '')
  expect_equal(splitLine('', Inf), '')
})

test_that('splitLine without hyphening behaves correctly', {
  widthSpaced <- function(x, max.width, type='max'){
    if (type == 'max')
      max(sapply(strsplit(splitLine(x, max.width), '\n'), function(x) nchar(x) * grepl(' ', x)))
    else if (type == 'min')
      min(sapply(strsplit(splitLine(x, max.width), '\n'), nchar))
  }
  x <- 'foo bar'
  expect_equal(splitLine(x, 7), 'foo bar')
  expect_equal(splitLine(x, 6), 'foo\nbar')
  x <- 'Each character string in the input is first split into paragraphs (or lines containing whitespace only)'
  expect_equal(splitLine(x, 10),
               'Each\ncharacter\nstring in\nthe input\nis first\nsplit into\nparagraphs\n(or lines\ncontaining\nwhitespace\nonly)') #nolint
  expect_equal(splitLine(x, 20),
               'Each character\nstring in the input\nis first split into\nparagraphs (or lines\ncontaining\nwhitespace only)') #nolint
  expect_equal(splitLine(x, 0), paste(strwrap(x, 0), collapse = '\n')) # zero width replaces whitespaces by line breaks
  expect_equal(splitLine(x, Inf), x) # Infinite does not change
  x <- 'Really do clean up and test your code and think twice before you even start contemplating optimizing the code'
  expect_true(widthSpaced(x, 5, 'max') <= 5) # max width of line is less than 5 (in case it has spaces)
  expect_true(widthSpaced(x, 10, 'max') <= 10) # max width of line is less than 10 (in case it has spaces)
  expect_true(widthSpaced(x, 10, 'min') >= 1) # every line has at least one character
  expect_equal('character', splitLine('character', 2)) # string without spaces remains unchanged
  expect_equal('', splitLine('', 2)) # for empty string, returns empty string
  expect_equal(length(strsplit(splitLine(x, 2), '\n')[[1]]), 19) # when max.width param is small, every word is a line
})

test_that('splitLine with hyphening behaves correctly', {
  x <- 'Really do clean up and test your code and think twice before you even start contemplating optimizing the code'
  widthSpaced <- function(x, max.width, type='max'){
    if (type == 'max')
      max(sapply(strsplit(splitLine(x, max.width, TRUE), '\n'), function(x) nchar(x) * grepl(' ', x)))
    else if (type == 'min')
      min(sapply(strsplit(splitLine(x, max.width, TRUE), '\n'), nchar))
  }
  expect_true(widthSpaced(x, 5, 'max') <= 5) # max width of line is less than 5 (in case it has spaces)
  expect_true(widthSpaced(x, 10, 'max') <= 10) # max width of line is less than 10 (in case it has spaces)
  expect_true(widthSpaced(x, 10, 'min') >= 1) # every line has at least one character
  # check correctness for predifined words/phrases
  expect_equal('char-\nac-\nter', splitLine('character', 2, TRUE))
  expect_equal(splitLine('start testing', 5, T), 'start\ntest-\ning')
  expect_equal(splitLine('Pander Package', 5, T), 'Pan-\nder\nPack-\nage')
  expect_equal(splitLine('Pander Package', 6, T), 'Pander\nPack-\nage')
  expect_equal(splitLine('Pander Package', 7, T), 'Pander\nPackage')
  # predifined example
  # when max.width param is small, every word is a line
  expect_equal(length(strsplit(splitLine(x, 10, TRUE), '\n')[[1]]), 12)
  expect_false(grepl('- ', splitLine(x, 10, TRUE))) # no space after hyphen
  expect_false(grepl(' -', splitLine(x, 10, TRUE))) # no space before hyphen
  expect_false(grepl('\n ', splitLine(x, 20, TRUE))) # line does not start with a space
  # infinite width
  expect_equal(splitLine(x, Inf, TRUE), x)
})

test_that('options', {
    expect_error(panderOptions('foobar'))
    expect_error(evalsOptions('foobar'))
})

context('p function')
test_that('p', {
    expect_equal(p(character(0)), '')
    expect_equal(p(1), '_1_')
    expect_equal(p(1:3), '_1_, _2_ and _3_')
    expect_equal(p(1, wrap = ''), '1')
    expect_equal(p(1:3, wrap = '', sep = '|', copula = '|'), '1|2|3')
    expect_equal(p(seq(1, 1.2, 0.1)), '_1_, _1.1_ and _1.2_')
    expect_equal(p(seq(1, 1.2, 0.1), keep.trailing.zeros = TRUE),
                 '_1.0_, _1.1_ and _1.2_')
    expect_error(p(1:10, limit = 3))
})

library(testthat)
library(pander)

## reset later
original_contrasts_options <- getOption('contrasts')

context('pandoc.table')

tables <- list(
    mtcars,
    mtcars$am,
    mtcars[1:2, ],
    mtcars[1:2, 5],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    summary(lm(mtcars$hp~1))$coeff,
    table(mtcars$am, mtcars$gear, mtcars$carb),
    addmargins(table(mtcars$gear, mtcars$carb)),
    summary(Sys.Date() + 0:5)
)

test_that('no error: multiline', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'multiline'), is_a('character'))
})
test_that('no error: simple', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'simple'), is_a('character'))
})
test_that('no error: grid', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'grid'), is_a('character'))
})
test_that('no error: rmarkdown', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'rmarkdown'), is_a('character'))
})

tables <- list(
    mtcars,
    mtcars$am,
    mtcars[1:2, ],
    mtcars[1:2, 5],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    lm(mtcars$hp~1),
    t.test(extra ~ group, data = sleep),
    prcomp(USArrests),
    density(mtcars$hp),
    table(mtcars$am, mtcars$gear, mtcars$carb) + 0.1
    )

test_that('split.tables', {
    # test exact values (#164)
    t <- data.frame(a = '7 chars', b = paste(rep('Δ', 5), collapse = ' '))
    expect_equal(nchar('Δ Δ Δ Δ Δ', type = 'width'), 9)
    res_grid <- pander_return(t, style = 'grid', split.tables = 23)
    expect_equal(length(res_grid), 8)
    expect_false(any(grepl('Table', res_grid)))
    res_grid <- pander_return(t, style = 'grid', split.tables = 22)
    expect_equal(length(res_grid), 18)
    expect_true(any(grepl('Table', res_grid)))
    res_simple <- pander_return(t, style = 'simple', split.tables = 21)
    expect_equal(length(res_simple), 6)
    expect_false(any(grepl('Table', res_simple)))
    res_simple <- pander_return(t, style = 'simple', split.tables = 20)
    expect_equal(length(res_simple), 14)
    expect_true(any(grepl('Table', res_simple)))
})

test_that('rmarkdown pipe-delimited table is correct (#186)', {
    d <- data.frame(a = 'foo|bar', b = 'my missing cell')
    res <- pander_return(d, style = 'rmarkdown')
    expect_true(any(grep('foo', res)))
    expect_true(any(grep('bar', res)))
    expect_true(any(grep('my missing cell', res)))
    rownames(d) <- 'x|y'
    res <- pander_return(d, style = 'rmarkdown')
    expect_true(any(grep('x', res)))
    expect_true(any(grep('y', res)))
    expect_true(any(grep('foo', res)))
    expect_true(any(grep('bar', res)))
    expect_true(any(grep('my missing cell', res)))
})

test_that('pandoc.table.return behaves correctly', {
    expect_warning(pander_return(mtcars[, 1:2], split.tables = Inf, split.cells = c('50%', '50%')))
    t <- mtcars[1:3, 1:2]
    attr(t, 'alignment') <- 'left'
    attr(t, 'caption') <- 'simplified mtcars'
    res <- pander_return(t, emphasize.rownames = FALSE)
    expect_false(any(grep('^[[:space:]]', res))) # because of left alignment
    expect_true(any(grep('simplified mtcars', res)))
    expect_error(pander_return(t, justify = c('left', 'right'))) # needs 3 because of rownames
    expect_error(pander_return(t, justify = 'lr')) # needs 3 because of rownames
    res <- pander_return(t, justify = 'lrr')
    expect_false(any(grep('[[:space:]]$', res)))
    expect_error(pander_return(t, justify = 'laft'))
    res <- pander_return(t, split.tables = 1)
    expect_equal(length(res), 24)
    res <- pander_return(t, split.cells = c(10, 10, 10))
    expect_equal(length(res), 16)
    expect_warning(pander_return(t, split.cells = vector()))
})

dm <- panderOptions('decimal.mark')
panderOptions('decimal.mark', ',')
test_that('decimal mark', {
    for (t in tables)
        expect_true(grepl(',', paste(pander_return(t), collapse = '\n')))
})

panderOptions('decimal.mark', dm)

test_that('Date is preserved', {
    expect_equal(grep('2015', pander_return(summary(as.Date('2015-01-01') + 0:5))), 5)
})

context('highlight tables')

t <- mtcars[1:3, 1:5]
test_that('highlight 1D: no error', {
    expect_that(pandoc.table.return(t$mpg, emphasize.italics.cells = 1), is_a('character'))
    expect_that(pandoc.table.return(t$mpg, emphasize.italics.cells = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t$mpg, emphasize.strong.cells = 1), is_a('character'))
    expect_that(pandoc.table.return(t$mpg, emphasize.strong.cells = 1:2), is_a('character'))
})

t <- table(mtcars$am, mtcars$gear)
test_that('emphasize 2D: no error', {
    expect_that(pandoc.table.return(t, emphasize.italics.rows = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.rows = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cols = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cols = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cells = which(t > 10, arr.ind = TRUE)), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cells = which(t > 20, arr.ind = TRUE)), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.rows = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.rows = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cols = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cols = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cells = which(t > 10, arr.ind = TRUE)), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cells = which(t > 20, arr.ind = TRUE)), is_a('character'))
})

test_that('emphasize: error', {
    expect_that(pandoc.table(t, emphasize.italics.cols = 1:5), throws_error())
    expect_that(pandoc.table(t, emphasize.italics.cols = 1.5), throws_error())
    expect_that(pandoc.table(t, emphasize.strong.cols = 1:5), throws_error())
    expect_that(pandoc.table(t, emphasize.strong.cols = 1.5), throws_error())
})

test_that('no warning for highlight NA/empty strings', {
    expect_warning(pandoc.table(data.frame(x = 1:2, y = c(1, NA)), emphasize.italics.cols = 2), regexp = NA)
})

test_that('emphasize.italics.rows works correctly', {
    # test for issue 176
    df <- data.frame(a=1:3, b=1:3, c=1:3)
    res <- pander_return(df, emphasize.italics.rows = c(1,2), style = 'simple')
    expect_equal(res[5], ' *1*   *1*   *1* ')
    expect_equal(res[6], ' *2*   *2*   *2* ')
    res <- pander_return(df, emphasize.strong.rows = c(1,2), style = 'simple')
    expect_equal(res[5], ' **1**   **1**   **1** ')
    expect_equal(res[6], ' **2**   **2**   **2** ')
})

test_that('emphasize.verbatim works correctly', {
    df <- data.frame(a = 1:3, b = 4:6, c = 7:9)
    res <- pander_return(df, emphasize.verbatim.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' `1`   `4`   `7` ')
    expect_equal(res[6], ' `2`   `5`   `8` ')
    res <- pander_return(df, emphasize.verbatim.cols = c(1,2), style = 'simple')
    expect_equal(res[5], ' `1`   `4`   7 ')
    expect_equal(res[6], ' `2`   `5`   8 ')
    df <- data.frame(a = 1:3, b = 1:3, c = 1:3)
    res <- pander_return(df, emphasize.italics.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' *1*   *1*   *1* ')
    expect_equal(res[6], ' *2*   *2*   *2* ')
    res <- pander_return(df, emphasize.strong.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' **1**   **1**   **1** ')
    expect_equal(res[6], ' **2**   **2**   **2** ')
})

test_that('emphasize.verbatim works correctly', {
    df <- data.frame(a = 1:3, b = 4:6, c = 7:9)
    res <- pander_return(df, emphasize.verbatim.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' `1`   `4`   `7` ')
    expect_equal(res[6], ' `2`   `5`   `8` ')
    res <- pander_return(df, emphasize.verbatim.cols = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' `1`   `4`   7 ')
    expect_equal(res[6], ' `2`   `5`   8 ')
    res <- pander_return(df, emphasize.verbatim.cells =
                                 which(df > 5, arr.ind = TRUE),
                         style = 'simple')
    expect_equal(res[5], ' 1    4    `7` ')
    expect_equal(res[6], ' 2    5    `8` ')
    res <- pander_return(df, emphasize.verbatim.cells =
                                 which(df > 5, arr.ind = TRUE),
                         emphasize.strong.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' **1**   **4**   **`7`** ')
    expect_equal(res[7], '   3      `6`      `9`   ')
})

test_that('emphasize.strong.rows works correctly with no rows to emphasize #354', {
    df <- data.frame(a = 1:3, b = 4:6, c = 7:9)
    res <- pander_return(df, emphasize.strong.rows = integer(0), style = 'simple')
    expect_equal(res[5], ' 1   4   7 ')
    expect_equal(res[6], ' 2   5   8 ')
    expect_equal(res[7], ' 3   6   9 ')
})

context('captions')

tables <- list(
    mtcars,
    mtcars[1:2, ],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    lm(mtcars$hp~1),
    t.test(extra ~ group, data = sleep),
    prcomp(USArrests),
    density(mtcars$hp)
    )

has.caption <- function(ttt, evals = FALSE) {
    set.caption('foo')
    if (!evals)
        any(grepl('Table:', pander_return(ttt)))
    else
        !is.null(attr(evals('get("ttt")', env = parent.frame())[[1]]$result, 'caption'))
}

test_that('direct call', {
    for (ttt in tables)
        expect_that(has.caption(ttt), is_true())
})

cache.dir <- evalsOptions('cache.dir')
graph.dir <- evalsOptions('graph.dir')
wd <- getwd()
setwd(tempdir())
evalsOptions('cache.dir',  file.path(tempdir(), '.cache'))
evalsOptions('graph.dir',  file.path(tempdir(), 'plots'))
test_that('evals', {
    for (ttt in tables)
        expect_that(has.caption(ttt, evals = TRUE), is_true())
})
evalsOptions('cache.dir',  cache.dir)
evalsOptions('graph.dir',  graph.dir)
setwd(wd)

context('row and column names')

library(testthat)
library(pander)

tables <- list(
  mtcars[1:2, 1:2],
  data.frame(x = 1:2, y = 2:3),
  data.frame(x = 1:3, y = 2:4)[c(1, 3), ]
  )

t <- tables[[1]]

test_that('row names can be suppressed', {
  for (t in tables) {
    expect_false(grepl("&nbsp;", pandoc.table.return(t, row.names = FALSE)))
  }
})

test_that('row names can be set', {
  for (t in tables) {
    expect_true(grepl("\\n \\*\\*a\\*\\*", pandoc.table.return(t, row.names = c("a", "b"))))
  }
})

test_that('obvious row names can be preserved', {
    expect_true(grepl(".*1.*2.*3.*", pandoc.table.return(data.frame(a = 7:9), row.names = 1:3)))
})

test_that('column names can be set', {
  for (t in tables) {
    res <- capture.output(pandoc.table(t, col.names = c("a", "b")))
    expect_true(grepl("a *b", res[3]))
  }
})

context('default alignments')

tables <- list(
    mtcars,
    mtcars$am,
    mtcars[1:2, ],
    mtcars[1:2, 5],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    summary(lm(mtcars$hp~1))$coeff,
    table(mtcars$am, mtcars$gear, mtcars$carb)
    )

tad <- panderOptions('table.alignment.default')
tar <- panderOptions('table.alignment.rownames')

panderOptions('table.alignment.default', 'centre')
panderOptions('table.alignment.rownames', 'right')
test_that('no error: allright', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'multiline'), is_a('character'))
})

f <- function(df) {
        if (class(df) != 'matrix' && !is.table(df) || length(dim(df)) < 2)
            ifelse(sapply(df, is.numeric), 'right', 'left')
        else
            ifelse(apply(df, 2, is.numeric), 'right', 'left')
}
panderOptions('table.alignment.default', f)
test_that('no error: functions', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'multiline'), is_a('character'))
})
panderOptions('table.alignment.default', tad)
panderOptions('table.alignment.rownames', tar)

test_that('digits param', {
    m <- matrix(rep(0.111111, 6), nrow = 2)
    res <- pander_return(m, digits = c(2, 4, 1))
    expect_equal(res[3], ' 0.11   0.1111   0.1 ')
    expect_warning(pander_return(m, digits = c(1, 2)))
    res <- pander_return(m, digits = 2)
    expect_equal(res[3],' 0.11   0.11   0.11 ')
    mt <- mtcars[1:4, 5:8]
    res <- pander_return(mt, digits = c(1, 4, 3, 4),
                         keep.trailing.zeros = TRUE)
    expect_equal(res[5],
                 '   **Mazda RX4**       4     2.620   16.5   0  ')
})

test_that('round param', {
    m <- matrix(rep(0.111111, 6), nrow = 2)
    res <- pander_return(m, round = c(2, 4, 1))
    expect_equal(res[3], ' 0.11   0.1111   0.1 ')
    expect_warning(pander_return(m, round = c(1, 2)))
    res <- pander_return(m, round = 2)
    expect_equal(res[3],' 0.11   0.11   0.11 ')
})

context('keep.trailing.zeros')
test_that('digits param', {
    expect_true(grepl('3.90', pander_return(mtcars[2:3, 1:5], keep.trailing.zeros = TRUE)[5]))
    expect_false(grepl('3.90', pander_return(mtcars[2:3, 1:5], keep.trailing.zeros = FALSE)[5]))
})

context('keep.line.breaks')
test_that('keep.line.breaks works correctly', {
  # keeping line breaks in a simple data.frame with one line breaks differs lines amount by one
  x <- data.frame(a = 'Pander\nPackage')
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + 1, lines.x.keep.breaks)

  # keeping line breaks in a simple data.frame with 2 rows with 1 line breaks differst lines amount by 2
  x <- data.frame(a = c('Pander\nPackage', 'Pander\nPackage'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + 2, lines.x.keep.breaks)

  #if there are no line breaks originally, they do not get introduced
  x <- data.frame(a = c('Pander Package', 'Pander Package'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks, lines.x.keep.breaks)

  # works with random number of rows added
  rows <- sample(1:100, 1)
  x <- data.frame(a = rep('Pander\nPackage', rows))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + rows, lines.x.keep.breaks)

  # random number of line breaks in one cell
  n <- sample(1:10, 1)
  x <- data.frame(a = paste(rep('pander', n), collapse = '\n'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE, split.cells = Inf), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + n - 1, lines.x.keep.breaks)

  # random number of line breaks in cells, 3 columns
  n <- sample(1:10, 3)
  x <- data.frame(a = paste(rep('pander', n[1]), collapse = '\n'),
                  b = paste(rep('pander', n[2]), collapse = '\n'),
                  c = paste(rep('pander', n[3]), collapse = '\n'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE, split.cells = Inf, split.tables = Inf), '\n')[[1]]) #nolint
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + max(n) - 1, lines.x.keep.breaks)

  # 3 columns, 2 rows
  x <- rbind(x, x)
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE, split.cells = Inf, split.tables = Inf), '\n')[[1]]) #nolint
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + 2 * max(n) - 2, lines.x.keep.breaks)
})

context('split.cells')
test_that('split.cells works correctly', {
  x <- data.frame(a = 'foo bar\nfo bar')
  # single line break behaves correctly combines with keep line breaks
  expect_equal(pandoc.table.return(x, keep.line.breaks = TRUE, split.cells = 7),
               '\n---------\n    a    \n---------\n foo bar \n fo bar  \n---------\n\n')
  expect_equal(pandoc.table.return(x, keep.line.breaks = TRUE, split.cells = 6),
               '\n--------\n   a    \n--------\n  foo   \n  bar   \n fo bar \n--------\n\n')
  # Corner values of split.cells
  x <- data.frame(a = 'foo bar', b = 'foo bar')
  expect_equal(length(strsplit(pandoc.table.return(x, split.cells = c(6, Inf)), '\n')[[1]]), 8)
  expect_equal(pandoc.table.return(x, split.cells = c(6, Inf)),
               '\n---------------\n  a       b    \n----- ---------\n foo   foo bar \n bar           \n---------------\n\n')
  expect_equal(pandoc.table.return(x, split.cells = c(Inf, 6)),
               '\n---------------\n    a       b  \n--------- -----\n foo bar   foo \n           bar \n---------------\n\n')
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)),
               '\n-------------------\n    a         b    \n--------- ---------\n foo bar   foo bar \n-------------------\n\n')
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)), pandoc.table.return(x, split.cells = Inf))
  expect_equal(pandoc.table.return(x, split.cells = c(6, 6)), pandoc.table.return(x, split.cells = 6))
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)), pandoc.table.return(x, split.cells = 7))
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)), pandoc.table.return(x, split.cells = 7))
  # relative split.cells
  expect_equal(pandoc.table.return(x, split.cells = c('50%', '50%')),
               pandoc.table.return(x, split.cells = c(7)))
  expect_equal(pandoc.table.return(x, split.cells = c('20%', '80%'), split.tables = 30),
               pandoc.table.return(x, split.cells = c(6, Inf)))
  expect_equal(pandoc.table.return(x, split.cells = c('80%', '20%'), split.tables = 30),
               pandoc.table.return(x, split.cells = c(Inf, 6)))
})

test_that('split.cells param produces expected warnings', {
  mt <- mtcars[1:2, 1:4]
  expect_warning(pander(mt, split.cells = c(1, 2)))
  expect_warning(pander(mt, split.cells = c(1, 2, 3)))
  expect_warning(pander(mt, split.cells = c('10%', '10%', '10%')))
  expect_warning(pander(mt, split.cells = c('30%', '30%', '40%')))
})

context('tableExpand')

test_that('produces.errors', {
  x <- data.frame(a = 'Pander\nPackage')
  expect_error(pander(x, style = 'simple', keep.line.breaks = TRUE))
  expect_error(pander(x, style = 'rmarkdown', keep.line.breaks = TRUE))
})

test_that('tableExpand behaves correctly', {
  ## multiline style check
  argv <-  structure(list(txt = structure(c(1L, 4L, 2L, 3L), .Label = c('&nbsp;',  'cyl', 'disp', 'mpg'), class = 'factor'), width = c(19, 5, 5,  6), justify = structure(c(1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c(NA, -4L), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)),
               length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '      &nbsp;         mpg   cyl   disp ');

  ## grid style check
  argv <-  structure(list(txt = structure(c(1L, 5L, 2L, 3L, 4L), .Label = c('&nbsp;',  'cyl', 'disp', 'hp', 'mpg'), class = 'factor'), width = c(20,  5, 5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c(NA, -5L), class = 'data.frame') #nolint
  sep.cols <-  c('| ', ' | ', ' |')
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res), nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2])) #nolint
  expect_equal(res, '|        &nbsp;        |  mpg  |  cyl  |  disp  |  hp  |' )

  ## rmarkdown style check
  argv <-  structure(list(txt = structure(c(1L, 4L, 5L, 3L, 2L), .Label = c('**Mazda RX4**',  '110', '160', '21', '6'), class = 'factor'), width = c(20, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('| ', ' | ', ' |')
  style <-  'rmarkdown'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '|    **Mazda RX4**     |  21   |   6   |  160   | 110  |')

  ## simple style check
  argv <-  structure(list(txt = structure(c(1L, 3L, 4L, 2L, 5L), .Label = c('**Datsun 710**',  '108', '22.8', '4', '93'), class = 'factor'), width = c(20, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'simple'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '   **Datsun 710**    22.8    4    108    93 ')

  ## left justification
  argv <-  structure(list(txt = structure(c(1L, 4L, 5L, 3L, 2L), .Label = c('**Mazda',  '110', '160', '21', '6'), class = 'factor'), width = c(10, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c(NA, -5L), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '**Mazda    21    6     160    110 ');

  ## right justification
  argv <-  structure(list(txt = structure(c(1L, 3L, 5L, 4L, 2L), .Label = c('**Hornet 4 Drive**',  '110', '21.4', '258', '6'), class = 'factor'), width = c(20,  5, 5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'right', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'simple'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '  **Hornet 4 Drive**  21.4     6    258  110' );

  ## multiple lines
  argv <-  structure(list(txt = structure(c(1L, 3L, 5L, 4L, 2L), .Label = c('**Hornet\n4\nDrive**',  '110', '21.4', '258', '6'), class = 'factor'), width = c(10,  5, 5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)),
               length(strsplit(res, '\n')[[1]]))
  # max number of line breaks equals number of lines in the result
  expect_equal(res, '**Hornet   21.4  6     258    110 \n4                                 \nDrive**                           '); #nolint


  argv <-  structure(list(txt = structure(c(1L, 3L, 4L, 2L, 5L), .Label = c('**Datsun\n710**',  '108', '22.8', '4', '93'), class = 'factor'), width = c(10, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)),
               length(strsplit(res, '\n')[[1]]))
  expect_equal(res, '**Datsun   22.8  4     108    93  \n710**                             ');

  argv <-  structure(list(txt = structure(c(1L, 4L, 5L, 3L, 2L), .Label = c('**Mazda\nRX4\nWag**',  '110', '160', '21', '6'), class = 'factor'), width = c(10, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- tableExpand_cpp(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(res, '**Mazda    21    6     160    110 \nRX4                               \nWag**                             '); #nolint

  # empty cells
  cells <- c('', '', '')
  cols.width <- c(2, 2, 2)
  justify <- c('centre', 'centre', 'centre')
  sep.cols <- c('', ' ', '')
  res <- tableExpand_cpp(cells, cols.width, justify, sep.cols, 'multiline')
  # when max.width param is small, every word is a line
  expect_equal(max(sapply(strsplit(as.character(cells), '\n'), length)), 0)
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(cells) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(cols.width))
  expect_equal(res, '        ')

  # backslashes issue (#22)
  expect_equal(pandoc.table.return(
      data.frame(a = '\\1 \\ 32', b ='23')),
      '\n--------------\n    a      b  \n--------- ----\n \\1 \\ 32   23 \n--------------\n\n') #nolint
  expect_equal(pandoc.table.return(
      data.frame(a = '\\1 \\ 32', b = '23'),
      justify = 'right'),
      '\n--------------\n        a    b\n--------- ----\n  \\1 \\ 32   23\n--------------\n\n') #nolint
  expect_equal(pandoc.table.return(data.frame(a = '\\1', b = '23')),
               '\n---------\n a    b  \n---- ----\n \\1   23 \n---------\n\n') #nolint

  # unicode string issue
  expect_equal(pandoc.table.return(data.frame(a = 'ßß')),
               '\n----\n a  \n----\n ßß \n----\n\n')
})

context('Empty objects')

test_that('Behavior for empty objects is correct', {
    mt <- mtcars[mtcars$mpg < 0, 1:4]
    res <- pander_return(mt)
    expect_equal(res[3], ' **mpg**   **cyl**   **disp**   **hp** ')
    expect_equal(length(res), 6)
    colnames(mt) <- NULL
    res <- suppressWarnings(pander_return(mt))
    expect_equal(length(res), 0)
    expect_warning(pander_return(mt))
    mt <- matrix(0, nrow = 0, ncol = 5)
    res <- suppressWarnings(pander_return(mt))
    expect_equal(length(res), 0)
    expect_warning(pander_return(mt))
    colnames(mt) <- 1:5
    res <- pander_return(mt)
    expect_equal(res[3], ' **1**   **2**   **3**   **4**   **5** ')
    expect_equal(length(res), 6)
    expect_equal(length(suppressWarnings(pander_return(data.frame()))), 0)
    expect_warning(pander_return(data.frame()))
})

context('plain.ascii')

test_that('plain.ascii option works correctly', {
    # dim is NULL
    x <- 1:10
    res <- pandoc.table.return(x, emphasize.cells = c(3, 4), plain.ascii = TRUE)
    res <- strsplit(res, '\n')[[1]]
    expect_false(any(grepl('\\*', res)))
    expect_equal(res[3], ' 1   2   3   4   5   6   7   8   9   10 ')
    # length(dim) == 1
    x <- array(1:10)
    res <- pandoc.table.return(x, emphasize.cells = c(3, 4), plain.ascii = TRUE)
    res <- strsplit(res, '\n')[[1]]
    expect_false(any(grepl('\\*', res)))
    expect_equal(res[3], ' 1   2   3   4   5   6   7   8   9   10 ')
    # length(dim) > 1
    x <- mtcars[1:3, 1:4]
    res <- pandoc.table.return(x, emphasize.rows = 2, plain.ascii = TRUE)
    expect_false(grepl('&nbsp;', res))
    expect_false(grepl('\\*', res))
})

context('S3 methods')

test_that('pander.tabular behaves correctly', {
    suppressMessages(require(tables))
    tab <- pander_return(tables::tabular(as.factor(am) ~ (mpg + hp + qsec) * (mean + median),
                                         data = mtcars),
                         emphasize.rownames = FALSE,
                         split.tables = Inf)
    expect_equal(length(tab), 10)
    tab <- pander_return(tables::tabular( (Species + 1) ~ (n = 1) + Format(digits = 2) * (Sepal.Length + Sepal.Width) * (mean + sd), data = iris ), #nolint
                         split.tables = Inf)
    expect_equal(length(tab), 14)
})

test_that('pander.CrossTable behaves correctly', {
    suppressMessages(require(descr))
    # issue https://github.com/Rapporter/pander/issues/163
    x <- CrossTable(mtcars$cyl, mtcars$gear, prop.c = FALSE, prop.t = FALSE, chisq = FALSE, prop.chisq = FALSE)
    res <- pander_return(x)
    expect_true(any(grepl(gsub('\\$', '\\\\$', x$ColData), res))) #nolint
    expect_true(any(grepl(gsub('\\$', '\\\\$', x$RowData), res))) #nolint
    # expected N, residual, std residual, adj std residual rownames was not included
    x <- suppressWarnings(CrossTable(mtcars$cyl, mtcars$gear, expected = TRUE,
                                     resid = TRUE, sresid = TRUE, asresid = TRUE))
    res <- pander_return(x)
    expect_true(any(grepl('Expected N', res)))
    expect_true(any(grepl('Residual', res)))
    expect_true(any(grepl('Std Residual', res)))
    expect_true(any(grepl('Adj Std Resid', res)))
    # issue 211 support for total.r total.c
    x <- suppressWarnings(CrossTable(mtcars$cyl, mtcars$gear))
    res <- pander_return(x, total.c = FALSE)
    expect_equal(length(res), 27)
    expect_equal(length(grep('Total', res)), 4)
    res <- pander_return(x, total.r = FALSE)
    expect_equal(length(res), 30)
    expect_equal(length(grep('Total', res)), 4)
    res <- pander_return(x)
    expect_equal(length(res), 30)
    expect_equal(length(grep('Total', res)), 5)
})

test_that('pander.NULL behaves correctly', {
    expect_equal(length(pander_return(NULL)), 0)
    expect_equal(length(pander_return(c(NULL, NULL))), 0)
})

test_that('pander.cast_df behaves correctly', {
    df <- data.frame(type = c(1, 1, 2, 2, 3, 3), variable = 'n', value = c(71, 72, 68, 80, 21, 20))
    df.cast <- reshape::cast(df, type~., sum)
    expect_equal(pander_return(df.cast, style = 'simple'),
                 c('', '', ' type   (all) ', '------ -------',
                   '  1      143  ', '  2      148  ', '  3      41   ', ''))
})

test_that('pander.lm/pander.summary.lm behaves correctly', {
    ctl <- c(4.17, 5.58, 5.18, 6.11, 4.50, 4.61, 5.17, 4.53, 5.33, 5.14)
    trt <- c(4.81, 4.17, 4.41, 3.59, 5.87, 3.83, 6.03, 4.89, 4.32, 4.69)
    group <- gl(2, 10, 20, labels = c('Ctl', 'Trt'))
    weight <- c(ctl, trt)
    lm.D9 <- lm(weight ~ group)
    res1 <- pander_return(lm.D9)
    expect_equal(length(res1), 11)
    expect_equal(max(nchar(res1)), 63)
    res <- pander_return(summary(lm.D9))
    expect_true(any(grepl('Fitting linear model', res)))
    expect_true(any(grepl('Observations', res)))
    expect_equal(length(res), 18)
})

test_that('pander.glm/pander.summary.glm behaves correctly', {
    clotting <- data.frame(
        u = c(5, 10, 15, 20, 30, 40, 60, 80, 100),
        lot1 = c(118, 58, 42, 35, 27, 25, 21, 19, 18),
        lot2 = c(69, 35, 26, 21, 18, 16, 13, 12, 12))
    glmm <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
    pglmm <- pander_return(glmm)
    expect_equal(length(pglmm), 11)
    expect_equal(max(nchar(pglmm)), 70)
    res <- pander_return(summary(glmm))
    expect_true(any(grepl('Null deviance', res)))
    expect_equal(length(res), 21)
})

test_that('pander.aov/pander.summary.aov behaves correctly', {
    npk.aovE <- aov(yield ~  N * P * K + Error(block), npk)
    paov <- pander_return(npk.aovE)
    psaov <- pander_return(summary(npk.aovE))
    expect_equal(paov, psaov) # choice of similar result for summary and standard
    expect_equal(length(psaov), 23)
})

test_that('pander.anova behaves correctly', {
    fit <- lm(sr ~ ., data = LifeCycleSavings)
    a <- anova(fit)
    pa <- pander_return(a, style = 'simple')
    expect_true(all(sapply(names(a)[-5], grepl, pa[3])))
    #more complicated run
    fit0 <- lm(sr ~ 1, data = LifeCycleSavings)
    fit1 <- update(fit0, . ~ . + pop15)
    fit2 <- update(fit1, . ~ . + pop75)
    fit3 <- update(fit2, . ~ . + dpi)
    fit4 <- update(fit3, . ~ . + ddpi)
    a <- anova(fit0, fit1, fit2, fit3, fit4, test = 'F')
    pa <- pander_return(a, style = 'simple')
    expect_true(all(sapply(names(a)[-6], grepl, pa[3])))
})

test_that('pander.aovlist/pander.summary.aovlist behaves correctly', {
    options(contrasts = c('contr.helmert', 'contr.poly'))
    N <- c(0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0)
    P <- c(1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0)
    K <- c(1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0)
    yield <- c(49.5, 62.8, 46.8, 57.0, 59.8, 58.5, 55.5, 56.0, 62.8, 55.8, 69.5,
               55.0, 62.0, 48.8, 45.5, 44.2, 52.0, 51.5, 49.8, 48.8, 57.2, 59.0, 53.2, 56.0)
    npk <- data.frame(block = gl(6, 4), N = factor(N), P = factor(P),
                      K = factor(K), yield = yield)
    a <- aov(yield ~  N * P * K + Error(block), npk)
    pa <- pander_return(a, style = 'simple')
    expect_equal(length(pa), 14)
})

test_that('pander.ts behaves correctly', {
    # ncol NULL
    res <- pander_return(ts(1:10, frequency = 4, start = c(1959, 2)), style = 'simple')
    expect_equal(res[3], '  &nbsp;    Q1   Q2   Q3   Q4 ')
    expect_equal(length(res), 8)
    # ncol not NULL
    res <- pander_return(ts(matrix(rnorm(30), 10, 3), start = c(1961, 1), frequency = 12), style = 'simple')
    expect_equal(res[3], '    &nbsp;      Series.1   Series.2   Series.3 ')
    expect_equal(length(res), 15)
})

test_that('pander.coxph behaves correctly', {
    suppressMessages(require(survival))
    test1 <- list(time = c(4, 3, 1, 1, 2, 2, 3),
                  status = c(1, 1, 1, 0, 1, 1, 0),
                  x = c(0, 2, 1, 1, 1, 0, 0),
                  sex = c(0, 0, 0, 0, 1, 1, 1))
    # Fit a stratified model
    res <- pander_return(coxph(Surv(time, status) ~ x + strata(sex), test1))
    expect_equal(res[3], ' &nbsp;    coef    exp(coef)   se(coef)     z        p    ')
    expect_equal(length(res), 10)
})

test_that('pander.clogit works correctly', {
    resp <- levels(logan$occupation)
    n <- nrow(logan)
    indx <- rep(1:n, length(resp))
    logan2 <- data.frame(logan[indx, ],
                         id = indx,
                         tocc = factor(rep(resp, each = n)))
    logan2$case <- (logan2$occupation == logan2$tocc)
    res <- pander_return(suppressWarnings(clogit(case ~ tocc + tocc:education + strata(id), logan2)), split.table = Inf)
    expect_true(grepl('Fitting Conditional logistic regression', res[24]))
    expect_equal(length(res), 26)
})

test_that('pander.zoo works correctly', {
    suppressMessages(require(zoo))
    x.Date <- as.Date('2003-02-01') + c(1, 3, 7, 9, 14) - 1
    res <- pander_return(zoo(rnorm(5), x.Date), style = 'simple')
    expect_true(grepl('Value', res[3]))
    expect_equal(length(res), 10)
    # more complex example with colnames
    res <- pander_return(zoo(cbind(foo = rnorm(5), bar = rnorm(5))), style = 'simple')
    expect_true(grepl('foo', res[3]))
    expect_equal(length(res), 10)
})

test_that('pander.lme/pander.summary.lme behaves correctly', {
    suppressMessages(require(nlme))
    l1 <- lme(distance ~ age, Orthodont, random = ~ age | Subject)
    sl <- summary(l1)
    pl <- pander_return(l1)
    spl <- pander_return(sl)
    expect_equal(length(pl), 11)
    expect_equal(length(grep('Table', pl)), 1)
    expect_equal(length(spl), 29)
    expect_equal(length(grep('Table', spl)), 3)
})

# test_that('pander.describe works correctly', {
#     suppressMessages(require(psych))
#     x <- data.frame(a = rnorm(10), b = rnorm(10, 2, 2), c = rnorm(10, 3, 4))
#     res <- pander_return(describe(x))
#     expect_equal(length(res), 37)
#     expect_equal(length(grep('Table', res)), 2)
#     res <- pander_return(describe(x), split.tables = Inf)
#     expect_equal(length(res), 11)
# })

test_that('pander.survdiff works correctly', {
    suppressMessages(require(survival))
    res <- pander_return(survdiff(Surv(futime, fustat) ~ rx, data = ovarian))
    expect_equal(length(res), 12)
    expect_equal(res[3], '  &nbsp;    N    Observed   Expected   (O-E)^2/E   (O-E)^2/V ')
    # length(x$n) == 1
    expect <- survexp(futime ~ ratetable(age = (accept.dt - birth.dt),
                                         sex = 1,
                                         year = accept.dt,
                                         race = 'white'),
                      jasa, cohort = FALSE, ratetable = survexp.usr)
    res <- pander_return(survdiff(Surv(jasa$futime, jasa$fustat) ~ offset(expect)))
    expect_equal(res[3], ' Observed   Expected     Z      p ')
    expect_equal(length(res), 9)
})

test_that('pander.survfit works correctly', {
    suppressMessages(require(survival))
    res <- pander_return(survfit(Surv(time, status) ~ x, data = aml))
    expect_equal(length(res), 20)
    expect_true(any(grepl('Table', res)))
    # using additional options
    res <- pander_return(survfit(Surv(time, status) ~ x, data = aml), print.rmean = TRUE)
    expect_equal(length(res), 21)
    expect_equal(res[21], '* restricted mean with upper limit =  161')
})

test_that('pander.sessionInfo works correctly', {
    suppressMessages(require(utils))
    res <- pander_return(sessionInfo())
    expect_true(any(grepl('locale', res)))
    expect_true(any(grepl('attached base package', res)))
    res <- pander_return(sessionInfo(), locale = FALSE, compact = FALSE)
    expect_false(any(grepl('locale', res)))
    expect_true(any(grepl('utils', res)))
})

test_that('pander.stat.table works correctly', {
    suppressMessages(require(Epi))
    res <- pander_return(stat.table(
        tension,
        list(count(), mean(breaks)),
        data = warpbreaks))
    expect_equal(length(res), 11)
    expect_equal(res[3], ' &nbsp;   count()   mean(breaks) ')
    res <- pander_return(stat.table(index = list(tension, wool),
                                    mean(breaks), data = warpbreaks))
    expect_equal(length(res), 13)
    # here add test
})

test_that('pander.microbenchmark works correctly', {
    suppressMessages(require(microbenchmark))
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)))
    expect_true(any(grepl('Unit', res)))
    expect_equal(length(res), 11)
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)), split.tables = Inf, expr.labels = c('A'))
    expect_true(any(grepl('A', res)))
    expect_true(any(grepl('paste0\\(1:10\\)', res)))
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)), split.tables = Inf, expr.labels = c('A', 'B'))
    expect_true(any(grepl('A', res)))
    expect_true(any(grepl('B', res)))
    expect_false(any(grepl('paste0\\(1:10\\)', res)))
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)), split.tables = Inf, expr.labels = c('A', 'B', 'C'))
    expect_true(any(grepl('A', res)))
    expect_true(any(grepl('B', res)))
    expect_false(any(grepl('paste0\\(1:10\\)', res)))
})

test_that('pander.function works correctly', {
    testf <- function(x) {
        y <- x + 1
        y
    }
    res <- pander_return(testf)
    expect_true(all(grepl('\t', res)))
    expect_equal(length(res), 5)
    res <- pander_return(testf, syntax.highlighting = TRUE, add.name = TRUE)
    expect_equal(length(res), 7)
    expect_equal(res[1], '```r')
    expect_true(grepl('testf', res[2]))
})

test_that('pander.rlm works correctly', {
    suppressMessages(require(MASS))
    res <- pander_return(rlm(stack.loss ~ ., stackloss))
    expect_equal(res[4], ' (Intercept)   Air.Flow   Water.Temp   Acid.Conc. ')
    expect_equal(length(res), 13)
    res <- pander_return(rlm(stack.loss ~ ., stackloss, psi = psi.hampel, init = 'lts'))
    expect_equal(res[11], 'Degrees of freedom: 21 total; 17 residual')
    expect_equal(length(res), 13)
})

test_that('pander.summary.table works correctly', {
    ts <- summary(xtabs(cbind(ncases, ncontrols) ~ ., data = esoph))
    res <- pander_return(ts)
    expect_equal(res[6], ' Chisq   df     p.value  ')
    expect_equal(length(res), 13)
    res <- pander_return(ts, print.call = FALSE)
    expect_false(any(grepl('Calls', res)))
    expect_equal(length(res), 12)
    res <- pander_return(ts, caption = 'Factor')
    expect_equal(res[11], 'Table: Factor')
    res <- pander_return(summary(table(1:3)))
    expect_false(any(grepl('-', res)))
    expect_equal(length(res), 2)
})

test_that('pander.randomForest works correctly', {
    suppressMessages(require(randomForest))
    iris.rf <- randomForest(Species ~ ., data = iris, importance = TRUE,
                            proximity = TRUE)
    res <- pander_return(iris.rf)
    expect_equal(length(res), 20)
    expect_equal(res[19], 'Table: Confusion Matrix')
    ozone.rf <- randomForest(Ozone ~ ., data = airquality, mtry = 3,
                             importance = TRUE, na.action = na.omit)
    res <- pander_return(ozone.rf)
    expect_equal(length(res), 8)
    expect_false(any(grep('-', res)))
    # with test
    index <- 1:nrow(iris)
    trainindex <- sample(index, trunc(length(index) / 2))
    trainset <- iris[trainindex, ]
    testset <- iris[-trainindex, ]
    res <- pander_return(randomForest(x = trainset[, -1],
                                      y = trainset[, 1],
                                      xtest = testset[, -1],
                                      ytest = testset[, 1]))
    expect_equal(length(res), 10)
    expect_true(any(grepl('Test set MSE', res)))
    res <- pander_return(randomForest(x = trainset[, -ncol(trainset)],
                                      y = trainset[, ncol(trainset)],
                                      xtest = testset[, -ncol(testset)],
                                      ytest = testset[, ncol(testset)]))
    expect_equal(length(res), 34)
    expect_equal(res[33], 'Table: Test Confusion Matrix')
})

test_that('pander.irts works correctly', {
    suppressMessages(require(tseries))
    n <- 10
    t <- cumsum(rexp(n, rate = 0.1))
    v <- rnorm(n)
    res <- pander_return(as.irts(cbind(t, v)))
    expect_equal(length(res), 23)
    u <- rnorm(n)
    res <- pander_return(irts(t, cbind(u, v)))
    expect_equal(length(res), 23)
})

test_that('pander.manova/summary.manova works correctly', {
    npk2 <- within(npk, foo <- rnorm(24))
    x <- manova(cbind(yield, foo) ~ block + N * P * K, npk2)
    res1 <- pander_return(x)
    xs <- summary(x)
    res2 <- pander_return(xs)
    expect_equal(res1, res2)
    expect_equal(length(res1), 21)
})

test_that('pander.gtable works correctly', {
    suppressMessages(require(gtable))
    suppressMessages(require(grid))
    a <- gtable(unit(1:3, c('cm')), unit(5, 'cm'))
    rect <- rectGrob(gp = gpar(fill = 'black'))
    a <- gtable_add_grob(a, rect, 1, 1)
    res <- pander_return(a)
    expect_equal(length(res), 7)
})

test_that('pander.nls/pander.summary.nls works correctly', {
    utils::data(muscle, package = 'MASS')
    with(muscle, table(Strip))
    musc.1 <- nls(Length ~ cbind(1, exp(-Conc / th)), muscle,
                start = list(th = 1), algorithm = 'plinear')
    res <- pander_return(musc.1, show.convergence = TRUE)
    expect_equal(length(res), 17)
    res <- pander_return(musc.1)
    expect_equal(length(res), 14)
    musc.1.s <- summary(musc.1)
    res <- pander_return(musc.1.s)
    expect_equal(length(res), 18)
    res <- pander_return(musc.1.s, show.convergence = TRUE)
    expect_equal(length(res), 21)
    musc.1.s <- summary(musc.1, correlation = TRUE)
    res <- pander_return(musc.1.s)
    expect_true(any(grepl('Correlation', res)))
    expect_equal(length(res), 31)
})

test_that('pander.arima works correctly', {
    res <- pander_return(arima(lh, order = c(1, 0, 0)))
    expect_equal(length(res), 17)
    expect_true(any(grep('s\\.e', res)))
    res <- pander_return(arima(lh, order = c(3, 0, 0)))
    expect_equal(length(res), 17)
    expect_true(any(grep('s\\.e', res)))
    res <- pander_return(arima(lh, order = c(3, 0, 0), method = 'CSS'), se = FALSE)
    expect_equal(length(res), 15)
    expect_false(any(grep('s\\.e', res)))
})

test_that('pander.polr/summary.polr works correctly', {
    house.plr <- polr(Sat ~ Infl + Type + Cont, weights = Freq, data = housing)
    res <- pander_return(house.plr)
    expect_equal(length(res), 25)
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(suppressMessages(summary(house.plr)))
    expect_equal(length(res), 39)
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(suppressMessages(summary(house.plr, correlation = TRUE)), split.table = Inf)
    expect_equal(length(res), 60)
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.survreg/summary.survreg works correctly', {
    suppressMessages(require(survival))
    x <- survreg(Surv(futime, fustat) ~ ecog.ps + rx, ovarian, dist = 'exponential')
    res <- pander_return(x)
    expect_equal(length(res), 27)
    expect_equal(length(grep('Table', res)), 1)
    res <- pander_return(suppressMessages(summary(x)))
    expect_equal(length(res), 33)
    expect_equal(length(grep('Table', res)), 1)
    res <- pander_return(suppressMessages(summary(x, correlation = TRUE)), split.table = Inf)
    expect_equal(length(res), 44)
    expect_equal(length(grep('Table', res)), 2)
})

test_that('pander.ols works correctly', {
    suppressMessages(require(rms))
    set.seed(1)
    x1 <- runif(200)
    x2 <- sample(0:3, 200, TRUE)
    distance <- (x1 + x2 / 3 + rnorm(200)) ^ 2
    d <- datadist(x1, x2)
    f <- ols(sqrt(distance) ~ rcs(x1, 4) + scored(x2), x = TRUE)
    res <- pander_return(f)
    expect_equal(length(res), 50)
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.lrm works correctly', {
    suppressMessages(require(rms))
    x    <- 1:5
    y    <- c(0, 1, 0, 1, 0)
    reps <- c(1, 2, 3, 2, 1)
    res <- pander_return(lrm(y ~ x))
    expect_equal(length(res), 39)
    expect_true(any(grep('y ~ x', res)))
    expect_equal(length(grep('Table', res)), 2)

    x <- rep(x, reps)
    y <- rep(y, reps)
    res <- pander_return(lrm(y ~ x), coefs = FALSE)
    expect_equal(length(res), 28)
    expect_true(any(grep('y ~ x', res)))
    expect_equal(length(grep('Table', res)), 1)

    res <- pander_return(lrm(y ~ x, penalty = 0.1), coefs = TRUE)
    expect_true(any(grep('y ~ x', res)))
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.orm works correctly', {
    suppressMessages(require(rms))
    n <- 100
    y <- round(runif(n), 2)
    x1 <- sample(c(-1, 0, 1), n, TRUE)
    x2 <- sample(c(-1, 0, 1), n, TRUE)
    res <- pander_return(orm(y ~ x1 + x2, eps = 1e-5))
    expect_equal(length(res), 39)
    expect_true(any(grep('y ~ x1 \\+ x2', res)))
    expect_equal(length(grep('Table', res)), 2)

    res <- pander_return(orm(y ~ x1 + x2, eps = 1e-5), coefs = FALSE)
    expect_equal(length(res), 28)
    expect_true(any(grep('y ~ x1 \\+ x2', res)))
    expect_equal(length(grep('Table', res)), 1)
})

test_that('pander.Glm works correctly', {
    suppressMessages(require(rms))
    clotting <- data.frame(
        u = c(5, 10, 15, 20, 30, 40, 60, 80, 100),
        lot1 = c(118, 58, 42, 35, 27, 25, 21, 19, 18),
        lot2 = c(69, 35, 26, 21, 18, 16, 13, 12, 12))
    f <- Glm(lot1 ~ log(u), data = clotting, family = Gamma)
    res <- pander_return(f)
    expect_equal(length(res), 31)
    expect_true(any(grep('lot1 ~ log\\(u\\)', res)))
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(f, coefs = FALSE)
    expect_equal(length(res), 20)
    expect_true(any(grep('ot1 ~ log\\(u\\)', res)))
    expect_equal(length(grep('Table', res)), 1)
})

test_that('pander.cph', {
    options(contrasts = original_contrasts_options)
    suppressMessages(require(rms))
    n <- 1000
    set.seed(731)
    age <- 50 + 12 * rnorm(n)
    label(age) <- 'Age'
    sex <- factor(sample(c('Male', 'Female'), n, rep = TRUE, prob = c(.6, .4)))
    cens <- 15 * runif(n)
    h <- .02 * exp(.04 * (age - 50) + .8 * (sex == 'Female'))
    dt <- -log(runif(n)) / h
    label(dt) <- 'Follow-up Time'
    e <- ifelse(dt <= cens, 1, 0)
    dt <- pmin(dt, cens)
    units(dt) <- 'Year'
    dd <- datadist(age, sex)
    S <- Surv(dt, e)
    f <- cph(S ~ rcs(age, 4) + sex, x = TRUE, y = TRUE)
    res <- pander_return(f)
    expect_equal(length(res), 43)
    expect_true(any(grep('S ~ rcs\\(age, 4\\) \\+ sex', res)))
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(f, conf.int = 0.95)
    expect_equal(length(res), 58)
    expect_true(any(grep('S ~ rcs\\(age, 4\\) \\+ sex', res)))
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.ets works correctly', {
  suppressMessages(require(forecast))
  res <- pander_return(ets(lh, model = 'AAN', lambda = .5))
  expect_equal(length(res), 27)
  expect_true(any(grep('Box-Cox', res)))
  res <- pander_return(ets(lh, model = 'ANN'))
  expect_equal(length(res), 25)
  expect_false(any(grep('Box-Cox', res)))
  res <- pander_return(ets(lh, model = 'AAN', damped = TRUE))
  expect_equal(length(res), 25)
  expect_true(any(grep('phi', res)))
})

test_that('pander.list works correctly', {
  res <- pander_return(list(1:10))
  expect_equal(length(res), 7)
  expect_true(any(grep('end of list', res)))
  res <- pander_return(list(a = 1:10, b = list()))
  expect_equal(length(res), 8)
  expect_true(any(grep('end of list', res)))
  res <- pander_return(list(a = list(b = list(c = 5))))
  expect_equal(length(res), 13)
  expect_true(any(grep('end of list', res)))
})

library(testthat)
library(pander)

context('replacing missing values')

pom <- panderOptions('missing')

panderOptions('missing', '<missing>')
test_that('replacing missing values works with string', {
    expect_equal(pander_return(c(1, NA)), '_1_ and _<missing>_')
    expect_equal(paste(pander_return(list(1, NA)), collapse = '\n'),
                 '\n\n  * _1_\n  * <missing>\n\n<!-- end of list -->\n\n')
    expect_equal(paste(pander_return(
        data.frame(a = 1:2, b = c(1, NA), c = c(NA, 'z'))),
        collapse = '\n'),
                 '\n---------------------------\n a       b           c     \n--- ----------- -----------\n 1       1       <missing> \n\n 2   <missing>       z     \n---------------------------\n') #nolint
})

panderOptions('missing', '')
test_that('suppressing missing values works', {
    expect_equal(pander_return(c(1, NA)), '_1_ and __')
    expect_equal(paste(pander_return(list(1, NA)), collapse = '\n'),
                 '\n\n  * _1_\n  *\n\n<!-- end of list -->\n\n')
    expect_equal(paste(pander_return(data.frame(a = 1:2, b = c(1, NA), c = c(NA, 'z'))), collapse = '\n'),
                 '\n-----------\n a   b   c \n--- --- ---\n 1   1     \n\n 2       z \n-----------\n')
})

panderOptions('missing', pom)

library(testthat)
library(pander)
context('Pandoc R5 reference class')

graph.dir <- evalsOptions('graph.dir')
evalsOptions('graph.dir',  file.path(tempdir(), 'plots'))

test_that('quick demo on R5 refclass', {
    myReport <- Pandoc$new()
    myReport$add.paragraph('Hello there!')
    myReport$add(head(mtcars))
    myReport$add(chisq.test(table(mtcars$am, mtcars$gear)))
    expect_true(class(myReport) == 'Pandoc')
    expect_equal(length(capture.output(myReport)), 63)
    t <- tempfile()
    myReport$format <- 'html'
    expect_equal(myReport$export(t, open = FALSE), paste0(t, '.html'))
    unlink(t)
})

evalsOptions('graph.dir',  graph.dir)

#' Minor tick marks
#'
#' Adds minor tick marks to an existing \code{base} plot.
#' @param nx intervals to split on x axis
#' @param ny intervals to split on x axis
#' @param grid show grid lines at minor ticks?
#' @keywords internal
#' @references This function is a forked version of \code{Hmisc::minor.tick}
#' @importFrom graphics par axis abline
add.minor.ticks <- function (nx = 4, ny = 4, grid = TRUE) {

    ax <- function(w, n, grid) {
        range <- par('usr')[ifelse(w == 'x', 1:2, 3:4)]
        tick.pos <- ifelse(w == 'x', par('xaxp'), par('yaxp'))
        distance.between.minor <- (tick.pos[2] - tick.pos[1]) / tick.pos[3] / n
        possible.minors <- tick.pos[1] - (0:100) * distance.between.minor
        low.minor <- min(possible.minors[possible.minors >= range[1]])
        if (is.na(low.minor)) {
            low.minor <- tick.pos[1]
        }
        possible.minors <- tick.pos[2] + (0:100) * distance.between.minor
        hi.minor <- max(possible.minors[possible.minors <= range[2]])
        if (is.na(hi.minor)) {
            hi.minor <- tick.pos[2]
        }
        ticks.at <- seq(low.minor, hi.minor, by = distance.between.minor)
        axis(if (w == 'x')
             1
        else 2, ticks.at,
             labels = FALSE, tcl = par('tcl'))
        if (w == 'x') {
            abline(v = ticks.at,
                   col = panderOptions('graph.grid.color'),
                   lwd = 0.2,
                   lty = panderOptions('graph.grid.lty'))
        } else {
            abline(h = ticks.at,
                   col = panderOptions('graph.grid.color'),
                   lwd = 0.2,
                   lty = panderOptions('graph.grid.lty'))
        }
    }

    if (nx > 1) {
        ax('x', nx, grid = grid)
    }
    if (ny > 1) {
        ax('y', ny, grid = grid)
    }

    invisible()

}


#' Add grids
#'
#' Exactly the same fn as  see \code{latticeExtra::axis.grid} but using \code{grid} colors instead.
#' @param side see \code{latticeExtra::axis.grid}
#' @param ... see \code{latticeExtra::axis.grid}
#' @param ticks see \code{latticeExtra::axis.grid}
#' @param scales see \code{latticeExtra::axis.grid}
#' @param components see \code{latticeExtra::axis.grid}
#' @param line.col see \code{latticeExtra::axis.grid}
#' @keywords internal
#' @references This function is a forked version of \code{latticeExtra::axis.grid}
add.lattice.grid <- function (side = c('top', 'bottom', 'left', 'right'), ..., ticks = c('default', 'yes', 'no'), scales, components, line.col) { #nolint

    side <- match.arg(side)
    ticks <- match.arg(ticks)
    scales.tck <- switch(side, left = , bottom = scales$tck[1], right = , top = scales$tck[2]) #nolint
    comps.major <- components
    mycomps <- components[[side]]

    if (is.list(mycomps)) {
        lab <- as.character(mycomps$labels$labels)
        if (any(lab != '')) {
            tck <- mycomps$ticks$tck
            if (any(tck * scales.tck != 0)) {
                tck <- rep(tck, length = length(lab))
                comps.major[[side]]$ticks$tck <- ifelse(lab == '', NA, tck)
            }
        }
    } else {
        ticks <- 'no'
    }

    lattice::axis.default(side,
                          scales = scales,
                          ticks = ticks,
                          components = comps.major,
                          ...,
                          line.col = panderOptions('graph.grid.color'))

    if (side %in% c('top', 'left')) {
        return()
    }
    if (scales$draw == FALSE) {
        return()
    }
    ref.line <- lattice::trellis.par.get('reference.line')

    if (side == 'bottom') {
        tck <- abs(mycomps$ticks$tck)
        lattice::panel.refline(v = mycomps$ticks$at,
                               lwd = ref.line$lwd * tck,
                               alpha = ref.line$alpha * tck / max(tck, na.rm = TRUE))
    }
    if (side == 'right') {
        if (!is.list(mycomps)) {
            mycomps <- components[['left']]
        }
        tck <- abs(mycomps$ticks$tck)
        lattice::panel.refline(h = mycomps$ticks$at,
                               lwd = ref.line$lwd * tck,
                               alpha = ref.line$alpha * tck / max(tck, na.rm = TRUE))
    }

}

#' Add subticks
#'
#' See \code{latticeExtra::xscale.components.subticks}.
#' @param lim see \code{latticeExtra::xscale.components.subticks}
#' @param ... see \code{latticeExtra::xscale.components.subticks}
#' @param n see \code{latticeExtra::xscale.components.subticks}
#' @keywords internal
#' @aliases add.lattice.ysubticks
#' @references This is a simplified verwion of \code{latticeExtra::xscale.components.subticks}.
add.lattice.xsubticks <- function (lim, ..., n = 2) {

    ans    <- lattice::xscale.components.default(lim = lim, ..., n = n)
    ticks  <- ans$bottom$ticks$at
    ticks2 <- head(ticks, -1) + diff(head(ticks, 2)) / 2
    ans$bottom$ticks$at <- c(ticks, ticks2)
    ans$bottom$ticks$tck <- c(rep(1, length(ticks)), rep(0.5, length(ticks2)))
    ans$bottom$labels$at <- ans$bottom$ticks$at
    ans$bottom$labels$labels <- c(ans$bottom$labels$labels, rep(' ', length(ticks2)))
    ans$bottom$labels$check.overlap <- FALSE
    ans

}

#' @keywords internal
add.lattice.ysubticks <- function (lim, ..., n = 2) {

    ans    <- lattice::yscale.components.default(lim = lim, ..., n = n)
    ticks  <- ans$left$ticks$at
    ticks2 <- head(ticks, -1) + diff(head(ticks, 2)) / 2
    ans$left$ticks$at <- c(ticks, ticks2)
    ans$left$ticks$tck <- c(rep(1, length(ticks)), rep(0.5, length(ticks2)))
    ans$left$labels$at <- ans$left$ticks$at
    ans$left$labels$labels <- c(ans$left$labels$labels, rep(' ', length(ticks2)))
    ans$left$labels$check.overlap <- FALSE
    ans

}

#' Evaluate with messages
#'
#' This function takes text(s) of R code and \code{eval}s all at one run - returning a list with four elements. See \code{Details}.
#'
#' \code{eval.msgs} returns a detailed list of the result of evaluation:
#' \itemize{
#'     \item \emph{src} - character vector of specified R code.
#'     \item \emph{result} - result of evaluation. \code{NULL} if nothing is returned. If any R code returned an R object while evaluating then the \emph{last} R object will be returned as a raw R object. If a graph is plotted in the end of the given R code (remember: \emph{last} R object), it would be automatically printed (see e.g. \code{lattice} and \code{ggplot2}).
#'      \item \emph{output} - character vector of printed version (\code{capture.output}) of \code{result}
#'     \item \emph{type} - class of generated output. 'NULL' if nothing is returned, 'error' if some error occurred.
#'     \item \emph{msg} - possible messages grabbed while evaluating specified R code with the following structure:
#'     \itemize{
#'         \item \emph{messages} - character vector of possible diagnostic message(s)
#'         \item \emph{warnings} - character vector of possible warning message(s)
#'         \item \emph{errors} - character vector of possible error message(s)
#'     }
#'     \item \emph{stdout} - character vector of possibly printed texts to standard output (console)
#' }
#' @param src character values containing R code
#' @param env environment where evaluation takes place. If not set (by default), a new temporary environment is created.
#' @param showInvisible return \code{invisible} results?
#' @param graph.unify should \code{eval.msgs} try to unify the style of (\code{lattice} and \code{ggplot2}) plots? If set to \code{TRUE} (by default), some \code{panderOptions()} would apply. Please note that this argument has no effect on \code{base} plots, use \code{evals} instead.
#' @return a list of parsed elements each containing: \code{src} (the command run), \code{result} (R object: \code{NULL} if nothing returned), \code{print}ed \code{output}, \code{type} (class of returned object if any), informative/wawrning and error messages (if any returned by the command run, otherwise set to \code{NULL}) and possible \code{stdout}t value. See Details above.
#' @seealso \code{\link{evals}}
#' @export
#' @examples \dontrun{
#' eval.msgs('1:5')
#' eval.msgs('x <- 1:5')
#' eval.msgs('lm(mtcars$hp ~ mtcars$wt)')
#'
#' ## plots
#' eval.msgs('plot(runif(100))')
#' eval.msgs('histogram(runif(100))')
#'
#' ## error handling
#' eval.msgs('runiff(23)')
#' eval.msgs('runif is a nice function')
#' eval.msgs('no.R.object.like.that')
#'
#' ## messages
#' eval.msgs(c('message("FOO")', '1:2'))
#' eval.msgs(c('warning("FOO")', '1:2'))
#' eval.msgs(c('message("FOO");message("FOO");warning("FOO")', '1:2'))
#' eval.msgs('warning("d");warning("f");1')
#'
#' ## stdout
#' eval.msgs('cat("writing to console")')
#' eval.msgs('cat("writing to console");1:4')
#' }
eval.msgs <- function(src, env = NULL, showInvisible = FALSE, graph.unify = evalsOptions('graph.unify')) {

    if (is.null(env)) {
        env <- new.env()
    }

    ## grab warnings and messages
    warnings <- NULL
    warning.handler <- function(w) {
        warnings <<- c(warnings, w$message)
        invokeRestart('muffleWarning')
    }

    messages <- NULL
    message.handler <- function(m) {
        messages <<- c(messages, sub('\n$', '', m$message))
    }

    ## grab stdout
    stdout <- vector('character')
    con <- textConnection('stdout', 'wr', local = TRUE)
    sink(con, split = FALSE)

    result <- suppressMessages(withCallingHandlers(tryCatch(withVisible(eval(parse(text = src), envir = env)),
                                                            error = function(e) e),
                                                   warning = warning.handler, message = message.handler))

    sink()
    close(con)
    if (length(stdout) == 0)
        stdout <- NULL

    ## error handling
    if (inherits(result, 'error')) {

        error  <- result$message
        result <- NULL

        if (grepl('^<text>:[0-9]*:[0-9]: unexpected ', error)) {
            error <- sub('<text>:([0-9]*):([0-9]*): unexpected ([a-zA-Z ]*)\n.*[0-9]*:(.*)\n.*[ \t]*$',
                         'Unexpected \\3 at character \\2 in line \\1: `\\4`',
                         error)
        }

    } else
        error <- NULL

    ## check if printing is needed
    if (!is.null(result)) {
        rv  <- result$value
        rvc <- class(rv)

        if (result$visible | showInvisible) {

            ## unify images
            if (graph.unify) {

                fs <- panderOptions('graph.fontsize')
                ff <- panderOptions('graph.fontfamily')
                fc <- panderOptions('graph.fontcolor')
                gc <- panderOptions('graph.grid.color')
                gl <- panderOptions('graph.grid.lty')
                cs <- panderOptions('graph.colors')
                if (panderOptions('graph.color.rnd'))
                    cs <- sample(cs)
                cb <- cs[1] # base color
                aa <- panderOptions('graph.axis.angle')
                bc <- panderOptions('graph.background')
                pc <- panderOptions('graph.panel.background')
                tc <- ifelse(pc == 'transparent', bc, pc) # 'transparent' color
                gs <- panderOptions('graph.symbol')

                ## lattice/trellis
                if ('trellis' %in% rvc) {

                    ## margin
                    if (panderOptions('graph.nomargin')) {
                        rv$par.settings$layout.heights <- list(top.padding = 0.4,
                                                               bottom.padding = 0,
                                                               main = 1.4,
                                                               main.key.padding = 0)
                        rv$par.settings$layout.widths  <- list(right.padding = -1,
                                                               left.padding = 0.4)
                    }

                    ## font family
                    rv$par.settings$axis.text <- rv$par.settings$add.text <- rv$par.settings$par.xlab.text <- rv$par.settings$par.ylab.text <- rv$par.settings$par.zlab.text <- rv$par.settings$par.sub.text <- rv$par.settings$par.main.text <- list(fontfamily = ff, col = fc) #nolint
                    rv$par.settings$fontsize  <- list(text = fs, points = fs * 0.8)

                    ## boxes
                    rv$par.settings$strip.background$col     <- 'transparent'
                    if (!panderOptions('graph.boxes')) {
                        rv$par.settings$strip.border$col     <- 'transparent'
                        rv$par.settings$axis.line$col        <- 'transparent'
                    } else {
                        rv$par.settings$strip.border$col     <- gc
                        rv$par.settings$axis.line$col        <- gc
                    }
                    rv$par.settings$between                  <- list(x = 0.4, y = 0.4)

                    ## colors
                    rv$par.settings$background$col           <- bc
                    rv$par.settings$panel.background$col     <- pc
                    rv$par.settings$plot.line$col            <- rv$par.settings$box.rectangle$fill <- rv$par.settings$box.rectangle$col <- rv$par.settings$plot.polygon$col <- cb #nolint
                    rv$par.settings$superpose.symbol$col     <- rv$par.settings$superpose.symbol$col <- cs
                    rv$par.settings$superpose.polygon$border <- rv$par.settings$plot.polygon$border <- tc
                    rv$par.settings$box.umbrella             <- list(col = 'black', lty = 'solid', lwd = 2)
                    rv$par.settings$plot.line$lwd            <- 2
                    ## pch
                    rv$par.settings$plot.symbol$pch          <- rv$par.settings$superpose.symbol$pch <- gs
                    ##if (gs > 20 & gs <= 25 ) {}
                    rv$par.settings$plot.symbol$col          <- cb
                    rv$par.settings$superpose.symbol$col     <- cs
                    rv$par.settings$plot.symbol$fill         <- rv$par.settings$superpose.symbol$fill <- tc

                    ## grid
                    if (panderOptions('graph.grid')) {
                        rv$par.settings$reference.line$col <- gc
                        rv$par.settings$reference.line$lty <- gl
                        rv$par.settings$reference.line$lwd <- 0.7
                        rv$axis <- add.lattice.grid
                        if (panderOptions('graph.grid.minor')) {
                            rv$xscale.components <- add.lattice.xsubticks
                            rv$yscale.components <- add.lattice.ysubticks
                        }
                    }

                    ## axis angle
                    switch(aa,
                           '0' = rv$y.scales$rot <- c(90, 90),
                           '2' = rv$x.scales$rot <- c(90, 90),
                           '3' = {
                                    rv$y.scales$rot <- c(90, 90)
                                    rv$x.scales$rot <- c(90, 90)
                            })
                    ## legend
                    if (!is.null(rv$legend)) {
                        rv$legend <- rv$legend[1]
                        names(rv$legend) <- panderOptions('graph.legend.position')
                    }

                }

                ## ggplot2
                if ('ggplot' %in% rvc) {

                    ## margin
                    if (panderOptions('graph.nomargin')) {
                        rv$theme$plot.margin  <- grid::unit(c(0.1, 0.1, 0.1, 0), 'lines')
                    }

                    ## font family
                    rv$theme$plot.title       <- ggplot2::element_text(colour = fc,
                                                                       family = ff,
                                                                       face = 'bold',
                                                                       size = fs * 1.2)
                    rv$theme$axis.text.x      <- rv$theme$axis.text.y <- rv$theme$legend.text <- ggplot2::element_text(colour = fc, family = ff, face = 'plain', size = fs * 0.8) #nolint
                    rv$theme$axis.title.x     <- ggplot2::element_text(colour = fc,
                                                                       family = ff,
                                                                       face = 'plain',
                                                                       size = fs)
                    rv$theme$strip.text.x     <- ggplot2::element_text(colour = fc,
                                                                       family = ff,
                                                                       face = 'plain',
                                                                       size = fs)
                    rv$theme$axis.title.y     <- ggplot2::element_text(colour = fc,
                                                                       family = ff,
                                                                       face = 'plain',
                                                                       size = fs,
                                                                       angle = 90)
                    rv$theme$legend.title     <- ggplot2::element_text(colour = fc,
                                                                       family = ff,
                                                                       face = 'italic',
                                                                       size = fs)
                    rv$theme$strip.text.y     <- ggplot2::element_text(colour = fc,
                                                                       family = ff,
                                                                       face = 'plain',
                                                                       size = fs,
                                                                       angle = -90)

                    ## boxes
                    if (!panderOptions('graph.boxes')) {
                        rv$theme$legend.key       <- rv$theme$strip.background <- ggplot2::element_rect(colour = 'transparent', fill = 'transparent') #nolint
                        rv$theme$panel.border     <- ggplot2::element_rect(fill = NA,
                                                                           colour = tc)
                        rv$theme$panel.background <- ggplot2::element_rect(fill = pc,
                                                                           colour = tc)
                    } else {
                        rv$theme$legend.key       <- rv$theme$strip.background <- ggplot2::element_rect(colour = gc, fill = 'transparent') #nolint
                        rv$theme$panel.border     <- ggplot2::element_rect(fill = NA,
                                                                           colour = gc)
                        rv$theme$panel.background <- ggplot2::element_rect(fill = pc,
                                                                           colour = gc)
                    }

                    ## colors
                    rv$theme$plot.background  <- ggplot2::element_rect(fill = bc, colour = NA)
                    rv$theme$axis.ticks       <- ggplot2::element_line(colour = gc, size = 0.2)

                    ## point shape still has to be updated
                    for (i in length(rv$layers)) {
                        if (packageVersion('ggplot2') <= '1.0.1') {
                            if (rv$layers[[i]]$geom$objname %in% c('point')) {
                                rv$layers[[i]]$geom_params$size   <- 3
                                rv$layers[[i]]$geom_params$shape  <- gs
                            }
                        } else {
                            if (inherits(rv$layers[[i]]$geom, 'GeomPoint')) {
                                rv$layers[[i]]$aes_params$size  <- 3
                                rv$layers[[i]]$aes_params$shape <- gs
                            }
                        }
                    }

                    ## geom colors
                    if (is.null(rv$labels$colour) & is.null(rv$labels$fill)) {

                        ## update layers with one color
                        ## this is an ugly hack but `update_geom_defaults`
                        ## is not reversible :(
                        if (packageVersion('ggplot2') <= '1.0.1') {
                            for (i in 1:length(rv$layers)) {
                                if (rv$layers[[i]]$geom$objname %in% c('histogram', 'bar')) {
                                    rv$layers[[i]]$geom_params$fill   <- cs[i]
                                    rv$layers[[i]]$geom_params$colour <- tc
                                } else {
                                    if (rv$layers[[i]]$geom$objname %in% c('boxplot')) {
                                        rv$layers[[i]]$geom_params$fill   <- cs[i]
                                        rv$layers[[i]]$geom_params$colour <- 'black'
                                    } else {
                                        if (rv$layers[[i]]$geom$objname %in% c('point')) {
                                            rv$layers[[i]]$geom_params$fill   <- tc
                                            rv$layers[[i]]$geom_params$colour <- cs[i]
                                        } else
                                            rv$layers[[i]]$geom_params$colour <- cs[i]
                                    }
                                }
                            }
                        } else {
                            for (i in 1:length(rv$layers)) {
                                if (inherits(rv$layers[[i]]$geom, 'GeomBar')) {
                                    rv$layers[[i]]$aes_params$fill   <- cs[i]
                                    rv$layers[[i]]$aes_params$colour <- tc
                                } else {
                                    if (inherits(rv$layers[[i]]$geom, 'GeomBoxplot')) {
                                        rv$layers[[i]]$aes_params$fill   <- cs[i]
                                        rv$layers[[i]]$aes_params$colour <- 'black'
                                    } else {
                                        if (inherits(rv$layers[[i]]$geom, 'GeomPoint')) {
                                            rv$layers[[i]]$aes_params$fill   <- tc
                                            rv$layers[[i]]$aes_params$colour <- cs[i]
                                        } else
                                            rv$layers[[i]]$aes_params$colour <- cs[i]
                                    }
                                }
                            }
                        }

                    } else {

                        ## we have a possible color scale (only dealing with discrete scales)
                        if (is.null(rv$labels$colour)) {

                            ## the scale of geom_tile is continuous
                            if (packageVersion('ggplot2') <= '1.0.1') {
                                tile <- rv$layers[[1]]$geom$objname %in% 'tile'
                            } else {
                                tile <- inherits(rv$layers[[1]]$geom, 'GeomTile')
                            }

                            if (length(rv$scales$scales) == 0 & !tile) {
                                rv <- rv + ggplot2::scale_fill_manual(values = cs)
                            } else {
                                ## we still might have something without a guide
                                if (!'continuous' %in% unlist(lapply(rv$scales$scales, class)) & !tile)
                                    rv <- rv + ggplot2::scale_fill_manual(values = cs, guide = FALSE)
                            }

                        } else {

                            if (length(rv$scales$scales) == 0) {
                                rv <- rv + ggplot2::scale_colour_manual(values = cs)
                            } else {
                                ## we still might have something without a guide
                                if (!'continuous' %in% unlist(lapply(rv$scales$scales, class)))
                                    rv <- rv + ggplot2::scale_colour_manual(values = cs, guide = FALSE)
                            }

                        }

                        ## no solid shapes
                        rv <- rv + ggplot2::scale_shape(solid = FALSE)

                    }

                    ## grid
                    if (!panderOptions('graph.grid')) {
                        rv$theme$panel.grid.minor <- rv$theme$panel.grid.major <- ggplot2::element_blank()
                    } else {
                        if (!panderOptions('graph.grid.minor')) {
                            rv$theme$panel.grid.major <- ggplot2::element_line(colour = gc,
                                                                               size = 0.2,
                                                                               linetype = gl)
                            rv$theme$panel.grid.minor <- ggplot2::element_blank()
                        } else {
                            rv$theme$panel.grid.minor <- ggplot2::element_line(colour = gc,
                                                                               size = 0.1,
                                                                               linetype = gl)
                            rv$theme$panel.grid.major <- ggplot2::element_line(colour = gc,
                                                                               size = 0.2,
                                                                               linetype = gl)
                        }
                    }

                    ## axis angle
                    if (aa == 0) {
                        rv$theme$axis.text.y <- ggplot2::element_text(colour = fc,
                                                                      family = ff,
                                                                      face = 'plain',
                                                                      size = fs * 0.8,
                                                                      angle = 90)
                    }
                    if (aa == 2) {
                        rv$theme$axis.text.x <- ggplot2::element_text(colour = fc,
                                                                      family = ff,
                                                                      face = 'plain',
                                                                      size = fs * 0.8,
                                                                      angle = 90,
                                                                      hjust = 1)
                    }
                    if (aa == 3) {
                        rv$theme$axis.text.y <- ggplot2::element_text(colour = fc,
                                                                      family = ff,
                                                                      face = 'plain',
                                                                      size = fs * 0.8,
                                                                      angle = 90)
                        rv$theme$axis.text.x <- ggplot2::element_text(colour = fc,
                                                                      family = ff,
                                                                      face = 'plain',
                                                                      size = fs * 0.8,
                                                                      angle = 90,
                                                                      hjust = 1)
                    }

                    ## legend
                    if (!is.null(rv$theme$legend.position)) {
                        if (rv$theme$legend.position != 'none') {
                            rv$theme$legend.position <- panderOptions('graph.legend.position')
                        }
                    } else {
                        rv$theme$legend.position <- panderOptions('graph.legend.position')
                    }

                }

            }

            ## grab output
            output <- vector('character')
            con <- textConnection('output', 'wr', local = TRUE)
            sink(con, split = FALSE)

            p.result <- tryCatch(print(rv), error = function(e) e)

            ## error while printing
            if (inherits(p.result, 'error')) {
                error  <- p.result$message
            }

            result <- rv

            sink()
            close(con)

        } else {

            result <- output <- NULL

        }
    }

    if (is.null(error)) {
        type  <- rvc
    } else {
        result <- output <- NULL
        type   <- 'error'
    }

    ## return
    list(src    = src,
         result = result,
         output = output,
         type   = type,
         msg    = list(
             messages = messages,
             warnings = warnings,
             errors   = error),
         stdout = stdout)

}


#' Evaluate and Process R Code
#'
#' This function takes either a vector/list of \emph{strings} with actual R code, which it to be \code{parse}d to separate elements. Each list element is \code{eval}uated in a special environment, and a detailed list of results is returned for each logical part of the R code: a character value with R code, resulting R object, printed output, class of resulting R object, possible informative/warning/error messages and anything written to \code{stdout}. If a graph is plotted in the given text, the returned object is a string specifying the path to the saved file. Please see Details below.
#' If \code{parse} option set to \code{FALSE}, then the returned list's length equals to the length of the \code{parse}d input - as each string is evaluated as separate R code in the same environment. If a nested list of R code or a concatenated string (separated by \code{\\n} or \code{;}) is provided like \code{list(c('runif(1)', 'runif(1)'))} with \code{parse=FALSE}, then everything is \code{eval}ed at one run so the length of returned list equals to one or the length of the provided nested list. See examples below.
#'
#' As \code{\link{evals}} tries to grab the plots internally, pleas do not run commands that set graphic device or \code{dev.off}. E.g. running \code{evals(c('png("/tmp/x.png")', 'plot(1:10)', 'dev.off()'))} would fail. \code{print}ing of \code{lattice} and \code{ggplot2} objects is not needed, \code{evals} would deal with that automatically.
#'
#' The generated image file(s) of the plots can be fine-tuned by some specific options, please check out \code{graph.output}, \code{width}, \code{height}, \code{res}, \code{hi.res}, \code{hi.res.width}, \code{hi.res.height} and \code{hi.res.res} parameters. Most of these options are better not to touch, see details of parameters below.
#'
#' Returned result values: list with the following elements
#' \itemize{
#'     \item \emph{src} - character vector of specified R code.
#'     \item \emph{result} - result of evaluation. \code{NULL} if nothing is returned. If any R code returned an R object while evaluating then the \emph{last} R object will be returned as a raw R object. If a graph is plotted in the given text, the returned object is a string (with \code{class} set to \code{image}) specifying the path to the saved image file. If graphic device was touched, then no other R objects will be returned.
#'      \item \emph{output} - character vector of printed version (\code{capture.output}) of \code{result}
#'     \item \emph{type} - class of generated output. 'NULL' if nothing is returned, 'error' if some error occurred.
#'     \item \emph{msg} - possible messages grabbed while evaluating specified R code with the following structure:
#'     \itemize{
#'         \item \emph{messages} - character vector of possible diagnostic message(s)
#'         \item \emph{warnings} - character vector of possible warning message(s)
#'         \item \emph{errors} - character vector of possible error message(s)
#'     }
#'     \item \emph{stdout} - character vector of possibly printed texts to standard output (console)
#' }
#'
#' By default \code{evals} tries to \emph{cache} results. This means that if evaluation of some R commands take too much time (specified in \code{cache.time} parameter), then \code{evals} would save the results in a file and return from there on next exact R code's evaluation. This caching algorithm tries to be smart as checks not only the passed R sources, but all variables inside that and saves the hash of those.
#'
#' Technical details of the caching algorithm:
#' \itemize{
#'      \item Each passed R chunk is \code{parse}d to single commands.
#'      \item Each parsed command's part (let it be a function, variable, constant etc.) \code{eval}uated (as a \code{name}) separately to a \code{list}. This list describes the unique structure and the content of the passed R commands, and has some IMHO really great benefits (see examples below).
#'      \item A hash if computed to each list element and cached too in \code{pander}'s local environments. This is useful if you are using large data frames, just imagine: the caching algorithm would have to compute the hash for the same data frame each time it's touched! This way the hash is recomputed only if the R object with the given name is changed.
#'      \item The list is \code{serialize}d and an \code{SHA-1} hash is computed for that - which is unique and there is no real risk of collision.
#'      \item If \code{evals} can find the cached results in a file named to the computed hash, then it is returned on the spot.
#'      \item Otherwise the call is evaluated and the results are optionally saved to cache (e.g. if \code{cache} is active, if the \code{proc.time()} of the evaluation is higher then it is defined in \code{cache.time} etc.).
#' }
#'
#' This is a quite secure way of caching, but if you would encounter any issues, just set \code{cache} to \code{FALSE} or tweak other cache parameters. While setting \code{cache.dir}, please do think about what you are doing and move your \code{graph.dir} accordingly, as \code{evals} might result in returning an image file path which is not found any more on your file system!
#'
#' Also, if you have generated a plot and rendered that to e.g. \code{png} before and later try to get e.g. \code{pdf} - it would fail with \code{cache} on. Similarly you cannot render a high resolution image of a cached image, but you have to (temporary) disable caching.
#'
#' The default \code{evals} options could be set globally with \code{\link{evalsOptions}}, e.g. to switch off the cache just run \code{evalsOptions('cache', FALSE)}.
#'
#' Please check the examples carefully below to get a detailed overview of \code{\link{evals}}.
#' @param txt a character vector containing R code. This could be a list/vector of lines of code or a simple string holding R code separated by \code{;} or \code{\\n}.
#' @param parse if \code{TRUE} the provided \code{txt} elements would be merged into one string and parsed to logical chunks. This is useful if you would want to get separate results of your code parts - not just the last returned value, but you are passing the whole script in one string. To manually lock lines to each other (e.g. calling a \code{plot} and on next line adding an \code{abline} or \code{text} to it), use a plus char (\code{+}) at the beginning of each line which should be evaluated with the previous one(s). If set to \code{FALSE}, \code{evals} would not try to parse R code, it would get evaluated in separate runs - as provided. Please see examples below.
#' @param cache caching the result of R calls if set to \code{TRUE}. Please note the caching would not work if \code{parse} set to \code{FALSE} or syntax error is to be found.
#' @param cache.mode cached results could be stored in an \code{environment} in \emph{current} R session or let it be permanent on \code{disk}.
#' @param cache.dir path to a directory holding cache files if \code{cache.mode} set to \code{disk}. Default to \code{.cache} in current working directory.
#' @param cache.time number of seconds to limit caching based on \code{proc.time}. If set to \code{0}, all R commands, if set to \code{Inf}, none is cached (despite the \code{cache} parameter).
#' @param cache.copy.images copy images to new file names if an image is returned from the \emph{disk} cache? If set to \code{FALSE} (default), the cached path would be returned.
#' @param showInvisible return \code{invisible} results?
#' @param classes a vector or list of classes which should be returned. If set to \code{NULL} (by default) all R objects will be returned.
#' @param hooks list of hooks to be run for given classes in the form of \code{list(class = fn)}. If you would also specify some parameters of the function, a list should be provided in the form of \code{list(fn, param1, param2=NULL)} etc. So the hooks would become \code{list(class1=list(fn, param1, param2=NULL), ...)}. See example below. A default hook can be specified too by setting the class to \code{'default'}. This can be handy if you do not want to define separate methods/functions to each possible class, but automatically apply the default hook to all classes not mentioned in the list. You may also specify only one element in the list like: \code{hooks=list('default' = pander_return)}. Please note, that nor error/warning messages, nor stdout is captured (so: updated) while running hooks!
#' @param length any R object exceeding the specified length will not be returned. The default value (\code{Inf}) does not filter out any R objects.
#' @param output a character vector of required returned values. This might be useful if you are only interested in the \code{result}, and do not want to save/see e.g. \code{messages} or \code{print}ed \code{output}. See examples below.
#' @param env environment where evaluation takes place. If not set (by default), a new temporary environment is created.
#' @param graph.unify should \code{evals} try to unify the style of (\code{base}, \code{lattice} and \code{ggplot2}) plots? If set to \code{TRUE}, some \code{panderOptions()} would apply. By default this is disabled not to freak out useRs :)
#' @param graph.name set the file name of saved plots which is \code{\link{tempfile}} by default. A simple character string might be provided where \code{\%d} would be replaced by the index of the generating \code{txt} source, \code{\%n} with an incremented integer in \code{graph.dir} with similar file names and \code{\%t} by some unique random characters. While running in \code{\link{Pandoc.brew}} other indices could be triggered like \code{\%i} and \code{\%I}.
#' @param graph.dir path to a directory where to place generated images. If the directory does not exist, \code{evals} try to create that. Default set to \code{plots} in current working directory.
#' @param graph.output set the required file format of saved plots. Currently it could be any of  \code{grDevices}': \code{png}, \code{bmp}, \code{jpeg}, \code{jpg}, \code{tiff}, \code{svg} or \code{pdf}.
#' @param width width of generated plot in pixels for even vector formats
#' @param height height of generated plot in pixels for even vector formats
#' @param res nominal resolution in \code{ppi}. The height and width of vector images will be calculated based in this.
#' @param hi.res generate high resolution plots also? If set to \code{TRUE}, each R code parts resulting an image would be run twice.
#' @param hi.res.width width of generated high resolution plot in pixels for even vector formats
#' @param hi.res.height height of generated high resolution plot in pixels for even vector formats. This value can be left blank to be automatically calculated to match original plot aspect ratio.
#' @param hi.res.res nominal resolution of high resolution plot in ppi. The height and width of vector plots will be calculated based in this. This value can be left blank to be automatically calculated to fit original plot scales.
#' @param graph.env save the environments in which plots were generated to distinct files (based on \code{graph.name}) with \code{env} extension?
#' @param graph.recordplot save the plot via \code{recordPlot} to distinct files (based on \code{graph.name}) with \code{recodplot} extension?
#' @param graph.RDS save the raw R object returned (usually with \code{lattice} or \code{ggplot2}) while generating the plots to distinct files (based on \code{graph.name}) with \code{RDS} extension?
#' @param log an optionally passed \emph{logger name} from \pkg{futile.logger} to record all info, trace, debug and error messages. Logging to the console can be done by specifying e.g. \code{flog.namespace()}, and log to a file by previously calling \code{flog.appender} and \code{appender.file} on the given \emph{logger name}.
#' @param ... optional parameters passed to graphics device (e.g. \code{bg}, \code{pointsize} etc.)
#' @return a list of parsed elements each containing: \code{src} (the command run), \code{result} (R object: \code{NULL} if nothing returned, path to image file if a plot was generated), \code{print}ed \code{output}, \code{type} (class of returned object if any), informative/wawrning and error messages (if any returned by the command run, otherwise set to \code{NULL}) and possible \code{stdout}t value. See Details above.
#' @seealso \code{\link{eval.msgs}} \code{\link{evalsOptions}}
#' @examples \dontrun{
#' # parsing several lines of R code
#' txt <- readLines(textConnection('x <- rnorm(100)
#'   runif(10)
#'   warning('Lorem ipsum foo-bar-foo!')
#'   plot(1:10)
#'   qplot(rating, data = movies, geom = 'histogram')
#'   y <- round(runif(100))
#'   cor.test(x, y)
#'   crl <- cor.test(runif(10), runif(10))
#'   table(mtcars$am, mtcars$cyl)
#'   ggplot(mtcars) + geom_point(aes(x = hp, y = mpg))'))
#' evals(txt)
#'
#' ## parsing a list of commands
#' txt <- list('df <- mtcars',
#'  c('plot(mtcars$hp, pch = 19)','text(mtcars$hp, label = rownames(mtcars), pos = 4)'),
#'  'ggplot(mtcars) + geom_point(aes(x = hp, y = mpg))')
#' evals(txt)
#'
#' ## the same commands in one string but also evaluating the `plot` with `text`
#' ## (note the leading '+' on the beginning of `text...` line)
#' txt <- 'df <- mtcars
#'  plot(mtcars$hp, pch = 19)
#'  +text(mtcars$hp, label = rownames(mtcars), pos = 4)
#'  ggplot(mtcars) + geom_point(aes(x = hp, y = mpg))'
#' evals(txt)
#' ## but it would fail without parsing
#' evals(txt, parse = FALSE)
#'
#' ## handling messages
#' evals('message(20)')
#' evals('message(20);message(20)', parse = FALSE)
#'
#' ## adding a caption to a plot
#' evals('set.caption("FOO"); plot(1:10)')
#' ## `plot` is started with a `+` to eval the codes in the same chunk
#' ## (no extra chunk with NULL result)
#' evals('set.caption("FOO"); +plot(1:10)')
#'
#' ## handling warnings
#' evals('chisq.test(mtcars$gear, mtcars$hp)')
#' evals(list(c('chisq.test(mtcars$gear, mtcars$am)', 'pi',
#'   'chisq.test(mtcars$gear, mtcars$hp)')), parse = FALSE)
#' evals(c('chisq.test(mtcars$gear, mtcars$am)',
#'   'pi',
#'   'chisq.test(mtcars$gear, mtcars$hp)'))
#'
#' ## handling errors
#' evals('runiff(20)')
#' evals('Old MacDonald had a farm\\dots')
#' evals('## Some comment')
#' evals(c('runiff(20)', 'Old MacDonald had a farm?'))
#' evals(list(c('runiff(20)', 'Old MacDonald had a farm?')), parse = FALSE)
#' evals(c('mean(1:10)', 'no.R.function()'))
#' evals(list(c('mean(1:10)', 'no.R.function()')), parse = FALSE)
#' evals(c('no.R.object', 'no.R.function()', 'very.mixed.up(stuff)'))
#' evals(list(c('no.R.object', 'no.R.function()', 'very.mixed.up(stuff)')), parse = FALSE)
#' evals(c('no.R.object', 'Old MacDonald had a farm\\dots', 'pi'))
#' evals('no.R.object;Old MacDonald had a farm\\dots;pi', parse = FALSE)
#' evals(list(c('no.R.object', 'Old MacDonald had a farm\\dots', 'pi')), parse = FALSE)
#'
#' ## graph options
#' evals('plot(1:10)')
#' evals('plot(1:10);plot(2:20)')
#' evals('plot(1:10)', graph.output = 'jpg')
#' evals('plot(1:10)', height = 800)
#' evals('plot(1:10)', height = 800, hi.res = TRUE)
#' evals('plot(1:10)', graph.output = 'pdf', hi.res = TRUE)
#' evals('plot(1:10)', res = 30)
#' evals('plot(1:10)', graph.name = 'myplot')
#' evals(list('plot(1:10)', 'plot(2:20)'), graph.name = 'myplots-%d')
#' evals('plot(1:10)', graph.env = TRUE)
#' evals('x <- runif(100);plot(x)', graph.env = TRUE)
#' evals(c('plot(1:10)', 'plot(2:20)'), graph.env = TRUE)
#' evals(c('x <- runif(100)', 'plot(x)','y <- runif(100)', 'plot(y)'), graph.env = TRUE)
#' evals(list(
#'     c('x <- runif(100)', 'plot(x)'),
#'     c('y <- runif(100)', 'plot(y)')),
#'   graph.env = TRUE, parse = FALSE)
#' evals('plot(1:10)', graph.recordplot = TRUE)
#' ## unprinted lattice plot
#' evals('histogram(mtcars$hp)', graph.recordplot = TRUE)
#'
#' ## caching
#' system.time(evals('plot(mtcars)'))
#' system.time(evals('plot(mtcars)'))                # running again to see the speed-up :)
#' system.time(evals('plot(mtcars)', cache = FALSE)) # cache disabled
#'
#' ## caching mechanism does check what's inside a variable:
#' x <- mtcars
#' evals('plot(x)')
#' x <- cbind(mtcars, mtcars)
#' evals('plot(x)')
#' x <- mtcars
#' system.time(evals('plot(x)'))
#'
#' ## stress your CPU - only once!
#' evals('x <- sapply(rep(mtcars$hp, 1e3), mean)')   # run it again!
#'
#' ## play with cache
#' require(lattice)
#' evals('histogram(rep(mtcars$hp, 1e5))')
#' ## nor run the below call
#' ## that would return the cached version of the above call :)
#' f <- histogram
#' g <- rep
#' A <- mtcars$hp
#' B <- 1e5
#' evals('f(g(A, B))')#'
#'
#' ## or switch off cache globally:
#' evalsOptions('cache', FALSE)
#' ## and switch on later
#' evalsOptions('cache', TRUE)
#'
#' ## evaluate assignments inside call to evals
#' ## changes to environments are cached properly and retreived
#' evalsOptions('cache.time', 0)
#' x <- 2
#' evals('x <- x^2')[[1]]$result
#' evals('x <- x^2; x + 1')[[2]]$result
#' evalsOptions('cache.time', 0.1)
#'
#' ## returning only a few classes
#' txt <- readLines(textConnection('rnorm(100)
#'   list(x = 10:1, y = 'Godzilla!')
#'   c(1,2,3)
#'    matrix(0,3,5)'))
#' evals(txt, classes = 'numeric')
#' evals(txt, classes = c('numeric', 'list'))
#'
#' ## hooks
#' txt <- 'runif(1:4); matrix(runif(25), 5, 5); 1:5'
#' hooks <- list('numeric' = round, 'matrix' = pander_return)
#' evals(txt, hooks = hooks)
#' ## using pander's default hook
#' evals(txt, hooks = list('default' = pander_return))
#' evals('22/7', hooks = list('numeric' = round))
#' evals('matrix(runif(25), 5, 5)', hooks = list('matrix' = round))
#'
#' ## setting default hook
#' evals(c('runif(10)', 'matrix(runif(9), 3, 3)'),
#'   hooks = list('default'=round))
#' ## round all values except for matrices
#' evals(c('runif(10)', 'matrix(runif(9), 3, 3)'),
#'   hooks = list(matrix = 'print', 'default' = round))
#'
#' # advanced hooks
#' hooks <- list('numeric' = list(round, 2), 'matrix' = list(round, 1))
#' evals(txt, hooks = hooks)
#'
#' # return only returned values
#' evals(txt, output = 'result')
#'
#' # return only messages (for checking syntax errors etc.)
#' evals(txt, output = 'msg')
#'
#' # check the length of returned values and do not return looong R objects
#' evals('runif(10)', length = 5)
#'
#' # note the following will not be filtered!
#' evals('matrix(1,1,1)', length = 1)
#'
#' # if you do not want to let such things be eval-ed in the middle of a string
#' # use it with other filters :)
#' evals('matrix(1,1,1)', length = 1, classes = 'numeric')
#'
#' # hooks & filtering
#' evals('matrix(5,5,5)',
#'   hooks = list('matrix' = pander_return),
#'   output = 'result')
#'
#' # eval-ing chunks in given environment
#' myenv <- new.env()
#' evals('x <- c(0,10)', env = myenv)
#' evals('mean(x)', env = myenv)
#' rm(myenv)
#' # note: if you had not specified 'myenv', the second 'evals' would have failed
#' evals('x <- c(0,10)')
#' evals('mean(x)')
#'
#' # log
#' x <- evals('1:10', log = 'foo')
#' # trace log
#' evalsOptions('cache.time', 0)
#' x <- evals('1:10', log = 'foo')
#' x <- evals('1:10', log = 'foo')
#' # log to file
#' t <- tempfile()
#' flog.appender(appender.file(t), name = 'evals')
#' x <- evals('1:10', log = 'evals')
#' readLines(t)
#' # permanent log for all events
#' evalsOptions('log', 'evals')
#' flog.threshold(TRACE, 'evals')
#' evals('foo')
#' }
#' @export
#' @importFrom digest digest
#' @importFrom grDevices dev.list dev.off dev.control dev.list recordPlot
#' @importFrom utils packageVersion object.size
evals <- function(txt, parse = evalsOptions('parse'), cache = evalsOptions('cache'), cache.mode = evalsOptions('cache.mode'), cache.dir = evalsOptions('cache.dir'), cache.time = evalsOptions('cache.time'), cache.copy.images = evalsOptions('cache.copy.images'), showInvisible = FALSE, classes = evalsOptions('classes'), hooks = evalsOptions('hooks'), length = evalsOptions('length'), output = evalsOptions('output'), env = NULL, graph.unify = evalsOptions('graph.unify'), graph.name = evalsOptions('graph.name'), graph.dir = evalsOptions('graph.dir'), graph.output = evalsOptions('graph.output'), width = evalsOptions('width'), height = evalsOptions('height'), res = evalsOptions('res'), hi.res = evalsOptions('hi.res'), hi.res.width = evalsOptions('hi.res.width'), hi.res.height = 960 * (height / width), hi.res.res = res * (hi.res.width / width), graph.env = evalsOptions('graph.env'), graph.recordplot = evalsOptions('graph.recordplot'), graph.RDS = evalsOptions('graph.RDS'), log = evalsOptions('log'), ...) { #nolint

    if (missing(txt)) {
        stop('No R code provided to evaluate!')
    }
    txt.original <- paste(txt, collapse = '\n')

    ## logging constant
    logging <- !is.null(log) && requireNamespace('futile.logger', quietly = TRUE)

    ## parse provided code after concatenating
    if (parse) {

        txt.parsed <- tryCatch(parse(text = txt), error = function(e) e)

        ## skip parsing on syntax error and disable cache
        if (inherits(txt.parsed, 'error')) {
            cache <- FALSE
        } else {

            txt <- sapply(txt.parsed, function(x) paste(deparse(x), collapse = '\n'))

            ## return NULL on missing R code (e.g. comments)
            if (length(txt) == 0) {
                return(list(structure(list(src      = txt.original,
                                           result   = NULL,
                                           output   = NULL,
                                           type     = NULL,
                                           msg      = list(
                                               messages = NULL,
                                               warnings = NULL,
                                               errors   = NULL),
                                           stdout   = NULL
                                           ), 'class' = 'evals')))
            }

            ## (re)merge lines on demand (based on `+` at the beginning of line)
            txt.sep <- c(which(!grepl('^\\+', txt)), length(txt) + 1)
            txt <-  lapply(1:(length(txt.sep) - 1), function(i) txt[txt.sep[i] : (txt.sep[i + 1] - 1)])
            txt <- rapply(txt, function(x) sub('^\\+', '', x), how = 'replace')

        }
    } else {
        cache <- FALSE
    }

    ## check provided dirs
    if (!identical(file.info(graph.dir)$isdir, TRUE) &&
        !dir.create(graph.dir, showWarnings = FALSE, recursive = TRUE)) {
        stop(sprintf('Something is definitely wrong with `graph.dir`: %s!', graph.dir))
    }
    if (cache.mode == 'disk' &&
        !identical(file.info(cache.dir)$isdir, TRUE) &&
        !dir.create(cache.dir, showWarnings = FALSE, recursive = TRUE)) {
                stop(sprintf('Something is definitely wrong with `cache.dir`: %s!', cache.dir))
    }

    ## check provided parameters
    if ('all' %in% output) {
        output <- c('src', 'result', 'output', 'type', 'msg', 'stdout')
    } else if (length(setdiff(output, c('src', 'result', 'output', 'type', 'msg', 'stdout'))) != 0) {
        stop('Wrong parameter supplied to output')
    }

    if (!is.null(hooks) && !is.list(hooks)) {
        stop('Wrong list of hooks provided!')
    }

    if (!is.character(graph.name)) {
        stop('Wrong graph.name (!character) specified!')
    }

    if (!is.na(graph.output) && graph.output == 'jpg') {
        graph.output <- 'jpeg'
    }

    ## env for running all lines of code
    if (is.null(env) || !is.environment(env)) {
        env <- new.env()
    }
    for (p in c('plot', 'barplot', 'lines', 'pie', 'boxplot', 'polygon',
                'points', 'legend', 'hist', 'pairs', 'stripchart')) {
        if (exists(p, envir = env, inherits = FALSE)) {
            stop(paste0('Using a reserved word as variable: `', p, '`'))
        }
    }

    `%d` <- 0

    ## main loop
    lapply(txt, function(src) {

        ## log R expression
        if (logging) {
            futile.logger::flog.info(paste('Command run:', gsub('[ ]+', ' ', gsub('\n', ' ', src))), name = log)
        }

        if (!is.na(graph.output)) {

            ## get image file name
            `%d` <<- `%d` + 1
            file.name <- gsub('%d', `%d`, graph.name, fixed = TRUE)

            ## chunk ID
            if (length(debug$chunkID) > 0) {
                if ((length(debug$nestedID) > 0) && (debug$nestedID > 1)) {
                    file.name <- gsub('%i', paste0(debug$nestedID, '_', debug$chunkID), file.name, fixed = TRUE)
                } else {
                    file.name <- gsub('%i', debug$chunkID, file.name, fixed = TRUE)
                }
            }
            if (length(debug$cmdID) > 0) {
                assign('cmdID', debug$cmdID + 1, envir = debug)
                file.name <- gsub('%I', debug$cmdID, file.name, fixed = TRUE)
            }
            file <- sprintf('%s.%s', file.name, graph.output)

            ## tempfile
            if (grepl('%t', graph.name)) {
                if (length(strsplit(sprintf('placeholder%splaceholder', file.name), '%t')[[1]]) > 2) {
                    stop('File name contains more then 1 "%t"!')
                }
                rep <- strsplit(file, '%t')[[1]]
                file <- tempfile(pattern = rep[1], tmpdir = graph.dir, fileext = rep[2])
                file.name <- sub(sprintf('.%s$', graph.output), '', file)
            } else {
                file <- file.path(graph.dir, file)
                file.name <- file.path(graph.dir, file.name)
            }
            file <- gsub('\\', '/', file, fixed = TRUE)
            file.name <- gsub('\\', '/', file.name, fixed = TRUE)

            ## similar files counter
            if (grepl('%n', file.name)) {
                if (length(strsplit(sprintf('placeholder%splaceholder', file.name), '%n')[[1]]) > 2) {
                    stop('File name contains more then 1 "%n"!')
                }
                similar.files <- list.files(graph.dir, pattern = sprintf('^%s\\.(jpeg|tiff|png|svg|bmp|pdf)$', gsub('%t', '[a-z0-9]*', gsub('%d|%n|%i', '[[:digit:]]*', basename(file.name))))) #nolint
                if (length(similar.files) > 0) {
                    similar.files <- sub('\\.(jpeg|tiff|png|svg|bmp|pdf)$', '', similar.files)
                    rep <- gsub('%t', '[a-z0-9]*',
                                gsub('%d|%i', '[[:digit:]]*',
                                     strsplit(basename(file.name), '%n')[[1]]))
                    `%n` <- max(as.numeric(gsub(paste(rep, collapse = '|'), '', similar.files))) + 1
                } else {
                    `%n` <- 1
                }
                file.name <- gsub('%n', `%n`, file.name, fixed = TRUE)
                file <- gsub('%n', `%n`, file, fixed = TRUE)
            }

        }

        ## checking cache
        if (cache) {

            ## helper functions
            hash_of_eval_or_deparse <- function(x, x.deparse) {
                ## compute the hash of the given 'name' by evaluating that or by deparsing if the prior would fail
                v <- tryCatch(eval(x, envir = env), error = function(e) e)
                if (inherits(v, 'error')) {
                    return(digest(x.deparse))
                }
                hash_from_cache(v, x.deparse)
            }

            hash_from_cache <- function(x, x.deparse) {
                ## get the hash of the object from local cache if possible, compute it and save to cache otherwise
                if (exists(x.deparse, envir = hash.cache.obj, inherits = FALSE)) {
                    if (identical(x, get(x.deparse, envir = hash.cache.obj))) {
                        assign(x.deparse, as.integer(Sys.time()), envir = hash.cache.last.used)
                        return(get(x.deparse, envir = hash.cache.hash))
                    }
                }
                x.hash <- digest(x, 'sha1')
                assign(x.deparse, x, envir = hash.cache.obj)
                assign(x.deparse, x.hash, envir = hash.cache.hash)
                assign(x.deparse, as.integer(Sys.time()), envir = hash.cache.last.used)
                return(x.hash)
            }

            get_call_parts <- function(call) {
                ## extracting each function's and variable's hash from the call
                lapply(call, function(x){
                           switch(mode(x),
                                  'name' = hash_of_eval_or_deparse(x, deparse(x)),
                                  'call' = get_call_parts(x),
                                  digest(deparse(x), 'sha1')
                                  )
                       })
            }

            ## get the hash of the call based on the hash of all `names`
            cached <- digest(list(call = get_call_parts(parse(text = src)),
                                  storage = digest(list(storage, panderOptions(), evalsOptions()), 'sha1')),
                             'sha1')

            if (cache.mode == 'disk') {

                cached <- file.path(cache.dir, cached)
                cached.env <- paste0(cached, '.ENV')

                ## load cached result
                if (file.exists(cached)) {
                    cached.result <- readRDS(cached)
                }

                ## load the modified R objects of the cached code
                if (file.exists(cached.env)) {
                    load(file = cached.env, envir = env)
                }

            } else {
                ## cache is in environment
                ## load cached result
                if (exists(cached, envir = cached.results, inherits = FALSE)) {
                    cached.result <- get(cached, envir = cached.results)
                }

                ## if the cached expression changed the environment (for example assignment),
                ## retrieve the respected changes too (see examples)
                if (exists(cached, envir = cached.environments, inherits = FALSE)) {
                    cached.objs <- get(cached, envir = cached.environments)
                    sapply(names(cached.objs), function(x) assign(x, cached.objs[[x]], envir = env))
                }

            }

            if (exists('cached.result')) {

                if (inherits(cached.result$result, 'image')) {

                    if (cache.copy.images && cache.mode == 'disk' && !is.na(graph.output)) {

                        ## we are copying img file + possibly extra from cache dir
                        file.copy(paste0(cached, '.', graph.output), file)
                        if (graph.recordplot) {
                            file.copy(paste0(cached, '.recordedplot'), sprintf('%s.recordplot', file.name))
                        }
                        if (graph.RDS) {
                            file.copy(paste0(cached, '.RDS'), sprintf('%s.RDS', file.name))
                        }
                        if (graph.env) {
                            file.copy(paste0(cached, '.env'), sprintf('%s.env', file.name))
                        }
                        if (hi.res) {
                            file.copy(paste0(cached, '-hires.', graph.output),
                                      sprintf('%s-hires.%s', file.name, graph.output))
                        }

                        cached.result$result <- file
                        class(cached.result$result) <- 'image'
                        if (logging) {
                            futile.logger::flog.trace(paste('Image copied from cache:', file), name = log)
                        }
                        return(cached.result)

                    } else {

                        ## we are checking in plots' dir if the img file exists
                        cached.image.file <- as.character(cached.result$result)
                        if (file.exists(cached.image.file)) {
                            if (logging) {
                                futile.logger::flog.trace(paste('Image found in cache:', cached.image.file), name = log)
                            }
                            return(cached.result)
                        } else {
                            warning(sprintf('The image file referenced in cache (%s) is no longer available: the image is recreated (%s).', shQuote(cached.image.file), shQuote(file)), call. = FALSE) #nolint
                        }

                    }

                } else {
                    if (logging) {
                        futile.logger::flog.trace('Returning cached R object.', name = log)
                    }
                    return(cached.result)
                }

            } # cached result not found

            ## starting timer
            timer <- proc.time()

        }

        ## clear graphics device (if there would be any open)
        clear.devs <- function() {
            if (!is.na(graph.output)) {
                sapply(dev.list(), dev.off)
            }
        }
        clear.devs()

        if (!is.na(graph.output)) {

            ## init (potential) img file
            pbg <- panderOptions('graph.background')
            if (graph.output %in% c('bmp', 'jpeg', 'png', 'tiff')) {
                if (capabilities('cairo')) {
                    do.call(graph.output, list(file,
                                               width = width,
                                               height = height,
                                               res = res,
                                               bg = pbg,
                                               type = 'cairo', ...))
                } else {
                    do.call(graph.output, list(file,
                                               width = width,
                                               height = height,
                                               res = res,
                                               bg = pbg, ...))
                }
            }
            if (graph.output == 'svg') {
                do.call(graph.output, list(file,
                                           width = width / res,
                                           height = height / res,
                                           bg = pbg, ...)) # TODO: font-family?
            }
            if (graph.output == 'pdf') {
                do.call('cairo_pdf', list(file,
                                          width = width / res,
                                          height = height / res,
                                          bg = pbg, ...)) # TODO: font-family?
            }

            ## start recordPlot
            dev.control(displaylist = 'enable')
        }

        ## if caching: save the initial environment's objects' hashes
        if (cache) {
            objs <- ls(envir = env)
            objs <- setdiff(objs, c('.storage', 'showCode', 'showText'))
            objs.hash <- sapply(objs, function(x) hash_of_eval_or_deparse(as.name(x), x))
        }

        ## add modified base plot functions to update colors, `par` settings and adding a grid
        if (graph.unify) {
            sapply(ls(envir = masked.plots), function(x) {
                assign(x, get(x, envir = masked.plots, inherits = FALSE), envir = env)
            })
        }

        ## env for optional high resolution images
        if (hi.res && !is.na(graph.output)) {
            env.hires <- env
        }

        ## eval
        res <- eval.msgs(src, env = env, showInvisible = showInvisible, graph.unify = graph.unify)

        ## grab recorded.plot
        if (!is.na(graph.output) && !is.null(dev.list())) {
            recorded.plot <- recordPlot()
            dev.control('inhibit')
        }

        ## did we produce a plot?
        if (is.na(graph.output)) {
            graph <- FALSE
        } else {
            graph <- ifelse(exists('recorded.plot'), ifelse(is.null(recorded.plot[[1]]), FALSE, file), FALSE)
        }

        ## close grDevice
        clear.devs()

        ## check if image file was created
        if (is.character(graph)) {
            if (!file.exists(file)) {
                res$msg$errors <- c(res$msg$errors, paste('Image file not written by:', paste(src, collapse = ';')))
            }
        }

        ## remove dummy img file (1px) on Windows if created
        if (grepl('w|W', .Platform$OS.type)) {
            if (!is.character(graph)) {
                if (file.exists(file)) {
                    unlink(file)
                }
            }
        }

        ## error handling
        if (!is.null(res$msg$errors)) {

            ## removing injected base::plot fns
            if (graph.unify) {
                rm(list = ls(envir = masked.plots), envir = env)
            }

            class(res) <- 'evals'
            if ('plot.new has not been called yet' %in% res$msg$errors) {
                res$msg$errors <- 'plot.new has not been called yet - Please note that all R commands are parsed and evaluated separately. To override this default behavior, add a plus sign (+) as the first character of the line(s) to evaluate with the prior one(s).' #nolint
            }
            if (logging) {
                futile.logger::flog.error(res$msg$errors, name = log)
            }
            return(res)
        }

        result <- res$result

        ## we have a graph
        if (is.character(graph)) {

            ## log image file name
            if (logging) {
                futile.logger::flog.trace(paste('Image file written:', file), name = log)
            }

            ## save recorded plot on demand
            if (graph.recordplot) {
                saveRDS(recorded.plot, file = sprintf('%s.recordplot', file.name))
            }

            ## save plot RDS on demand
            if (graph.RDS) {
                if (!is.null(result)) {
                    saveRDS(result, file = sprintf('%s.RDS', file.name))
                }
            }

            result <- graph
            class(result) <- 'image'

            ## saving environment on demand
            if (graph.env) {
                save(list = ls(envir = env), file = sprintf('%s.env', file.name), envir = env)
            }

            ## generate high resolution images on demand
            if (hi.res) {

                file.hi.res <- sprintf('%s-hires.%s', file.name, graph.output)

                ## initialize high resolution image file
                if (graph.output %in% c('bmp', 'jpeg', 'png', 'tiff')) {

                    if (capabilities('cairo')) {
                        do.call(graph.output, list(file.hi.res,
                                                   width = hi.res.width,
                                                   height = hi.res.height,
                                                   res = hi.res.res,
                                                   bg = pbg,
                                                   type = 'cairo', ...))
                    } else {
                        do.call(graph.output, list(file.hi.res,
                                                   width = hi.res.width,
                                                   height = hi.res.height,
                                                   res = hi.res.res,
                                                   bg = pbg, ...))
                    }

                } else {

                    if (.Platform$OS.type == 'unix') {
                        ## a symlink would be fine for vector formats on a unix-like OS
                        file.symlink(file, file.hi.res)
                    } else {
                        ## we have no option to do so on Windows (to be backward compatible)
                        do.call(graph.output, list(file.hi.res,
                                                   width = hi.res.width / hi.res.res,
                                                   height = hi.res.height / hi.res.res,
                                                   bg = pbg, ...)) # TODO: font-family?
                    }
                }

                ## render high resolution image (if needed)
                if ((graph.output %in% c('bmp', 'jpeg', 'png', 'tiff')) | (.Platform$OS.type != 'unix')) {
                    ## we need eval.msgs() here instead of simple eval()
                    ## to prevent unprinted lattice/ggplot2 objects' issues
                    eval.msgs(src, env = env.hires)
                    clear.devs()
                }

                ## add 'href' attribute to returned R object
                attr(result, 'href') <- file.hi.res
            }
        }

        ## check length
        if (length(result) > length) {
            result <- NULL
        }

        ## check classes
        if (!is.null(classes)) {
            if (!inherits(result, classes)) {
                result <- output <- NULL
            }
        }

        ## caption
        if (!is.null(storage$caption) & !is.null(result)) {
            attr(result, 'caption') <- get.caption()
        }

        ## alignment of tables
        if (!is.null(storage$alignment) & !is.null(result)) {
            attr(result, 'alignment') <- get.alignment(result)
        }

        ## highlight cells
        if (!is.null(result)) {
            result <- get.emphasize(result)
        }

        ## run hooks if specified
        if (!is.null(hooks)) {
            hook.name <- ifelse(inherits(result, names(hooks)), class(result), 'default')
            fn <- hooks[[hook.name]];
            params <- list(result)
            if (!is.null(fn)) {
                if (is.list(fn)) {
                    params <- list(result, fn[[-1]])
                    fn <- fn[[1]]
                }
                if (logging) {
                    futile.logger::flog.trace(paste('Calling hook for', hook.name), name = log)
                }
                result <- do.call(fn, params)
            }
        }

        ## return list at last
        res <- list(src      = src,
                    result   = result,
                    output   = res$output,
                    type     = class(result),
                    msg      = list(
                        messages = res$msg$messages,
                        warnings = res$msg$warnings,
                        errors   = res$msg$errors),
                    stdout   = res$msg$stdout
                    )

        res <- res[output]
        class(res) <- 'evals'

        ## removing injected base::plot fns
        if (graph.unify) {
            rm(list = ls(envir = masked.plots), envir = env)
        }

        ## save to cache
        if (cache) {

            ## comparing the resulting environment's objects' hashes with the original ones
            objs.res      <- ls(envir = env)
            objs.res      <- setdiff(objs.res, c('.storage', 'showCode', 'showText', '.graph.dir', '.graph.name'))
            objs.res.hash <- sapply(objs.res, function(x) hash_of_eval_or_deparse(as.name(x), x))
            change        <- setdiff(objs.res, objs)
            common        <- intersect(objs.res, objs)
            changed       <- c(change, unlist(sapply(common, function(x) {
                                                                if (objs.hash[[x]] != objs.res.hash[[x]])
                                                                    return(x)
                                                            },
                                                     USE.NAMES = FALSE)))
            if (cache.mode == 'disk') {
                if (as.numeric(proc.time() - timer)[3] > cache.time) {

                    ## save cached result
                    saveRDS(res, file = cached)

                    ## save the modified R objects of the cached code
                    if (length(changed) > 0) {
                        save(list = changed, envir = env, file = paste0(cached, '.ENV'))
                    }

                    ## save plot related files
                    if ('image' %in% class(result)) {
                        file.copy(file, paste0(cached, '.', graph.output))
                        if (graph.recordplot) {
                            file.copy(sprintf('%s.recordplot', file.name), paste0(cached, '.recordedplot'))
                        }
                        if (graph.RDS) {
                            if (!is.null(result)) {
                                file.copy(sprintf('%s.RDS', file.name), paste0(cached, '.RDS'))
                            }
                        }
                        if (graph.env) {
                            file.copy(sprintf('%s.env', file.name), paste0(cached, '.env'))
                        }
                        if (hi.res) {
                            file.copy(sprintf('%s-hires.%s', file.name, graph.output),
                                      paste0(cached, '-hires.', graph.output))
                        }
                    }
                    if (logging) {
                        futile.logger::flog.trace('Cached result', name = log)
                    }
                }

            } else {

                if (as.numeric(proc.time() - timer)[3] > cache.time) {

                    ## save cached result
                    assign(cached, res, envir = cached.results)

                    ## save changes to environment so when expression result is retreived from cache
                    ## changes to environment will be retreived also (see example)
                    if (length(changed) > 0) {
                        assign(cached, mget(changed, envir = env), envir = cached.environments)
                    }

                    if (logging) {
                        futile.logger::flog.trace('Cached result', name = log)
                    }
                }
            }
        }

        ## log
        if (logging) {
            if (!is.null(res$msg$warnings)) {
                futile.logger::flog.warn(res$msg$warnings, name = log)
            }
            if (!is.null(res$result) && res$type != 'image') {
                futile.logger::flog.debug(paste0(
                    'Returned object: class = ',
                    res$type,
                    ', length = ',
                    length(res$result),
                    ', dim = ',
                    paste(dim(res$result), collapse = '/'),
                    ', size = ',
                    object.size(res$result),
                    ' bytes'
                ), name = log)
            }
        }

        ## return
        res

    })
}

#' Redraws plot saved in file
#'
#' This function is a wrapper around \code{redrawPlot}.
#' @param file path and name of an rds file containing a plot object to be redrawn
#' @references Thanks to Jeroen Ooms \url{https://stat.ethz.ch/pipermail/r-devel/2012-January/062973.html}, JJ Allaire \url{https://github.com/rstudio/rstudio/commit/eb5f6f1db4717132c2ff111f068ffa6e8b2a5f0b}, and Gabriel Becker.
#' @seealso \code{\link{evals}}
#' @export
redraw.recordedplot <- function(file) {

    plot <- tryCatch(readRDS(file), error = function(e) e)

    if (inherits(plot, 'error')) {
        stop(paste('Cannot read file:', plot$message))
    }

    redrawPlot(plot)
}


#' Redraw a recordedplot, grid, trellis, or ggplot2 plot.
#'
#' This function redraws the plot represented by \code{rec_plot}. It can redraw grid/trellis/ggplot2/etc plots, as well as \code{recordedplot} objects. For \code{recordedplot} objects it acts as a wrapper around \code{replayPlot} with memory tweaks to fix native symbol address errors when the recordedplot was loaded from an rda/rds file.
#' @param rec_plot the plot object to redraw
#' @references Thanks to Jeroen Ooms \url{https://stat.ethz.ch/pipermail/r-devel/2012-January/062973.html}, JJ Allaire \url{https://github.com/rstudio/rstudio/commit/eb5f6f1db4717132c2ff111f068ffa6e8b2a5f0b}, and Gabriel Becker.
#' @seealso \code{\link{redraw.recordedplot}}
#' @export
#' @importFrom methods is
#' @importFrom grDevices replayPlot
redrawPlot <- function(rec_plot) {
    ## this allows us to deal with trellis/grid/ggplot objects as well ...
    if (!is(rec_plot, 'recordedplot')) {
        res <- try(print(rec_plot))
        if (is(res, 'error')) {
            stop(res)
        }
    } else {
        if (getRversion() < '3.0.0') {
            for (i in 1:length(rec_plot[[1]])) {
                #@jeroenooms
                if ('NativeSymbolInfo' %in% class(rec_plot[[1]][[i]][[2]][[1]])) {
                    rec_plot[[1]][[i]][[2]][[1]] <- getNativeSymbolInfo(rec_plot[[1]][[i]][[2]][[1]]$name)
                }
            }
        } else {
            for (i in 1:length(rec_plot[[1]])) {
                #@jjallaire
                symbol <- rec_plot[[1]][[i]][[2]][[1]]
                if ('NativeSymbolInfo' %in% class(symbol)) {
                    if (!is.null(symbol$package)) {
                        name <- symbol$package[['name']]
                    } else {
                        name <- symbol$dll[['name']]
                    }
                    pkg_dll <- getLoadedDLLs()[[name]]
                    native_sumbol <- getNativeSymbolInfo(name = symbol$name,
                                                        PACKAGE = pkg_dll, withRegistrationInfo = TRUE)
                    rec_plot[[1]][[i]][[2]][[1]] <- native_sumbol
                }
            }
        }
        if (is.null(attr(rec_plot, 'pid')) || attr(rec_plot, 'pid') != Sys.getpid()) {
            warning('Loading plot snapshot from a different session with possible side effects or errors.')
            attr(rec_plot, 'pid') <- Sys.getpid()
        }
        suppressWarnings(replayPlot(rec_plot))
    }
}

library(testthat)
library(pander)

cache.dir <- evalsOptions('cache.dir')
graph.dir <- evalsOptions('graph.dir')
wd <- getwd()
setwd(tempdir())
evalsOptions('cache.dir',  file.path(tempdir(), '.cache'))
evalsOptions('graph.dir',  file.path(tempdir(), 'plots'))

context('eval.msgs')

test_that('returns', {
    expect_equal(eval.msgs('1:5')$result, 1:5)
    expect_equal(eval.msgs('mtcars')$result, mtcars)
    expect_null(eval.msgs('x <- mtcars')$result)
    expect_null(evals('plot(mtcars)')$result)
    expect_equal(eval.msgs('cat(1:5);1:5')$result, 1:5)
    expect_null(eval.msgs('#comment')$result)
    expect_null(eval.msgs('')$result)
})

test_that('messages', {
    expect_null(eval.msgs('1:5')$msg$messages)
    expect_null(eval.msgs('mtcars')$msg$messages)
    expect_null(eval.msgs('x <- mtcars')$msg$messages)
    expect_null(evals('plot(mtcars)')$msg$messages)
    expect_null(eval.msgs('warning("d");warning("f");1')$msg$messages)
    expect_equal(eval.msgs('message("FOO")')$msg$messages, 'FOO')
    expect_equal(eval.msgs(c('message("FOO")', '1:2'))$msg$messages, 'FOO')
    expect_equal(eval.msgs(c('message("FOO");message("FOO");warning("FOO")', '1:2'))$msg$messages, c('FOO', 'FOO'))
})

test_that('warnings', {
    expect_null(eval.msgs('1:5')$msg$warnings)
    expect_null(eval.msgs('mtcars')$msg$warnings)
    expect_null(eval.msgs('x <- mtcars')$msg$warnings)
    expect_null(evals('plot(mtcars)')$msg$warnings)
    expect_null(eval.msgs('message("FOO")')$msg$warnings)
    expect_null(eval.msgs(c('message("FOO")', '1:2'))$msg$warnings)
    expect_equal(eval.msgs('warning("d");warning("f");1')$msg$warnings, c('d', 'f'))
    expect_equal(eval.msgs(c('warning("FOO")', '1:2'))$msg$warnings, 'FOO')
    expect_equal(eval.msgs(c('warning("FOO");message("FOO");warning("FOO")', '1:2'))$msg$warnings, c('FOO', 'FOO'))
})

test_that('errors', {
    expect_null(eval.msgs('1:5')$msg$errors)
    expect_null(eval.msgs('mtcars')$msg$errors)
    expect_null(eval.msgs('x <- mtcars')$msg$errors)
    expect_null(evals('plot(mtcars)')$msg$errors)
    expect_null(eval.msgs('warning("d");warning("f");1')$msg$errors)
    expect_null(eval.msgs('message("FOO")')$msg$errors)
    expect_null(eval.msgs(c('message("FOO")', '1:2'))$msg$errors)
    expect_null(eval.msgs(c('warning("FOO")', '1:2'))$msg$errors)
    expect_null(eval.msgs(c('warning("FOO");message("FOO");warning("FOO")', '1:2'))$msg$errors)
    expect_is(eval.msgs('runiff(2)')$msg$errors, 'character')
    expect_is(eval.msgs('runif would be nice to run')$msg$errors, 'character')
    expect_is(eval.msgs('no.R.object.like.that')$msg$errors, 'character')
    expect_error(evals())
    expect_error(evals('1:5', hooks = 1:10))
    expect_error(evals('plot(mtcars)', graph.name = 1))
    env <- new.env()
    env$plot <- 1
    expect_error(evals(1:10, env = env))
})

test_that('output', {
    expect_is(eval.msgs('1:5')$output, 'character')
    expect_is(eval.msgs('mtcars')$output, 'character')
    expect_is(eval.msgs('cat(1:5);1:5')$output, 'character')
    expect_null(eval.msgs('x <- mtcars')$output)
    expect_null(evals('plot(mtcars)')$output)
    expect_null(eval.msgs('cat(1:5)')$output)
})

test_that('stdout', {
    expect_null(eval.msgs('1:5')$stdout)
    expect_null(eval.msgs('mtcars')$stdout)
    expect_null(eval.msgs('x <- mtcars')$stdout)
    expect_null(evals('plot(mtcars)')$stdout)
    expect_equal(eval.msgs('cat(1:5)')$stdout, '1 2 3 4 5')
    expect_equal(eval.msgs('cat(1:5);1:5')$stdout, '1 2 3 4 5')
})

context('evals')

test_that('Variable assignement', {
    expect_null(evals('x <- rnorm(100)')[[1]]$result)
})

test_that('Multiple variable assignement', {
    expect_null(evals(list(c('x <- rnorm(100)', 'y <- rnorm(10)')), parse = F)[[1]]$result)
})

test_that('Comment', {
    expect_null(evals('## comment')[[1]]$result)
})

test_that('Comment & variable assignement', {
    expect_null(evals(list(c('## comment', 'y <- rnorm(10)')), parse = F)[[1]]$result)
})

test_that('Basic: numerics', {
    expect_is(evals('rnorm(100)')[[1]]$result, 'numeric')
})

test_that('Basic: character', {
    expect_equal(evals('"Hello world!"')[[1]]$result, 'Hello world!')
})

test_that('Basic: list', {
    expect_is(evals('lapply(rnorm(10), round)')[[1]]$result, 'list')
})

test_that('Basic: data.frame', {
    expect_is(evals('mtcars')[[1]]$result, 'data.frame')
})

test_that('Plot: graphics package', {
    expect_is(evals('plot(1:10)')[[1]]$result, 'image')
})

test_that('Plot: lattice package', {
    expect_is(evals('require(lattice);+histogram(mtcars$hp)')[[1]]$result, 'image')
    expect_is(evals('require(lattice);histogram(mtcars$hp)', parse = F)[[1]]$result, 'image')
})

test_that('Plot: multiline code', {
    expect_is(evals(list(c('plot(cars, main="Stopping Distance versus Speed")',
                           'plot(cars, main="Stopping Distance versus Speed")')),
                    parse = F)[[1]]$result,
              'image')
    expect_is(evals('plot(cars, main="Stopping Distance versus Speed"); plot(cars, main="Stopping Distance versus Speed")')[[2]]$result, 'image') #nolint
})

test_that('Plot: pdf', {
    expect_is(evals('plot(1:10)', graph.output = 'pdf')[[1]]$result, 'image')
})

test_that('Plot: pdf without plotting', {
    expect_is(evals('runif(10)', graph.output = 'pdf')[[1]]$result, 'numeric')
})

test_that('NULL', {
    expect_is(evals('rnorm(100); NULL')[[1]]$result, 'numeric')
    expect_null(evals('rnorm(100); NULL')[[2]]$result)
    expect_null(evals('rnorm(100); NULL', parse = FALSE)[[1]]$result)
})

test_that('## comment', {
    expect_is(evals('rnorm(100);## comment')[[1]]$result, 'numeric')
    expect_is(evals('rnorm(100);## comment', parse = FALSE)[[1]]$result, 'numeric')
})

test_that('var assignement', {
    expect_is(evals('rnorm(100);x <- 1')[[1]]$result, 'numeric')
    expect_null(evals('rnorm(100);x <- 1', parse = F)[[1]]$result)
})

test_that('creating function', {
    expect_is(evals('rnorm(100);f <- function(x) mean(x)')[[1]]$result, 'numeric')
    expect_null(evals('rnorm(100);f <- function(x) mean(x)')[[2]]$result)
    expect_null(evals('rnorm(100);f <- function(x) mean(x)', parse = F)[[1]]$result)
})


test_that('string+num', {
    expect_is(evals('"X";rnorm(100)')[[2]]$result, 'numeric')
    expect_is(evals('"X";rnorm(100)')[[1]]$result, 'character')
    expect_is(evals('"X";rnorm(100)', parse = FALSE)[[1]]$result, 'numeric')
})

test_that('data.frame+num', {
    expect_is(evals(list(c('mtcars', 'rnorm(100)')), parse = FALSE)[[1]]$result, 'numeric')
})

test_that('simple error', {
    expect_is(evals('dadaffaSFA+A')[[1]]$msg$error, 'character')
})

test_that('simple error', {
    expect_is(evals('runiff(10)')[[1]]$msg$error, 'character')
})

test_that('multiple errors', {
    expect_is(evals(list(c('runiff(20)', 'Old MacDonald had a farm?')), parse = F)[[1]]$msg$error, 'character')
})

test_that('multiple errors', {
    expect_is(evals('no.R.object;no.R.function();very.mixed.up(stuff)')[[1]]$msg$error, 'character')
    expect_is(evals('no.R.object;no.R.function();very.mixed.up(stuff)')[[2]]$msg$error, 'character')
    expect_is(evals('no.R.object;no.R.function();very.mixed.up(stuff)')[[3]]$msg$error, 'character')
    expect_is(evals('no.R.object;no.R.function();very.mixed.up(stuff)', parse = F)[[1]]$msg$error, 'character')
})

test_that('output+errors', {
    expect_is(evals(list(c('mean(1:10)', 'no.R.function()')), parse = F)[[1]]$msg$error, 'character')
})

test_that('output+multiple errors', {
    expect_is(evals(list(c('no.R.object', 'Old MacDonald had a farm\\dots', 'pi')),
                    parse = F)[[1]]$msg$error,
              'character')
})

test_that('simple warning', {
    expect_is(evals('chisq.test(mtcars$gear, mtcars$hp)')[[1]]$msg$warning, 'character')
})

test_that('multiple warnings+output', {
    expect_is(evals(list(c('chisq.test(mtcars$gear, mtcars$am)', 'pi', 'chisq.test(mtcars$gear, mtcars$hp)')),
                    parse = F)[[1]]$msg$warnings,
              'character')
    expect_is(evals('chisq.test(mtcars$gear, mtcars$am);pi;chisq.test(mtcars$gear, mtcars$hp)')[[1]]$msg$warnings,
              'character')
})

test_that('long lines with line breaks', {
    expect_is(evals('if (TRUE) {\nset.caption("FOO")\nplot(c(1,2,3,4,5,6,7,8,9,10,11,12))\n}')[[1]]$result, 'image')
    expect_equal(length(evals('x <- "this is a very-very looooooooooong line which might be split on `parse`, but that is not a big deal at all"\nx\n1')), 3) #nolint
})


test_that('cache.dir option works correctly', {
    dir <- tempdir()
    suppressWarnings(file.remove(list.files(dir)))
    t1 <- system.time(evals('1:1e5', cache.mode = 'disk', cache.dir = dir, cache.time = 0))
    lf <- list.files(dir)
    expect_true(length(lf) > 0)
    t2 <- system.time(evals('1:1e5', cache.mode = 'disk', cache.dir = dir, cache.time = 0))
    expect_equal(list.files(dir), lf)
    # plots
    suppressWarnings(file.remove(list.files(dir)))
    t1 <- system.time(evals('plot(mtcars)',
            cache.mode = 'disk',
            cache.dir = dir,
            cache.time = 0,
            cache.copy.images = T))
    lf <- list.files(dir)
    expect_true(length(lf) > 0)
    t2 <- system.time(evals('plot(mtcars)',
            cache.mode = 'disk',
            cache.dir = dir,
            cache.time = 0,
            cache.copy.images = TRUE))
    expect_equal(list.files(dir), lf)
})

test_that('caching works correctly', {
    env <- new.env()
    evals('x <- 1:10', env = env, cache.time = 0)
    expect_true(length(ls(pander:::cached.results)) > 0)
    evals('x <- 1:100', env = env, cache.time = 0)
    res <- evals('x', env = env)
    expect_equal(res[[1]]$result, 1:100)
    t1 <- system.time(evals('plot(mtcars)', hi.res = T))
    lf <- list.files('plots')
    t2 <- system.time(evals('plot(mtcars)', hi.res = T))
    expect_equal(list.files('plots'), lf)
})

test_that('plots', {
    r1 <- evals('ggplot(mtcars, aes(wt, mpg)) + geom_point() + geom_smooth(aes(wt,mpg), method = lm, se=FALSE)',
        graph.unify = T)
    r2 <- evals('xyplot(wt ~ mpg, mtcars, type = c("p","smooth"))', graph.unify = T)
    expect_equal(r1$type, r2$type)
    dir <- 'temp'
    r1 <- evals('plot(mtcars)',
        graph.recordplot = TRUE,
        cache.time = 0,
        cache.copy.images = TRUE,
        cache.mode = 'disk',
        cache.dir = dir)
    lf <- list.files(dir)
    expect_true(length(list.files(dir, pattern = 'recordedplot$')) > 0)
    png(filename = file.path(dir, 'recorded.png'))
    redraw.recordedplot(list.files(dir, pattern = 'recordedplot$', full.names = T)[1])
    dev.off()
})

test_that('hooks work correctly', {
    set.seed(1)
    txt <- 'runif(1:4); matrix(runif(25), 5, 5); 1:5'
    hooks <- list('numeric' = round, 'matrix' = pander_return)
    res <- evals(txt, hooks = hooks)
    set.seed(1)
    expect_equal(res[[1]]$result, round(runif(1:4)))
    expect_equal(res[[2]]$result, pander_return(matrix(runif(25), 5, 5)))
    ## using pander's default hook
    set.seed(1)
    res <- evals(txt, hooks = list('default' = pander_return))
    set.seed(1)
    expect_equal(res[[1]]$result, pander_return(runif(1:4)))
    expect_equal(res[[2]]$result, pander_return(matrix(runif(25), 5, 5)))
    expect_equal(res[[3]]$result, pander_return(1:5))
})

test_that('output param works correctly', {
    res <- evals('1:10', output = 'all')
    expect_equal(names(res[[1]]), c('src', 'result', 'output', 'type', 'msg', 'stdout'))
    res <- evals('1:10', output = c('src', 'result'))
    expect_equal(names(res[[1]]), c('src', 'result'))
    expect_error(evals('1:10', output = c('src', 'result', '123')))
    res <- evals('matrix(5,5,5)',
          hooks = list('matrix' = list(pander_return, 'Cap')))[[1]]$result
    expect_equal(res[14], 'Table: Cap')
})

evalsOptions('cache.dir',  cache.dir)
evalsOptions('graph.dir',  graph.dir)
setwd(wd)

test_that('switching cache works', {
    co <- evalsOptions('cache')
    evalsOptions('cache', FALSE)
    expect_false(evalsOptions('cache'))
    evalsOptions('cache', TRUE)
    expect_true(evalsOptions('cache'))
    evalsOptions('cache', co)
})

#' Add trailing and leading blank line
#'
#' Adds a line break before \emph{and} after the character string(s).
#' @param x character vector
#' @export
add.blank.lines <- function(x)
    sprintf('\n%s\n', x)


#' Remove more then two joined newlines
#' @param x character vector
#' @export
#' @examples
#' remove.extra.newlines(c('\n\n\n', '\n\n', '\n'))
remove.extra.newlines <- function(x)
    gsub('[\n][\n]+', '\n\n', x)


#' Trim leading and trailing spaces
#' @param x character vector
#' @return character vector
#' @export
#' @seealso \code{trim.space} in \code{rapport} package
trim.spaces <- function(x)
    gsub('^[[:space:]]+|[[:space:]]+$', '', x)

#' Repeating chars
#'
#' Repeating a string \code{n} times and returning a concatenated character vector.
#' @param x string to repeat
#' @param n integer
#' @param sep separator between repetitions
#' @return character vector
#' @export
repChar <- function(x, n, sep = '')
    paste(rep.int(x, n), collapse = sep)


#' Inline Printing
#'
#' \code{\link{p}} merges elements of a vector in one string for the sake of pretty inline printing. Default parameters are read from appropriate \code{option} values (see argument description for details). This function allows you to put the results of an expression that yields a variable \emph{inline}, by wrapping the vector elements with the string provided in \code{wrap}, and separating elements by main and ending separator (\code{sep} and \code{copula}). In case of a two-length vector, value specified in \code{copula} will be used as a separator. You can also control the length of provided vector by altering an integer value specified in \code{limit} argument (defaults to \code{Inf}).
#' @param x an atomic vector to get merged for inline printing
#' @param wrap a string to wrap vector elements (uses value set in \code{p.wrap} option: \code{'_'} by default, which is a markdown-friendly wrapper and it puts the string in \emph{italic})
#' @param sep a string with the main separator, i.e. the one that separates all vector elements but the last two (uses the value set in \code{p.sep} option - \code{','} by default)
#' @param copula a string with ending separator - the one that separates the last two vector elements (uses the value set in \code{p.copula} option, \code{'and'} by default)
#' @param limit maximum character length (defaults to \code{Inf}initive  elements)
#' @param keep.trailing.zeros to show or remove trailing zeros in numbers
#' @param missing string to replace missing values
#' @param digits numeric (default: 2) passed to format
#' @param round numeric (default: Inf) passed to round
#' @return a string with concatenated vector contents
#' @examples
#' p(c('fee', 'fi', 'foo', 'fam'))
#' # [1] '_fee_, _fi_, _foo_ and _fam_'
#' p(1:3, wrap = '')
#' # [1] '1, 2 and 3'
#' p(LETTERS[1:5], copula = 'and the letter')
#' # [1] '_A_, _B_, _C_, _D_ and the letter _E_'
#' p(c('Thelma', 'Louise'), wrap = '', copula = '&')
#' # [1] 'Thelma & Louise'
#' @export
#' @author Aleksandar Blagotic
#' @references This function was moved from \code{rapport} package: \url{http://rapport-package.info}.
p <- function(x, wrap = panderOptions('p.wrap'), sep = panderOptions('p.sep'), copula = panderOptions('p.copula'), limit = Inf, keep.trailing.zeros = panderOptions('keep.trailing.zeros'), missing = panderOptions('missing'), digits = panderOptions('digits'), round = panderOptions('round')){ #nolint

    attributes(x) <- NULL
    stopifnot(is.vector(x))
    stopifnot(all(sapply(list(wrap, sep, copula), function(x) is.character(x) && length(x) == 1)))
    x.len <- length(x)
    if (x.len == 0) {
        return('')
    }
    stopifnot(x.len <= limit)

    ## store missing values
    w <- which(is.na(x))

    ## prettify numbers
    if (is.numeric(x)) {

        x <- round(x, digits = round)

        ## optionally remove trailing zeros by running format separately on each element of the vector
        if (!keep.trailing.zeros) {
            x <- sapply(x, format, trim = TRUE, digits = digits, decimal.mark = panderOptions('decimal.mark')) #nolint
        } else {
            ## otherwise force using the same number format for all vector elements
            x <- format(x, trim = TRUE, digits = digits, decimal.mark = panderOptions('decimal.mark'))
        }

    }

    ## replace missing values
    if (length(w) > 0) {
        x[w] <- missing
    }

    if (x.len == 1) {
        wrap(x, wrap)
    } else if (x.len == 2) {
        paste(wrap(x, wrap), collapse = copula)
    } else {
        paste0(paste(wrap(head(x, -1), wrap), collapse = sep), copula, wrap(tail(x, 1), wrap))
    }
}


#' Wrap Vector Elements
#'
#' Wraps vector elements with string provided in \code{wrap} argument.
#' @param x a vector to wrap
#' @param wrap a string to wrap around vector elements
#' @return a string with wrapped elements
#' @examples \dontrun{
#' wrap('foobar')
#' wrap(c('fee', 'fi', 'foo', 'fam'), '_')
#' }
#' @export
#' @author Aleksandar Blagotic
#' @references This function was moved from \code{rapport} package: \url{http://rapport-package.info}.
wrap <- function(x, wrap = '"'){
    attributes(x) <- NULL
    stopifnot(is.vector(x))
    sprintf('%s%s%s', wrap, x, wrap)
}

#' Check if rownames are available
#'
#' Dummy helper to check if the R object has real rownames or not.
#' @param x a tabular-like R object
#' @return \code{TRUE} OR \code{FALSE}
#' @export
has.rownames <- function(x) {
    length(dim(x)) != 1 && length(rownames(x)) > 0 && !all(rownames(x) == 1:nrow(x)) && !all(rownames(x) == '')
}

#' Adds caption in current block
#'
#' This is a helper function to add a caption to the returning image/table.
#' @param x string
#' @param permanent (default \code{FALSE}) if caption is permanent (for all future tables) or not
#' @export
set.caption <- function(x, permanent = FALSE){
    assign('caption', x, envir = storage)
    if (!is.null(x)) {
        attr(storage$caption, 'permanent') <- permanent
    }
}


#' Get caption
#'
#' Get caption from temporary environment and truncates that
#' @return stored caption as string
#' @keywords internal
get.caption <- function()
    get.storage('caption')


#' Sets alignment for tables
#'
#' This is a helper function to update the alignment (\code{justify} parameter in \code{pandoc.table}) of the next returning table. Possible values are: \code{centre} or \code{center}, \code{right}, \code{left}.
#' @param default character vector which length equals to one (would be repeated \code{n} times) ot \code{n} - where \code{n} equals to the number of columns in the following table
#' @param row.names string holding the alignment of the (optional) row names
#' @param permanent (default \code{FALSE}) if alignment is permanent (for all future tables) or not. It's cleaner to use \code{panderOptions} instead.
#' @export
set.alignment <- function(default = panderOptions('table.alignment.default'), row.names = panderOptions('table.alignment.rownames'), permanent = FALSE) { #nolint
    assign('alignment', list(default = default, row.names = row.names), envir = storage)
    attr(storage$alignment, 'permanent') <- permanent
}


#' Get alignment
#'
#' Get alignment from temporary environment, truncating that and applying rownames and other columns alignment to passed \code{df}.
#' @return vector of alignment parameters
#' @keywords internal
#' @param df data.frame
#' @param remove.obious.rownames if this rule should be applied or not
get.alignment <- function(df, remove.obious.rownames = TRUE) {

    if (is.null(attr(df, 'alignment'))) {

        a <- get.storage('alignment')

        if (is.null(a)) {
            ad <- panderOptions('table.alignment.default')
            ar <- panderOptions('table.alignment.rownames')
            if (remove.obious.rownames && !has.rownames(df)) {
                ar <- NULL
            }
            if (is.function(ar)) {
                ar <- ar()
            }
            if (is.function(ad)) {
                return(c(ar, ad(df)))
            }
            a <- list(default = ad, row.names = ar)
        }

        if (length(a) == 1) {
            a <- list(default = as.character(a), row.names = as.character(a))
        }

        if (length(dim(df)) < 2) {
            w <- length(df)
            n <- NULL
        } else {
            w <- ncol(df)
            n <- rownames(df)
            if (isTRUE(remove.obious.rownames) && all(n == 1:nrow(df))) {
                n <- NULL
            }
        }

        if (is.null(n)) {
            return(rep(a$default, length.out = w))
        } else {
            return(c(a$row.names, rep(a$default, length.out = w)))
        }

    }

    attr(df, 'alignment')
}


#' Emphasize rows/columns/cells
#'
#' Storing indexes of cells to be (strong) emphasized of a tabular data in an internal buffer that can be released and applied by \code{\link{pandoc.table}}, \code{\link{pander}} or \code{\link{evals}} later.
#' @param x vector of row/columns indexes or an array like returned by \code{which(..., arr.ind = TRUE)}
#' @aliases emphasize.rows emphasize.cols emphasize.cells emphasize.strong.rows emphasize.strong.cols emphasize.strong.cells emphasize.italics.rows emphasize.italics.cols emphasize.italics.cells emphasize.verbatim.rows emphasize.verbatim.cols emphasize.verbatim.cells
#' @usage
#' emphasize.rows(x)
#'
#' emphasize.cols(x)
#'
#' emphasize.cells(x)
#'
#' emphasize.strong.rows(x)
#'
#' emphasize.strong.cols(x)
#'
#' emphasize.strong.cells(x)
#'
#' emphasize.italics.rows(x)
#'
#' emphasize.italics.cols(x)
#'
#' emphasize.italics.cells(x)
#'
#' emphasize.verbatim.rows(x)
#'
#' emphasize.verbatim.cols(x)
#'
#' emphasize.verbatim.cells(x)
#' @export
#' @examples \dontrun{
#' n <- data.frame(x = c(1,1,1,1,1), y = c(0,1,0,1,0))
#' emphasize.cols(1)
#' emphasize.rows(1)
#' pandoc.table(n)
#'
#' emphasize.strong.cells(which(n == 1, arr.ind = TRUE))
#' pander(n)
#' }
emphasize.rows <- function(x)
    assign(deparse(match.call()[[1]]), x, envir = storage)
#' @export
emphasize.strong.rows <- emphasize.rows
#' @export
emphasize.italics.rows <- emphasize.rows
#' @export
emphasize.verbatim.rows <- emphasize.rows
#' @export
emphasize.cols <- emphasize.rows
#' @export
emphasize.strong.cols <- emphasize.rows
#' @export
emphasize.italics.cols <- emphasize.rows
#' @export
emphasize.verbatim.cols <- emphasize.rows
#' @export
emphasize.cells <- emphasize.rows
#' @export
emphasize.strong.cells <- emphasize.rows
#' @export
emphasize.italics.cells <- emphasize.rows
#' @export
emphasize.verbatim.cells <- emphasize.rows


#' Get emphasize params from internal buffer
#'
#' And truncate content.
#' @param df tabular data
#' @return R object passed as \code{df} with possibly added \code{attr}s captured from internal buffer
#' @keywords internal
get.emphasize <- function(df) {
    for (v in c('emphasize.rows',
                'emphasize.cols',
                'emphasize.cells',
                'emphasize.strong.rows',
                'emphasize.strong.cols',
                'emphasize.strong.cells',
                'emphasize.italics.rows',
                'emphasize.italics.cols',
                'emphasize.italics.cells',
                'emphasize.verbatim.rows',
                'emphasize.verbatim.cols',
                'emphasize.verbatim.cells')) {
        if (is.null(attr(df, v))) {
            attr(df, v) <- get.storage(v)
        }
    }
    return(df)
}


#' Get a value from internal buffer
#'
#' And truncate content.
#' @param what string
#' @keywords internal
get.storage <- function(what) {
    res <- tryCatch(get(what, envir = storage, inherits = FALSE), error = function(e) NULL)
    if (is.null(attr(res, 'permanent')) || !attr(res, 'permanent')) {
        assign(what, NULL, envir = storage)
    }
    return(res)
}


#' Add significance stars
#'
#' This function adds significance stars to passed \code{p} value(s) as: one star for value below \code{0.05}, two for \code{0.01} and three for \code{0.001}.
#' @param p numeric vector or tabular data
#' @param cutoffs the cutoffs for the 1/2/3 significance stars
#' @return character vector
#' @export
add.significance.stars <- function(p, cutoffs = c(0.05, 0.01, 0.001)) {

    stopifnot(length(cutoffs) == 3)

    if (inherits(p, c('matrix', 'data.frame')) && length(dim(p)) == 2) {
        apply(p, c(1,2), add.significance.stars, cutoffs = cutoffs)
    } else {
        if (length(p) > 1) {
            sapply(p, add.significance.stars, cutoffs = cutoffs)
        } else {
            ifelse(p > cutoffs[1], '',
                   ifelse(p > cutoffs[2], ' *',
                          ifelse(p > cutoffs[3], ' * *', ' * * *')))
        }
    }
}


#' Toggle cache
#'
#' This function is just a wrapper around \code{\link{evalsOptions}} to switch pander's cache on or off easily, which might be handy in some brew documents to prevent repetitive strain injury :)
#' @aliases cache.on
#' @usage
#' cache.on()
#'
#' cache.off()
#' @export
cache.off <- function()
    evalsOptions('cache', FALSE)

#' @export
cache.on <- function()
    evalsOptions('cache', TRUE)

#' Split line with line breaks depending on max.width
#'
#' This is a helper function to insert line breaks depending on (\code{split.cells} parameter of \code{pandoc.table}) of the returning table.
#' @param x string to be split. Works only with one string. Non-string arguments and multi-dimensional arguments are returned unchaged
#' @param max.width default integer value specyfing max number of characters between line breaks
#' @param use.hyphening (default: \code{FALSE}) if try to use hyphening when splitting large cells according to table.split.cells. Requires \pkg{sylly}.
#' @return character string with line breaks
#' @export
#' @examples
#' splitLine('foo bar', 6)
#' splitLine('foo bar', 7)
#' splitLine('Pandoc Package', 3, TRUE)
splitLine <- function(x, max.width = panderOptions('table.split.cells'), use.hyphening = FALSE) {
    if (any(is.na(x))) {
        return(x)
    }
    if (!is.character(x) || !is.null(dim(x)) || length(x) != 1 || x == '') {
        return(x)
    }
    if (suppressWarnings(!is.na(as.numeric(x)))) {
        return(x)
    }
    if (is.infinite(max.width)) {
        max.width <- .Machine$integer.max
    }
    if (use.hyphening) {
        if (requireNamespace('sylly', quietly = TRUE) && requireNamespace('sylly.en', quietly = TRUE)) {
            sylly.en::hyph.support.en()
        } else {
            use.hyphening <- FALSE
        }
    }
    hyphen_f <- function(s)
        sylly::hyphen(s, hyph.pattern = 'en', quiet = TRUE)@hyphen[1, 2]
    .Call('pander_splitLine_cpp', PACKAGE = 'pander', x, max.width, use.hyphening, hyphen_f)
}


#' Check if caption is valid
#' @param caption R object to check
#' @return boolean
#' @keywords internal
check_caption <- function(caption) {

    if (length(caption) > 1) {
        stop('The caption should be exactly one string.')
    }

    if (!(is.character(caption) | is.null(caption))){
        stop('The caption should be string (character class) or NULL.')
    }
    invisible(TRUE)
}


#' Check if vector parameter for round/digits and adjust accodingly
#' @param param vector to be checked
#' @param name parameter name
#' @param n needed size of vector
#' @return original vector if size is good, vector of default values otherwise
#' @keywords internal
check_digits <- function(param, name, n) {
    if (length(param) == 1) {
        param <- rep(param, n)
    } else if (length(param) < n) {
        warning(sprintf('%s is not equal to ncol(t), reverting to default value', name))
        param <- rep(panderOptions(name), n)
    }
    param
}

#' Create a multitable used for rendering objects from rms package
#'
#' When ols/lrm/orm from rms package get rendered, main statistics are group in table of tables.
#' Since pandoc doesn't support row or col-span,
#' we chose to group those statistics in a column each.
#' This function takes care of that
#' @param v list of vectors/lists to be merge in the table
#' @return data.frame in specified format
#' @examples
#' pander:::multitable(list(list(a=1, b=2),list(c=3, d=4)))
#' @keywords internal
multitable <- function(v) {
    ml <- max(sapply(v, length))
    mod <- lapply(1:length(v),
                  function(i)  {
                      uv <- unlist(c(rbind(pandoc.strong.return(names(v[[i]])),
                                           sapply(v[[i]], p, wrap = ''))))
                      if (length(v[[i]]) < ml)
                          uv <- c(uv, rep('', 2 * (ml - length(v[[i]]))))
                      uv
                  })
    do.call(cbind, mod)
}

#' Calculate coef matrix for models from rms package
#' Forked from prModFit from rms
#'
#' @param obj object list
#' @param coefs numeric value if to print only the first n regression coefficients in the model.
#' @return coeficients matrix
#' @importFrom stats pt pchisq
coef_mat <- function(obj, coefs) {
    errordf <- obj$errordf
    beta <- obj$coef
    se <- obj$se
    Z <- beta / se
    if (length(errordf)) {
        P <- 2 * (1 - pt(abs(Z), errordf))
    } else {
        P <- 1 - pchisq(Z ^ 2, 1)
    }
    U <- cbind(beta, se, Z, P)
    colnames(U) <- c('Coef', 'S.E.', 'Wald Z', 'Pr(>|Z|)')
    if (length(errordf)) {
        colnames(U)[3:4] <- c('t', 'Pr(>|t|)')
    }
    rownames(U) <- names(beta)
    if (length(obj$aux)) {
        U <- cbind(U, obj$aux)
        colnames(U)[ncol(U)] <- obj$auxname
    }
    if (is.numeric(coefs)) {
        U <- U[1:coefs, , drop = FALSE] #nolint
        U <- rbind(U, rep('', ncol(U)))
        rownames(U)[nrow(U)] <- '. . .'
    }
    U
}

#' Generic pander method
#'
#' Prints an R object in Pandoc's markdown.
#' @param x an R object
#' @param ... optional parameters passed to special methods and/or raw \code{pandoc.*} functions
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @note This function can be called by \code{pander} and \code{pandoc} too.
#' @references \itemize{
#'   \item John MacFarlane (2013): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#'   \item David Hajage (2011): _ascii. Export R objects to several markup languages._ \url{https://cran.r-project.org/package=ascii}
#'   \item Hlavac, Marek (2013): _stargazer: LaTeX code for well-formatted regression and summary statistics tables._ \url{https://cran.r-project.org/package=stargazer}
#' }
#' @export
#' @examples
#'
#' ## Vectors
#' pander(1:10)
#' pander(letters)
#' pander(mtcars$am)
#' pander(factor(mtcars$am))
#'
#' ## Lists
#' pander(list(1, 2, 3, c(1, 2)))
#' pander(list(a = 1, b = 2, c = table(mtcars$am)))
#' pander(list(1, 2, 3, list(1, 2)))
#' pander(list(a = 1, 2, 3, list(1, 2)))
#' pander(list('FOO', letters[1:3], list(1:5), table(mtcars$gear), list('FOOBAR', list('a', 'b'))))
#' pander(list(a = 1, b = 2, c = table(mtcars$am), x = list(myname = 1, 2), 56))
#' pander(unclass(chisq.test(table(mtcars$am, mtcars$gear))))
#'
#' ## Arrays
#' pander(mtcars)
#' pander(table(mtcars$am))
#' pander(table(mtcars$am, mtcars$gear))
#'
#' ## Tests
#' pander(ks.test(runif(50), runif(50)))
#' pander(chisq.test(table(mtcars$am, mtcars$gear)))
#' pander(t.test(extra ~ group, data = sleep))
#'
#' ## Models
#' ml <- with(lm(mpg ~ hp + wt), data = mtcars)
#' pander(ml)
#' pander(anova(ml))
#' pander(aov(ml))
#' ## Dobson (1990) Page 93: Randomized Controlled Trial (examples from: ?glm)
#' counts <- c(18, 17, 15, 20, 10, 20, 25, 13, 12)
#' outcome <- gl(3, 1, 9)
#' treatment <- gl(3, 3)
#' m <- glm(counts ~ outcome + treatment, family = poisson())
#' pander(m)
#' pander(anova(m))
#' pander(aov(m))
#' ## overwriting labels
#' pander(lm(Sepal.Width ~ Species, data = iris), covariate.labels = c('Versicolor', 'Virginica'))
#'
#' ## Prcomp
#' pander(prcomp(USArrests))
#'
#' ## Others
#' pander(density(runif(10)))
#' pander(density(mtcars$hp))
#'
#' ## default method
#' x <- chisq.test(table(mtcars$am, mtcars$gear))
#' class(x) <- 'I heave never heard of!'
#' pander(x)
pander <- function(x = NULL, ...) {

    ## save current knitr.auto.asis option
    kaao <- panderOptions('knitr.auto.asis')

    if (isTRUE(panderOptions('knitr.auto.asis')) &&
        isTRUE(getOption('knitr.in.progress')) &&
        requireNamespace('knitr', quietly = TRUE)) {

        ## override knitr.auto.asis option for nested calls
        panderOptions('knitr.auto.asis', FALSE)

        ## grab stdout
        stdout <- vector('character')
        con    <- textConnection('stdout', 'wr', local = TRUE)
        sink(con)
        sink(con, type = 'message')

        ## close
        on.exit({

            ## restore knitr.auto.asis option
            if (panderOptions('knitr.auto.asis') != kaao) {
                panderOptions('knitr.auto.asis', kaao)
            }

            ## revert R output back to normal
            sink()
            sink(type = 'message')
            close(con)

            ## restore the final line break
            if (tail(stdout, 1) == '') {
                stdout <- c(stdout, '')
            }

            return(knitr::asis_output(paste(stdout, collapse = '\n')))
        })
    }

    UseMethod('pander', x)

}


#' Pander and capture output
#'
#' This is a wrapper function around \code{pander} but instead of printing to \code{stdout}, this function returns a character vector of the captured lines.
#' @param ... everything passed to \code{pander}
#' @export
#' @seealso pander
pander_return <- function(...)
    capture.output(pander(...))


#' Pander method for a NULL object
#'
#' Prints a NULL object in Pandoc's markdown.
#' @param x a NULL object
#' @param ... ignored parameters
#' @export
pander.NULL <- function(x, ...)
    return(invisible(NULL))


#' Helper function to deal with atomic vectors
#' @param x vector
#' @param ... ignored parameters
#' @keywords internal
pander.vector <- function(x, ...) {

    if (!is.null(names(x))) {
        return(pandoc.table(x))
    }

    cat(p(x))

}


#' Pander method for logical class
#'
#' Prints a logical object in Pandoc's markdown.
#' @param x a logical object
#' @param ... ignored parameters
#' @export
pander.logical <- function(x, ...)
    pander.vector(x, ...)


#' Pander method for numeric class
#'
#' Prints a numeric class in Pandoc's markdown.
#' @param x a numeric object
#' @param ... igroned parameter
#' @export
pander.numeric <- function(x, ...)
    pander.vector(x, ...)


#' Pander method for factor class
#'
#' Prints a factor object in Pandoc's markdown.
#' @param x a factor object
#' @param ... igroned parameters
#' @export
pander.factor <- function(x, ...)
    pander.vector(x, ...)


#' Pander method for character class
#'
#' Prints a character class in Pandoc's markdown.
#' @param x a character object
#' @param ... igroned parameters
#' @export
pander.character <- function(x, ...) {

    if (!is.null(names(x))) {
        return(pandoc.table(x))
    }

    if (length(x) < 2) {
        cat(x)
    } else {
        cat(p(x))
    }

}


#' Pander method for image class
#'
#' Prints a image object in Pandoc's markdown.
#' @param x a image object
#' @param caption caption (string) to be shown under the table
#' @param href link that image should be linked with
#' @param ... ignored parameters
#' @export
pander.image <- function(x, caption = attr(x, 'caption'), href = attr(x, 'href'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    res <- pandoc.image.return(as.character(x), caption)

    if (is.null(href)) {
        cat(res)
    } else {
        pandoc.link(href, res)
    }

}


#' Pander method for table class
#'
#' Prints a table object in Pandoc's markdown.
#' @param x a table object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.table <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    pandoc.table(x, caption = caption, ...)

}

#' Pander method for data.table class
#'
#' Prints a data.table object in Pandoc's markdown. Data.tables drop attributes (like row names) when called.
#' @param x a data.table object
#' @param caption caption (string) to be shown under the table
#' @param keys.as.row.names controls whether to use data.table key as row names when calling pandoc.table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.data.table <- function(x, caption = attr(x, 'caption'),
                              keys.as.row.names = TRUE, ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    requireNamespace('data.table', quietly = TRUE)
    xx <- data.table::copy(x)
    if (keys.as.row.names && data.table::haskey(xx)) {
        firstkey <- data.table::key(xx)[1]
        row.names.dt <- as.character(xx[[firstkey]])
        xx <- xx[, setdiff(colnames(xx), firstkey), with = FALSE, drop = FALSE]
        data.table::setattr(xx, 'row.names', row.names.dt)
    }

    pandoc.table(xx, caption = caption, ...)

}

#' Pander method for data.frame class
#'
#' Prints a data.frame object in Pandoc's markdown.
#' @param x a data.frame object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.data.frame <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    pandoc.table(x, caption = caption, ...)

}


#' Pander method for matrix class
#'
#' Prints a matrix object in Pandoc's markdown.
#' @param x a matrix object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.matrix <- function(x, caption = attr(x, 'caption'),  ...) {
    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }
    pandoc.table(x, caption = caption, ...)
}


#' Pander method for cast_df class
#'
#' Prints a cast_df object in Pandoc's markdown.
#' @param x a cast_df object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.cast_df <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    pandoc.table(as.data.frame(x), caption = caption, ...)

}


#' Pander method for summary.lm class
#'
#' Prints a summary.lm object in Pandoc's markdown.
#' @param x an summary.lm object
#' @param caption caption (string) to be shown under the table
#' @param covariate.labels vector to replace covariate lables in the table
#' @param omit vector of variable to omit for priting in resulting table
#' @param summary (defaut:\code{TRUE}) if used for summary.lm or lm
#' @param add.significance.stars if significance stars should be shown for P value
#' @param move.intercept by default, the Intercept is the first coefficient in the table, which can be moved to the bottom of the table
#' @param ... optional parameters passed to special methods and/or raw \code{pandoc.*} functions
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
pander.summary.lm <- function(x, caption = attr(x, 'caption'), covariate.labels,
                              omit, summary = TRUE, add.significance.stars = FALSE, move.intercept = FALSE, ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- pandoc.formula.return(x$call$formula, text = 'Fitting linear model:')
        } else {
            caption <- get.caption()
        }
    }

    res <- as.data.frame(x$coeff)

    if (move.intercept && rownames(res)[1] == '(Intercept)' & nrow(res) > 1) {
        res <- res[c(2:nrow(res), 1), ]
    }

    if (!missing(omit)) {
        res <- res[!apply(sapply(omit, grepl, row.names(res)), 1, any), ]
    }

    if (!missing(covariate.labels)) {
        row.names(res)[1:length(covariate.labels)] <- covariate.labels
    }

    if (add.significance.stars) {
        res <- cbind(res, ' ' = add.significance.stars(res[, 4]))
    }

    if (summary) {
        pandoc.table(res, ...)
        if (class(x) == 'summary.glm') {
            cat('\n(Dispersion parameter for ', x$family$family, ' family taken to be ',
                format(x$dispersion), ')\n\n')
            stats <- cbind(paste(format(c('Null', 'Residual'), justify = 'right'), 'deviance:'),
                           apply(cbind(
                               format(unlist(x[c('null.deviance', 'deviance')]), digits = panderOptions('digits')),
                               ' on',
                               format(unlist(x[c('df.null', 'df.residual')])), ' degrees of freedom\n'),
                               1L, paste, collapse = ' '))
            rownames(stats) <- NULL
            colnames(stats) <- NULL
            pandoc.table(stats, keep.trailing.zeros = TRUE, ...)
        } else {
            pandoc.table(data.frame(
                'Observations'        = length(x$residuals),
                'Residual Std. Error' = x$sigma,
                '$R^2$'               = x$r.squared,
                'Adjusted $R^2$'      = x$adj.r.squared,
                check.names = FALSE), keep.trailing.zeros = TRUE, caption = caption, digits = panderOptions('digits'))
        }
    } else {

        pandoc.table(res, caption = caption, ...)

    }

}


#' Pander method for summary.glm class
#'
#' Prints a summary.glm object in Pandoc's markdown.
#' @param x an summary.glm object
#' @param caption caption (string) to be shown under the table
#' @param covariate.labels vector to replace covariate lables in the table
#' @param omit vector of variable to omit for priting in resulting table
#' @param summary (defaut:\code{TRUE}) if used for summary.lm or lm
#' @param ... optional parameters passed to special methods and/or raw \code{pandoc.*} functions
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
pander.summary.glm <- function(x, caption = attr(x, 'caption'), covariate.labels, omit, summary = TRUE, ...)
    pander.summary.lm(x, caption = caption, summary = summary, covariate.labels = covariate.labels, omit = omit, ...)


#' Pander method for summary.lm class
#'
#' Prints a summary.lm object in Pandoc's markdown.
#' @param x a summary.glm object
#' @param caption caption (string) to be shown under the table
#' @param covariate.labels vector to replace covariate lables in the table
#' @param omit vector of variable to omit for priting in resulting table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.lm <- function(x, caption = attr(x, 'caption'), covariate.labels, omit, ...)
    pander.summary.lm(summary(x),
                      caption = caption,
                      summary = FALSE,
                      covariate.labels = covariate.labels,
                      omit = omit, ...)


#' Pander method for summary.glm class
#'
#' Prints a summary.glm object in Pandoc's markdown.
#' @param x a summary.glm object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.glm <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- sprintf('Fitting generalized (%s) linear model: %s',
                               paste(x$family$family, x$family$link, sep = '/'),
                               pandoc.formula.return(x$call$formula))
        } else {
            caption <- get.caption()
        }
    }

    pander.summary.glm(summary(x), caption = caption, summary = FALSE, ...)

}


#' Pander method for summary.aov class
#'
#' Prints a summary.aov object in Pandoc's markdown.
#' @param x a summary.aov object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.aov <- function(x, caption = attr(x, 'caption'), ...) {

    res <- unclass(x)[[1]]

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- 'Analysis of Variance Model'
        } else {
            caption <- get.caption()
        }
    }

    pandoc.table(res, caption = caption, ...)
}


#' Pander method for aov class
#'
#' Prints an aov object in Pandoc's markdown.
#' @param x an aov object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.aov <- function(x, caption = attr(x, 'caption'), ...) {
    pander(summary(x), caption = caption, ...)
}


#' Pander method for anova class
#'
#' Prints an anova object in Pandoc's markdown.
#' @param x an anova object
#' @param caption caption (string) to be shown under the table
#' @param add.significance.stars if significance stars should be shown for P value
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.anova <- function(x, caption = attr(x, 'caption'), add.significance.stars = FALSE, ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            if (!is.null(attr(x, 'heading'))) {
                caption <- strsplit(attr(x, 'heading'), '\n')[[1]][1]
            }
        }
    }
    if (is.null(caption)) {
        caption <- get.caption()
    }
    if (add.significance.stars) {
        x <- cbind(x, ' ' = add.significance.stars(x[, 5]))
    }
    pandoc.table(x, caption = caption, ...)
    if (add.significance.stars) {
        cat('Signif. codes:  0 \'\\*\\*\\*\' 0.001 \'\\*\\*\' 0.01 \'\\*\' 0.05 \'.\' 0.1 \' \' 1\n')
    }
}


#' Pander method for summary.aovlist class
#'
#' Prints a summary.aovlist object in Pandoc's markdown.
#' @param x a summary.aovlist object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.aovlist <- function(x, caption = attr(x, 'caption'), ...) {
    n <- length(x)
    if (n == 1) {
        pandoc.table(unclass(x[[1]][[1]]), caption, ...)
    } else {
        z <- x[[1]][[1]]
        for (i in 2:n) {
            z <- rbind(z, x[[i]][[1]])
        }
        pandoc.table(z, caption = caption, ...)
    }
}


#' Pander method for aovlist class
#'
#' Prints an aovlist object in Pandoc's markdown.
#' @param x an aovlist object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.aovlist <- function(x, caption = attr(x, 'caption'), ...) {
    pander(summary(x), caption = caption, ...)
}


#' Pander method for htest class
#'
#' Prints a htest object in Pandoc's markdown.
#' @param x a htest object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.htest <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- paste0(x$method, ': `', gsub('( and | by )', '`\\1`', paste(x$data.name, collapse='')), '`')
        } else {
            caption <- get.caption()
        }
    }

    ## we do not know which values are provided
    res <- data.frame(placeholder = 'FOO')

    ## add what we know
    if (!is.null(x$statistic)) {
        res$'Test statistic' <- as.numeric(x$statistic)
    }
    if (!is.null(x$parameter)) {
        res[names(x$parameter)] <- x$parameter
    }
    if (!is.null(x$p.value)) {
        res$'P value' <- paste(
          format(round(x$p.value, panderOptions('round')),
                 trim         = TRUE,
                 digits       = panderOptions('digits'),
                 decimal.mark = panderOptions('decimal.mark')),
          add.significance.stars(x$p.value))
    }
    if (!is.null(x$alternative)) {
        res['Alternative hypothesis'] <- x$alternative
    }
    if (!is.null(x$estimate)) {
        if (!is.null(names(x$estimate))) {
            res[names(x$estimate)] <- x$estimate
        } else {
            res['Estimate'] <- x$estimate
        }
    }

    ## drop placeholder
    res$placeholder <- NULL

    ## return
    pandoc.table(res, caption = caption, ...)

}


#' Pander method for summary.prcomp class
#'
#' Prints a summary.prcomp object in Pandoc's markdown.
#' @param x a summary.prcomp object
#' @param caption caption (string) to be shown under the table
#' @param summary (default:\code{TRUE}) if extended summary should be printed
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.prcomp <- function(x, caption = attr(x, 'caption'), summary = TRUE, ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- 'Principal Components Analysis'
        } else {
            caption <- get.caption()
        }
    }

    pandoc.table(x$rotation, caption = caption, ...)
    if (summary) {
        pandoc.table(x$importance, ...)
    }
}


#' Pander method for prcomp class
#'
#' Prints a prcomp object in Pandoc's markdown.
#' @param x a prcomp object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.prcomp <- function(x, caption = attr(x, 'caption'),  ...) {
    pander(summary(x), caption = caption, summary = FALSE, ...)
}


#' Pander method for density class
#'
#' Prints a density object in Pandoc's markdown.
#' @param x a density object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.density <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- sprintf('Kernel density of *%s* (bandwidth: %s)', x$data.name, format(x$bw))
        } else {
            caption <- get.caption()
        }
    }

    res <- data.frame('Coordinates' = as.numeric(summary(x$x)),
                      'Density values' = as.numeric(summary(x$y)),
                      check.names = FALSE)
    rownames(res) <- names(summary(1))

    pandoc.table(res, caption = caption, ...)
}


#' Pander method for list class
#'
#' Prints a list object in Pandoc's markdown.
#' @param x a list object
#' @param ... ignored parameters
#' @export
pander.list <- function(x, ...) {

    ## match call
    mc <- match.call()
    if (is.null(mc$indent)) {
        indent <- 0
    } else {
        indent <- eval(mc$indent, parent.frame(1))
    }

    ## replace missing values
    w <- which(is.na(x))
    if (length(w) > 0) {
        x[w] <- panderOptions('missing')
    }

    ## grab elements name (if any)
    x.names <- sapply(names(x), function(x) ifelse(x == '', '  *', sprintf('  * **%s**:', x)))
    if (length(x.names) == 0) {
        x.names <- rep('  *', length(x))
    }

    ## capture pandoc output of list element
    res <- paste(unlist(lapply(1:length(x), function(i) {

        res.i <- paste(capture.output(pander(x[[i]], indent = indent + 1)), collapse = '\n')
        if (grepl('\n', res.i) & !grepl('\n *\\* ', res.i)) {
            res.i <- sub('^\n', '\n\n', res.i)
            res.i <- pandoc.indent(res.i, 1)
        }

        paste(x.names[i], res.i)

    })), collapse = '\n') #nolint

    ## indent output
    res <- pandoc.indent(res, indent)

    ## add (comment): end of list (preventing conflicts with forthcoming pandoc blocks)
    if (indent == 0) {
        res <- paste0(res, '\n\n<!-- end of list -->\n')
    }

    pandoc.p(res)

}


#' Default Pander method
#'
#' Method to be used, when no exact S3 method for given object is found. Tries to render object as a list
#' @param x an object
#' @param ... optional parameters passed to raw \code{pandoc.list} function
#' @export
pander.default <- function(x, ...) {
    warning(sprintf('No pander.method for "%s", reverting to default.', class(x)))
    class(x) <- 'list'
    pander(x, ...)
}


#' Pander method for evals class
#'
#' Prints a evals object in Pandoc's markdown.
#' @param x a evals object
#' @param ... ignored parameters
#' @export
pander.evals <- function(x, ...) {

    o <- pander(x$result)

    if (panderOptions('evals.messages')) {
        if (!is.null(x$msg$messages)) {
            o <- paste0(o, ' **MESSAGE**', pandoc.footnote.return(x$msg$messages))
        }
        if (!is.null(x$msg$warnings)) {
            o <- paste0(o, ' **WARNING**', pandoc.footnote.return(x$msg$warnings))
        }
        if (!is.null(x$msg$error)) {
            o <- paste0(o, ' **ERROR**', pandoc.footnote.return(x$msg$errors))
        }
    }

    cat(o)

}


#' Pander method for rapport class
#'
#' Prints a rapport object in Pandoc's markdown.
#' @param x a rapport object
#' @param ... ignored parameters
#' @export
pander.rapport <- function(x, ...)
    print(x)


#' Pander method for POSIXlt class
#'
#' Prints a POSIXlt object in Pandoc's markdown.
#' @param x a POSIXlt object
#' @param ... optional parameters passed to raw \code{pandoc.date} function
#' @export
pander.POSIXlt <- function(x, ...)
    pandoc.date(x, ...)


#' Pander method for POSIXct class
#'
#' Prints a POSIXct object in Pandoc's markdown.
#' @param x a POSIXct object
#' @param ... optional parameters passed to raw \code{pandoc.date} function
#' @export
pander.POSIXct <- function(x, ...)
    pandoc.date(x, ...)


#' Pander method for Date class
#'
#' Prints a Date object in Pandoc's markdown.
#' @param x a Date object
#' @param ... optional parameters passed to raw \code{pandoc.date} function
#' @export
pander.Date <- function(x, ...)
    pandoc.date(x, ...)


#' Pander method for ftable class
#'
#' Prints a ftable object in Pandoc's markdown.
#' @param x a ftable object
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.ftable <- function(x, ...)
    pandoc.table(x, ...)


#' Pander method for CrossTable class
#'
#' Prints a CrossTable object in Pandoc's markdown.
#' @param x a CrossTable object
#' @param caption caption (string) to be shown under the table
#' @param digits number of digits of precision
#' @param total.r if to print row totals. Default values is taken from CrossTable object
#' @param total.c if to print column totals. Default values is taken from CrossTable object
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.CrossTable <- function(x, caption = attr(x, 'caption'), digits = panderOptions('digits'),
                              total.r = x$total.r, total.c = x$total.c, ...) {
    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }
    nr <- dim(x$t)[1]
    nc <- dim(x$t)[2]
    if (!is.null(rownames(x$t))) {
        nt <- cbind(rownames(x$t), x$t, x$rs)
    } else {
        nt <- cbind('&nbsp;', x$t, x$rs)
    }
    hdd <- 100
    appendlines <- function(nt, xx, hasttl = FALSE, haslbl = FALSE) {
        if (!hasttl) {
            xx <- cbind(xx, rep('', nr))
        }
        if (!haslbl) {
            xx <- cbind(rep('', nr), xx)
        }
        n <- dim(nt)[1] / nr
        nt <- rbind(nt, xx)
        idx <- integer()
        k <- 1
        l <- nr * n + 1
        for (i in 1:nr) {
            for (j in 1:n) {
                idx <- c(idx, k)
                k <- k + 1
            }
            idx <- c(idx, l)
            l <- l + 1
        }
        nt <- nt[idx, ]
        nt
    }
    if (!is.na(x$expected) && x$expected == TRUE) {
        xex <- outer(x$rs, x$cs, '*')
        xex <- xex / x$gt
        if (is.null(digits)) {
            digits <- 1
        }
        xx <- format(round(xex, digits), ...)
        xx <- cbind(rep('Expected N', nr), xx, rep('', nr))
        nt <- rbind(nt, xx)
        idx <- integer()
        for (i in 1:nr) {
            idx <- c(idx, i, i + nr)
        }
        nt <- nt[idx, ]
    }
    if (x$prop.chisq) {
        xx <- (x$CST$expected - x$t) ^ 2 / x$CST$expected
        xx <- format(round(xx, digits), trim = TRUE, ...)
        xx <- cbind('Chi-square', xx)
        nt <- appendlines(nt, xx, haslbl = TRUE)
    }
    if (!is.na(x$prop.row[1])) {
        xx <- cbind(x$prop.row, x$rs / x$gt)
        xx <- format(round(xx * hdd, digits), trim = TRUE, ...)
        xx <- matrix(paste(xx, '%', sep = ''), nrow = nr,
                 ncol = nc + 1)
        xx <- cbind('Row(%)', xx)
        nt <- appendlines(nt, xx, TRUE, haslbl = TRUE)
    }
    if (!is.na(x$prop.col[1])) {
        xx <- format(round(x$prop.col * hdd, digits), trim = TRUE,
                 ...)
        xx <- matrix(paste(xx, '%', sep = ''), nrow = nr,
                 ncol = nc)
        xx <- cbind('Column(%)', xx)
        nt <- appendlines(nt, xx, haslbl = TRUE)
    }
    if (!is.na(x$prop.tbl[1])) {
        xx <- format(round(x$prop.tbl * hdd, digits), trim = TRUE,
                 ...)
        xx <- matrix(paste(xx, '%', sep = ''), nrow = nr,
                 ncol = nc)
        xx <- cbind('Total(%)', xx)
        nt <- appendlines(nt, xx, haslbl = TRUE)
    }
    if (!is.na(x$resid) && x$resid == TRUE && x$expected == TRUE) {
        xx <- x$t - xex
        xx <- format(round(xx, digits), trim = TRUE, ...)
        xx <- cbind('Residual', xx)
        nt <- appendlines(nt, xx, haslbl = TRUE)
    }
    if (!is.na(x$sresid) && x$sresid == TRUE
        && x$expected == TRUE) {
        xx <- x$CST$residual
        xx <- format(round(xx, digits), trim = TRUE, ...)
        xx <- cbind('Std Residual', xx)
        nt <- appendlines(nt, xx, haslbl = TRUE)
    }
    if (!is.na(x$asr[1])) {
        xx <- format(round(x$asr, digits), trim = TRUE, ...)
        xx <- cbind('Adj Std Resid', xx)
        nt <- appendlines(nt, xx, haslbl = TRUE)
    }
    n <- dim(nt)[1] / nr
    idx <- seq(1, dim(nt)[1], n)
    nt <- rbind(nt, c(gettext('Total', domain = 'R-descr'), x$cs,
                    x$gt))
    if (!is.na(x$prop.col[1])) {
        nt <- rbind(nt, c('', sapply(round(hdd * x$cs / x$gt, digits), function(x) paste(x, '%', sep = '')), ''))
    }
    len <- dim(nt)[1]
    rownames(nt) <- as.character(1:len)

    ## merging and print
    ts <- ifelse(!is.na(x$prop.col[1]), 2, 1)
    or <- (nrow(nt) - ts) / nr
    nt.nr <- nrow(nt)
    nc <- ncol(nt)
    if (!is.null(total.r) && !total.r) {
        nc <- nc - 1
    }
    RowData <- x$RowData
    ColData <- x$ColData
    res <- NULL
    for (i in 1:nr) {
        res.r <- paste(pandoc.strong.return(nt[1 + or * (i - 1), 1]),
                       'N',
                       paste(nt[ (2 + or * (i - 1)) : (i * or), 1], collapse = '\\ \n'),
                   sep = '\\ \n')
        for (j in 2:nc) {
            res.r <- cbind(res.r,
                           paste('&nbsp;',
                                 paste(nt[ (1 + or * (i - 1)) : (i * or), j], collapse = '\\  \n'),
                                 sep = '\\ \n'))
        }
        res <- rbind(res, res.r)
    }
    res.r <- NULL
    if (is.null(total.c) || total.c) {
        for (j in 1:nc) {
            res.r <- cbind(res.r, paste(nt[ (nt.nr - ts + 1) : nt.nr, j], collapse = '\\ \n'))
        }
        res <- rbind(res, res.r)
    }
    cln <- c(ifelse(RowData != '', RowData, '&nbsp;'), colnames(x$t))
    if (is.null(total.r) || total.r) {
        cln <- c(cln, 'Total')
    }
    if (ColData != '') {
        cln.t <- paste(c('&nbsp;', ColData, rep('&nbsp;', nc - 2)), rep('\\\n', nc - 1), sep = '')
        cln <- paste(cln.t, cln, sep = '')
    }
    colnames(res) <- cln
    pandoc.table(res, caption = caption, keep.line.breaks = TRUE, ...)
}


#' Pander method for timeseries class
#'
#' Prints a timeseries object in Pandoc's markdown.
#' @param x a timeseries object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
#' @importFrom stats time cycle frequency start end time
pander.ts <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    if (!is.null(ncol(x))) {
        tp.1 <- trunc(time(x))
        tp.2 <- trunc(cycle(x))
        day.abb <- c('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri',
                     'Sat')
        row.names <- switch(frequency(x), tp.1, 'Arg2', 'Arg3',
                            paste(tp.1, c('Q1', 'Q2', 'Q3', 'Q4')[tp.2], sep = ' '),
                            'Arg5', 'Arg6', paste('Wk.', tp.1, ' ', day.abb[tp.2],
                                                  sep = ''), 'Arg8', 'Arg9', 'Arg10', 'Arg11',
                            paste(tp.1, month.abb[tp.2], sep = ' '))
        t <- data.frame(x, row.names = row.names)
    } else {
        col.names <- switch(frequency(x), 'Value', 'Arg2', 'Arg3',
                            c('Q1', 'Q2', 'Q3', 'Q4'), 'Arg5', 'Arg6', day.abb,
                            'Arg8', 'Arg9', 'Arg10', 'Arg11', month.abb)
        row.names <- seq(from = start(x)[1], to = end(x)[1])
        t <- data.frame(matrix(c(rep(NA, start(x)[2] - 1),
                                 x, rep(NA, frequency(x) - end(x)[2])), ncol = frequency(x),
                               byrow = TRUE), row.names = row.names)
        names(t) <- col.names
    }

    pandoc.table(t, caption = caption, ...)

}


#' Pander method for formula class
#'
#' Prints a formula object in Pandoc's markdown.
#' @param x a formula object
#' @param max.width maximum width in characters per line
#' @param caption caption (string) to be shown under the formula
#' @param ... optional parameters passed to raw \code{pandoc.formula} function
#' @export
pander.formula <- function(x, max.width = 80, caption = attr(x, 'caption'), ...) {
    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }
    pandoc.formula(x, max.width = max.width, caption = caption, ...)
}


#' Pander method for call class
#'
#' Prints a call object in Pandoc's markdown.
#' @param x a call object
#' @param ... optional parameters passed to raw \code{pandoc.formula} function
#' @export
pander.call <- function(x, ...)
    pandoc.verbatim(deparse(x))

#' Pander method for name class
#'
#' Prints a call object in Pandoc's markdown.
#' @param x a name language object
#' @param ... ignored parameters
#' @export
pander.name <- function(x, ...)
    pander(deparse(x))

#' Pander method for coxph class
#'
#' Prints a coxph object in Pandoc's markdown.
#' @param x an coxph object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
#' @importFrom stats pchisq naprint
pander.coxph <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- pandoc.formula.return(x$call$formula, text = 'Fitting Proportional Hazards Regression Model:')
        } else {
            caption <- get.caption()
        }
    }

    if (!is.null(x$fail)) {
        cat('  Coxph failed.', x$fail, '\n')
        return(invisible())
    }

    beta <- x$coef
    se <- sqrt(diag(x$var))

    if (is.null(x$naive.var)) {
        c.tab <- cbind(beta, exp(beta), se, beta / se, 1 - pchisq( (beta / se) ^ 2, 1))
        dimnames(c.tab) <- list(names(beta), c('coef', 'exp(coef)', 'se(coef)', 'z', 'p'))
    } else {
        c.tab <- cbind(beta, exp(beta), se, beta / se, signif(1 - pchisq( (beta / se) ^ 2, 1), 1))
        dimnames(c.tab) <- list(names(beta), c('coef', 'exp(coef)', 'robust se', 'z', 'p'))
    }

    pandoc.table(c.tab, caption = caption, ...)
    logtest <- -2 * (x$loglik[1] - x$loglik[2])

    if (is.null(x$df)) {
        df <- sum(!is.na(beta))
    } else {
        df <- round(sum(x$df), 2)
    }

    cat('Likelihood ratio test=', format(round(logtest, 2)), '  on ',
        df, ' df,', ' p=', format(1 - pchisq(logtest, df)), sep = '')

    omit <- x$na.action
    cat('  n=', x$n)
    if (!is.null(x$nevent)) {
        cat(', number of events=', x$nevent, '\n')
    } else {
        cat('\n')
    }

    if (length(omit)) {
        cat('   (', naprint(omit), ')\n', sep = '')
    }

}


#' Pander method for clogit class
#'
#' Prints a clogit object in Pandoc's markdown.
#' @param x an clogit object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.clogit <- function (x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- pandoc.formula.return(x$userCall, text = 'Fitting Conditional logistic regression:')
        } else {
            caption <- get.caption()
        }
    }

    pander.coxph(x, caption = caption, ...)

}


#' Pander method for zoo class
#'
#' Prints a zoo object in Pandoc's markdown.
#' @param x an zoo object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.zoo <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    c.tab <- as.data.frame(x)
    c.tab <- cbind(pandoc.date.return(trunc(time(x)), simplified = TRUE), c.tab)

    if (length(colnames(x)) != 0) {
        colnames(c.tab) <- c('Period', colnames(x))
    } else {
        colnames(c.tab) <- c('Period', 'Value')
    }

    rownames(c.tab) <- NULL
    pandoc.table(c.tab, caption = caption, ...)

}

#' Pander method for summary.lme class
#'
#' Prints a lme object in Pandoc's markdown.
#' @param x a lme object
#' @param caption caption (string) to be shown under the table
#' @param summary (default:\code{TRUE}) if to print expender summary
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.lme <- function(x, caption = attr(x, 'caption'), summary = TRUE, ...) {
    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- sprintf('Linear mixed-effects model fit by %s : %s',
                               paste(sub('^[ ]*', '', ifelse(x$method == 'REML', 'REML', 'maximum likelihood'))),
                               pandoc.formula.return(x$call$fixed), collapse = '')
        } else {
            caption <- get.caption()
        }
    }
    res <- as.data.frame(x$tTable)
    if (summary) {
        pandoc.table(res, caption = pandoc.formula.return(x$call$fixed, text = 'Fixed effects: '),
                     split.tables = Inf, ...)
        pandoc.table(x$residuals, caption = 'Standardized Within-Group Residuals')
        pandoc.table(data.frame(
            'Observations'        = x$dims[['N']],
            'Groups'              = x$dims$ngrps[1:x$dims$Q],
            'Log-restricted-likelihood' = x$logLik,
            check.names = FALSE), keep.trailing.zeros = TRUE, caption = caption, digits = 4)
    } else {
        pandoc.table(res, caption = caption, ...)
    }
}

#' Pander method for lme class
#'
#' Prints a lme object in Pandoc's markdown.
#' @param x a lme object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.lme <- function(x, caption = attr(x, 'caption'), ...)
    pander(summary(x), caption = caption, summary = FALSE, ...)


#' Pander method for describe class
#'
#' Prints a describe object in Pandoc's markdown.
#' @param x an describe object
#' @param caption caption (string) to be shown under the table
#' @param digits number of digits of precision
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.describe <- function(x, caption = attr(x, 'caption'), digits = panderOptions('digits'), ...) {
    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }
    if (length(dim(x)) == 1) {
        class(x) <- 'list'
        attr(x, 'call') <- NULL
        pander(x, digits = digits, caption = caption, ...)
    } else {
        class(x) <- 'data.frame'
        pander(x, digits = digits, caption = caption, ...)
    }
    invisible()
}


#' Pander method for survdiff class
#'
#' Prints an survdiff object in Pandoc's markdown.
#' @param x an survdiff object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.survdiff <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- pandoc.formula.return(x$call$formula, text = 'Call:')
        } else {
            caption <- get.caption()
        }
    }

    if (length(x$n) == 1) {
        z <- sign(x$exp - x$obs) * sqrt(x$chisq)
        temp <- c(x$obs, x$exp, z, signif(1 - pchisq(x$chisq, 1), panderOptions('digits')))
        names(temp) <- c('Observed', 'Expected', 'Z', 'p')
        temp <- t(temp)
    } else {
        if (is.matrix(x$obs)) {
            otmp <- apply(x$obs, 1, sum)
            etmp <- apply(x$exp, 1, sum)
        } else {
            otmp <- x$obs
            etmp <- x$exp
        }
        df <- sum(1 * (etmp > 0)) - 1
        p <- 1 - pchisq(x$chisq, df[!is.na(df)])
        temp <- cbind(x$n, otmp, etmp, (otmp - etmp) ^ 2 / etmp, (otmp - etmp) ^ 2 / diag(x$var))
        dimnames(temp) <- list(names(x$n), c('N', 'Observed', 'Expected', '(O-E)^2/E', '(O-E)^2/V'))
        caption <- paste(caption, sprintf('Chisq = %f \non %d degrees of freedom, p = %f', x$chisq, df, p), sep = ' ')
    }

    temp <- as.data.frame(temp, checknames = FALSE)
    pandoc.table(temp, caption = caption, ...)

}


#' Pander method for survfit class
#'
#' Prints an survfit object in Pandoc's markdown.
#'
#' @param x the result of a call to the survfit function.
#' @param caption caption (string) to be shown under the table
#' @param scale	a numeric value to rescale the survival time, e.g., if the input data to survfit were in days, scale=365 would scale the printout to years.
#' @param print.rmean,rmean options for computation and display of the restricted mean
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
#' @importFrom utils getFromNamespace
pander.survfit <- function (x, caption = attr(x, 'caption'),
                            scale = 1, print.rmean = getOption('survfit.print.rmean'),
                            rmean = getOption('survfit.rmean'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    omit <- x$na.action

    if (length(omit)){
        pander(list(naprint(omit)), list.type = 'none')
    }

    if (!missing(print.rmean) && is.logical(print.rmean) && missing(rmean)) {
        rmean <- ifelse(print.rmean, 'common', 'none')
    }

    if (is.null(rmean)) {
        if (is.logical(print.rmean)) {
            rmean <- ifelse(print.rmean, 'common', 'none')
        } else {
            rmean <- 'none'
        }
    }

    if (is.numeric(rmean)) {
        if (is.null(x$start.time)) {
            if (rmean < min(x$time)) {
                stop('Truncation point for the mean is < smallest survival')
            }
        } else {
            if (rmean < x$start.time) {
                stop('Truncation point for the mean is < smallest survival')
            }
        }
    } else {
        rmean <- match.arg(rmean, c('none', 'common', 'individual'))
        if (length(rmean) == 0) {
            stop('Invalid value for rmean option')
        }
    }

    temp <- getFromNamespace('survmean', 'survival')(x, scale = scale, rmean)
    pandoc.table(temp$matrix, caption = ...)

    if (rmean != 'none') {
        if (rmean == 'individual') {
            pander('* restricted mean with variable upper limit')
        } else {
            pander(paste('* restricted mean with upper limit = ', format(temp$end.time[1])))
        }
    }

}

#' Pander method for smooth.spline class
#'
#' Prints an smooth.spline object in Pandoc's markdown.
#' @param x an smooth.spline object
#' @param ... igroned parameters
#' @export
pander.smooth.spline <- function(x, ...) {

    x <- as.list(capture.output(x)[-1:-3])
    pandoc.list(x, add.end.of.list = FALSE)

}


#' Pander method for rlm class
#'
#' Prints an rlm object in Pandoc's markdown.
#' @param x an rlm object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.rlm <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            if (!is.null(x$call)) {
                caption <- pandoc.formula.return(x$call$formula, text = 'Fitting linear model by robust regression:')
            } else {
                caption <- 'Fitting linear model by robust regression'
            }
        } else {
            caption <- get.caption()
        }
    }

    if (x$converged) {
        cat('Converged in', length(x$conv), 'iterations\n')
    } else {
        cat('Ran', length(x$conv), 'iterations without convergence\n')
    }

    coef <- x$coefficients
    pandoc.table(coef, caption = caption, ...)
    nobs <- length(x$residuals)
    rdf <- nobs - length(coef)
    cat('Degrees of freedom:', nobs, 'total;', rdf, 'residual\n\n')

    if (nzchar(mess <- naprint(x$na.action))) {
        cat('  (', mess, ')\n', sep = '')
    }

    cat('Scale estimate:', format(signif(x$s, 3)), '\n')

}


#' Pander method for stat.table class
#'
#' Prints an stat.table object in Pandoc's markdown.
#' @param x an stat.table object
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.stat.table <- function(x, caption = attr(x, 'caption'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }

    if (length(dim(x)) == 2) {
        pandoc.table(t(x), caption = caption, ...)
        return(invisible())
    }

    if (length(dim(x)) == 3) {

        xx <- list()

        for (i in 1:dim(x)[2]) {
            xx[[i]] <- x[, i, ]
        }

        xx <- do.call(rbind, xx)
        xx <- apply(xx, c(1, 2), format, digits = panderOptions('digits'))
        dn <- dimnames(x)
        lgroup <- list(names(dn)[2], dn[[2]])
        tgroup <- names(dn)[3]
        c.s <- length(dn[[3]])
        xx <- rbind(colnames(xx), xx)
        xx <- cbind(unlist(lgroup), xx)
        xx <- rbind(c(rep('', c.s - 1), tgroup, ''), xx)
        colnames(xx) <- NULL
        pandoc.table(xx, caption = caption, emphasize.rows = c(1, 2), emphasize.cols = 1, ...)
    }

}


#' Pander method for sessionInfo class
#'
#' Prints an sessionInfo object in Pandoc's markdown.
#' @param x an sessionInfo object
#' @param locale (defaut:\code{TRUE}) if to print locale output
#' @param compact (defaut:\code{TRUE}) if output shoud be compact (ommiting extra line breaks and spaces, inline printing of lists)
#' @param ... ignored parameters
#' @export
pander.sessionInfo <- function (x, locale = TRUE, compact = TRUE, ...) {

    mkLabel <- function(L, n) {
        vers <- sapply(L[[n]], function(x) x[['Version']])
        pkg <- sapply(L[[n]], function(x) x[['Package']])
        sprintf('%s(v.%s)', pkg, vers)
    }

    cat(pandoc.strong(x$R.version$version.string), '\n\n', sep = '')
    cat(pandoc.strong('Platform:'), x$platform, '\n\n', sep = ' ')

    if (locale) {
        cat(pandoc.strong('locale:'))
        cat('\n')
        pander(gsub('[/]', '||', strsplit(x$locale, ';', fixed = TRUE)[[1]]), ...)
        cat('\n')
    }

    if (compact) {
        attached.base.packages <- pander_return(x$basePkgs, quote = FALSE, ...)
        other.attached.packages <- pander_return(mkLabel(x, 'otherPkgs'), quote = FALSE, ...)
        load.via.namespaces <- pander_return(mkLabel(x, 'loadedOnly'), quote = FALSE, ...)
    } else {
        attached.base.packages <- pandoc.list.return(x$basePkgs, add.end.of.list = FALSE, ...)
        other.attached.packages <- pandoc.list.return(mkLabel(x, 'otherPkgs'), add.end.of.list = FALSE, ...)
        load.via.namespaces <- pandoc.list.return(mkLabel(x, 'loadedOnly'), add.end.of.list = FALSE, ...)
    }

    cat('\n')
    cat(pandoc.strong('attached base packages:'), '\n')
    cat(attached.base.packages)

    if (!is.null(x$otherPkgs)) {
        cat('\n\n')
        cat(pandoc.strong('other attached packages:'), '\n')
        cat(other.attached.packages)
    }

    if (!is.null(x$loadedOnly)) {
        cat('\n\n')
        cat(pandoc.strong('loaded via a namespace (and not attached):'), '\n')
        cat(load.via.namespaces)
    }

    invisible(x)

}


#' Pander method for microbenchmark class
#'
#' Prints an microbenchmark object in Pandoc's markdown.
#' @param x an microbenchmark object
#' @param caption caption (string) to be shown under the table
#' @param expr.labels expression labels that will replace default ones (similar to rownames, which microbenchmark class table does not have)
#' @param unit units in which values should be printed (for example second, microseconds, etc.). Should be one of ns, us, ms, s, t, hz, khz, mhz, eps, f
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.microbenchmark <- function(x, caption = attr(x, 'caption'), expr.labels, unit, ...) {

    xs <- summary(x, unit = unit)

    if (is.null(caption)) {
        if (is.null(storage$caption)) {
            caption <- paste('Unit: ', attr(xs, 'unit'), sep = '')
        } else {
            caption <- get.caption()
        }
    }

    if (!missing(expr.labels)) {
        xs[, 1] <- as.vector(xs[, 1])
        xs[1:length(expr.labels), 1] <- expr.labels
    }

    pander(xs, caption = caption, ...)

}


#' Pander method for function class
#'
#' Prints an function object in Pandoc's markdown.
#' @param x an function object
#' @param add.name (defaut:\code{FALSE}) if to add function name to output or just to print a body
#' @param verbatim (defaut:\code{TRUE}) if to add tabulation, so pandoc conversion will rander it properly
#' @param syntax.highlighting (defaut:\code{FALSE}) if to add hyghlighting tag for R syntax
#' @param ... ignored parameters
#' @export
pander.function <- function(x, add.name = FALSE, verbatim = TRUE, syntax.highlighting = FALSE, ...) {

    fname <- substitute(x)
    ps <- ''

    if (syntax.highlighting) {
        cat('```r\n')
    } else {
        ps <- ifelse(verbatim, '\t', '')
    }

    if (!is.null(add.name) && add.name) {
        cat(ps, fname, ' <- ', sep = '')
    }

    for (line in deparse(x)) {
        cat(ps, line, '\n', sep = '')
    }

    if (syntax.highlighting) {
        cat('```')
    }

}


#' Pander method for tabular class
#'
#' Renders an tabular object in Pandoc's markdown.
#' @param x an function object
#' @param caption caption (string) to be shown under the table
#' @param digits number of digits of precision
#' @param emphasize.rownames (defaut:\code{TRUE}) if rownames should be highlighted
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.tabular <- function(x, caption = attr(x, 'caption'),
                           emphasize.rownames = TRUE, digits = panderOptions('digits'), ...) {
    if (is.null(caption) & !is.null(storage$caption)) {
        caption <- get.caption()
    }
    data <- as.matrix(x, format = T, rowLabels = F, colLabels = F, digits = digits)
    rlabels <- attr(x, 'rowLabels')
    rlabels[is.na(rlabels)] <- ''
    clabels <- attr(x, 'colLabels')
    clabels[is.na(clabels)] <- ''
    if (!is.null(colnames(rlabels))) {
        ## needed for case of more complex tabular structure (see examples)
        cl <- colnames(rlabels)
        data <- cbind(rlabels, data)
        clabels <- cbind(rbind(matrix('',
                                      nrow = (nrow(clabels) - 1),
                                      ncol = length(cl)),
                               colnames(rlabels)),
                         clabels)
    }
    clabels <- apply(clabels, c(2), paste, collapse = '\\ \n')
    colnames(data) <- clabels
    if (emphasize.rownames) {
        pandoc.table(data, caption = caption, keep.line.breaks = TRUE, emphasize.cols = 1:length(cl), ...)
    } else {
        pandoc.table(data, caption = caption, keep.line.breaks = TRUE, ...)
    }
}

#' Pander method for summary.table class
#'
#' Renders an summary.table object in Pandoc's markdown.
#' @param x an function object
#' @param caption caption (string) to be shown under the table
#' @param print.call (defaut:\code{TRUE}) if call should be printed
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.table <- function(x, caption = attr(x, 'caption'), print.call = T, ...) {
    if (is.null(caption)) {
        if (!is.null(storage$caption)) {
            caption <- get.caption()
        } else {
            caption <- 'Test for independence of all factors'
        }
    }
    if (!is.null(x$call) && print.call) {
        cat('Call: ')
        print(x$call)
    }
    cat('Number of cases in table:', x$n.cases, '\n')
    cat('Number of factors:', x$n.vars, '\n')
    if (x$n.vars > 1) {
        ch <- x$statistic
        tdf <- data.frame('Chisq' = ch, 'df' = x$parameter, 'p-value' = x$p.value)
        pandoc.table(tdf, caption = caption, ...)
        if (!x$approx.ok) {
            cat('Chi-squared approximation may be incorrect\n')
        }
    }
}

#' Pander method for randomForest class
#'
#' Renders an randomForest object in Pandoc's markdown.
#' @param x an randomForest object
#' @param digits number of digits of precision
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.randomForest <- function (x, digits = panderOptions('digits'), ...) {
    cat('\nCall: ', pandoc.formula.return(x$call), '\n')
    cat('Type of random forest: ', x$type, '\n', sep = '')
    cat('Number of trees: ', x$ntree, '\n', sep = '')
    cat('No. of variables tried at each split: ', x$mtry, '\n\n', sep = '')
    if (x$type == 'classification') {
        if (!is.null(x$confusion)) {
            cat('OOB estimate of  error rate: ',
                round(x$err.rate[x$ntree, 'OOB'] * 100, digits = digits), '%\n', sep = '')
            pandoc.table(x$confusion, caption = 'Confusion Matrix', digits = digits, ...)
            if (!is.null(x$test$err.rate)) {
                cat('Test set error rate: ',
                    round(x$test$err.rate[x$ntree, 'Test'] * 100, digits = 2),
                    '%\n',
                    sep = '')
                pandoc.table(x$test$confusion, caption = 'Test Confusion Matrix', digits = digits, ...)
            }
        }
    }
    if (x$type == 'regression') {
        if (!is.null(x$mse)) {
            cat('Mean of squared residuals: ', x$mse[length(x$mse)], '\n', sep = '')
            cat('% Var explained: ', round(100 * x$rsq[length(x$rsq)], digits = 2), '\n', sep = '')
            if (!is.null(x$test$mse)) {
                cat('Test set MSE: ',
                    round(x$test$mse[length(x$test$mse)], digits = digits),
                    '\n', sep = '')
                cat('% Test Var explained: ',
                    round(100 * x$test$rsq[length(x$test$rsq)],
                          digits = 2), '\n', sep = '')
            }
        }
        if (!is.null(x$coefs)) {
            cat('Bias correction applied:\n')
            cat('Intercept: ', x$coefs[1], '\n')
            cat('Slope: ', x$coefs[2], '\n')
        }
    }
}

#' Prints an irts object from tseries package in Pandoc's markdown.
#' @param x an irts object
#' @param caption caption (string) to be shown under the table
#' @param format string passed to format when printing dates (POSIXct or POSIXt)
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.irts <- function(x, caption = attr(x, 'caption'), format = panderOptions('date'), ...) {

    if (is.null(caption) & !is.null(storage$caption)) {
            caption <- get.caption()
    }
    m <- as.matrix(x$value)
    colnames(m) <- NULL
    rownames(m) <- format(x$time, format = format)
    pander(m, caption = caption, ...)
}

#' Prints an summary.manova object from stats package in Pandoc's markdown.
#' @param x an summary.manova object
#' @param add.significance.stars if significance stars should be shown for P value
#' @param caption caption (string) to be shown under the table
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.manova <- function (x, caption = attr(x, 'caption'), add.significance.stars = FALSE, ...) {
    if (length(stats <- x$stats)) {
        pander.anova(stats, caption = caption, add.significance.stars = add.significance.stars, ...)
    } else {
        cat('No error degrees of freedom\n\n')
        pander(data.frame(Df = x$Df, row.names = x$row.names), caption = caption, ...)
    }
    invisible(x)
}

#' Pander method for manova class
#'
#' Prints an manova object in Pandoc's markdown.
#' @param x an manovv object
#' @param caption caption (string) to be shown under the table
#' @param add.significance.stars if significance stars should be shown for P value
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.manova <- function(x, caption = attr(x, 'caption'), add.significance.stars = FALSE, ...)
    pander(summary(x), caption = caption, add.significance.stars = add.significance.stars, ...)

#' Pander method for gtable class
#'
#' Renders an gtable object in Pandoc's markdown.
#' @param x an gtable object
#' @param zsort Sort by z values? Default \code{FALSE}
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.gtable <- function(x, zsort = FALSE, ...) {
    if (nrow(x$layout) == 0) {
        return()
    }
    pos <- as.data.frame(format(as.matrix(x$layout[c('t', 'r',
                                                     'b', 'l')])), stringsAsFactors = FALSE)
    grobNames <- vapply(x$grobs, as.character, character(1))
    info <- data.frame(z = x$layout$z, cells = paste('(', pos$t, '-', pos$b, ',', pos$l, '-', pos$r, ')', sep = ''),
                       name = x$layout$name,
                       grob = grobNames)
    if (zsort) {
        info <- info[order(x$layout$z), ]
    }
    pander(info)
}

#' Prints an summary.nls object from stats package in Pandoc's markdown.
#' @param x an summary.nls object
#' @param summary (defaut:\code{TRUE}) if used for summary.lm or lm
#' @param add.significance.stars if significance stars should be shown for P value
#' @param digits number of digits of precision
#' @param show.convergence (defaut:\code{FALSE}) if to print convergence info
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.nls <- function(x, summary = TRUE, add.significance.stars = FALSE,
                               digits = panderOptions('digits'), show.convergence = FALSE, ...) {
    cat('\n')
    pandoc.formula(x$call$formula,
                   text = 'Fitting nonlinear regression model:',
                   add.line.breaks = TRUE)

    if (summary) {
        coef <- x$coefficients
        if (add.significance.stars) {
            coef[, 4] <- pander::add.significance.stars(coef[, 4])
        }
        pander(x$coefficients, caption = 'Parameters', ...)
        if (add.significance.stars) {
            cat('\nSignif. codes:  0 \'***\' 0.001 \'**\' 0.01 \'*\' 0.05 \'.\' 0.1 \' \' 1\n')
        }
        if (!is.null(x$correlation) && NCOL(x$correlation) > 1) {
            pander(x$correlation, caption = 'Correlation of Parameter Estimates', ...)
        }
        cat('\nResidual standard error:',
            format(signif(x$sigma, digits)), 'on', x$df[2L], 'degrees of freedom\n')
    } else {
        pandoc.table(x$coefficients[, 1], caption = 'Parameter Estimates', digits = digits, ...)
        cat('\nresidual sum-of-squares:', format(signif(x$sigma, digits)), '\n')
    }

    if (show.convergence && !is.null(x$convInfo)) {
        if (identical(x$call$algorithm, 'port')) {
            cat('\nAlgorithm \'port\', convergence message: ',
                x$convInfo$stopMessage, '\n', sep = '')
        } else {
            cat('\nNumber of iterations', ifelse(x$convInfo$isConv, 'to convergence:', 'till stop:'),
                x$convInfo$finIter, '\nAchieved convergence tolerance:',
                format(x$convInfo$finTol, digits = digits), '\n')
            if (!x$convInfo$isConv) {
                cat('Reason stopped:', x$convInfo$stopMessage, '\n')
            }
        }
    }
}

#' Prints an nls object from stats package in Pandoc's markdown.
#' @param x an nls object
#' @param show.convergence (defaut:\code{FALSE}) if to print convergence info
#' @param digits number of digits of precision
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.nls <- function(x, digits = panderOptions('digits'), show.convergence = FALSE, ...) {
    pander.summary.nls(summary(x), summary = FALSE, add.significance.stars = FALSE,
                       digits = digits, show.convergence = show.convergence, ...)
}

#' Prints an arima object from stats package in Pandoc's markdown.
#' @param x an arima object
#' @param digits number of digits of precision
#' @param se if to include standard error in coefficients table (default \code{TRUE})
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.Arima <- function(x, digits = panderOptions('digits'), se = TRUE, ...) {
    cat('\nCall:', pandoc.formula.return(x$call), '', sep = '\n')
    cn <- names(x$coef)
    coef <- matrix(x$coef, nrow = 1)
    colnames(coef) <- cn
    if (se && nrow(x$var.coef)) {
        ses <- rep_len(0, length(coef))
        ses[x$mask] <- sqrt(diag(x$var.coef))
        coef <- rbind(coef, s.e. = ses)
    }
    pandoc.table(coef, caption = 'Coefficients', digits = digits, ...)
    cm <- x$call$method
    if (is.null(cm) || cm != 'CSS')
        cat('\nsigma^2 estimated as ', format(x$sigma2, digits = digits),
            ':  log likelihood = ', format(round(x$loglik, 2)),
            ',  aic = ', format(round(x$aic, 2)), '\n', sep = '')
    else cat('\nsigma^2 estimated as ', format(x$sigma2, digits = digits),
             ':  part log likelihood = ', format(round(x$loglik, 2)),
             '\n', sep = '')
    invisible(x)
}

#' Prints an polr object from MASS package in Pandoc's markdown.
#' @param x an polr object
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.polr <- function (x, ...) {
    if (!is.null(cl <- x$call)) {
        cat('\nCall:', pandoc.formula.return(cl), '', sep = '\n')
    }
    if (length(coef(x))) {
        pandoc.table(coef(x), caption = 'Coefficients', ...)
    } else {
        cat('\nNo coefficients\n')
    }
    pandoc.table(x$zeta, caption = 'Intercepts', ...)
    cat('\nResidual Deviance:', format(x$deviance, nsmall = 2L), '\n')
    cat('AIC:', format(x$deviance + 2 * x$edf, nsmall = 2L), '\n')
    if (nzchar(mess <- naprint(x$na.action))) {
        cat('(', mess, ')\n', sep = '')
    }
    if (x$convergence > 0) {
        cat('Warning: did not converge as iteration limit reached\n')
    }
    invisible()
}

#' Prints an ols object from rms package in Pandoc's markdown.
#' @param x an ols object
#' @param long if to print the correlation matrix of parameter estimates. default(\code{FALSE})
#' @param coefs if to the table of model coefficients, standard errors, etc. default(\code{TRUE})
#' @param digits passed to \code{format}. Can be a vector specifying values for each column (has to be the same length as number of columns).
#' @param round passed to \code{round}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
#' @importFrom stats quantile
pander.ols <- function (x, long = FALSE, coefs = TRUE,
                        digits = panderOptions('digits'), round = panderOptions('round'), ...) {
    requireNamespace('rms', quietly = TRUE)
    stats <- x$stats
    pen <- length(x$penalty.matrix) > 0
    resid <- x$residuals
    n <- length(resid)
    p <- length(x$coef) - (names(x$coef)[1] == 'Intercept')
    if (length(stats) == 0) {
        cat('n=', n, '   p=', p, '\n\n', sep = '')
    }
    ndf <- stats['d.f.']
    df <- c(ndf, n - ndf - 1, ndf)
    r2 <- stats['R2']
    sigma <- stats['Sigma']
    rdf <- df[2]
    rsqa <- 1 - (1 - r2) * (n - 1) / rdf
    lrchisq <- stats['Model L.R.']
    ci <- x$clusterInfo
    if (lst <- length(stats)) {
        misc <- rms::reListclean(Obs = stats['n'], sigma = sigma, d.f. = df[2], `Cluster on` = ci$name, Clusters = ci$n) #nolint
        lr <- rms::reListclean(`LR chi2` = lrchisq, d.f. = ndf, `Pr(> chi2)` = 1 - pchisq(lrchisq, ndf)) #nolint
        disc <- rms::reListclean(R2 = r2, `R2 adj` = rsqa, g = stats['g']) #nolint
        sdf <- multitable(list(misc, lr, disc))
        colnames(sdf) <- c('', 'Model Likelihood\nRatio Test', 'Discrimination\nIndexes')
        caption <- pandoc.formula.return(x$call$formula, text = 'Fitting linear model:')
        pandoc.table(sdf, keep.line.breaks = TRUE, caption = caption, ...)
    }
    if (rdf > 5) {
        if (length(dim(resid)) == 2) {
            rq <- apply(t(resid), 1, quantile)
            dimnames(rq) <- list(c('Min', '1Q', 'Median', '3Q',
                                   'Max'), dimnames(resid)[[2]])
        } else {
            rq <- quantile(resid)
            names(rq) <- c('Min', '1Q', 'Median', '3Q', 'Max')
        }
        pandoc.table(rq, caption = 'Residuals', ...)
    }
    else if (rdf > 0) {
        pandoc.table(resid, caption = 'Residuals', ...)
    }
    if (nsingular <- df[3] - df[1]) {
        cat(nsingular, 'coefficients not defined because of singularities', '\n')
    }
    se <- sqrt(diag(x$var))
    if (coefs) {
        obj <- list(coef = x$coefficients, se = se, errordf = rdf)
        U <- coef_mat(obj, coefs = coefs)
        pandoc.table(U, caption = 'Coeficients', ...)
    }
    if (!pen) {
        if (long && p > 0) {
            correl <- diag(1 / se) %*% x$var %*% diag(1 / se)
            dimnames(correl) <- dimnames(x$var)
            ll <- lower.tri(correl)
            correl[ll] <- format(round(correl[ll], digits = round), digits = digits, ...)
            correl[!ll] <- ''
            pandoc.table(correl[-1, - (p + 1), drop = FALSE],
                         caption = 'Correlation of Coefficients',
                         digits = digits,
                         round = round, ...)
        }
    }
    invisible()
}

#' Prints an summary.polr object from MASS package in Pandoc's markdown.
#' @param x an summary.polr object
#' @param digits number of digits of precision passed to format
#' @param round number of rounding digits passed to round
#' @param keep.trailing.zeros to show or remove trailing zeros in numbers on a column basis width
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.polr <- function(x, digits = panderOptions('digits'), round = panderOptions('round'),
                                keep.trailing.zeros = panderOptions('keep.trailing.zeros'), ...) {
    if (!is.null(cl <- x$call)) {
        cat('\nCall:', pandoc.formula.return(cl), '', sep = '\n')
    }
    pc <- x$pc
    if (pc > 0) {
        pander(x$coefficients[seq_len(pc), , drop = FALSE], caption = 'Coeficients', #nolint
               digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros, ...)
    } else {
        cat('\nNo coefficients\n')
    }
    pander(x$coefficients[ (pc + 1L):nrow(x$coefficients), , drop = FALSE], caption = 'Intercepts', #nolint
          digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros, ...)
    cat('\nResidual Deviance:', format(x$deviance, nsmall = 2L), '\n\n')
    cat('AIC:', format(x$deviance + 2 * x$edf, nsmall = 2L), '\n\n')
    if (nzchar(mess <- naprint(x$na.action))) {
        cat('(', mess, ')\n\n', sep = '')
    }
    if (!is.null(correl <- x$correlation)) {
        ll <- lower.tri(correl)
        correl <- apply(correl, c(1, 2),
                        p, wrap = '', digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros)
        correl[!ll] <- ''
        pander(correl[-1L, -ncol(correl)],
               digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros,
               caption = 'Correlation of Coefficients', ...)
    }
    invisible(x)
}

#' Prints an survreg object from survival package in Pandoc's markdown.
#' @param x an survreg object
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.survreg <- function (x, ...) {
        pander(summary(x), summary = FALSE, ...)
}

#' Prints an survreg object from survival package in Pandoc's markdown.
#' @param x an survreg object
#' @param summary if summary should be printed
#' @param digits number of digits of precision passed to format
#' @param round number of rounding digits passed to round
#' @param keep.trailing.zeros to show or remove trailing zeros in numbers on a column basis width
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.survreg <- function(x, summary = TRUE, digits = panderOptions('digits'),
                                   round = panderOptions('round'),
                                   keep.trailing.zeros = panderOptions('keep.trailing.zeros'), ...) {
    if (!is.null(cl <- x$call)) {
        cat('\nCall:', pandoc.formula.return(cl), '', sep = '\n\n')
    }
    if (!is.null(x$fail)) {
        cat(' Survreg failed.', x$fail, '\n\n')
        return(invisible())
    }
    if (summary) {
        pandoc.table(x$table, caption = 'Model statistics',
                     digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros, ...)
    } else {
        coef <- x$coef
        if (any(nas <- is.na(coef))) {
            if (is.null(names(coef))) {
                names(coef) <- paste('b', 1:length(coef), sep = '')
            }
            cat('\nCoefficients: (', sum(nas), ' not defined because of singularities)\n',
                sep = '')
        }
        pandoc.table(coef, caption = 'Coefficients',
                     digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros, ...)

    }
    if (nrow(x$var) == length(coef)) {
        cat('\nScale fixed at', format(x$scale), '\n')
    } else if (length(x$scale) == 1) {
        cat('\nScale=', format(x$scale), '\n')
    } else {
        pandoc.table(x$scale, caption = 'Scale', ...)
    }
    nobs <- length(x$linear)
    chi <- 2 * diff(x$loglik)
    df <- sum(x$df) - x$idf
    pandoc.table(data.frame('Loglik(model)' = x$loglik[2], 'Loglik(intercept only)' = x$loglik[1]), ...)
    if (df > 0) {
        cat('Chisq=', p(chi, wrap = ''), 'on', p(df, wrap = ''),
            'degrees of freedom, p=', p(signif(1 - pchisq(chi, df), 2), wrap = ''), '\n\n')
    } else {
        cat('\n')
    }
    if (summary) {
        if (x$robust) {
            cat('(Loglikelihood assumes independent observations)\n\n')
        }
        cat('Number of Newton-Raphson Iterations:', p(trunc(x$iter), wrap =), '\n\n') #nolint
    }
    omit <- x$na.action
    if (length(omit)) {
        cat('n=', nobs, ' (', naprint(omit), ')\n', sep = '')
    } else {
        cat('n=', nobs, '\n')
    }
    if (summary) {
        if (!is.null(correl <- x$correlation)) {
            p <- dim(correl)[2]
            if (p > 1) {
                ll <- lower.tri(correl)
                correl <- apply(correl, c(1, 2),
                                p, wrap = '', digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros)
                correl[!ll] <- ''
                pander(correl[-1L, -ncol(correl)],
                       digits = digits, round = round, keep.trailing.zeros = keep.trailing.zeros,
                       caption = 'Correlation of Coefficients', ...)
            }
        }
    }
    invisible()
}

#' Prints an lrm object from rms package in Pandoc's markdown.
#' @param x an lrm object
#' @param coefs if to the table of model coefficients, standard errors, etc. default(\code{TRUE})
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.lrm <- function (x, coefs = TRUE, ...)  {
    requireNamespace('rms', quietly = TRUE)
    ns <- x$non.slopes
    nstrata <- x$nstrata
    if (!length(nstrata)) {
        nstrata <- 1
    }
    pm <- x$penalty.matrix
    penaltyFactor <- NULL
    if (length(pm)) {
        psc <- ifelse(length(pm) == 1, sqrt(pm), sqrt(diag(pm)))
        penalty.scale <- c(rep(0, ns), psc)
        cof <- matrix(x$coef[ - (1:ns)], ncol = 1)
        pandoc.table(as.data.frame(x$penalty, row.names = ''), caption = 'Penalty factors', ...)
        penaltyFactor <- as.vector(t(cof) %*% pm %*% cof)
    }
    est.exp <- 1:ns
    if (length(x$est)) {
        est.exp <- c(est.exp, ns + x$est[x$est + ns <= length(x$coefficients)])
    }
    vv <- diag(x$var)
    cof <- x$coef
    stats <- x$stats
    maxd <- signif(stats['Max Deriv'], 1)
    ci <- x$clusterInfo
    misc <- rms::reListclean(Obs = stats['Obs'],
                     `Sum of weights` = stats['Sum of Weights'],
                     Strata = if (nstrata > 1) nstrata,
                     `Cluster on` = ci$name, Clusters = ci$n,
                     `max |deriv|` = maxd)
    if (length(x$freq) < 4) {
        names(x$freq) <- paste(' ', names(x$freq), sep = '')
        misc <- c(misc[1], x$freq, misc[-1])
    }
    lr <- rms::reListclean(`LR chi2` = stats['Model L.R.'],
                   d.f. = stats['d.f.'],
                   `Pr(> chi2)` = stats['P'],
                   Penalty = penaltyFactor)
    disc <- rms::reListclean(R2 = stats['R2'], g = stats['g'], gr = stats['gr'],
                     gp = stats['gp'], Brier = stats['Brier'])
    discr <- rms::reListclean(C = stats['C'], Dxy = stats['Dxy'], gamma = stats['Gamma'],
                      `tau-a` = stats['Tau-a'])
    sdf <- multitable(list(misc, lr, disc, discr))
    colnames(sdf) <- c('', 'Model Likelihood\nRatio Test',
                       'Discrimination\nIndexes', 'Rank Discrim.\nIndexes')
    caption <- pandoc.formula.return(x$call$formula, text = 'Fitting logistic regression model:')
    pandoc.table(sdf, keep.line.breaks = TRUE, caption, ...)
    if (coefs) {
        obj <- list(coef = cof, se = sqrt(vv), aux = if (length(pm)) penalty.scale, auxname = 'Penalty Scale')
        U <- coef_mat(obj, coefs = coefs)
        pandoc.table(U, caption = 'Coeficients', ...)
    }
    invisible()
}

#' Prints an orm object from rms package in Pandoc's markdown.
#' @param x an orm object
#' @param coefs if to the table of model coefficients, standard errors, etc. default(\code{TRUE})
#' @param intercepts if to print intercepts, by default, intercepts are only printed if there are fewer than 10 of them
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.orm <- function (x, coefs = TRUE, intercepts = x$non.slopes < 10, ...) {
    requireNamespace('rms', quietly = TRUE)
    ns <- x$non.slopes
    cik <- attr(x$coef, 'intercepts')
    if (length(cik) && intercepts) {
        warning('intercepts=TRUE not implemented for fit.mult.impute objects')
        intercepts <- FALSE
    }
    pm <- x$penalty.matrix
    penaltyFactor <- NULL
    if (length(pm)) {
        psc <- ifelse(length(pm) == 1, sqrt(pm), sqrt(diag(pm)))
        penalty.scale <- c(rep(0, ns), psc)
        cof <- matrix(x$coef[ - (1:ns)], ncol = 1)
        pandoc.table(as.data.frame(x$penalty, row.names = ''), caption = 'Penalty factors', ...)
        penaltyFactor <- as.vector(t(cof) %*% pm %*% cof)
    }
    vv <- diag(vcov(x, intercepts = ifelse(intercepts, 'all', 'none')))
    if (!intercepts) {
        if (!length(cik)) {
            nints <- ns
        } else if (length(cik) == 1 && cik == 0) {
            nints <- 0
        } else {
            length(cik)
        }
        ints.to.delete <- vector()
        if (ns != 0 && nints != 0) {
            ints.to.delete <- 1:nints
        }
        vv <- c(rep(NA, nints), vv)
    }
    cof <- x$coef
    stats <- x$stats
    maxd <- signif(stats['Max Deriv'], 1)
    ci <- x$clusterInfo
    misc <- rms::reListclean(Obs = stats['Obs'], `Unique Y` = stats['Unique Y'],
                     `Cluster on` = ci$name, Clusters = ci$n, `Median Y` = stats['Median Y'],
                     `max |deriv|` = maxd)
    if (length(x$freq) < 4) {
        names(x$freq) <- paste(' ', names(x$freq), sep = '')
        misc <- c(misc[1], x$freq, misc[-1])
    }
    lr <- rms::reListclean(`LR chi2` = stats['Model L.R.'], #nolint
                   d.f. = stats['d.f.'],
                   `Pr(> chi2)` = stats['P'],
                   `Score chi2` = stats['Score'],
                   `Pr(> chi2)` = stats['Score P'], Penalty = penaltyFactor)
    disc <- rms::reListclean(R2 = stats['R2'], g = stats['g'], gr = stats['gr'], #nolint
                     `|Pr(Y>=median)-0.5|` = stats['pdm'])
    discr <- rms::reListclean(rho = stats['rho']) #nolint
    sdf <- multitable(list(misc, lr, disc, discr))
    colnames(sdf) <- c('', 'Model Likelihood\nRatio Test',
                       'Discrimination\nIndexes', 'Rank Discrim.\nIndexes')
    caption <- switch(x$family, logistic = 'Logistic (Proportional Odds)',
                      probit = 'Probit', cauchit = 'Cauchy', loglog = '-log-log',
                      cloglog = 'Complementary log-log')
    caption <- paste(caption, 'Ordinal Regression Model')
    caption <- pandoc.formula.return(x$call$formula, text = caption)
    pandoc.table(sdf, keep.line.breaks = TRUE, caption, ...)
    if (coefs) {
        if (!intercepts) {
            j <- -ints.to.delete
            cof <- cof[j]
            vv <- vv[j]
            if (length(pm)) {
                penalty.scale <- penalty.scale[j]
            }
        }
        obj <- list(coef = cof, se = sqrt(vv), aux = if (length(pm)) penalty.scale, auxname = 'Penalty Scale')
        U <- coef_mat(obj, coefs = coefs)
        pandoc.table(U, caption = 'Coeficients', ...)
    }
    invisible()
}

#' Prints an Grm object from rms package in Pandoc's markdown.
#' @param x an Grm object
#' @param coefs if to the table of model coefficients, standard errors, etc. default(\code{TRUE})
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
#' @importFrom stats coef pchisq vcov
#' @rdname pander.glm.rms
pander.Glm <- function (x, coefs = TRUE, ...) {
    requireNamespace('rms', quietly = TRUE)
    cof <- coef(x)
    lr <- x$null.deviance - x$deviance
    dof <- x$rank - (names(cof)[1] == 'Intercept')
    pval <- 1 - pchisq(lr, dof)
    ci <- x$clusterInfo
    misc <- rms::reListclean(Obs = length(x$residuals), `Residual d.f.` = x$df.residual,
                     `Cluster on` = ci$name, Clusters = ci$n, g = x$g)
    lr <- rms::reListclean(`LR chi2` = lr, d.f. = dof, `Pr(> chi2)` = pval) #nolint
    sdf <- multitable(list(misc, lr))
    colnames(sdf) <- c('', 'Model Likelihood\nRatio Test')
    caption <- pandoc.formula.return(x$call$formula, text = 'General Linear Model')
    pandoc.table(sdf, keep.line.breaks = TRUE, caption, ...)
    if (coefs) {
        se <- sqrt(diag(vcov(x)))
        obj <- list(coef = cof, se = se)
        U <- coef_mat(obj, coefs = coefs)
        pandoc.table(U, caption = 'Coeficients', ...)
    }
    invisible()
}

#' Prints an cph object from rms package in Pandoc's markdown.
#' @param x an cph object
#' @param table if to print event frequency statistics. default(\code{TRUE})
#' @param conf.int set to e.g. .95 to print 0.95 confidence intervals on simple hazard ratios (which are usually meaningless as one-unit changes are seldom relevant and most models contain multiple terms per predictor)
#' @param coefs if to the table of model coefficients, standard errors, etc. default(\code{TRUE})
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
#' @importFrom stats qnorm
pander.cph <- function (x, table = TRUE, conf.int = FALSE, coefs = TRUE, ...) {
    requireNamespace('rms', quietly = TRUE)
    if (table && length(x$n) && is.matrix(x$n)) {
       pandoc.table(x$n, caption = 'Status', ...)
    }
    if (length(x$coef)) {
        stats <- x$stats
        ci <- x$clusterInfo
        misc <- rms::reListclean(Obs = stats['Obs'], Events = stats['Events'],
                         `Cluster on` = ci$name, Clusters = ci$n,
                         Center = x$center)
        lr <- rms::reListclean(`LR chi2` = stats['Model L.R.'], d.f. = stats['d.f.'],
                       `Pr(> chi2)` = stats['P'], `Score chi2` = stats['Score'],
                       `Pr(> chi2)` = stats['Score P'])
        disc <- rms::reListclean(R2 = stats['R2'], Dxy = stats['Dxy'],
                         g = stats['g'], gr = stats['gr'])
        sdf <- multitable(list(misc, lr, disc))
        colnames(sdf) <- c('', 'Model Likelihood\nRatio Test',
                           'Discrimination\nIndexes')
        caption <- pandoc.formula.return(x$call$formula, text = 'Cox Proportional Hazards Model')
        pandoc.table(sdf, keep.line.breaks = TRUE, caption, ...)
        beta <- x$coef
        se <- sqrt(diag(x$var))
        if (coefs) {
            se <- sqrt(diag(x$var))
            obj <- list(coef = x$coef, se = se)
            U <- coef_mat(obj, coefs = coefs)
            pandoc.table(U, caption = 'Coeficients', ...)
        }
        if (conf.int) {
            zcrit <- qnorm( (1 + conf.int) / 2 )
            tmp <- cbind(exp(beta), exp(-beta),
                         exp(beta - zcrit * se),
                         exp(beta + zcrit * se))
            dimnames(tmp) <- list(names(beta),
                                  c('exp(coef)', 'exp(-coef)',
                                    paste('lower ', p(conf.int, wrap = ''), sep = ''),
                                    paste('upper ', p(conf.int, wrap = ''), sep = '')))
            pandoc.table(tmp, caption = 'Confidence interval', ...)
        }
    }
    invisible()
}

#' Prints an summary.rms from rms package in Pandoc's markdown.
#' @param x an summary.rms object
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.summary.rms <- function (x, ...) {
    cstats <- dimnames(x)[[1]]
    for (i in 1:7) cstats <- cbind(cstats, signif(x[, i], 5))
    dimnames(cstats) <- list(rep('', nrow(cstats)), c('Factor', dimnames(x)[[2]][1:7]))
    pandoc.table(cstats, ...)
    if ( (A <- attr(x, 'adjust')) != '')
        cat('\nAdjusted to:', A, '\n')
    blab <- switch(attr(x, 'conf.type'),
                   `bootstrap nonparametric percentile` = 'Bootstrap nonparametric percentile confidence intervals',
                   `bootstrap BCa` = 'Bootstrap BCa confidence intervals',
                   `basic bootstrap` = 'Basic bootstrap confidence intervals',
                   '')
    if (blab != '') {
        cat('\n', blab, '\n', sep = '')
    }
    cat('\n')
    invisible()
}

#' Prints an ets object from forecast package in Pandoc's markdown.
#' @param x an ets object
#' @param digits number of digits of precision
#' @param ... optional parameters passed to raw \code{pandoc.table} function
#' @export
pander.ets <- function(x, digits = panderOptions('digits'), ...) {
    cat('\nCall:', pandoc.formula.return(x$call), '', sep = '\n')
    cat('Type of ets: ', x$method, '\n', sep = '')
    lambda <- x$lambda
    isn <- names(x$initstate)
    initstate <- matrix(x$initstate, nrow = 1)
    colnames(initstate) <- isn
    sp <- x$par['alpha']
    if (x$components[2] != 'N'){
        sp <- c(sp, x$par['beta'])
    }
    if (x$components[3] != 'N'){
        sp <- c(sp, x$par['gamma'])
    }
    if (x$components[4] != 'FALSE'){
        sp <- c(sp, x$par['phi'])
    }
    if (!is.null(lambda)){
        cat('\nBox-Cox transformation: lambda =', round(lambda, panderOptions('digits')), '\n')
    }
    pandoc.table(sp, caption = 'Smoothing parameters', digits = digits, ...)
    pandoc.table(initstate, caption = 'Initial states', digits = digits, ...)
    cat('\nsigma^2 estimated as', format(x$sigma2, digits = digits))
    if (!is.null(x$loglik)){
        cat(': log likelihood = ', format(round(x$loglik, 2)))
    }
    if (!is.null(x$aic)){
        cat(', aic = ', format(round(x$aic, 2)), '\n', sep = '')
    }
    invisible(x)
}

#' Reporting with Pandoc
#'
#' This \code{R5} reference class can hold bunch of elements (text or R objects) from which it tries to create a Pandoc's markdown text file. Exporting the report to several formats (like: PDF, docx, odt etc. - see Pandoc's documentation) is also possible, see examples below.
#' @param ... this is an R5 object without any direct params but it should be documented, right?
#' @export Pandoc
#' @examples \dontrun{
#' ## Initialize a new Pandoc object
#' myReport <- Pandoc$new()
#'
#' ## Add author, title and date of document
#' myReport$author <- 'Anonymous'
#' myReport$title  <- 'Demo'
#'
#' ## Or it could be done while initializing
#' myReport <- Pandoc$new('Anonymous', 'Demo')
#'
#' ## Add some free text
#' myReport$add.paragraph('Hello there, this is a really short tutorial!')
#'
#' ## Add maybe a header for later stuff
#' myReport$add.paragraph('# Showing some raw R objects below')
#'
#' ## Adding a short matrix
#' myReport$add(matrix(5,5,5))
#'
#' ## Or a table with even # TODO: caption
#' myReport$add.paragraph('Hello table:')
#' myReport$add(table(mtcars$am, mtcars$gear))
#'
#' ## Or a "large" data frame which barely fits on a page
#' myReport$add(mtcars)
#'
#' ## And a simple linear model with Anova tables
#' ml <- with(lm(mpg ~ hp + wt), data = mtcars)
#' myReport$add(ml)
#' myReport$add(anova(ml))
#' myReport$add(aov(ml))
#'
#' ## And do some principal component analysis at last
#' myReport$add(prcomp(USArrests))
#'
#' ## Sorry, I did not show how Pandoc deals with plots:
#' myReport$add(plot(1:10)) # TODO: caption
#'
#' ## Want to see the report? Just print it:
#' myReport
#'
#' ## Exporting to PDF (default)
#' myReport$export()
#'
#' ## Or to docx in tempdir:
#' myReport$format <- 'docx'
#' myReport$export(tempfile())
#'
#' ## You do not want to see the generated report after generation?
#' myReport$export(open = FALSE)
#' }
#' @importFrom methods setRefClass new getGeneric
Pandoc <- setRefClass('Pandoc', fields = list('author' = 'character',
                                              'title' = 'character',
                                              'date' = 'character',
                                              'body' = 'list',
                                              'format' = 'character',
                                              'proc.time' = 'numeric'))

Pandoc$methods(initialize = function(author = 'Anonymous',
                                     title = base::sprintf('%s\'s report', author),
                                     date = base::date(),
                                     format = 'pdf', ...) {

    .self$author    <- author
    .self$title     <- title
    .self$date      <- date
    .self$format    <- format
    .self$proc.time <- 0
    callSuper(...)

})

Pandoc$methods(add = function(x) {

    timer           <- proc.time()
    res             <- evals(deparse(match.call()[[2]]),
                             env = parent.frame(),
                             graph.name = evalsOptions('graph.name'),
                             graph.dir = evalsOptions('graph.dir'),
                             graph.output = evalsOptions('graph.output'),
                             width = evalsOptions('width'),
                             height = evalsOptions('height'))
    .self$body      <- c(.self$body, res)
    .self$proc.time <- .self$proc.time + as.numeric(proc.time() - timer)[3]

})

Pandoc$methods(add.paragraph = function(x) .self$body <- c(.self$body, list(list(result = pandoc.p.return(x)))))

Pandoc$methods(show = function(x) {

    ## starting timer for proc.time
    timer  <- proc.time()

    ## show header
    cat(pandoc.header(.self$title, style = 'setext'))
    cat(' written by *', .self$author, '* at *', .self$date, '*\n', sep = '')
    cat('\n', sprintf(' This report holds %s block(s).', length(.self$body)), '\n')

    ## show body (if not empty)
    if (length(.self$body) > 0) {

        cat(pandoc.horizontal.rule())

        lapply(.self$body, function(x) {
            for (m in c('messages', 'warnings', 'errors')) {
                if (!is.null(x$msg[[m]])) {
                    cat('\n**', toupper(sub('s$', '', m)), ':** ', x$msg[[m]], sep = '')
                }
            }
            pander(x$result)
        })

        ## show proc.time
        cat(pandoc.horizontal.rule())
        cat('\nProc. time: ', .self$proc.time + as.numeric(proc.time() - timer)[3], 'seconds. \n\n')

    } else {
        cat('\n)')
    }

})

Pandoc$methods(export = function(f, ...) {

    if (missing(f)) {
        f <- tempfile('pander-', getwd())
    }
    fp    <- sprintf('%s.md', f)
    timer <- proc.time()

    ## create pandoc file
    cat(pandoc.title.return(.self$title, .self$author, .self$date), file = fp)
    lapply(.self$body,
           function(x) cat(paste(capture.output(pander(x$result)), collapse = '\n'), file = fp, append = TRUE))

    ## convert
    fe <- Pandoc.convert(fp, format = .self$format, proc.time = as.numeric(proc.time() - timer)[3], ...)

    ## return
    cat('\nExported to *', f, '.[md|', format, ']* under ',
        .self$proc.time + as.numeric(proc.time() - timer)[3], ' seconds.\n\n', sep = '')

    return(invisible(fe))

})

#' @keywords internal
ess.pander.evals <- function(x, show.src = TRUE) {
    require(pander, quietly = T)
    wd <- getwd()
    setwd(tempdir())
    cat('\n')
    res <- evals(x)
    for (x in res) {
        if (show.src) {
            cat('\n```r\n', paste(x$src, collapse = '\n'), '\n```\n\n', sep = '')
        }
        cat(pander(x), sep = '\n')
    }
    setwd(wd)
    cat('\n')
}

#' @keywords internal
ess.evals <- function(x) {
    require(pander, quietly = T)
    wd <- getwd()
    setwd(tempdir())
    cat('\n')
    evals(x)
    setwd(wd)
    cat('\n')
}

#' Open file
#'
#' Tries to open a file with operating system's default program.
#' @param f file (with full path)
#' @references This function is a fork of David Hajage's \code{convert} function: \url{https://github.com/eusebe/ascii/blob/master/R/export.r}
#' @export
openFileInOS <- function(f) {

    if (missing(f)) {
        stop('No file to open!')
    }

    f <- path.expand(f)

    if (!file.exists(f)) {
        stop('File not found!')
    }

    if (grepl('w|W', .Platform$OS.type)) {
        ## we are on Windows
        shell.exec(f) #nolint
    } else {
        if (grepl('darwin', version$os)) {
            ## Mac
            system(paste(shQuote('open'), shQuote(f)), wait = FALSE, ignore.stderr = TRUE)
        } else {
            ## Linux
            system(paste(shQuote('/usr/bin/xdg-open'), shQuote(f)), #nolint
                   wait = FALSE,
                   ignore.stdout = TRUE)
        }
    }

}

#' Converts Pandoc to other format
#'
#' Calling John MacFarlane's great program to convert specified file (see \code{f} parameter below) or character vector {see \code{text} paramater} to other formats like \code{HTML}, \code{pdf}, \code{docx}, \code{odt} etc.
#' @param f Pandoc's markdown format file path. If URL is provided then the generated file's path is \code{tempfile()} but please bear in mind that this way only images with absolute path would shown up in the document.
#' @param text Pandoc's markdown format character vector. Treated as the content of \code{f} file - so the \code{f} parameter is ignored. The generated file's path is \code{tempfile()}.
#' @param format required output format. For all possible values here check out Pandoc homepage: \url{http://johnmacfarlane.net/pandoc/}
#' @param open try to open converted document with operating system's default program
#' @param options optionally passed arguments to Pandoc (instead of \code{pander}'s default)
#' @param footer add footer to document with meta-information
#' @param proc.time optionally passed number in seconds which would be shown in the generated document's footer
#' @param portable.html instead of using local files, rather linking JS/CSS files to an online CDN for portability and including base64-encoded images if converting to \code{HTML} without custom \code{options}
#' @param pandoc.binary path to \code{pandoc}'s binary if not found in the path
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @note This function depends on \code{Pandoc} which should be pre-installed on user's machine. See the \code{INSTALL} file of the package.
#' @return Converted file's path.
#' @importFrom tools file_path_sans_ext
#' @importFrom utils packageDescription
#' @export
#' @examples \dontrun{
#' Pandoc.convert(text = c('# Demo', 'with a paragraph'))
#' Pandoc.convert('http://rapporter.github.io/pander/minimal.md')
#' # Note: the generated HTML is not showing images with relative path from the above file.
#' # Based on that `pdf`, `docx` etc. formats would not work! If you want to convert an
#' # online markdown file to other formats with this function, please pre-process the file
#' # to have absolute paths instead.
#' }
Pandoc.convert <- function(f, text, format = 'html', open = TRUE, options = '',
                           footer = FALSE, proc.time, portable.html = TRUE,
                           pandoc.binary = panderOptions('pandoc.binary')) {

    ## check for Pandoc
    if (pandoc.binary == '') {
        if (grepl('w|W', .Platform$OS.type))
            message('You may install Pandoc easily with "install.pandoc()" from the "installr" package.')
        stop("It seems Pandoc is not installed or path of binary is not found. Did you restarted R after Pandoc install? See installation details by running:\n\n\t readLines(system.file('includes/html/footer.html', package='pander'))\n") #nolint
    }
    if (!file.exists(pandoc.binary)) {
        stop(paste('Pandoc binary is not found at provided location:', pandoc.binary))
    }

    ## dealing with provided character vector
    if (!missing(text)) {
        f <- tempfile()
        cat(text, file = f, sep = '\n')
    }

    ## content
    rl <- readLines(f, warn = FALSE)

    ## dealing with URLs
    if (grepl('^https*://.*', f)) {
        f.dir <- tempdir()
        f.out <- paste0(tempfile(), '.', format)
    } else {
        f.dir <- dirname(f)
        f.out <- paste0(file_path_sans_ext(f), '.', format)
    }

    ## force UTF-8 encoding #nolint
    ## if (!grepl('utf', Sys.getlocale())) { #nolint

        ## convert content to UTF-8
        text <- iconv(readLines(f, warn = FALSE), from = '', to = 'UTF-8')

        ## do not touch original input file
        if (!missing(f)) {
            f <- tempfile()
            ## remove tempfile if not needed any more
            on.exit(unlink(f))
        }

        ## write content with UTF-8 encoding
        con <- file(f, 'w', encoding = 'UTF-8')
        cat(text, file = con, sep = '\n')
        close(con)

    ## } #nolint

    ## add nifty HTML/CSS/JS components
    if (format == 'html') {

        if (options == '') {

            options <- sprintf('-A "%s"', system.file('includes/html/footer.html', package = 'pander'))

            if (portable.html) {
                options <- paste('--self-contain', options)
            }

        }

    } else {

        if (options == '' && any(grepl('^#', rl))) {
            options <- '--toc'
        }

    }

    ## add other formats' templates
    ## TODO

    ## add footer to file
    if (footer) {
        if (length(rl) > 0 && !grepl('This report was generated', tail(rl, 1))) {
            cat(sprintf('\n\n-------\nThis report was generated with [R](http://www.r-project.org/) (%s) and [pander](https://github.com/rapporter/pander) (%s)%son %s platform.', sprintf('%s.%s', R.version$major, R.version$minor), packageDescription('pander')$Version, ifelse(missing(proc.time), ' ', sprintf(' in %s sec ', format(proc.time))), R.version$platform), file = f, append = TRUE) #nolint
        }
    }

    ## set specified dir
    wd <- getwd()
    setwd(f.dir)

    ## call Pandoc
    cmd <- sprintf('%s -f markdown -s %s %s -o %s', pandoc.binary, options, shQuote(f), shQuote(f.out))
    if (grepl('w|W', .Platform$OS.type)) {
        res <- suppressWarnings(tryCatch(shell(cmd, intern = TRUE), error = function(e) e)) #nolint
    } else {
        res <- suppressWarnings(tryCatch(system(cmd, intern = TRUE), error = function(e) e))
    }

    ## inject HTML header
    if (format == 'html'
        && grepl(sprintf('^(--self-contain )*-A "%s"$', system.file('includes/html/footer.html', package='pander')), options)) { #nolint

        rl <- readLines(f.out, warn = FALSE)
        he <- grep('</head>', rl)
        ho <- readLines(system.file('includes/html/header.html', package = 'pander'), warn = FALSE)

        if (portable.html) {
            ch <- ho
        } else {
            ch <- gsub('http://cdn.rapporter.net/pander', system.file('includes/', package = 'pander'), ho)
        }

        writeLines(c(rl[1 : (he - 1)], ch, rl[he:length(rl)]), f.out)

    }

    ## revert settings
    setwd(wd)

    ## return
    if (!is.null(attr(res, 'status'))) {
        warning(paste0('Pandoc had some problems while converting to ',
                       format, ': \n\n', paste(res, collapse = '\n'),
                       '\n\nPandoc was called as:\n\n\t', cmd))
    } else {
        if (open) {
            openFileInOS(f.out)
        }
        return(f.out)
    }

}

library(testthat)
library(pander)
context('Pandoc.convert')

test_that('openFileInOS works correctly', {
    tf <- paste(tempfile(), '.txt', sep = '')
    sink(file = tf)
    cat('Some info')
    sink()
#    expect_equal(openFileInOS(tf), 0)
    expect_error(openFileInOS('NEWS2'))
    unlink(tf)
})

test_that('Pandoc.convert works correctly', {
    tf <- tempfile()
    cat('% test\n', file = tf)
    sink(tf, append = TRUE)
    pander(mtcars[1:3, 1:4])
    sink()
    res1 <- Pandoc.convert(tf, open = FALSE)
    expect_equal(dirname(tf), dirname(res1))
    res2 <- Pandoc.convert(text = readLines(tf), open = FALSE)
    expect_equal(readLines(res1), readLines(res2)) # text or file gives same result
    unlink(tf)
})

#' Brew in pandoc format
#'
#' This function behaves just like \code{brew} except for the \code{<\%=...\%>} tags, where \code{Pandoc.brew} first translate the R object found between the tags to Pandoc's markdown before passing to the \code{cat} function.
#'
#' This parser tries to be smart in some ways:
#'
#' \itemize{
#'      \item a block (R commands between the tags) could return any value at any part of the block and there are no restrictions about the number of returned R objects
#'      \item plots and images are grabbed in the document, rendered to a png file and \code{pander} method would result in a Pandoc's markdown formatted image link (so the image would be shown/included in the exported document). The images are put in \code{plots} directory in current \code{getwd()} or to the specified \code{output} file's directory.
#'      \item all warnings/messages and errors are recorded in the blocks and returned in the document as a footnote
#' }
#'
#' Please see my Github page for details (\url{http://rapporter.github.com/pander/#brew-to-pandoc}) and examples (\url{http://rapporter.github.com/pander/#examples}).
#' @param file file path of the brew template. As this is passed to \code{readLines}, \code{file} could be an URL too, but not over SSL (for that latter \code{RCurl} would be needed).
#' @param output (optional) file path of the output file
#' @param convert string: format of required output document (besides Pandoc's markdown). Pandoc is called if set via \code{Pandoc.convert} and the converted document could be also opened automatically (see below).
#' @param open try to open converted document with operating system's default program
#' @param graph.name character string (default to \code{\%t} when \code{output} is set to \code{stdout} and \code{paste0(basename(output), '-\%n')} otherwise) passed to \code{\link{evals}}.  Besides \code{\link{evals}}'s possible tags \code{\%i} is also available which would be replaced by the chunk number (and optionally an integer which would handle nested \code{brew} calls) and \code{\%I} with the order of the current expression.
#' @param graph.dir character string (default to \code{tempdir()} when \code{output} is set to \code{stdout} and \code{dirname(graph.name)} otherwise) passed to \code{\link{evals}}
#' @param graph.hi.res render high resolution images of plots? Default is \code{FALSE} except for HTML output.
#' @param text character vector (treated as the content of the \code{file}
#' @param envir environment where to \code{brew} the template
#' @param append should append or rather overwrite (default) the \code{output} markdown text file? Please note that this option only affects the markdown file and not the optionally created other formats.
#' @param ... additional parameters passed to \code{\link{Pandoc.convert}}
#' @note Only one of the input parameters (\code{file} or \code{text}) is to be used at once!
#' @export
#' @importFrom utils tail head capture.output
#' @return converted file name with full path if \code{convert} is set, none otherwise
#' @references \itemize{
#'      \item Jeffrey Horner (2011). _brew: Templating Framework for Report Generation._ \url{https://cran.r-project.org/package=brew}
#'      \item John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' }
#' @examples \dontrun{
#' text <- paste('# Header', '',
#'   'What a lovely list:\n<%=as.list(runif(10))%>',
#'   'A wide table:\n<%=mtcars[1:3, ]%>',
#'   'And a nice chart:\n\n<%=plot(1:10)%>', sep = '\n')
#' Pandoc.brew(text = text)
#' Pandoc.brew(text = text, output = tempfile(), convert = 'html')
#' Pandoc.brew(text = text, output = tempfile(), convert = 'pdf')
#'
#' ## pi is awesome
#' Pandoc.brew(text='<%for (i in 1:5) {%>\n Pi has a lot (<%=i%>) of power: <%=pi^i%><%}%>')
#'
#' ## package bundled examples
#' Pandoc.brew(system.file('examples/minimal.brew', package='pander'))
#' Pandoc.brew(system.file('examples/minimal.brew', package='pander'),
#'   output = tempfile(), convert = 'html')
#' Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'))
#' Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'),
#'   output = tempfile(), convert = 'html')
#'
#' ## brew returning R objects
#' str(Pandoc.brew(text='Pi equals to <%=pi%>.
#' And here are some random data:\n<%=runif(10)%>'))
#'
#' str(Pandoc.brew(text='# Header <%=1%>\nPi is <%=pi%> which is smaller then <%=2%>.
#' foo\nbar\n <%=3%>\n<%=mtcars[1:2,]%>'))
#'
#' str(Pandoc.brew(text='<%for (i in 1:5) {%>
#' Pi has a lot (<%=i%>) of power: <%=pi^i%><%}%>'))
#' }
Pandoc.brew <- function(file = stdin(), output = stdout(), convert = FALSE, open = TRUE,
                        graph.name, graph.dir, graph.hi.res = FALSE, text = NULL,
                        envir = parent.frame(), append = FALSE, ...) {

    timer <- proc.time()
    output.stdout <- deparse(substitute(output)) == 'stdout()'

    if (identical(convert, FALSE)) {
        open <- FALSE
    } else {
        if (output.stdout) {
            stop('A file name should be provided while converting a document.')
        }
    }

    if (!output.stdout) {
        basedir    <- dirname(output)
        if (missing(graph.name)) {
            graph.name <- paste0(basename(output), '-%n')
        }
        if (missing(graph.dir)) {
            graph.dir  <- file.path(basedir, 'plots')
        }
    } else {
        if (missing(graph.name)) {
            graph.name <- '%t'
        }
        if (missing(graph.dir)) {
            graph.dir  <- file.path(tempdir(), 'plots')
        }
    }

    if (is.null(text)) {
        text <- paste(readLines(file, warn = FALSE), collapse = '\n')
    }

    ## id of chunk
    assign('cmdID', 0, envir = debug)
    assign('chunkID', 0, envir = debug)
    assign('nested', debug$nested + 1, envir = debug)
    assign('nestedID', debug$nestedID + 1, envir = debug)

    ## helper fn
    showCode <- function(..., envir = parent.frame(), cache = evalsOptions('cache')) {

        ## increment chunk ID
        assign('chunkID', debug$chunkID + 1, envir = debug)

        ## evaluate chunk
        res <- evals(unlist(...), env = envir, graph.dir = graph.dir, graph.name = graph.name, hi.res = graph.hi.res)

        ## format 'em
        for (r in res) {

            r.pander <- tryCatch(pander_return(r), error = function(e) e)
            if (inherits(r.pander, 'error')) {
                r.pander <- paste0('Internal `pander` error: `',
                                   r.pander$message,
                                   '` while running: `',
                                   r$src,
                                   '`\n\nPlease [report the issue](https://github.com/Rapporter/pander/issues/new) with a reproducible example to help developers fix this ASAP.') #nolint
            }
            r$output <- r.pander
            cat(paste(r.pander, collapse = '\n'))

            localstorage <- get('.storage', envir = envir)
            localstorage.last <- tail(localstorage, 1)[[1]]
            localstorage.last.text <- ifelse(is.null(localstorage.last$text$eval), '', localstorage.last$text$eval)

            if ('image' %in% r$type
                || length(r.pander) > 1
                || grepl('\n$', localstorage.last.text)
                || is.null(localstorage.last$text$eval)) {
                type <- 'block'
            } else {
                type <- 'inline'
            }

            if (type == 'inline') {

                localstorage[[length(localstorage)]]$text <- list(raw = paste0(localstorage.last$text$raw, paste0('<%=', r$src, '%>')), #nolint
                                                                  eval = paste0(localstorage.last$text$eval, r.pander)) #nolint
                localstorage[[length(localstorage)]]$chunks <- list(raw = c(localstorage.last$chunks$raw, paste0('<%=', r$src, '%>')), #nolint
                                                                    eval = c(localstorage.last$chunks$eval, ifelse(length(r.pander) == 0, '', r.pander))) #nolint
                localstorage[[length(localstorage)]]$msg <- list(messages = c(localstorage.last$msg$messages, r$msg$messages), #nolint
                                                                 warnings = c(localstorage.last$msg$warnings, r$msg$warnings), #nolint
                                                                 errors = c(localstorage.last$msg$errors, r$msg$errors)) #nolint

            } else {
                localstorage <- c(localstorage, list(list(type = 'block', robject = r)))
            }

            assign('.storage', localstorage, envir = envir)

        }
    }
    assign('showCode', showCode, envir = envir)

    assign('.storage', NULL, envir = envir)
    res <- capture.output(brew(text = text, envir = envir))

    ## remove absolute path from image links
    if (!output.stdout) {
        res <- gsub(sprintf(']\\(%s/', basedir), ']\\(', res, fixed = TRUE)
    }

    cat(remove.extra.newlines(paste(res, collapse = '\n')), '\n', file = output, append = append)

    if (is.character(convert)) {
        Pandoc.convert(output, format = convert, open = open, proc.time = as.numeric(proc.time() - timer)[3], ...)
    }

    ## there is no sense of chunkID outside of brew
    assign('chunkID', NULL, envir = debug)
    assign('cmdID', NULL, envir = debug)
    assign('nested', debug$nested - 1, envir = debug)
    if (debug$nested == 0) {
        assign('nestedID', 0, envir = debug)
    }

    invisible(get('.storage', envir = envir))

}


######################################################################################
# This is a forked/patched version of `brew` package made by Jeffrey Horner (c) 2007.
# Original sources can be found at: https://cran.r-project.org/package=brew
######################################################################################

BRTEXT <- 1
BRCODE <- 2
BRCOMMENT <- 3
BRCATCODE <- 4
DELIM <- list()
DELIM[[BRTEXT]] <- c('', '')
DELIM[[BRCODE]] <- c('<%', '%>')
DELIM[[BRCOMMENT]] <- c('<%#', '%>')
DELIM[[BRCATCODE]] <- c('<%=', '%>')

#' Patched brew
#'
#' This is a forked/patched version of `brew` package made by Jeffrey Horner (c) 2007. See: \code{References} about the original version.
#'
#' This custom function can do more and also less compared to the original \code{brew} package. First of all the internal caching mechanism (and other, from \code{pander} package POV needless features) of `brew` is removed for some extra profits:
#' \itemize{
#'      \item multiple R expressions can be passed between \code{<\%= ... \%>} tags,
#'      \item the text of the file and also the evaluated R objects are (invisibly) returned in a structured list, which can be really useful while post-processing the results of `brew`.
#' }
#' @param text character vector
#' @param envir environment
#' @return \code{brew}ed document to \code{stdout} and raw results while evaluating the \code{text} in a structured list.
#' @note This function should be never called directly (use \code{brew::brew} instead) as being a helper function of \code{Pandoc.brew}.
#' @seealso \code{\link{Pandoc.brew}}
#' @references Jeffrey Horner (2011). _brew: Templating Framework for Report Generation._ \url{https://cran.r-project.org/package=brew}
#' @keywords internal
`brew` <- function(text = NULL, envir = parent.frame()) {

    if (is.character(text) && nchar(text[1]) > 0) {
        icon <- textConnection(text[1])
    } else {
        stop('Invalid input.')
    }

    if (!is.environment(envir)) {
        stop('Invalid environment')
    }

    state <- BRTEXT
    text <- code <- character(0)
    textLen <- codeLen <- as.integer(0)
    textStart <- as.integer(1)
    line <- ''

    while (TRUE) {
        if (!nchar(line)){
            line <- readLines(icon, 1)
            if (length(line) != 1){
                break
            }
            line <- paste(line, '\n', sep = '')
        }
        if (state == BRTEXT){

            spl <- strsplit(line, DELIM[[BRCODE]], fixed = TRUE)[[1]]

            ## Beginning markup found
            if (length(spl) > 1){

                if (nchar(spl[1])) {
                    text[textLen + 1] <- spl[1]
                    textLen <- textLen + 1
                }
                line <- paste(spl[-1], collapse = '<%')

                ## We know we've found this so far, so go ahead and set up state.
                state <- BRCODE

                ## Now let's search for additional markup.
                if (regexpr('^=', spl[2]) > 0){
                    state <- BRCATCODE
                    line <- sub('^=', '', line)
                } else if (regexpr('^#', spl[2]) > 0){
                    state <- BRCOMMENT
                }

                if (textStart <= textLen) {
                    code[codeLen + 1] <- paste('showText(', textStart, ',', textLen, ')', sep = '')
                    codeLen <- codeLen + 1
                    textStart <- textLen + 1
                }
            } else {
                text[textLen + 1] <- line
                textLen <- textLen + 1
                line <- ''
            }
        } else {
            if (regexpr('%>', line, perl = TRUE) > 0){
                spl <- strsplit(line, '%>', fixed = TRUE)[[1]]
                line <- paste(spl[-1], collapse = '%>')

                n <- nchar(spl[1])
                ## test  for '-' immediately preceding %> will strip trailing newline from line
                if (n > 0) {
                    if (substr(spl[1], n, n) == '-') {
                        line <- substr(line, 1, nchar(line) - 1)
                        spl[1] <- substr(spl[1], 1, n - 1)
                    }
                    text[textLen + 1] <- spl[1]
                    textLen <- textLen + 1
                }

                ## We've found the end of a brew section, but we only care if the
                ## section is a BRCODE or BRCATCODE. We just implicitly drop BRCOMMENT sections
                if (state == BRCODE){
                    code[codeLen + 1] <- paste(text[textStart:textLen], collapse = '')
                    codeLen <- codeLen + 1
                } else if (state == BRCATCODE){
                    code[codeLen + 1] <- paste0('showCode(',
                                                deparse(paste(text[textStart:textLen], collapse = '\n')), ')')
                    codeLen <- codeLen + 1
                }
                textStart <- textLen + 1
                state <- BRTEXT
            } else if (regexpr('<%', line, perl = TRUE) > 0){
                stop('Oops! Someone forgot to close a tag. We saw: ',
                     DELIM[[state]][1], ' and we need ',
                     DELIM[[state]][2])
            } else {
                text[textLen + 1] <- line
                textLen <- textLen + 1
                line <- ''
            }
        }
    }
    if (state == BRTEXT){
        if (textStart <= textLen) {
            code[codeLen + 1] <- paste('showText(', textStart, ',', textLen, ')', sep = '')
            codeLen <- codeLen + 1
            textStart <- textLen + 1
        }
    } else {
        stop('Oops! Someone forgot to close a tag. We saw: ',
             DELIM[[state]][1], ' and we need ',
             DELIM[[state]][2], call. = FALSE)
    }

    showText <- function(from, to) {

        localtexts <- text[from:to]
        for (localtext in localtexts) {

            cat(localtext)

            if (grepl('^#+[ \t]+', localtext)) {
                heading.level <- nchar(gsub('^(#{1,6})[ \t]+.*', '\\1', localtext))
                localtext <- gsub('^#{1,6}[ \t]+', '', localtext)
                type <- 'heading'
            } else {
                type <- 'text'
            }

            localstorage <- get('.storage', envir = envir)
            localstorage.last <- tail(localstorage, 1)[[1]]
            localstorage.last.text <- localstorage.last$text$eval
            localstorage.last.type <- ifelse(is.null(localstorage.last$type), '', localstorage.last$type)

            if (localstorage.last.type == 'block' & type == 'text' & localtext != '\n') {
                localstorage.last.pander  <- localstorage.last$robject$output

                ## we had an inline chunk in the beginning of the line converted to block
                if (!'image' %in% localstorage.last$robject$type && length(localstorage.last.pander) <= 1) {
                    localstorage <- c(localstorage[-length(localstorage)],
                                      list(list(type = 'text',
                                           text = list(
                                               raw  = paste0('<%=', localstorage.last$robject$src,  '%>', localtext),
                                               eval = paste0(localstorage.last.pander, localtext)),
                                           chunks = list(
                                               raw = paste0('<%=', localstorage.last$robject$src,  '%>'),
                                               eval = localstorage.last.pander
                                               ),
                                           msg = list(
                                               messages = localstorage.last$robject$msg$messages,
                                               warnings = localstorage.last$robject$msg$warnings,
                                               errors   = localstorage.last$robject$msg$errors
                                               ))))
                } else {
                ## leave that block as is and add localtext as new
                    localstorage <- c(localstorage,
                                      list(list(type = type,
                                                text = list(raw = localtext, eval = localtext),
                                                chunks = list(raw = NULL, eval = NULL),
                                                msg = list(messages = NULL, warnings = NULL, errors = NULL))))
                }

            } else {

                ## text continues
                if (is.character(localstorage.last.text) &&
                    (type == 'text') &&
                    ifelse(localstorage.last.type == 'heading', !grepl('\n', localstorage.last.text), TRUE)) {
                    localstorage[[length(localstorage)]]$text <- list(raw = paste0(localstorage.last$text$raw, localtext), eval = paste0(localstorage.last.text, localtext)) #nolint
                } else {
                ## new text starts here
                    localstorage <- c(localstorage,
                                      list(list(type = type,
                                                text = list(raw = localtext, eval = localtext),
                                                chunks = list(raw = NULL, eval = NULL),
                                                msg = list(messages = NULL, warnings = NULL, errors = NULL))))
                }
            }

            if (type == 'heading') {
                localstorage[[length(localstorage)]]$level <- heading.level
            }

            assign('.storage', localstorage, envir = envir)

        }

    }

    assign('showText', showText, envir = envir)

    e <- tryCatch(eval(parse(text = code), envir = envir), error = function(e) e)

    if (inherits(e, 'error')) {

        msg <- e$message
        assign('last', list(code = code, text = text, error = msg), envir = debug) # debug

        brcodes <- code[!grepl('^show', code)]
        if (length(brcodes) > 0) {
            brcodes <- p(brcodes, wrap = '`')
            if (grepl('[Uu]nexpected', msg)) {
                stop(paste0('`',
                            sub('.*([Uu]nexpected [a-zA-Z0-9\\(\\)\'\\{\\} ]*)( at character|\n).*', '\\1', msg),
                            '` in your BRCODEs: ', brcodes), call. = FALSE)
            } else {
                stop(sprintf('Error (`%s`) in your BRCODEs: %s', msg, brcodes), call. = FALSE)
            }
        } else {

            stop(paste0('Error: ', p(msg, wrap = '`')), call. = FALSE)
        }
    } else {
        assign('last', list(code = code, text = text, result = e), envir = debug) # debug
    }

    invisible()

}

library(testthat)
library(pander)
context('Pandoc.brew')

test_that('expected result', {
    expect_equal(capture.output(Pandoc.brew(text = '## foo')), '## foo ')
    expect_equal(capture.output(Pandoc.brew(text = '## <%=1%>')), '## _1_ ')
    expect_equal(capture.output(Pandoc.brew(text = '<%=1%>')), '_1_ ')
    expect_equal(capture.output(Pandoc.brew(text = '<%for (i in 1:3)%><%=i%>')), '_1__2__3_ ')
})

output <- '/tmp/dev.null' #nolint

test_that('correct number of list elements', {
    expect_equal(length(Pandoc.brew(text = '## foo', output = output)), 1)
    expect_equal(length(Pandoc.brew(text = '## foo <%=mtcars[1:2,]%>', output = output)), 3)
    expect_equal(length(Pandoc.brew(text = '## foo <%=mtcars[1:2,]%> bar <%=plot(1:10)%>', output = output)), 5)
    expect_equal(length(Pandoc.brew(text = '## foo <%=1%> bar <%=pi%>', output = output)), 1)
    expect_equal(length(Pandoc.brew(text = '<%for (i in 1:5){%>## foo<%}%>', output = output)), 5)
    expect_equal(length(Pandoc.brew(text = '# FOO <%=pi%> bar \n# sad', output = output)), 2)
    expect_equal(length(Pandoc.brew(text = '# FOO\n## bar<%=pi%> bar \n# sad', output = output)), 3)
    expect_equal(length(Pandoc.brew(text = '# FOO ## bar<%=pi%> barn# sad', output = output)), 1)
})

getChunkTypes <- function(x)
    sapply(x, function(x) x$type)

test_that('correct returned list element types', {
    expect_equal(getChunkTypes(Pandoc.brew(text = 'foo <%=1:10%>', output = output)), 'text')
    expect_equal(getChunkTypes(Pandoc.brew(text = '## foo <%=1:10%>', output = output)), 'heading')
    expect_equal(getChunkTypes(Pandoc.brew(text = 'foo\n<%=1:10%>', output = output)), c('text', 'block', 'text'))
    expect_equal(getChunkTypes(Pandoc.brew(text = '## foo\n<%=1:10%>', output = output)), c('heading', 'block', 'text'))
    expect_equal(getChunkTypes(Pandoc.brew(text = 'foo <%=list(1:10)%>', output = output)), c('text', 'block', 'text'))
    expect_equal(getChunkTypes(Pandoc.brew(text = '## foo <%=mtcars%>', output = output)),
                 c('heading', 'block', 'text'))
    expect_equal(getChunkTypes(Pandoc.brew(text = 'foo has <%=1%> apple and <%=3%> pears', output = output)), 'text')
    expect_equal(getChunkTypes(Pandoc.brew(text = 'foo has \nsome neat apples. Exactly <%=5%>', output = output)),
                  'text')
    expect_equal(getChunkTypes(Pandoc.brew(text = '<%=5%> apple', output = output)), 'text')
    expect_equal(getChunkTypes(Pandoc.brew(text = '## header\n<%=5%> apple', output = output)), c('heading', 'text'))
    expect_equal(getChunkTypes(Pandoc.brew(text = '<%=plot(1:10)%> ', output = output)), c('block', 'text'))

})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

splitLine_cpp <- function(str, max_width, use_hyphening, hyphen) {
    .Call('pander_splitLine_cpp', PACKAGE = 'pander', str, max_width, use_hyphening, hyphen)
}

tableExpand_cpp <- function(cells, colsWidth, justify, sepCols, style) {
    .Call('pander_tableExpand_cpp', PACKAGE = 'pander', cells, colsWidth, justify, sepCols, style)
}

#' Indent text
#'
#' Indent all (optionally concatenated) lines of provided text with given level.
#' @param x character vector
#' @param level integer
#' @export
#' @examples
#' pandoc.indent('FOO', 1)
#' pandoc.indent(pandoc.table.return(table(mtcars$gear)), 2)
#' cat(pandoc.indent(pandoc.table.return(table(mtcars$gear)), 3))
pandoc.indent <- function(x, level = 0) {

    if (!is.character(x)) {
        stop('Only character strings are allowed.')
    }
    indent <- repChar(' ', level * 4)
    res <- paste0(indent, gsub('\n', paste0('\n', indent), x))

    ## remove wasted space
    res <- gsub(' *\n', '\n', res)
    res <- sub(' *$', '', res)

    res

}


#' Paragraphs
#'
#' Pandoc's markdown paragraph.
#' @param x character vector
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.p
#' @seealso \code{\link{pandoc.emphasis}} \code{\link{pandoc.strikeout}} \code{\link{pandoc.verbatim}}
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.p('FOO')
#' pandoc.p(c('Lorem', 'ipsum', 'lorem ipsum'))
pandoc.p.return <- function(x)
    add.blank.lines(add.blank.lines(paste(x, collapse = '\n')))

#' @export
pandoc.p <- function(...)
    cat(pandoc.p.return(...))


#' @keywords internal
pandoc.add.formatting <- function(x, f) {

    attributes(x) <- NULL
    if (!is.vector(x)) {
        stop('Sorry, vectors only!')
    }

    ## escape chars
    f.e  <- gsub('*', '\\*', f, fixed = TRUE)

    ## remove trailing or leading spaces
    x    <- trim.spaces(x)

    ## do not stack formatting chars
    w    <- which(!grepl(sprintf('^%s.*%s$', f.e, f.e), x) & x != '')

    ## add an extra space if the string starts with a formatting char
    x[w] <- sapply(x[w],
                   function(x) ifelse(grepl(paste0('^', f.e), x), paste0('\\ ', x), x),
                   USE.NAMES = FALSE)

    ## add formatting chars
    x[w] <- paste0(f, x[w], f)

    return(x)

}


#' Strong emphasis
#'
#' Pandoc's markdown strong emphasis format (e.g. \code{**FOO**}) is added to character string.
#' @param x character vector
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.strong
#' @seealso \code{\link{pandoc.emphasis}} \code{\link{pandoc.strikeout}} \code{\link{pandoc.verbatim}}
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.strong('FOO')
#' pandoc.strong(c('FOO', '**FOO**'))
#' pandoc.strong.return('FOO')
pandoc.strong.return <- function(x)
    pandoc.add.formatting(x, '**')

#' @export
pandoc.strong <- function(...)
    cat(pandoc.strong.return(...))


#' Emphasis
#'
#' Pandoc's markdown emphasis format (e.g. \code{*FOO*}) is added to character string.
#' @param x character vector
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.emphasis
#' @seealso \code{\link{pandoc.strong}} \code{\link{pandoc.strikeout}} \code{\link{pandoc.verbatim}}
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.emphasis('FOO')
#' pandoc.emphasis(c('FOO', '*FOO*'))
#' pandoc.emphasis.return('FOO')
pandoc.emphasis.return <- function(x)
    pandoc.add.formatting(x, '*')

#' @export
pandoc.emphasis <- function(...)
    cat(pandoc.emphasis.return(...))


#' Add strikeout
#'
#' Pandoc's markdown strikeout format (e.g. \code{~~FOO~~}) is added to character string.
#' @param x character vector
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.strikeout
#' @seealso \code{\link{pandoc.emphasis}} \code{\link{pandoc.strong}} \code{\link{pandoc.verbatim}}
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.strikeout('FOO')
#' pandoc.strikeout(c('FOO', '~~FOO~~'))
#' pandoc.strikeout.return('FOO')
pandoc.strikeout.return <- function(x)
    pandoc.add.formatting(x, '~~') #nolint

#' @export
pandoc.strikeout <- function(...)
    cat(pandoc.strikeout.return(...))


#' Add verbatim
#'
#' Pandoc's markdown verbatim format (e.g. \code{`FOO`}) is added to character string.
#' @param x character vector
#' @param style show code \code{inline} or in a separate (\code{indent}ed or \code{delim}ited) block
#' @param attrs (optionally) pass ID, classes and any attribute to the \code{delimited} block
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.verbatim
#' @seealso \code{\link{pandoc.emphasis}} \code{\link{pandoc.strikeout}} \code{\link{pandoc.strong}}
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' # different styles/formats
#' pandoc.verbatim('FOO')
#'
#' src <- c('FOO', 'indent', 'BAR' )
#' pandoc.verbatim(src)
#' pandoc.verbatim.return(src)
#' pandoc.verbatim(c('FOOO\nBAR  ', ' I do R'), 'indent')
#' pandoc.verbatim(c('FOOO\nBAR  ', ' I do R'), 'delim')
#'
#' # add highlighting and HTML/LaTeX ID and classes (even custom attribute)
#' pandoc.verbatim(c('cat("FOO")', 'mean(bar)'), 'delim', '.R #MyCode custom_var="10"')
pandoc.verbatim.return <- function(x, style = c('inline', 'indent', 'delim'), attrs = '') {

    style <- match.arg(style)
    if (style != 'delim' & !missing(attrs)) {
        warning('Providing attrs is only meaningful with delimited blocks.')
    }

    switch(style,
           'inline' = paste0('`', trim.spaces(paste(x, collapse = ' ')), '`'),
           'indent' = sprintf('\n%s\n',
                              paste(paste0(repChar(' ', 4), unlist(strsplit(trim.spaces(x), '\n')), collapse = '\n'))),
           'delim'  = paste0('\n', repChar('`', 7), ifelse(attrs == '', '', sprintf('{%s}', attrs)), '\n', paste(trim.spaces(x), collapse = '\n'), '\n', repChar('`', 7), '\n') #nolint
           )

}

#' @export
pandoc.verbatim <- function(...)
    cat(pandoc.verbatim.return(...))


#' Create pandoc link
#' Pandoc's markdown format link.
#' @param url hyperlink
#' @param text link text
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.link
#' @examples
#' pandoc.link('http://r-project.org')
#' pandoc.link('http://r-project.org', 'R')
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
pandoc.link.return <- function(url, text = url)
    sprintf('[%s](%s)', text, url)

#' @export
pandoc.link <- function(...)
    cat(pandoc.link.return(...))


#' Create pandoc image tags
#'
#' Creates a Pandoc's markdown format image hyperlink.
#' @param img image path
#' @param caption text
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.image
#' @seealso \code{\link{set.caption}}
#' @note The \code{caption} text is read from an internal buffer which defaults to \code{NULL}. To update that, call \code{link{set.caption}} before.
#' @examples
#' pandoc.image('foo.png')
#' pandoc.image('foo.png', 'Nice image, huh?')
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
pandoc.image.return <- function(img, caption = storage$caption) {

    check_caption(caption)

    if (is.null(caption)) {
        caption <- ''
    }

    ## truncating caption buffer if needed
    if (!is.null(storage$caption)) {
        storage$caption <- NULL
    }

    sprintf('![%s](%s)', caption, img)
}

#' @export
pandoc.image <- function(...)
    cat(pandoc.image.return(...))


#' Footnote
#'
#' Creates a Pandoc's markdown format footnote.
#' @param x character vector
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.footnote
#' @examples
#' pandoc.footnote('Automatically numbered footnote, right?')
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
pandoc.footnote.return <- function(x)
    gsub('[\n][\n]*', '\n', sprintf('^[%s]', paste(x, collapse = ' ')))

#' @export
pandoc.footnote <- function(...)
    cat(pandoc.footnote.return(...))


#' Create horizontal rule
#'
#' Creates a Pandoc's markdown format horizontal line with trailing and leading newlines.
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.horizontal.rule
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
pandoc.horizontal.rule.return <- function()
    add.blank.lines('---')

#' @export
pandoc.horizontal.rule <- function(...)
    cat(pandoc.horizontal.rule.return(...))


#' Create header
#'
#' Creates a (Pandoc's) markdown style header with given level.
#' @param x character vector
#' @param level integer
#' @param style atx or setext type of heading
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.header
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.header('Foo!', 4)
#' pandoc.header('Foo!', 2, 'setext')
#' pandoc.header('Foo **bar**!', 1, 'setext')
pandoc.header.return <- function(x, level = 1, style = c('atx', 'setext')) {

    if (missing(style)) {
        style <- panderOptions('header.style')
    } else {
        style <- match.arg(style)
    }

    if (!is.numeric(level)) {
        stop('Wrong level provided!')
    }
    if (any( (style == 'atx' & level > 6), (style == 'setext' & level > 2))) {
        stop('Too high level provided!')
    }
    if (level < 1) {
        stop('Too low level provided!')
    }

    res <- switch(style,
                  'atx'    = paste(repChar('#', level), x),
                  'setext' = paste(x, repChar(ifelse(level == 1, '=', '-'), nchar(x, type = 'width')), sep = '\n')
                  )

    add.blank.lines(res)

}

#' @export
pandoc.header <- function(...)
    cat(pandoc.header.return(...))


#' Create title block
#'
#' Creates a Pandoc's markdown style title block with optional author, title and date fields.
#' @param author character vector or semicolon delimited list of authors without line break
#' @param title character vector of lines of title or multiline string with \code{\\n} separators
#' @param date any string fit in one line
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.title
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.title('Tom', 'Render pandoc in R', '2012-05-16')
#' pandoc.title(c('Tom', 'Jerry'), 'Render pandoc in R', '2012-05-16')
#' pandoc.title('Tom; Jerry', 'Render pandoc in R', '2012-05-16')
#' pandoc.title('Tom; Jerry', c('Render', 'pandoc', 'in R'), '2012-05-16')
#' pandoc.title('Tom; Jerry', 'Render\n    pandoc \n    in R', '2012-05-16')
#'
#' ## missing fields
#'
#' pandoc.title('Tom; Jerry', 'Render pandoc in R')
#' pandoc.title('Tom; Jerry')
#' pandoc.title(title = 'Render pandoc in R', date= '2012-05-16')
pandoc.title.return <- function(author = '', title = '', date = '') {

    if (author == '' && title == '' && date != '') {
        stop('You cannot create a title with only date specified!')
    }
    ## updating title tags
    if (length(author) > 1 && author != '') {
        author <- paste('%', paste(author[author != ''], collapse = '; '))
    }
    if (length(title) > 1 && title != '') {
        title  <- paste0(
            '% ',
            gsub('[\t ][\t ]*', '  ', gsub('\n', '\n  ', paste(title[title != ''], collapse = '\n'))))
    }

    ## formatting result
    if (title == '') {
        ## author
        res <- paste0('%\n', author)
    } else {
        if (date == '') {
            if (author == '') {
                ## title
                res <- title
            } else {
                ## author and title
                res <- paste(title, author, sep = '\n')
            }
        } else {
            date <- paste0('% ', gsub('\n', ' ', date)[1])
            if (author == '') {
                ## title and date
                res <- paste(title, '%', date, sep = '\n')
            } else {
                ## author, title and date
                res <- paste(title, author, date, sep = '\n')
            }
        }
    }

    sprintf('%s\n', res)

}

#' @export
pandoc.title <- function(...)
    cat(pandoc.title.return(...))


#' Create a list
#'
#' Creates a Pandoc's markdown format list from provided character vector/list.
#' @param elements character vector of strings
#' @param style the required style of the list
#' @param loose adding a newline between elements
#' @param add.line.breaks adding a leading and trailing newline before/after the list
#' @param add.end.of.list adding a separator comment after the list
#' @param indent.level the level of indent
#' @param missing string to replace missing values
#' @param ... extra arguments passed by from parent call, disregarded
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.list
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' ## basic lists
#' pandoc.list(letters[1:5])
#' pandoc.list(letters[1:5])
#' pandoc.list(letters[1:5], 'ordered')
#' pandoc.list(letters[1:5], 'roman')
#' pandoc.list(letters[1:5], loose = TRUE)
#'
#' ## nested lists
#' l <- list("First list element",
#'   rep.int('sub element', 5),
#'   "Second element",
#'   list('F', 'B', 'I', c('phone', 'pad', 'talics')))
#' pandoc.list(l)
#' pandoc.list(l, loose = TRUE)
#' pandoc.list(l, 'roman')
#'
#' ## complex nested lists
#' pandoc.list(list('one', as.list(2)))
#' pandoc.list(list('one', list('two')))
#' pandoc.list(list('one', list(2:3)))
#' @importFrom utils as.roman
pandoc.list.return <- function(elements, style = c('bullet', 'ordered', 'roman'), loose = FALSE, add.line.breaks = TRUE, add.end.of.list = TRUE, indent.level = 0, missing = panderOptions('missing'), ...) { #nolint

    ## checks
    if (!is.logical(loose)) {
        stop('Wrong argument provided: loose')
    }

    ## default values
    if (missing(style)) {
        style <- panderOptions('list.style')
    } else {
        style <- match.arg(style)
    }

    ## replace missing values
    w <- which(is.na(elements))
    if (length(w) > 0) {
        elements[w] <- missing
    }

    ## helpers
    elements.l <- length(elements)
    marker     <- switch(style,
                         'bullet'  = rep('* ', elements.l),
                         'ordered' = paste0(1:elements.l, '. '),
                         'roman'   = paste0(as.roman(1:elements.l), '. '))

    ## number of elements should be more than one
    if (elements.l == 0) {
        return('')
    }

    ## recursive call
    i.lag <- 0
    res <- ifelse(add.line.breaks, '\n', '')
    res <- paste(sapply(1:elements.l, function(i) {
        if (length(elements[[i]]) == 1 && !is.list(elements[[i]])) {
            paste0(paste(rep(' ', indent.level * 4), collapse = ''), marker[i - i.lag], elements[i])
        } else {
            i.lag <<- i.lag + 1
            pandoc.list.return(elements[[i]], style, loose, FALSE, FALSE, indent.level + 1)
        }}),
        collapse = '\n', ifelse(loose, '\n', ''))

    ## closing tag
    if (add.end.of.list) {
        res <- paste0(res, ifelse(loose, '', '\n\n'), '<!-- end of list -->\n')
    }
    if (add.line.breaks) {
        res <- add.blank.lines(res)
    }

    return(res)

}

#' @export
pandoc.list <- function(...)
    cat(pandoc.list.return(...))


#' Create a table
#'
#' Creates a Pandoc's markdown style table with optional caption and some other tweaks. See 'Details' below.
#'
#' This function takes any tabular data as its first argument and will try to make it pretty like: rounding and applying \code{digits} and custom \code{decimal.mark} to numbers, auto-recognizing if row names should be included, setting alignment of cells and dropping trailing zeros by default.
#'
#' \code{pandoc.table} also tries to split large cells with line breaks or even the whole table to separate parts on demand. Other arguments lets the use to highlight some rows/cells/cells in the table with italic or bold text style.
#'
#' For more details please see the parameters above and passed arguments of \code{\link{panderOptions}}.
#' @param t data frame, matrix or table
#' @param caption caption (string) to be shown under the table
#' @param digits passed to \code{format}. Can be a vector specifying values for each column (has to be the same length as number of columns).
#' @param decimal.mark passed to \code{format}
#' @param big.mark passed to \code{format}
#' @param round passed to \code{round}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
#' @param missing string to replace missing values
#' @param justify defines alignment in cells passed to \code{format}. Can be \code{left}, \code{right} or \code{centre}, which latter can be also spelled as \code{center}. Defaults to \code{centre}. Can be abbreviated to a string consisting of the letters \code{l}, \code{c} and \code{r} (e.g. 'lcr' instead of c('left', 'centre', 'right').
#' @param style which Pandoc style to use: \code{simple}, \code{multiline}, \code{grid} or \code{rmarkdown}
#' @param split.tables where to split wide tables to separate tables. The default value (\code{80}) suggests the conventional number of characters used in a line, feel free to change (e.g. to \code{Inf} to disable this feature) if you are not using a VT100 terminal any more :)
#' @param split.cells where to split cells' text with line breaks. Default to \code{30}, to disable set to \code{Inf}. Can be also supplied as a vector, for each cell separately (if length(split.cells) == number of columns + 1, then first value in split.cells if for row names, and others are for columns). Supports relative (percentage) parameters in combination with split.tables.
#' @param keep.trailing.zeros to show or remove trailing zeros in numbers on a column basis width
#' @param keep.line.breaks (default: \code{FALSE}) if to keep or remove line breaks from cells in a table
#' @param plain.ascii (default: \code{FALSE}) if output should be in plain ascii (without markdown markup) or not
#' @param use.hyphening boolean (default: \code{FALSE}) if try to use hyphening when splitting large cells according to table.split.cells. Requires \pkg{sylly}.
#' @param row.names if \code{FALSE}, row names are suppressed. A character vector of row names can also be specified here. By default, row names are included if \code{rownames(t)} is neither
#'   \code{NULL} nor identical to \code{1:nrow(x)}
#' @param col.names a character vector of column names to be used in the table
#' @param emphasize.rownames boolean (default: \code{TRUE}) if row names should be highlighted
#' @param emphasize.rows deprecated for \code{emphasize.italics.rows} argument
#' @param emphasize.cols deprecated for \code{emphasize.italics.cols} argument
#' @param emphasize.cells deprecated for \code{emphasize.italics.cells} argument
#' @param emphasize.italics.rows a vector for a two dimensional table specifying which rows to emphasize
#' @param emphasize.italics.cols a vector for a two dimensional table specifying which cols to emphasize
#' @param emphasize.italics.cells a vector for one-dimensional tables or a matrix like structure with two columns for row and column indexes to be emphasized in two dimensional tables. See e.g. \code{which(..., arr.ind = TRUE)}
#' @param emphasize.strong.rows see \code{emphasize.italics.rows} but in bold
#' @param emphasize.strong.cols see \code{emphasize.italics.cols} but in bold
#' @param emphasize.strong.cells see \code{emphasize.italics.cells} but in bold
#' @param emphasize.verbatim.rows see \code{emphasize.italics.rows} but in verbatim
#' @param emphasize.verbatim.cols see \code{emphasize.italics.cols} but in verbatim
#' @param emphasize.verbatim.cells see \code{emphasize.italics.cells} but in verbatim
#' @param ... unsupported extra arguments directly placed into \code{/dev/null}
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call \code{pandoc.table.return} instead.
#' @export
#' @useDynLib pander
#' @importFrom Rcpp evalCpp
#' @importFrom stats ftable
#' @aliases pandoc.table
#' @seealso \code{\link{set.caption}}, \code{\link{set.alignment}}
#' @note If \code{caption} is missing, then the value is first checked in \code{t} object's \code{caption} attribute and if not found in an internal buffer set by \code{link{set.caption}}. \code{justify} parameter works similarly, see \code{\link{set.alignment}} for details.
#' @references John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
#' @examples
#' pandoc.table(mtcars)
#'
#' # caption
#' pandoc.table(mtcars, 'Motor Trend Car Road Tests')
#'
#' # other input/output formats
#' pandoc.table(mtcars[, 1:3], decimal.mark = ',')
#' pandoc.table(mtcars[, 1:3], decimal.mark = ',', justify = 'right')
#' pandoc.table(matrix(sample(1:1000, 25), 5, 5))
#' pandoc.table(matrix(runif(25), 5, 5))
#' pandoc.table(matrix(runif(25), 5, 5), digits = 5)
#' pandoc.table(matrix(runif(25),5,5), round = 1)
#' pandoc.table(table(mtcars$am))
#' pandoc.table(table(mtcars$am, mtcars$gear))
#' pandoc.table(table(state.division, state.region))
#' pandoc.table(table(state.division, state.region), justify = 'centre')
#'
#' m <- data.frame(a = c(1, -500, 10320, 23, 77),
#'   b = runif(5),
#'   c = c('a', 'bb', 'ccc', 'dddd', 'eeeee'))
#' pandoc.table(m)
#' pandoc.table(m, justify = c('right', 'left', 'centre'))
#' pandoc.table(m, justify = 'rlc') # Same as upper statement
#'
#' ## splitting up too wide tables
#' pandoc.table(mtcars)
#' pandoc.table(mtcars, caption = 'Only once after the first part!')
#'
#' ## tables with line breaks in cells
#' ## NOTE: line breaks are removed from table content in case keep.line.breaks is set to FALSE
#' ## and added automatically based on "split.cells" parameter!
#' t <- data.frame(a = c('hundreds\nof\nmouses', '3 cats'), b=c('FOO is nice', 'BAR\nBAR2'))
#' pandoc.table(t)
#' pandoc.table(t, split.cells = 5)
#'
#' ## exporting tables in other Pandoc styles
#' pandoc.table(m)
#' pandoc.table(m, style = "grid")
#' pandoc.table(m, style = "simple")
#' pandoc.table(t, style = "grid")
#' pandoc.table(t, style = "grid", split.cells = 5)
#' tryCatch(pandoc.table(t, style = "simple", split.cells = 5),
#'   error = function(e) 'Yeah, no newline support in simple tables')
#'
#' ## highlight cells
#' t <- mtcars[1:3, 1:5]
#' pandoc.table(t$mpg, emphasize.italics.cells = 1)
#' pandoc.table(t$mpg, emphasize.strong.cells = 1)
#' pandoc.table(t$mpg, emphasize.italics.cells = 1, emphasize.strong.cells = 1)
#' pandoc.table(t$mpg, emphasize.italics.cells = 1:2)
#' pandoc.table(t$mpg, emphasize.strong.cells = 1:2)
#' pandoc.table(t, emphasize.italics.cells = which(t > 20, arr.ind = TRUE))
#' pandoc.table(t, emphasize.italics.cells = which(t == 6, arr.ind = TRUE))
#' pandoc.table(t, emphasize.verbatim.cells = which(t == 6, arr.ind = TRUE))
#' pandoc.table(t, emphasize.verbatim.cells = which(t == 6, arr.ind = TRUE),
#'  emphasize.italics.rows = 1)
#' ## with helpers
#' emphasize.cols(1)
#' emphasize.rows(1)
#' pandoc.table(t)
#'
#' emphasize.strong.cells(which(t > 20, arr.ind = TRUE))
#' pandoc.table(t)
#'
#' ### plain.ascii
#' pandoc.table(mtcars[1:3, 1:3], plain.ascii = TRUE)
#'
#' ### keep.line.breaks
#' x <- data.frame(a="Pandoc\nPackage")
#' pandoc.table(x)
#' pandoc.table(x, keep.line.breaks = TRUE)
#'
#' ## split.cells
#' x <- data.frame(a = "foo bar", b = "foo bar")
#' pandoc.table(x, split.cells = 4)
#' pandoc.table(x, split.cells = 7)
#' pandoc.table(x, split.cells = c(4, 7))
#' pandoc.table(x, split.cells = c("20%", "80%"), split.tables = 30)
#'
#' y <- c("aa aa aa", "aaa aaa", "a a a a a", "aaaaa", "bbbb bbbb bbbb", "bb bbb bbbb")
#' y <- matrix(y, ncol = 3, nrow = 2)
#' rownames(y) <- c("rowname one", "rowname two")
#' colnames(y) <- c("colname one", "colname two", "colname three")
#' pandoc.table(y, split.cells = 2)
#' pandoc.table(y, split.cells = 6)
#' pandoc.table(y, split.cells = c(2, 6, 10))
#' pandoc.table(y, split.cells = c(2, Inf, Inf))
#'
#' ## first value used for rownames
#' pander(y, split.cells = c(5, 2, Inf, Inf))
#' pandoc.table(y, split.cells = c(5, 2, Inf, 5, 3, 10))
#'
#' ## when not enough reverting to default values
#' pandoc.table(y, split.cells = c(5, 2))
#'
#' ## split.cells with hyphenation
#' x <- data.frame(a = "Can be also supplied as a vector, for each cell separately",
#'        b = "Can be also supplied as a vector, for each cell separately")
#' pandoc.table(x, split.cells = 10, use.hyphening = TRUE)
pandoc.table.return <- function(t, caption, digits = panderOptions('digits'), decimal.mark = panderOptions('decimal.mark'), big.mark = panderOptions('big.mark'), round = panderOptions('round'), missing = panderOptions('missing'), justify, style = c('multiline', 'grid', 'simple', 'rmarkdown', 'jira'), split.tables = panderOptions('table.split.table'), split.cells = panderOptions('table.split.cells'), keep.trailing.zeros = panderOptions('keep.trailing.zeros'), keep.line.breaks = panderOptions('keep.line.breaks'), plain.ascii = panderOptions('plain.ascii'), use.hyphening = panderOptions('use.hyphening'), row.names, col.names, emphasize.rownames = panderOptions('table.emphasize.rownames'), emphasize.rows, emphasize.cols, emphasize.cells, emphasize.strong.rows, emphasize.strong.cols, emphasize.strong.cells, emphasize.italics.rows, emphasize.italics.cols, emphasize.italics.cells, emphasize.verbatim.rows, emphasize.verbatim.cols, emphasize.verbatim.cells, ...) { #nolint

    row.names.provided <- !missing(row.names)

    ## expands cells for output
    table.expand <- function(cells, cols.width, justify, sep.cols) {
        .Call('pander_tableExpand_cpp', PACKAGE = 'pander', cells, cols.width, justify, sep.cols, style)
    }

    ## cell conversion to plain-ascii (deletion of markup characters)
    to.plain.ascii <- function(x){
        x <- gsub('[\\\\]', '', x) # backslashes
        x <- gsub('&nbsp;', ' ', x)  # table non-breaking space
        x <- gsub('[*]+([^\\*.]*)[*]+', '\\1', x) # emphasis and strong
        x <- gsub('^[`]|[`]$', '', x) # verbatium
        x <- gsub('^[~]{2}|[~]{2}$', '', x) # strikeout
        gsub('^[_]|[_]$', '', x) # italic
    }

    ## split single cell with line breaks based on max.width
    split.single.cell <- function(x, max.width){
        if (!is.character(x)) {
            x <- as.character(x)
        }
        ## as.character(NA) remains NA, which causes isses with nchar since 2015-04-23
        ## https://stat.ethz.ch/pipermail/r-devel/2015-April/071007.html
        if (is.na(x)) {
            x <- 'NA'
        }
        if (!style %in% c('simple', 'rmarkdown')) {
            ## split
            if (nchar(x) == nchar(encodeString(x)) && !use.hyphening) {
                x <- paste(strwrap(x, width = max.width + 1), collapse = '\n')
            } else {
                ## dealing with CJK chars + also it does not count \n, \t, etc.
                ## this happens because width - counts only the number of columns
                ## cat will use to print the string in a monospaced font.
                if (!keep.line.breaks){
                    x <- gsub('\n', ' ', x)
                    x <- splitLine(x, max.width, use.hyphening)
                } else {
                    lines <- strsplit(x, '\\n')[[1]]
                    x <- ''
                    for (line in lines) {
                        sl <- splitLine(line, max.width, use.hyphening)
                        x <- paste0(x, sl, sep = '\n')
                    }
                }
            }
        }else{
            x <- gsub('^\\s+|\\s+$', '', x)
        }
        x
    }

    split.large.cells <- function(cells, for.rownames = FALSE) {

        ## if we have a single value, extend it to a vector to do less checks laters
        if (length(split.cells) == 1) {
            split.cells <- rep(split.cells, length(cells))
        }
        if (for.rownames) {
            ## in case it is used for rownames, we only need the first value
            split.cells <- rep(split.cells[1], length(cells))
        }

        res <- NULL
        rn <- rownames(cells)
        ## single value and vectors/lists
        if (length(dim(cells)) < 2) {

                ## discard first value which was for rownames
                if (!for.rownames && (length(split.cells) >= length(cells) + 1)) {
                    split.cells <- split.cells[-1]
                }

                if (length(cells) > length(split.cells)) {
                    warning('length of split.cells vector is smaller than data. Default value will be used for other cells') #nolint
                    split.cells <- c(split.cells, rep(panderOptions('table.split.cells'), length(cells) - length(split.cells))) #nolint
                }
                res <- sapply(seq_along(cells), function(x, i) split.single.cell(x[i], max.width = split.cells[i]), x = cells, USE.NAMES = FALSE) #nolint

        } else {
            ## matrixes and tables
            ## discard first value which was for rownames
            if (length(split.cells) >= dim(cells)[2] + 1) {
                split.cells <- split.cells[-1]
            }

            if (dim(cells)[2] > length(split.cells)) {
                warning('length of split.cells vector is smaller than data. Default value will be used for other cells')
                split.cells <- c(split.cells, rep(panderOptions('table.split.cells'),
                                                  dim(cells)[2] - length(split.cells)))
            }

            for (j in 1:dim(cells)[2]) {
                res <- cbind(res, sapply(cells[, j], split.single.cell, max.width = split.cells[j], USE.NAMES = FALSE))
            }

        }
        rownames(res) <- rn
        ## return
        res

    }

    align.hdr <- function(t.width, justify) {
        justify.vec <- rep(justify, length.out = length(t.width))
        dashes <- mapply(function(justify, width)
                         switch(
                             justify,
                             left = paste0(':', repChar('-', width + 1)),
                             right = paste0(repChar('-', width + 1), ':'),
                             centre = paste0(':', repChar('-', width), ':')
                             ),
                         justify.vec, t.width)
        hdr <- paste0('|', paste(dashes, collapse = '|'), '|')
        return(hdr)
    }
    is.wholenumber <- function(x, tol = .Machine$double.eps ^ 0.5)
        abs(x - round(x)) < tol
    check.highlight.parameters <- function(x, num, num2) {
        if (!all(is.wholenumber(x))){
            stop('Only integers (whole numbers) can be passed to highlight table cell(s), row(s) or column(s).')
        }
        if (!all(x > 0)) {
            stop('Only positive numbers can be passed to highlight table cell(s), row(s) or column(s).')
        }
        if (missing(num2)) {
            if (!is.vector(x)) {
                stop('Only a vector or NULL can be passed to highlight table cell(s), row(s) or column(s).')
            }
            if (!all(x < (num + 1))) {
                stop(paste('Too high number passed that should be kept below', num + 1))
            }
        } else {
            if (ncol(x) != 2) {
                stop('A matrix like structure should be passed to highlight cells of a table with two columns for row and column indexes - just like what is returned by `which(..., arr.ind = TRUE)`.') #nolint
            }
            if (!all(x[, 1] < (num + 1))) {
                stop(paste('Too high number passed for row indexes that should be kept below', num + 1))
            }
            if (!all(x[, 2] < (num2 + 1))) {
                stop(paste('Too high number passed for column indexes that should be kept below', num + 1))
            }
        }
    }

    ## #########################################################################
    ## converting the table into intermediate representation
    ## #########################################################################

    ## revert dplyr to a normal data.frame
    if (inherits(t, 'tbl_df')) {
        t <- as.data.frame(t)
    }

    ## we have a high-dimensional table to be converted to 2D
    if (length(dim(t)) > 2){
        t <- ftable(t)

    } else {

        ## we have a pseudo-table with only one or NULL dimension
        if (length(dim(t)) < 2) {

            tn <- names(t)

            ## print.summaryDefault stores NAs in a very special way
            ## see eg: pander(summary(c(Sys.Date(), NA)))
            if ( (inherits(t, 'Date') || inherits(t, 'POSIXct')) & length(attr(t, 'NAs'))) {
                tn <- c(tn, 'NAs')
            }

            ## matrix/rbind will coerce cells to atomic (e.g. Date to numeric)
            ## so we should first convert these values to a character vector
            if (!is.numeric(t)) {
                t <- as.character(t)
            }

            ## make a table
            t <- rbind(matrix(nrow = 0, ncol = length(t)), t)

            ## restore col/row names
            colnames(t) <- tn
            rownames(t) <- NULL

            ## special conversion for emphasize.cells, emphasize.strong.cells
            if (!missing(emphasize.cells)) {
                emphasize.cells <- cbind(rep(1, length(emphasize.cells)), emphasize.cells)
            }
            if (!missing(emphasize.verbatim.cells)) {
                emphasize.verbatim.cells <- cbind(rep(1, length(emphasize.verbatim.cells)), emphasize.verbatim.cells)
            }
            if (!missing(emphasize.italics.cells)) {
                emphasize.italics.cells <- cbind(rep(1, length(emphasize.italics.cells)), emphasize.italics.cells)
            }
            if (!missing(emphasize.strong.cells)) {
                emphasize.strong.cells <- cbind(rep(1, length(emphasize.strong.cells)), emphasize.strong.cells)
            }

        } else {
            if (dim(t)[1] == 0) {
                ## check for empty objects
                if (!is.null(colnames(t)) && length(colnames(t)) > 0) {
                    t <- matrix(data = pandoc.strong.return(colnames(t)), nrow = 1)
                } else {
                    warning('Object is empty and without header. No output will be produced')
                    return(invisible())
                }
            }
        }
    }


    ## #########################################################################
    ## check passed parameters and set sane defaults
    ## #########################################################################

    ## check correct split.cells param
    if (length(split.cells) == 0) {
        warning('split.cells is a vector of length 0, reverting to default value')
        split.cells <- panderOptions('table.split.cells')
    }

    ## check for relative split.cells
    if (all(grepl('%$', split.cells))){
        d <- dim(t)[2]
        split.cells <- as.numeric(gsub('%$', '', split.cells))
        if (sum(split.cells) == 100){
            if (is.infinite(split.tables)){
                warning('Split.tables is an infinite value, so split cells can\'t be suplied as relative value. Reverting to default') #nolint
                split.cells <- panderOptions('table.split.cells')
            } else{
                d <- ifelse(length(rownames(t)) != 0, d, d + 1)
                if (length(split.cells) < d) {
                    cat('d - ', d, '\n')
                    warning('Using relative split.cells require a value for every column and rownames. Reverting to default') #nolint
                    split.cells <- panderOptions('table.split.cells')
                } else {
                    split.cells <- round(split.cells * 0.01 * split.tables)
                }
            }
        } else {
            warning('Supplied relative values don\'t add up to 100%. Reverting to default')
            split.cells <- panderOptions('table.split.cells')
        }
    }

    ## initializing
    mc  <- match.call()
    if (is.null(mc$style)) {
        style <- panderOptions('table.style')
    } else {
        style <- match.arg(style)
    }
    if (row.names.provided) {
      if (identical(row.names, FALSE)) {
        rownames(t) <- NULL
        row.names.provided <- FALSE
      } else {
        rownames(t) <- row.names
      }
    }
    if (!missing(col.names)) {
      colnames(t) <- col.names
    }
    if (is.null(mc$justify)) {
        if (is.null(attr(t, 'alignment'))) {
            if (inherits(t, 'ftable')) {
                justify <- get.alignment(format(t), remove.obious.rownames = !row.names.provided)
            } else {
                justify <- get.alignment(t, remove.obious.rownames = !row.names.provided)
            }
        } else {
            justify <- attr(t, 'alignment')
        }
    }
    if (is.null(mc$caption)) {
        if (is.null(attr(t, 'caption'))) {
            caption <- get.caption()
        } else {
            caption <- attr(t, 'caption')
        }
    }


    ## check if emphasize parameters were passed
    emphasize.parameters <- c('emphasize.rows',
                              'emphasize.cols',
                              'emphasize.cells',
                              'emphasize.strong.rows',
                              'emphasize.strong.cols',
                              'emphasize.strong.cells',
                              'emphasize.italics.rows',
                              'emphasize.italics.cols',
                              'emphasize.italics.cells',
                              'emphasize.verbatim.rows',
                              'emphasize.verbatim.cols',
                              'emphasize.verbatim.cells')
    if (all(sapply(emphasize.parameters, function(p) is.null(mc[[p]]), USE.NAMES = FALSE))) {
        ## check if emphasize parameters were set in attributes
        if (all(sapply(emphasize.parameters, function(p) is.null(attr(t, p)), USE.NAMES = FALSE))) {
            t <- get.emphasize(t)
        }
        ## set emphasize parameters at last
        for (p in emphasize.parameters) {
            assign(p, attr(t, p))
        }
    } else {
        ## some emphasize parameters passed, other should be set to NULL
        for (p in emphasize.parameters) {
            if (is.null(mc[[p]])) {
                assign(p, NULL)
            }
        }
    }
    res <- ''

    ## store missing values
    wm <- which(is.na(t), arr.ind = TRUE)

    ## #########################################################################
    ## update numeric values
    ## #########################################################################

    ## round numbers & cut digits & apply decimal mark & optionally remove trailing zeros
    digits <- check_digits(digits, 'digits', ncol(t))

    ## Temporary conversion of matrix to data.frame,
    ## because matrix columns can't be formatted separately:
    ## as soon as first column is formatted all others are fomatted too.
    ## Formatting each column separately is needed
    ## to support digits and round params as vectors with values for each column.
    rn <- rownames(t)
    cln <- colnames(t)
    if (inherits(t, 'matrix') & !inherits(t, 'table')) {
      rownames(t) <- NULL
      temp.t <- t <- as.data.frame(t)
    } else {
      temp.t <- t
    }
    t.n <- which(sapply(1:ncol(t), function(x) is.numeric(t[, x])))
    if (length(t.n) > 0) {
        ## make sure it's numeric (eg convert integer64 first as sapply having issues)
        ## see eg: sapply(bit64::as.integer64(1:5), format)
        for (j in t.n) {
            t[, j] <- as.numeric(t[, j])
        }
        ## round digits as needed
        round <- check_digits(round, 'round', ncol(t))
        ## for-loop is needed to preserve row/col names and use index to get appropriate value from round vector
        for (j in 1:ncol(temp.t)) {
            if (j %in% t.n) {
                t[, j] <- round(t[, j], digits = round[j])
            }
        }
        if (!keep.trailing.zeros) {
            ## for-loop is needed to preserve row/col names and use index to get appropriate value from digits vector
            for (j in 1:ncol(temp.t)) {
                temp.t[, j] <- sapply(t[, j],
                                      format,
                                      trim         = TRUE,
                                      digits       = digits[j],
                                      decimal.mark = decimal.mark,
                                      big.mark     = big.mark,
                                      quote        = FALSE)
            }
        }
    }

    ## drop unexpected classes and revert back to a common format
    if (keep.trailing.zeros) {
        ## for-loop is needed to preserve row/col names and use index to get appropriate value from digits vector
        for (j in 1:ncol(t)) {
            temp.t[, j] <- format(t[, j],
                                  trim         = TRUE,
                                  digits       = digits[j],
                                  decimal.mark = decimal.mark,
                                  big.mark     = big.mark,
                                  quote        = FALSE)
        }
    } else {
        ## here adds unneeded zero's
        temp.t <- format(temp.t, trim = TRUE, quote = FALSE)
    }

    ## transform to matrix with proper row/colnames
    t <- as.matrix(temp.t)
    colnames(t) <- cln
    rownames(t) <- rn

    ## force all non-numeric cells to character vectors
    wf <- which(sapply(t, is.factor))
    if (length(wf) > 0) {
        t[, wf] <- apply(t[wf], 2, as.character)
    }

    ## replace missing values
    if (length(wm) > 0) {
        t[wm] <- missing
    }

    ## #########################################################################
    ## adding formatting (emphasis, strong etc.)
    ## #########################################################################

    if (is.null(emphasize.italics.rows)) {
        emphasize.italics.rows <- emphasize.rows
    }
    if (is.null(emphasize.italics.cols)) {
        emphasize.italics.cols <- emphasize.cols
    }
    if (is.null(emphasize.italics.cells)) {
        emphasize.italics.cells <- emphasize.cells
    }
    if (!is.null(emphasize.verbatim.rows) && !plain.ascii) {
        check.highlight.parameters(emphasize.verbatim.rows, nrow(t))
        t[emphasize.verbatim.rows, ] <- apply(t[emphasize.verbatim.rows, , drop = FALSE], #nolint
                                              c(1, 2),
                                              pandoc.verbatim.return)
    }
    if (!is.null(emphasize.verbatim.cols) && !plain.ascii) {
        check.highlight.parameters(emphasize.verbatim.cols, ncol(t))
        t[, emphasize.verbatim.cols] <- apply(t[, emphasize.verbatim.cols, drop = FALSE],
                                              c(1, 2), pandoc.verbatim.return)
    }
    if (!is.null(emphasize.verbatim.cells) && !plain.ascii) {
        t <- as.matrix(t)
        check.highlight.parameters(emphasize.verbatim.cells, nrow(t), ncol(t))
        t[emphasize.verbatim.cells] <- sapply(t[emphasize.verbatim.cells], pandoc.verbatim.return)
    }
    if (!is.null(emphasize.italics.rows) && !plain.ascii) {
        check.highlight.parameters(emphasize.italics.rows, nrow(t))
        t[emphasize.italics.rows, ] <- base::t(apply(t[emphasize.italics.rows, , drop = FALSE], #nolint
                                             c(1),
                                             pandoc.emphasis.return))
    }
    if (!is.null(emphasize.strong.rows) && !plain.ascii) {
        check.highlight.parameters(emphasize.strong.rows, nrow(t))
        t[emphasize.strong.rows, ] <- base::t(apply(t[emphasize.strong.rows, , drop = FALSE], #nolint
                                                    c(1),
                                                    pandoc.strong.return))
    }
    if (!is.null(emphasize.italics.cols) && !plain.ascii) {
        check.highlight.parameters(emphasize.italics.cols, ncol(t))
        t[, emphasize.italics.cols] <- apply(t[, emphasize.italics.cols, drop = FALSE], c(2), pandoc.emphasis.return)
    }
    if (!is.null(emphasize.strong.cols) && !plain.ascii) {
        check.highlight.parameters(emphasize.strong.cols, ncol(t))
        t[, emphasize.strong.cols] <- apply(t[, emphasize.strong.cols, drop = FALSE], c(2), pandoc.strong.return)
    }
    if (!is.null(emphasize.italics.cells) && !plain.ascii) {
        t <- as.matrix(t)
        check.highlight.parameters(emphasize.italics.cells, nrow(t), ncol(t))
        t[emphasize.italics.cells] <- pandoc.emphasis.return(t[emphasize.italics.cells])
    }
    if (!is.null(emphasize.strong.cells) && !plain.ascii) {
        t <- as.matrix(t)
        check.highlight.parameters(emphasize.strong.cells, nrow(t), ncol(t))
        t[emphasize.strong.cells] <- pandoc.strong.return(t[emphasize.strong.cells])
    }

    ## #########################################################################
    ## (re)set row and column names
    ## #########################################################################

    ## get (col/row)names if any
    t.colnames <- tryCatch(colnames(t), error = function(e) NULL)
    t.rownames <- tryCatch(rownames(t), error = function(e) NULL)

    ## fixed for incorrect pipelining with rmarkdown (#186)
    if (style == 'rmarkdown') {
        t <- apply(t, c(1,2), function(x) gsub('\\|', '\\\\|', x)) #nolint
        t.rownames <- sapply(t.rownames, function(x) gsub('\\|', '\\\\|', x)) #nolint
    }

    t <- split.large.cells(t)

    ## re-set col/rownames to be passed to split tables
    if (!is.null(t.rownames)) {
        rownames(t) <- t.rownames
    }
    if (!is.null(t.colnames)) {
        colnames(t) <- t.colnames
    }

    ## #########################################################################
    ## compute column width
    ## #########################################################################

    ## header width
    if (!is.null(t.colnames)) {
        t.colnames <- replace(t.colnames, which(t.colnames == ''), '&nbsp;')
        t.colnames <- split.large.cells(t.colnames)
        t.colnames.width <- sapply(t.colnames,
                                   function(x) max(nchar(strsplit(x, '\n')[[1]], type = 'width'), 0),
                                   USE.NAMES = FALSE)
    } else {
        t.colnames.width <- 0
    }

    ## remove traling spaces, because they affect formatting negatively
    t <- apply(t, c(1, 2), function(x) gsub('[[:space:]]*$', '', x))

    ## also dealing with cells split by newlines
    t.width <-  as.numeric(apply(cbind(t.colnames.width, apply(t, 2, function(x) max(sapply(strsplit(x,'\n'), function(x) max(nchar(x, type = 'width'), 0))))), 1, max)) #nolint

    ## remove obvious row.names
    if ((!row.names.provided && (all(t.rownames == 1:nrow(t)) | all(t.rownames == ''))) |
        row.names.provided && row.names == FALSE) {
        t.rownames <- NULL
    }

    if (!is.null(t.rownames) && emphasize.rownames) {
        t.rownames <- pandoc.strong.return(t.rownames)
    }

    if (length(t.rownames) != 0) {

        if ((length(split.cells) <= dim(t)[2]) && (length(split.cells) > 1)) {
            split.cells <- c(panderOptions('table.split.cells'), split.cells)
        }
        t.rownames <- split.large.cells(t.rownames, TRUE)

        if (!is.null(t.colnames)) {
            t.colnames <- c('&nbsp;', t.colnames)
        }
        t.width <- c(max(sapply(strsplit(t.rownames, '\n'), function(x) max(nchar(x, type = 'width'), 0))), t.width)
        t.width[1] <- t.width[1]

        ## if we have a non-breaking space in the header
        if (!is.null(t.colnames)) {
            t.width[1] <- max(t.width[1], 6)
        }

    }

    if (length(justify) != 1) {
        if (length(justify) != length(t.width)) {
            stop(sprintf('Wrong number of parameters (%s instead of *%s*) passed: justify',
                         length(justify), length(t.width)))
        }
    } else {
        if (all (strsplit(justify, '')[[1]] %in% c('c', 'l', 'r') )) {
          if (nchar(justify) != length(t.width)) {
            stop(sprintf('Wrong number of parameters (%s instead of *%s*) passed: justify',
                         nchar(justify), length(t.width)))
          }
          justify <- c(l = 'left', c = 'centre', r = 'right')[ strsplit(justify, '')[[1]] ]
        } else {
          justify <- rep(justify, length(t.width))
        }
    }
    justify <- sub('^center$', 'centre', justify)
    if (!all(justify %in% c('left', 'right', 'centre'))) {
        stop('Invalid values passed for `justify` that can be "left", "right" or "centre/center".')
    }

    ## #########################################################################
    ## Jira format is simple, let's print early
    ## #########################################################################

    if (style == 'jira') {

        if (length(t.rownames) != 0) {
            t <- cbind(t.rownames, t)
        }

        if (length(t.colnames) != 0) {
            res <- paste0('|', paste(wrap(t.colnames, '|'), collapse = ''), '|')
        } else {
            res <- ''
        }

        for (i in seq_len(nrow(t))) {
            res <- paste0(res, '\n|')
            for (j in seq_len(ncol(t))) {
                res <- paste0(res, t[i, j], '|')
            }
        }

        return(paste0(res, '\n\n'))

    }

    ## #########################################################################
    ## split too wide tables
    ## #########################################################################

    ## determine weather separator influences column's width (#164)
    extra.spaces.width <- switch(style,
                                 ## 3 because 2 spaces + 1 separator
                                 'grid' =, 'rmarkdown' = 3, #nolint
                                 'multiline' = , 'simple' = 0) #nolint

    ## add extra 2 spaces for alignment
    if (style %in% c('simple', 'multiline')) {
        t.width <- t.width + 2
    }

    ## +1 for the middle separator
    if (sum(t.width + extra.spaces.width) + 1 > split.tables
        & length(t.width) > 1 + (length(t.rownames) != 0)) {

        t.split <- max(which(cumsum(t.width + extra.spaces.width + 1) > split.tables + 1)[1] - 1, 1)
        if (t.split == 1 & length(t.rownames) != 0) {
            t.split <- 2
        }

        ## update caption
        if (!is.null(caption)) {
            caption <- paste(caption, panderOptions('table.continues.affix'))
        } else {
            caption <- panderOptions('table.continues')
        }

        ## split
        if (length(t.rownames) != 0) {
            justify <- list(justify[1:t.split], justify[c(1, (t.split + 1):length(t.width))])
            t.split <- t.split - 1
        } else {
            justify <- list(justify[1 : (t.split)], justify[c( (t.split + 1):length(t.width) )])
        }
        res <- list(t[, 1 : (t.split), drop = FALSE], t[, (t.split + 1):ncol(t), drop = FALSE])

        ## recursive call
        res <- paste(
            pandoc.table.return(res[[1]], caption = caption, digits = digits, decimal.mark = decimal.mark, round = round, missing = missing, justify = justify[[1]], style = style, split.tables = split.tables, split.cells = split.cells,  keep.trailing.zeros = keep.trailing.zeros, keep.line.breaks = keep.line.breaks, plain.ascii = plain.ascii, use.hyphening = use.hyphening, emphasize.rownames = emphasize.rownames), #nolint
            pandoc.table.return(res[[2]], caption = NULL, digits = digits, decimal.mark = decimal.mark, round = round, justify = justify[[2]], style = style, split.tables = split.tables, split.cells = split.cells,  keep.trailing.zeros = keep.trailing.zeros, keep.line.breaks = keep.line.breaks, plain.ascii = plain.ascii, use.hyphening = use.hyphening, emphasize.rownames = emphasize.rownames)) #nolint

        return(res)

    } else {

        ## #########################################################################
        ## define markdown dialects
        ## #########################################################################

        switch(style,
               'grid'      = {
                   sep.row <- paste0('\n+', paste(sapply(t.width + 2, function(x) repChar('-', x)), collapse = '+'), '+') #nolint
                   sep.top <- sep.row
                   sep.btn <- sep.row
                   sep.hdr <- paste0('+', paste(sapply(t.width + 2, function(x) repChar('=', x)), collapse = '+'), '+')
                   sep.col <- c('| ', ' | ', ' |')
               },
               'multiline' = {
                   sep.row <- '\n'
                   sep.hdr <- paste(sapply(t.width, function(x) repChar('-', x)), collapse = ' ')
                   if (length(t.colnames) != 0) {
                       sep.top <- gsub(' ', '-', sep.hdr)
                   } else {
                       sep.top <- paste(sapply(t.width, function(x) repChar('-', x)), collapse = ' ')
                   }
                   sep.btn <- sep.top
                   sep.col <- c('', ' ', '')
               },
               'simple'   = {
                   sep.row <- ''
                   if (length(t.colnames) == 0) {
                       sep.top <- paste(sapply(t.width, function(x) repChar('-', x)), collapse = ' ')
                       sep.btn <- paste0('\n', sep.top)
                   } else {
                       sep.top <- sep.btn <- ''
                   }
                   sep.hdr <- paste(sapply(t.width, function(x) repChar('-', x)), collapse = ' ')
                   sep.col <- c('', ' ', '')
               },
               'rmarkdown' = {
                   sep.row <- ''
                   sep.top <- ''
                   sep.btn <- ''
                   sep.hdr <- align.hdr(t.width, justify)
                   sep.col <- c('| ', ' | ', ' |')
               })

        if (plain.ascii){
            t <- apply(t, c(1, 2), to.plain.ascii)
            t.rownames <- sapply(t.rownames, to.plain.ascii)
            t.colnames <- sapply(t.colnames, to.plain.ascii)
        }

        ## #########################################################################
        ## Actual printing starts here
        ## #########################################################################

        ## header
        if (length(t.colnames) != 0) {
            res <- paste(res, sep.top,
                         table.expand(t.colnames, t.width, justify, sep.col), sep.hdr, sep = '\n')
        } else {
            if (style == 'rmarkdown') {
                blank.hdr <- paste0('| ', paste(sapply(t.width, function(x) repChar(' ', x)), collapse = ' | '), ' |')
                res <- paste(res, blank.hdr, sep.hdr, sep = '\n')
            } else {
                res <- paste(res, sep.top, sep = '\n')
            }
        }

        ## body
        res <- paste0(res, '\n')

        if (length(t.rownames) != 0) {
            t <- cbind(t.rownames, t)
        }

        res <- paste0(res, paste(apply(t, 1, function(x) paste0(table.expand(x, t.width, justify, sep.col), sep.row)), collapse = '\n')) #nolint

        ## It is possible for a multiline table to have just one row,
        ## but the row should be followed by a blank line (and then
        ## the row of dashes that ends the table), or the table may
        ## be interpreted as a simple table.
        if (style == 'multiline' & nrow(t) == 1 & length(t.colnames) == 0) {
            res <- paste0(res, '\n')
        }

        ## footer
        if (style != 'grid') {
            res <- paste0(res, sep.btn, '\n\n')
        } else {
            res <- paste0(res, '\n\n')
        }

        ## (optional) caption
        check_caption(caption)
        if (!is.null(caption) && caption != '') {
            res <- paste0(res, panderOptions('table.caption.prefix'), caption, '\n\n')
        }

        return(res)

    }
}

#' @export
pandoc.table <- function(...)
    cat(pandoc.table.return(...))

#' Formulas
#'
#' Pandoc's mardown formula.
#' @param x formula
#' @param text text to be written before result in the same line. Typically used by calls from other functions in the package
#' @param max.width maximum width in characters per line
#' @param caption caption (string) to be shown under the formula
#' @param add.line.breaks if to add 2 line breaks after formula
#' @param ... extra arguments passed by from parent call, disregarded
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.formula
#' @examples
#' pandoc.formula(y ~ x)
#' pandoc.formula(formula(paste('y ~ ', paste0('x', 1:12, collapse = ' + '))))
pandoc.formula.return <- function(x, text = NULL, max.width = 80, caption, add.line.breaks = FALSE, ...){
    mc  <- match.call()
    if (is.null(mc$caption)) {
        if (is.null(attr(t, 'caption'))) {
            caption <- get.caption()
        } else {
            caption <- attr(t, 'caption')
        }
    }
    res <- paste(sub('^[ ]*', '', deparse(x, width.cutoff = max.width)), collapse = '')
    if (!is.null(text)) {
        res <- paste(text, res, sep = ' ')
    }
    if (add.line.breaks) {
        res <- paste(res, '\n\n')
    }
    ## (optional) caption
    if (!is.null(caption) && caption != '' && check_caption(caption)) {
        res <- paste0(res, '\n\n', panderOptions('formula.caption.prefix'), caption, '\n\n')
    }
    return(res)
}

#' @export
pandoc.formula <- function(...)
    cat(pandoc.formula.return(...))


#' Dates
#'
#' Pandoc's mardown date.
#' @param x date or vector of dates
#' @param inline if to render vector of dates as inline paragraph or not (as list)
#' @param simplified if just add date formatting to vector of dates
#' @param ... extra arguments passed by from parent call, disregarded
#' @return By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
#' @export
#' @aliases pandoc.date
#' @examples
#' pandoc.date(Sys.Date())
#' pandoc.date(Sys.Date() - 1:10)
#' pandoc.date(Sys.Date() - 1:10, inline = FALSE)
pandoc.date.return <- function(x, inline = TRUE, simplified = FALSE, ...){
    if (length(x) == 1 || simplified){
        format(x, format = panderOptions('date'))
    } else {
        if (inline) {
            p(as.character(format(x, format = panderOptions('date'))))
        } else {
            pandoc.list.return(as.list(format(x, format = panderOptions('date'))), add.end.of.list = FALSE)
        }
    }
}

#' @export
pandoc.date <- function(...){
    cat(pandoc.date.return(...))
}

library(testthat)
library(pander)

## reset later
original_contrasts_options <- getOption('contrasts')

context('pandoc.table')

tables <- list(
    mtcars,
    mtcars$am,
    mtcars[1:2, ],
    mtcars[1:2, 5],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    summary(lm(mtcars$hp~1))$coeff,
    table(mtcars$am, mtcars$gear, mtcars$carb),
    addmargins(table(mtcars$gear, mtcars$carb)),
    summary(Sys.Date() + 0:5)
)

test_that('no error: multiline', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'multiline'), is_a('character'))
})
test_that('no error: simple', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'simple'), is_a('character'))
})
test_that('no error: grid', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'grid'), is_a('character'))
})
test_that('no error: rmarkdown', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'rmarkdown'), is_a('character'))
})

tables <- list(
    mtcars,
    mtcars$am,
    mtcars[1:2, ],
    mtcars[1:2, 5],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    lm(mtcars$hp~1),
    t.test(extra ~ group, data = sleep),
    prcomp(USArrests),
    density(mtcars$hp),
    table(mtcars$am, mtcars$gear, mtcars$carb) + 0.1
    )

test_that('split.tables', {
    # test exact values (#164)
    t <- data.frame(a = '7 chars', b = paste(rep('Δ', 5), collapse = ' '))
    expect_equal(nchar('Δ Δ Δ Δ Δ', type = 'width'), 9)
    res_grid <- pander_return(t, style = 'grid', split.tables = 23)
    expect_equal(length(res_grid), 8)
    expect_false(any(grepl('Table', res_grid)))
    res_grid <- pander_return(t, style = 'grid', split.tables = 22)
    expect_equal(length(res_grid), 18)
    expect_true(any(grepl('Table', res_grid)))
    res_simple <- pander_return(t, style = 'simple', split.tables = 21)
    expect_equal(length(res_simple), 6)
    expect_false(any(grepl('Table', res_simple)))
    res_simple <- pander_return(t, style = 'simple', split.tables = 20)
    expect_equal(length(res_simple), 14)
    expect_true(any(grepl('Table', res_simple)))
})

test_that('rmarkdown pipe-delimited table is correct (#186)', {
    d <- data.frame(a = 'foo|bar', b = 'my missing cell')
    res <- pander_return(d, style = 'rmarkdown')
    expect_true(any(grep('foo', res)))
    expect_true(any(grep('bar', res)))
    expect_true(any(grep('my missing cell', res)))
    rownames(d) <- 'x|y'
    res <- pander_return(d, style = 'rmarkdown')
    expect_true(any(grep('x', res)))
    expect_true(any(grep('y', res)))
    expect_true(any(grep('foo', res)))
    expect_true(any(grep('bar', res)))
    expect_true(any(grep('my missing cell', res)))
})

test_that('pandoc.table.return behaves correctly', {
    expect_warning(pander_return(mtcars[, 1:2], split.tables = Inf, split.cells = c('50%', '50%')))
    t <- mtcars[1:3, 1:2]
    attr(t, 'alignment') <- 'left'
    attr(t, 'caption') <- 'simplified mtcars'
    res <- pander_return(t, emphasize.rownames = FALSE)
    expect_false(any(grep('^[[:space:]]', res))) # because of left alignment
    expect_true(any(grep('simplified mtcars', res)))
    expect_error(pander_return(t, justify = c('left', 'right'))) # needs 3 because of rownames
    expect_error(pander_return(t, justify = 'lr')) # needs 3 because of rownames
    res <- pander_return(t, justify = 'lrr')
    expect_false(any(grep('[[:space:]]$', res)))
    expect_error(pander_return(t, justify = 'laft'))
    res <- pander_return(t, split.tables = 1)
    expect_equal(length(res), 24)
    res <- pander_return(t, split.cells = c(10, 10, 10))
    expect_equal(length(res), 16)
    expect_warning(pander_return(t, split.cells = vector()))
})

dm <- panderOptions('decimal.mark')
panderOptions('decimal.mark', ',')
test_that('decimal mark', {
    for (t in tables)
        expect_true(grepl(',', paste(pander_return(t), collapse = '\n')))
})

panderOptions('decimal.mark', dm)

test_that('Date is preserved', {
    expect_equal(grep('2015', pander_return(summary(as.Date('2015-01-01') + 0:5))), 5)
})

context('highlight tables')

t <- mtcars[1:3, 1:5]
test_that('highlight 1D: no error', {
    expect_that(pandoc.table.return(t$mpg, emphasize.italics.cells = 1), is_a('character'))
    expect_that(pandoc.table.return(t$mpg, emphasize.italics.cells = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t$mpg, emphasize.strong.cells = 1), is_a('character'))
    expect_that(pandoc.table.return(t$mpg, emphasize.strong.cells = 1:2), is_a('character'))
})

t <- table(mtcars$am, mtcars$gear)
test_that('emphasize 2D: no error', {
    expect_that(pandoc.table.return(t, emphasize.italics.rows = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.rows = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cols = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cols = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cells = which(t > 10, arr.ind = TRUE)), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.italics.cells = which(t > 20, arr.ind = TRUE)), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.rows = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.rows = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cols = 1), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cols = 1:2), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cells = which(t > 10, arr.ind = TRUE)), is_a('character'))
    expect_that(pandoc.table.return(t, emphasize.strong.cells = which(t > 20, arr.ind = TRUE)), is_a('character'))
})

test_that('emphasize: error', {
    expect_that(pandoc.table(t, emphasize.italics.cols = 1:5), throws_error())
    expect_that(pandoc.table(t, emphasize.italics.cols = 1.5), throws_error())
    expect_that(pandoc.table(t, emphasize.strong.cols = 1:5), throws_error())
    expect_that(pandoc.table(t, emphasize.strong.cols = 1.5), throws_error())
})

test_that('no warning for highlight NA/empty strings', {
    expect_warning(pandoc.table(data.frame(x = 1:2, y = c(1, NA)), emphasize.italics.cols = 2), regexp = NA)
})

test_that('emphasize.italics.rows works correctly', {
    # test for issue 176
    df <- data.frame(a=1:3, b=1:3, c=1:3)
    res <- pander_return(df, emphasize.italics.rows = c(1,2), style = 'simple')
    expect_equal(res[5], ' *1*   *1*   *1* ')
    expect_equal(res[6], ' *2*   *2*   *2* ')
    res <- pander_return(df, emphasize.strong.rows = c(1,2), style = 'simple')
    expect_equal(res[5], ' **1**   **1**   **1** ')
    expect_equal(res[6], ' **2**   **2**   **2** ')
})

test_that('emphasize.verbatim works correctly', {
    df <- data.frame(a = 1:3, b = 4:6, c = 7:9)
    res <- pander_return(df, emphasize.verbatim.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' `1`   `4`   `7` ')
    expect_equal(res[6], ' `2`   `5`   `8` ')
    res <- pander_return(df, emphasize.verbatim.cols = c(1,2), style = 'simple')
    expect_equal(res[5], ' `1`   `4`   7 ')
    expect_equal(res[6], ' `2`   `5`   8 ')
    df <- data.frame(a = 1:3, b = 1:3, c = 1:3)
    res <- pander_return(df, emphasize.italics.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' *1*   *1*   *1* ')
    expect_equal(res[6], ' *2*   *2*   *2* ')
    res <- pander_return(df, emphasize.strong.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' **1**   **1**   **1** ')
    expect_equal(res[6], ' **2**   **2**   **2** ')
})

test_that('emphasize.verbatim works correctly', {
    df <- data.frame(a = 1:3, b = 4:6, c = 7:9)
    res <- pander_return(df, emphasize.verbatim.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' `1`   `4`   `7` ')
    expect_equal(res[6], ' `2`   `5`   `8` ')
    res <- pander_return(df, emphasize.verbatim.cols = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' `1`   `4`   7 ')
    expect_equal(res[6], ' `2`   `5`   8 ')
    res <- pander_return(df, emphasize.verbatim.cells =
                                 which(df > 5, arr.ind = TRUE),
                         style = 'simple')
    expect_equal(res[5], ' 1    4    `7` ')
    expect_equal(res[6], ' 2    5    `8` ')
    res <- pander_return(df, emphasize.verbatim.cells =
                                 which(df > 5, arr.ind = TRUE),
                         emphasize.strong.rows = c(1, 2),
                         style = 'simple')
    expect_equal(res[5], ' **1**   **4**   **`7`** ')
    expect_equal(res[7], '   3      `6`      `9`   ')
})


context('captions')

tables <- list(
    mtcars,
    mtcars[1:2, ],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    lm(mtcars$hp~1),
    t.test(extra ~ group, data = sleep),
    prcomp(USArrests),
    density(mtcars$hp)
    )

has.caption <- function(ttt, evals = FALSE) {
    set.caption('foo')
    if (!evals)
        any(grepl('Table:', pander_return(ttt)))
    else
        !is.null(attr(evals('get("ttt")', env = parent.frame())[[1]]$result, 'caption'))
}

test_that('direct call', {
    for (ttt in tables)
        expect_that(has.caption(ttt), is_true())
})

cache.dir <- evalsOptions('cache.dir')
graph.dir <- evalsOptions('graph.dir')
wd <- getwd()
setwd(tempdir())
evalsOptions('cache.dir',  file.path(tempdir(), '.cache'))
evalsOptions('graph.dir',  file.path(tempdir(), 'plots'))
test_that('evals', {
    for (ttt in tables)
        expect_that(has.caption(ttt, evals = TRUE), is_true())
})
evalsOptions('cache.dir',  cache.dir)
evalsOptions('graph.dir',  graph.dir)
setwd(wd)

context('row and column names')

library(testthat)
library(pander)

tables <- list(
  mtcars[1:2, 1:2],
  data.frame(x = 1:2, y = 2:3),
  data.frame(x = 1:3, y = 2:4)[c(1, 3), ]
  )

t <- tables[[1]]

test_that('row names can be suppressed', {
  for (t in tables) {
    expect_false(grepl("&nbsp;", pandoc.table.return(t, row.names = FALSE)))
  }
})

test_that('row names can be set', {
  for (t in tables) {
    expect_true(grepl("\\n \\*\\*a\\*\\*", pandoc.table.return(t, row.names = c("a", "b"))))
  }
})

test_that('obvious row names can be preserved', {
    expect_true(grepl(".*1.*2.*3.*", pandoc.table.return(data.frame(a = 7:9), row.names = 1:3)))
})

test_that('column names can be set', {
  for (t in tables) {
    res <- capture.output(pandoc.table(t, col.names = c("a", "b")))
    expect_true(grepl("a *b", res[3]))
  }
})

context('default alignments')

tables <- list(
    mtcars,
    mtcars$am,
    mtcars[1:2, ],
    mtcars[1:2, 5],
    summary(mtcars$am),
    table(mtcars$am) + 0.1,
    table(mtcars$am, mtcars$gear) + 0.1,
    summary(lm(mtcars$hp~1))$coeff,
    table(mtcars$am, mtcars$gear, mtcars$carb)
    )

tad <- panderOptions('table.alignment.default')
tar <- panderOptions('table.alignment.rownames')

panderOptions('table.alignment.default', 'centre')
panderOptions('table.alignment.rownames', 'right')
test_that('no error: allright', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'multiline'), is_a('character'))
})

f <- function(df) {
        if (class(df) != 'matrix' && !is.table(df) || length(dim(df)) < 2)
            ifelse(sapply(df, is.numeric), 'right', 'left')
        else
            ifelse(apply(df, 2, is.numeric), 'right', 'left')
}
panderOptions('table.alignment.default', f)
test_that('no error: functions', {
    for (t in tables)
        expect_that(pandoc.table.return(t, style = 'multiline'), is_a('character'))
})
panderOptions('table.alignment.default', tad)
panderOptions('table.alignment.rownames', tar)

test_that('digits param', {
    m <- matrix(rep(0.111111, 6), nrow = 2)
    res <- pander_return(m, digits = c(2, 4, 1))
    expect_equal(res[3], ' 0.11   0.1111   0.1 ')
    expect_warning(pander_return(m, digits = c(1, 2)))
    res <- pander_return(m, digits = 2)
    expect_equal(res[3],' 0.11   0.11   0.11 ')
    mt <- mtcars[1:4, 5:8]
    res <- pander_return(mt, digits = c(1, 4, 3, 4),
                         keep.trailing.zeros = TRUE)
    expect_equal(res[5],
                 '   **Mazda RX4**       4     2.620   16.5   0  ')
})

test_that('round param', {
    m <- matrix(rep(0.111111, 6), nrow = 2)
    res <- pander_return(m, round = c(2, 4, 1))
    expect_equal(res[3], ' 0.11   0.1111   0.1 ')
    expect_warning(pander_return(m, round = c(1, 2)))
    res <- pander_return(m, round = 2)
    expect_equal(res[3],' 0.11   0.11   0.11 ')
})

context('keep.trailing.zeros')
test_that('digits param', {
    expect_true(grepl('3.90', pander_return(mtcars[2:3, 1:5], keep.trailing.zeros = TRUE)[5]))
    expect_false(grepl('3.90', pander_return(mtcars[2:3, 1:5], keep.trailing.zeros = FALSE)[5]))
})

context('keep.line.breaks')
test_that('keep.line.breaks works correctly', {
  # keeping line breaks in a simple data.frame with one line breaks differs lines amount by one
  x <- data.frame(a = 'Pander\nPackage')
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + 1, lines.x.keep.breaks)

  # keeping line breaks in a simple data.frame with 2 rows with 1 line breaks differst lines amount by 2
  x <- data.frame(a = c('Pander\nPackage', 'Pander\nPackage'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + 2, lines.x.keep.breaks)

  #if there are no line breaks originally, they do not get introduced
  x <- data.frame(a = c('Pander Package', 'Pander Package'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks, lines.x.keep.breaks)

  # works with random number of rows added
  rows <- sample(1:100, 1)
  x <- data.frame(a = rep('Pander\nPackage', rows))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + rows, lines.x.keep.breaks)

  # random number of line breaks in one cell
  n <- sample(1:10, 1)
  x <- data.frame(a = paste(rep('pander', n), collapse = '\n'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE, split.cells = Inf), '\n')[[1]])
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + n - 1, lines.x.keep.breaks)

  # random number of line breaks in cells, 3 columns
  n <- sample(1:10, 3)
  x <- data.frame(a = paste(rep('pander', n[1]), collapse = '\n'),
                  b = paste(rep('pander', n[2]), collapse = '\n'),
                  c = paste(rep('pander', n[3]), collapse = '\n'))
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE, split.cells = Inf, split.tables = Inf), '\n')[[1]]) #nolint
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + max(n) - 1, lines.x.keep.breaks)

  # 3 columns, 2 rows
  x <- rbind(x, x)
  lines.x.no.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = FALSE, split.cells = Inf, split.tables = Inf), '\n')[[1]]) #nolint
  lines.x.keep.breaks <- length(strsplit(pandoc.table.return(x, keep.line.breaks = TRUE), '\n')[[1]])
  expect_equal(lines.x.no.breaks + 2 * max(n) - 2, lines.x.keep.breaks)
})

context('split.cells')
test_that('split.cells works correctly', {
  x <- data.frame(a = 'foo bar\nfo bar')
  # single line break behaves correctly combines with keep line breaks
  expect_equal(pandoc.table.return(x, keep.line.breaks = TRUE, split.cells = 7),
               '\n---------\n    a    \n---------\n foo bar \n fo bar  \n---------\n\n')
  expect_equal(pandoc.table.return(x, keep.line.breaks = TRUE, split.cells = 6),
               '\n--------\n   a    \n--------\n  foo   \n  bar   \n fo bar \n--------\n\n')
  # Corner values of split.cells
  x <- data.frame(a = 'foo bar', b = 'foo bar')
  expect_equal(length(strsplit(pandoc.table.return(x, split.cells = c(6, Inf)), '\n')[[1]]), 8)
  expect_equal(pandoc.table.return(x, split.cells = c(6, Inf)),
               '\n---------------\n  a       b    \n----- ---------\n foo   foo bar \n bar           \n---------------\n\n')
  expect_equal(pandoc.table.return(x, split.cells = c(Inf, 6)),
               '\n---------------\n    a       b  \n--------- -----\n foo bar   foo \n           bar \n---------------\n\n')
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)),
               '\n-------------------\n    a         b    \n--------- ---------\n foo bar   foo bar \n-------------------\n\n')
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)), pandoc.table.return(x, split.cells = Inf))
  expect_equal(pandoc.table.return(x, split.cells = c(6, 6)), pandoc.table.return(x, split.cells = 6))
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)), pandoc.table.return(x, split.cells = 7))
  expect_equal(pandoc.table.return(x, split.cells = c(7, 7)), pandoc.table.return(x, split.cells = 7))
  # relative split.cells
  expect_equal(pandoc.table.return(x, split.cells = c('50%', '50%')),
               pandoc.table.return(x, split.cells = c(7)))
  expect_equal(pandoc.table.return(x, split.cells = c('20%', '80%'), split.tables = 30),
               pandoc.table.return(x, split.cells = c(6, Inf)))
  expect_equal(pandoc.table.return(x, split.cells = c('80%', '20%'), split.tables = 30),
               pandoc.table.return(x, split.cells = c(Inf, 6)))
})

test_that('split.cells param produces expected warnings', {
  mt <- mtcars[1:2, 1:4]
  expect_warning(pander(mt, split.cells = c(1, 2)))
  expect_warning(pander(mt, split.cells = c(1, 2, 3)))
  expect_warning(pander(mt, split.cells = c('10%', '10%', '10%')))
  expect_warning(pander(mt, split.cells = c('30%', '30%', '40%')))
})

context('table.expand')

test_that('produces.errors', {
  x <- data.frame(a = 'Pander\nPackage')
  expect_error(pander(x, style = 'simple', keep.line.breaks = TRUE))
  expect_error(pander(x, style = 'rmarkdown', keep.line.breaks = TRUE))
})

table.expand <- function(cells, cols.width, justify, sep.cols, style) {
  .Call('pander_tableExpand_cpp', PACKAGE = 'pander', cells, cols.width, justify, sep.cols, style)
}

test_that('table.expand behaves correctly', {
  ## multiline style check
  argv <-  structure(list(txt = structure(c(1L, 4L, 2L, 3L), .Label = c('&nbsp;',  'cyl', 'disp', 'mpg'), class = 'factor'), width = c(19, 5, 5,  6), justify = structure(c(1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c(NA, -4L), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)),
               length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '      &nbsp;         mpg   cyl   disp ');

  ## grid style check
  argv <-  structure(list(txt = structure(c(1L, 5L, 2L, 3L, 4L), .Label = c('&nbsp;',  'cyl', 'disp', 'hp', 'mpg'), class = 'factor'), width = c(20,  5, 5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c(NA, -5L), class = 'data.frame') #nolint
  sep.cols <-  c('| ', ' | ', ' |')
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res), nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2])) #nolint
  expect_equal(res, '|        &nbsp;        |  mpg  |  cyl  |  disp  |  hp  |' )

  ## rmarkdown style check
  argv <-  structure(list(txt = structure(c(1L, 4L, 5L, 3L, 2L), .Label = c('**Mazda RX4**',  '110', '160', '21', '6'), class = 'factor'), width = c(20, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('| ', ' | ', ' |')
  style <-  'rmarkdown'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '|    **Mazda RX4**     |  21   |   6   |  160   | 110  |')

  ## simple style check
  argv <-  structure(list(txt = structure(c(1L, 3L, 4L, 2L, 5L), .Label = c('**Datsun 710**',  '108', '22.8', '4', '93'), class = 'factor'), width = c(20, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'centre', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'simple'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '   **Datsun 710**    22.8    4    108    93 ')

  ## left justification
  argv <-  structure(list(txt = structure(c(1L, 4L, 5L, 3L, 2L), .Label = c('**Mazda',  '110', '160', '21', '6'), class = 'factor'), width = c(10, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c(NA, -5L), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '**Mazda    21    6     160    110 ');

  ## right justification
  argv <-  structure(list(txt = structure(c(1L, 3L, 5L, 4L, 2L), .Label = c('**Hornet 4 Drive**',  '110', '21.4', '258', '6'), class = 'factor'), width = c(20,  5, 5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'right', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'simple'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(argv[, 2]) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(argv[, 2]))
  expect_equal(res, '  **Hornet 4 Drive**  21.4     6    258  110' );

  ## multiple lines
  argv <-  structure(list(txt = structure(c(1L, 3L, 5L, 4L, 2L), .Label = c('**Hornet\n4\nDrive**',  '110', '21.4', '258', '6'), class = 'factor'), width = c(10,  5, 5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)),
               length(strsplit(res, '\n')[[1]]))
  # max number of line breaks equals number of lines in the result
  expect_equal(res, '**Hornet   21.4  6     258    110 \n4                                 \nDrive**                           '); #nolint


  argv <-  structure(list(txt = structure(c(1L, 3L, 4L, 2L, 5L), .Label = c('**Datsun\n710**',  '108', '22.8', '4', '93'), class = 'factor'), width = c(10, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)),
               length(strsplit(res, '\n')[[1]]))
  expect_equal(res, '**Datsun   22.8  4     108    93  \n710**                             ');

  argv <-  structure(list(txt = structure(c(1L, 4L, 5L, 3L, 2L), .Label = c('**Mazda\nRX4\nWag**',  '110', '160', '21', '6'), class = 'factor'), width = c(10, 5,  5, 6, 4), justify = structure(c(1L, 1L, 1L, 1L, 1L), .Label = 'left', class = 'factor')), .Names = c('txt',  'width', 'justify'), row.names = c('t.rownames', 'mpg', 'cyl',  'disp', 'hp'), class = 'data.frame') #nolint
  sep.cols <-  c('', ' ', '')
  style <-  'multiline'
  res <- table.expand(argv[, 1], argv[, 2], argv[, 3], sep.cols, style)
  # max number of line breaks equals number of lines in the result
  expect_equal(max(sapply(strsplit(as.character(argv[, 1]), '\n'), length)), length(strsplit(res, '\n')[[1]]))
  expect_equal(res, '**Mazda    21    6     160    110 \nRX4                               \nWag**                             '); #nolint

  # empty cells
  cells <- c('', '', '')
  cols.width <- c(2, 2, 2)
  justify <- c('centre', 'centre', 'centre')
  sep.cols <- c('', ' ', '')
  res <- table.expand(cells, cols.width, justify, sep.cols, 'multiline')
  # when max.width param is small, every word is a line
  expect_equal(max(sapply(strsplit(as.character(cells), '\n'), length)), 0)
  expect_equal(nchar(res),
               nchar(sep.cols)[1] + (length(cells) - 1) * nchar(sep.cols)[2] + nchar(sep.cols)[3] + sum(cols.width))
  expect_equal(res, '        ')

  # backslashes issue (#22)
  expect_equal(pandoc.table.return(
      data.frame(a = '\\1 \\ 32', b ='23')),
      '\n--------------\n    a      b  \n--------- ----\n \\1 \\ 32   23 \n--------------\n\n') #nolint
  expect_equal(pandoc.table.return(
      data.frame(a = '\\1 \\ 32', b = '23'),
      justify = 'right'),
      '\n--------------\n        a    b\n--------- ----\n  \\1 \\ 32   23\n--------------\n\n') #nolint
  expect_equal(pandoc.table.return(data.frame(a = '\\1', b = '23')),
               '\n---------\n a    b  \n---- ----\n \\1   23 \n---------\n\n') #nolint

  # unicode string issue
  expect_equal(pandoc.table.return(data.frame(a = 'ßß')),
               '\n----\n a  \n----\n ßß \n----\n\n')
})

context('Empty objects')

test_that('Behavior for empty objects is correct', {
    mt <- mtcars[mtcars$mpg < 0, 1:4]
    res <- pander_return(mt)
    expect_equal(res[3], ' **mpg**   **cyl**   **disp**   **hp** ')
    expect_equal(length(res), 6)
    colnames(mt) <- NULL
    res <- suppressWarnings(pander_return(mt))
    expect_equal(length(res), 0)
    expect_warning(pander_return(mt))
    mt <- matrix(0, nrow = 0, ncol = 5)
    res <- suppressWarnings(pander_return(mt))
    expect_equal(length(res), 0)
    expect_warning(pander_return(mt))
    colnames(mt) <- 1:5
    res <- pander_return(mt)
    expect_equal(res[3], ' **1**   **2**   **3**   **4**   **5** ')
    expect_equal(length(res), 6)
    expect_equal(length(suppressWarnings(pander_return(data.frame()))), 0)
    expect_warning(pander_return(data.frame()))
})

context('plain.ascii')

test_that('plain.ascii option works correctly', {
    # dim is NULL
    x <- 1:10
    res <- pandoc.table.return(x, emphasize.cells = c(3, 4), plain.ascii = TRUE)
    res <- strsplit(res, '\n')[[1]]
    expect_false(any(grepl('\\*', res)))
    expect_equal(res[3], ' 1   2   3   4   5   6   7   8   9   10 ')
    # length(dim) == 1
    x <- array(1:10)
    res <- pandoc.table.return(x, emphasize.cells = c(3, 4), plain.ascii = TRUE)
    res <- strsplit(res, '\n')[[1]]
    expect_false(any(grepl('\\*', res)))
    expect_equal(res[3], ' 1   2   3   4   5   6   7   8   9   10 ')
    # length(dim) > 1
    x <- mtcars[1:3, 1:4]
    res <- pandoc.table.return(x, emphasize.rows = 2, plain.ascii = TRUE)
    expect_false(grepl('&nbsp;', res))
    expect_false(grepl('\\*', res))
})

context('S3 methods')

test_that('pander.tabular behaves correctly', {
    suppressMessages(require(tables))
    tab <- pander_return(tables::tabular(as.factor(am) ~ (mpg + hp + qsec) * (mean + median),
                                         data = mtcars),
                         emphasize.rownames = FALSE,
                         split.tables = Inf)
    expect_equal(length(tab), 10)
    tab <- pander_return(tables::tabular( (Species + 1) ~ (n = 1) + Format(digits = 2) * (Sepal.Length + Sepal.Width) * (mean + sd), data = iris ), #nolint
                         split.tables = Inf)
    expect_equal(length(tab), 14)
})

test_that('pander.CrossTable behaves correctly', {
    suppressMessages(require(descr))
    # issue https://github.com/Rapporter/pander/issues/163
    x <- CrossTable(mtcars$cyl, mtcars$gear, prop.c = FALSE, prop.t = FALSE, chisq = FALSE, prop.chisq = FALSE)
    res <- pander_return(x)
    expect_true(any(grepl(gsub('\\$', '\\\\$', x$ColData), res))) #nolint
    expect_true(any(grepl(gsub('\\$', '\\\\$', x$RowData), res))) #nolint
    # expected N, residual, std residual, adj std residual rownames was not included
    x <- suppressWarnings(CrossTable(mtcars$cyl, mtcars$gear, expected = TRUE,
                                     resid = TRUE, sresid = TRUE, asresid = TRUE))
    res <- pander_return(x)
    expect_true(any(grepl('Expected N', res)))
    expect_true(any(grepl('Residual', res)))
    expect_true(any(grepl('Std Residual', res)))
    expect_true(any(grepl('Adj Std Resid', res)))
    # issue 211 support for total.r total.c
    x <- suppressWarnings(CrossTable(mtcars$cyl, mtcars$gear))
    res <- pander_return(x, total.c = FALSE)
    expect_equal(length(res), 27)
    expect_equal(length(grep('Total', res)), 4)
    res <- pander_return(x, total.r = FALSE)
    expect_equal(length(res), 30)
    expect_equal(length(grep('Total', res)), 4)
    res <- pander_return(x)
    expect_equal(length(res), 30)
    expect_equal(length(grep('Total', res)), 5)
})

test_that('pander.NULL behaves correctly', {
    expect_equal(length(pander_return(NULL)), 0)
    expect_equal(length(pander_return(c(NULL, NULL))), 0)
})

test_that('pander.cast_df behaves correctly', {
    df <- data.frame(type = c(1, 1, 2, 2, 3, 3), variable = 'n', value = c(71, 72, 68, 80, 21, 20))
    df.cast <- reshape::cast(df, type~., sum)
    expect_equal(pander_return(df.cast, style = 'simple'),
                 c('', '', ' type   (all) ', '------ -------',
                   '  1      143  ', '  2      148  ', '  3      41   ', ''))
})

test_that('pander.lm/pander.summary.lm behaves correctly', {
    ctl <- c(4.17, 5.58, 5.18, 6.11, 4.50, 4.61, 5.17, 4.53, 5.33, 5.14)
    trt <- c(4.81, 4.17, 4.41, 3.59, 5.87, 3.83, 6.03, 4.89, 4.32, 4.69)
    group <- gl(2, 10, 20, labels = c('Ctl', 'Trt'))
    weight <- c(ctl, trt)
    lm.D9 <- lm(weight ~ group)
    res1 <- pander_return(lm.D9)
    expect_equal(length(res1), 11)
    expect_equal(max(nchar(res1)), 63)
    res <- pander_return(summary(lm.D9))
    expect_true(any(grepl('Fitting linear model', res)))
    expect_true(any(grepl('Observations', res)))
    expect_equal(length(res), 18)
})

test_that('pander.glm/pander.summary.glm behaves correctly', {
    clotting <- data.frame(
        u = c(5, 10, 15, 20, 30, 40, 60, 80, 100),
        lot1 = c(118, 58, 42, 35, 27, 25, 21, 19, 18),
        lot2 = c(69, 35, 26, 21, 18, 16, 13, 12, 12))
    glmm <- glm(lot1 ~ log(u), data = clotting, family = Gamma)
    pglmm <- pander_return(glmm)
    expect_equal(length(pglmm), 11)
    expect_equal(max(nchar(pglmm)), 70)
    res <- pander_return(summary(glmm))
    expect_true(any(grepl('Null deviance', res)))
    expect_equal(length(res), 21)
})

test_that('pander.aov/pander.summary.aov behaves correctly', {
    npk.aovE <- aov(yield ~  N * P * K + Error(block), npk)
    paov <- pander_return(npk.aovE)
    psaov <- pander_return(summary(npk.aovE))
    expect_equal(paov, psaov) # choice of similar result for summary and standard
    expect_equal(length(psaov), 23)
})

test_that('pander.anova behaves correctly', {
    fit <- lm(sr ~ ., data = LifeCycleSavings)
    a <- anova(fit)
    pa <- pander_return(a, style = 'simple')
    expect_true(all(sapply(names(a)[-5], grepl, pa[3])))
    #more complicated run
    fit0 <- lm(sr ~ 1, data = LifeCycleSavings)
    fit1 <- update(fit0, . ~ . + pop15)
    fit2 <- update(fit1, . ~ . + pop75)
    fit3 <- update(fit2, . ~ . + dpi)
    fit4 <- update(fit3, . ~ . + ddpi)
    a <- anova(fit0, fit1, fit2, fit3, fit4, test = 'F')
    pa <- pander_return(a, style = 'simple')
    expect_true(all(sapply(names(a)[-6], grepl, pa[3])))
})

test_that('pander.aovlist/pander.summary.aovlist behaves correctly', {
    options(contrasts = c('contr.helmert', 'contr.poly'))
    N <- c(0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0)
    P <- c(1, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0)
    K <- c(1, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0)
    yield <- c(49.5, 62.8, 46.8, 57.0, 59.8, 58.5, 55.5, 56.0, 62.8, 55.8, 69.5,
               55.0, 62.0, 48.8, 45.5, 44.2, 52.0, 51.5, 49.8, 48.8, 57.2, 59.0, 53.2, 56.0)
    npk <- data.frame(block = gl(6, 4), N = factor(N), P = factor(P),
                      K = factor(K), yield = yield)
    a <- aov(yield ~  N * P * K + Error(block), npk)
    pa <- pander_return(a, style = 'simple')
    expect_equal(length(pa), 14)
})

test_that('pander.ts behaves correctly', {
    # ncol NULL
    res <- pander_return(ts(1:10, frequency = 4, start = c(1959, 2)), style = 'simple')
    expect_equal(res[3], '  &nbsp;    Q1   Q2   Q3   Q4 ')
    expect_equal(length(res), 8)
    # ncol not NULL
    res <- pander_return(ts(matrix(rnorm(30), 10, 3), start = c(1961, 1), frequency = 12), style = 'simple')
    expect_equal(res[3], '    &nbsp;      Series.1   Series.2   Series.3 ')
    expect_equal(length(res), 15)
})

test_that('pander.coxph behaves correctly', {
    suppressMessages(require(survival))
    test1 <- list(time = c(4, 3, 1, 1, 2, 2, 3),
                  status = c(1, 1, 1, 0, 1, 1, 0),
                  x = c(0, 2, 1, 1, 1, 0, 0),
                  sex = c(0, 0, 0, 0, 1, 1, 1))
    # Fit a stratified model
    res <- pander_return(coxph(Surv(time, status) ~ x + strata(sex), test1))
    expect_equal(res[3], ' &nbsp;    coef    exp(coef)   se(coef)     z        p    ')
    expect_equal(length(res), 10)
})

test_that('pander.clogit works correctly', {
    resp <- levels(logan$occupation)
    n <- nrow(logan)
    indx <- rep(1:n, length(resp))
    logan2 <- data.frame(logan[indx, ],
                         id = indx,
                         tocc = factor(rep(resp, each = n)))
    logan2$case <- (logan2$occupation == logan2$tocc)
    res <- pander_return(suppressWarnings(clogit(case ~ tocc + tocc:education + strata(id), logan2)), split.table = Inf)
    expect_true(grepl('Fitting Conditional logistic regression', res[24]))
    expect_equal(length(res), 26)
})

test_that('pander.zoo works correctly', {
    suppressMessages(require(zoo))
    x.Date <- as.Date('2003-02-01') + c(1, 3, 7, 9, 14) - 1
    res <- pander_return(zoo(rnorm(5), x.Date), style = 'simple')
    expect_true(grepl('Value', res[3]))
    expect_equal(length(res), 10)
    # more complex example with colnames
    res <- pander_return(zoo(cbind(foo = rnorm(5), bar = rnorm(5))), style = 'simple')
    expect_true(grepl('foo', res[3]))
    expect_equal(length(res), 10)
})

test_that('pander.lme/pander.summary.lme behaves correctly', {
    suppressMessages(require(nlme))
    l1 <- lme(distance ~ age, Orthodont, random = ~ age | Subject)
    sl <- summary(l1)
    pl <- pander_return(l1)
    spl <- pander_return(sl)
    expect_equal(length(pl), 11)
    expect_equal(length(grep('Table', pl)), 1)
    expect_equal(length(spl), 29)
    expect_equal(length(grep('Table', spl)), 3)
})

# test_that('pander.describe works correctly', {
#     suppressMessages(require(psych))
#     x <- data.frame(a = rnorm(10), b = rnorm(10, 2, 2), c = rnorm(10, 3, 4))
#     res <- pander_return(describe(x))
#     expect_equal(length(res), 37)
#     expect_equal(length(grep('Table', res)), 2)
#     res <- pander_return(describe(x), split.tables = Inf)
#     expect_equal(length(res), 11)
# })

test_that('pander.survdiff works correctly', {
    suppressMessages(require(survival))
    res <- pander_return(survdiff(Surv(futime, fustat) ~ rx, data = ovarian))
    expect_equal(length(res), 12)
    expect_equal(res[3], '  &nbsp;    N    Observed   Expected   (O-E)^2/E   (O-E)^2/V ')
    # length(x$n) == 1
    expect <- survexp(futime ~ ratetable(age = (accept.dt - birth.dt),
                                         sex = 1,
                                         year = accept.dt,
                                         race = 'white'),
                      jasa, cohort = FALSE, ratetable = survexp.usr)
    res <- pander_return(survdiff(Surv(jasa$futime, jasa$fustat) ~ offset(expect)))
    expect_equal(res[3], ' Observed   Expected     Z      p ')
    expect_equal(length(res), 9)
})

test_that('pander.survfit works correctly', {
    suppressMessages(require(survival))
    res <- pander_return(survfit(Surv(time, status) ~ x, data = aml))
    expect_equal(length(res), 20)
    expect_true(any(grepl('Table', res)))
    # using additional options
    res <- pander_return(survfit(Surv(time, status) ~ x, data = aml), print.rmean = TRUE)
    expect_equal(length(res), 21)
    expect_equal(res[21], '* restricted mean with upper limit =  103')
})

test_that('pander.sessionInfo works correctly', {
    suppressMessages(require(utils))
    res <- pander_return(sessionInfo())
    expect_true(any(grepl('locale', res)))
    expect_true(any(grepl('attached base package', res)))
    res <- pander_return(sessionInfo(), locale = FALSE, compact = FALSE)
    expect_false(any(grepl('locale', res)))
    expect_true(any(grepl('utils', res)))
})

test_that('pander.stat.table works correctly', {
    suppressMessages(require(Epi))
    res <- pander_return(stat.table(
        tension,
        list(count(), mean(breaks)),
        data = warpbreaks))
    expect_equal(length(res), 11)
    expect_equal(res[3], ' &nbsp;   count()   mean(breaks) ')
    res <- pander_return(stat.table(index = list(tension, wool),
                                    mean(breaks), data = warpbreaks))
    expect_equal(length(res), 13)
    # here add test
})

test_that('pander.microbenchmark works correctly', {
    suppressMessages(require(microbenchmark))
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)))
    expect_true(any(grepl('Unit', res)))
    expect_equal(length(res), 11)
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)), split.tables = Inf, expr.labels = c('A'))
    expect_true(any(grepl('A', res)))
    expect_true(any(grepl('paste0\\(1:10\\)', res)))
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)), split.tables = Inf, expr.labels = c('A', 'B'))
    expect_true(any(grepl('A', res)))
    expect_true(any(grepl('B', res)))
    expect_false(any(grepl('paste0\\(1:10\\)', res)))
    res <- pander_return(microbenchmark(paste(1:10), paste0(1:10)), split.tables = Inf, expr.labels = c('A', 'B', 'C'))
    expect_true(any(grepl('A', res)))
    expect_true(any(grepl('B', res)))
    expect_false(any(grepl('paste0\\(1:10\\)', res)))
})

test_that('pander.function works correctly', {
    testf <- function(x) {
        y <- x + 1
        y
    }
    res <- pander_return(testf)
    expect_true(all(grepl('\t', res)))
    expect_equal(length(res), 5)
    res <- pander_return(testf, syntax.highlighting = TRUE, add.name = TRUE)
    expect_equal(length(res), 7)
    expect_equal(res[1], '```r')
    expect_true(grepl('testf', res[2]))
})

test_that('pander.rlm works correctly', {
    suppressMessages(require(MASS))
    res <- pander_return(rlm(stack.loss ~ ., stackloss))
    expect_equal(res[4], ' (Intercept)   Air.Flow   Water.Temp   Acid.Conc. ')
    expect_equal(length(res), 13)
    res <- pander_return(rlm(stack.loss ~ ., stackloss, psi = psi.hampel, init = 'lts'))
    expect_equal(res[11], 'Degrees of freedom: 21 total; 17 residual')
    expect_equal(length(res), 13)
})

test_that('pander.summary.table works correctly', {
    ts <- summary(xtabs(cbind(ncases, ncontrols) ~ ., data = esoph))
    res <- pander_return(ts)
    expect_equal(res[6], ' Chisq   df     p.value  ')
    expect_equal(length(res), 13)
    res <- pander_return(ts, print.call = FALSE)
    expect_false(any(grepl('Calls', res)))
    expect_equal(length(res), 12)
    res <- pander_return(ts, caption = 'Factor')
    expect_equal(res[11], 'Table: Factor')
    res <- pander_return(summary(table(1:3)))
    expect_false(any(grepl('-', res)))
    expect_equal(length(res), 2)
})

test_that('pander.randomForest works correctly', {
    suppressMessages(require(randomForest))
    iris.rf <- randomForest(Species ~ ., data = iris, importance = TRUE,
                            proximity = TRUE)
    res <- pander_return(iris.rf)
    expect_equal(length(res), 20)
    expect_equal(res[19], 'Table: Confusion Matrix')
    ozone.rf <- randomForest(Ozone ~ ., data = airquality, mtry = 3,
                             importance = TRUE, na.action = na.omit)
    res <- pander_return(ozone.rf)
    expect_equal(length(res), 8)
    expect_false(any(grep('-', res)))
    # with test
    index <- 1:nrow(iris)
    trainindex <- sample(index, trunc(length(index) / 2))
    trainset <- iris[trainindex, ]
    testset <- iris[-trainindex, ]
    res <- pander_return(randomForest(x = trainset[, -1],
                                      y = trainset[, 1],
                                      xtest = testset[, -1],
                                      ytest = testset[, 1]))
    expect_equal(length(res), 10)
    expect_true(any(grepl('Test set MSE', res)))
    res <- pander_return(randomForest(x = trainset[, -ncol(trainset)],
                                      y = trainset[, ncol(trainset)],
                                      xtest = testset[, -ncol(testset)],
                                      ytest = testset[, ncol(testset)]))
    expect_equal(length(res), 34)
    expect_equal(res[33], 'Table: Test Confusion Matrix')
})

test_that('pander.irts works correctly', {
    suppressMessages(require(tseries))
    n <- 10
    t <- cumsum(rexp(n, rate = 0.1))
    v <- rnorm(n)
    res <- pander_return(as.irts(cbind(t, v)))
    expect_equal(length(res), 23)
    u <- rnorm(n)
    res <- pander_return(irts(t, cbind(u, v)))
    expect_equal(length(res), 23)
})

test_that('pander.manova/summary.manova works correctly', {
    npk2 <- within(npk, foo <- rnorm(24))
    x <- manova(cbind(yield, foo) ~ block + N * P * K, npk2)
    res1 <- pander_return(x)
    xs <- summary(x)
    res2 <- pander_return(xs)
    expect_equal(res1, res2)
    expect_equal(length(res1), 21)
})

test_that('pander.gtable works correctly', {
    suppressMessages(require(gtable))
    suppressMessages(require(grid))
    a <- gtable(unit(1:3, c('cm')), unit(5, 'cm'))
    rect <- rectGrob(gp = gpar(fill = 'black'))
    a <- gtable_add_grob(a, rect, 1, 1)
    res <- pander_return(a)
    expect_equal(length(res), 7)
})

test_that('pander.nls/pander.summary.nls works correctly', {
    utils::data(muscle, package = 'MASS')
    with(muscle, table(Strip))
    musc.1 <- nls(Length ~ cbind(1, exp(-Conc / th)), muscle,
                start = list(th = 1), algorithm = 'plinear')
    res <- pander_return(musc.1, show.convergence = TRUE)
    expect_equal(length(res), 17)
    res <- pander_return(musc.1)
    expect_equal(length(res), 14)
    musc.1.s <- summary(musc.1)
    res <- pander_return(musc.1.s)
    expect_equal(length(res), 18)
    res <- pander_return(musc.1.s, show.convergence = TRUE)
    expect_equal(length(res), 21)
    musc.1.s <- summary(musc.1, correlation = TRUE)
    res <- pander_return(musc.1.s)
    expect_true(any(grepl('Correlation', res)))
    expect_equal(length(res), 31)
})

test_that('pander.arima works correctly', {
    res <- pander_return(arima(lh, order = c(1, 0, 0)))
    expect_equal(length(res), 17)
    expect_true(any(grep('s\\.e', res)))
    res <- pander_return(arima(lh, order = c(3, 0, 0)))
    expect_equal(length(res), 17)
    expect_true(any(grep('s\\.e', res)))
    res <- pander_return(arima(lh, order = c(3, 0, 0), method = 'CSS'), se = FALSE)
    expect_equal(length(res), 15)
    expect_false(any(grep('s\\.e', res)))
})

test_that('pander.polr/summary.polr works correctly', {
    house.plr <- polr(Sat ~ Infl + Type + Cont, weights = Freq, data = housing)
    res <- pander_return(house.plr)
    expect_equal(length(res), 25)
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(suppressMessages(summary(house.plr)))
    expect_equal(length(res), 39)
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(suppressMessages(summary(house.plr, correlation = TRUE)), split.table = Inf)
    expect_equal(length(res), 60)
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.survreg/summary.survreg works correctly', {
    suppressMessages(require(survival))
    x <- survreg(Surv(futime, fustat) ~ ecog.ps + rx, ovarian, dist = 'exponential')
    res <- pander_return(x)
    expect_equal(length(res), 27)
    expect_equal(length(grep('Table', res)), 1)
    res <- pander_return(suppressMessages(summary(x)))
    expect_equal(length(res), 33)
    expect_equal(length(grep('Table', res)), 1)
    res <- pander_return(suppressMessages(summary(x, correlation = TRUE)), split.table = Inf)
    expect_equal(length(res), 44)
    expect_equal(length(grep('Table', res)), 2)
})

test_that('pander.ols works correctly', {
    suppressMessages(require(rms))
    set.seed(1)
    x1 <- runif(200)
    x2 <- sample(0:3, 200, TRUE)
    distance <- (x1 + x2 / 3 + rnorm(200)) ^ 2
    d <- datadist(x1, x2)
    f <- ols(sqrt(distance) ~ rcs(x1, 4) + scored(x2), x = TRUE)
    res <- pander_return(f)
    expect_equal(length(res), 50)
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.lrm works correctly', {
    suppressMessages(require(rms))
    x    <- 1:5
    y    <- c(0, 1, 0, 1, 0)
    reps <- c(1, 2, 3, 2, 1)
    res <- pander_return(lrm(y ~ x))
    expect_equal(length(res), 39)
    expect_true(any(grep('y ~ x', res)))
    expect_equal(length(grep('Table', res)), 2)

    x <- rep(x, reps)
    y <- rep(y, reps)
    res <- pander_return(lrm(y ~ x), coefs = FALSE)
    expect_equal(length(res), 28)
    expect_true(any(grep('y ~ x', res)))
    expect_equal(length(grep('Table', res)), 1)

    res <- pander_return(lrm(y ~ x, penalty = 0.1), coefs = TRUE)
    expect_true(any(grep('y ~ x', res)))
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.orm works correctly', {
    suppressMessages(require(rms))
    n <- 100
    y <- round(runif(n), 2)
    x1 <- sample(c(-1, 0, 1), n, TRUE)
    x2 <- sample(c(-1, 0, 1), n, TRUE)
    res <- pander_return(orm(y ~ x1 + x2, eps = 1e-5))
    expect_equal(length(res), 39)
    expect_true(any(grep('y ~ x1 \\+ x2', res)))
    expect_equal(length(grep('Table', res)), 2)

    res <- pander_return(orm(y ~ x1 + x2, eps = 1e-5), coefs = FALSE)
    expect_equal(length(res), 28)
    expect_true(any(grep('y ~ x1 \\+ x2', res)))
    expect_equal(length(grep('Table', res)), 1)
})

test_that('pander.Glm works correctly', {
    suppressMessages(require(rms))
    clotting <- data.frame(
        u = c(5, 10, 15, 20, 30, 40, 60, 80, 100),
        lot1 = c(118, 58, 42, 35, 27, 25, 21, 19, 18),
        lot2 = c(69, 35, 26, 21, 18, 16, 13, 12, 12))
    f <- Glm(lot1 ~ log(u), data = clotting, family = Gamma)
    res <- pander_return(f)
    expect_equal(length(res), 31)
    expect_true(any(grep('lot1 ~ log\\(u\\)', res)))
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(f, coefs = FALSE)
    expect_equal(length(res), 20)
    expect_true(any(grep('ot1 ~ log\\(u\\)', res)))
    expect_equal(length(grep('Table', res)), 1)
})

test_that('pander.cph', {
    options(contrasts = original_contrasts_options)
    suppressMessages(require(rms))
    n <- 1000
    set.seed(731)
    age <- 50 + 12 * rnorm(n)
    label(age) <- 'Age'
    sex <- factor(sample(c('Male', 'Female'), n, rep = TRUE, prob = c(.6, .4)))
    cens <- 15 * runif(n)
    h <- .02 * exp(.04 * (age - 50) + .8 * (sex == 'Female'))
    dt <- -log(runif(n)) / h
    label(dt) <- 'Follow-up Time'
    e <- ifelse(dt <= cens, 1, 0)
    dt <- pmin(dt, cens)
    units(dt) <- 'Year'
    dd <- datadist(age, sex)
    S <- Surv(dt, e)
    f <- cph(S ~ rcs(age, 4) + sex, x = TRUE, y = TRUE)
    res <- pander_return(f)
    expect_equal(length(res), 43)
    expect_true(any(grep('S ~ rcs\\(age, 4\\) \\+ sex', res)))
    expect_equal(length(grep('Table', res)), 2)
    res <- pander_return(f, conf.int = 0.95)
    expect_equal(length(res), 58)
    expect_true(any(grep('S ~ rcs\\(age, 4\\) \\+ sex', res)))
    expect_equal(length(grep('Table', res)), 3)
})

test_that('pander.ets works correctly', {
  suppressMessages(require(forecast))
  res <- pander_return(ets(lh, model = 'AAN', lambda = .5))
  expect_equal(length(res), 27)
  expect_true(any(grep('Box-Cox', res)))
  res <- pander_return(ets(lh, model = 'ANN'))
  expect_equal(length(res), 25)
  expect_false(any(grep('Box-Cox', res)))
  res <- pander_return(ets(lh, model = 'AAN', damped = TRUE))
  expect_equal(length(res), 25)
  expect_true(any(grep('phi', res)))
})

---
title: "Rendering markdown with pander"
author: "Roman Tsegelskyi, Gergely Daróczi"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Rendering markdown with pander}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(collapse = T, comment = "#>")
library(pander)
library(tables)
panderOptions('knitr.auto.asis', FALSE)
panderOptions('plain.ascii', TRUE)
```

`Pander` is designed to provide a minimal and easy tool for rendering `R` objects into [Pandoc](https://pandoc.org/)'s markdown. This vignette aims to introduce the `pander` package and its core pieces of functionality. It is intended to be a general overview with pointers to places with detailed information. This vignette will talk about:

* core functionality for rendering objects in `Pandoc`'s markdown with generic S3 [pander method](#rendering-r-objects).
* functionality for capturing various information when evaluating R expressions with [evals](#evals).
* report generation with [Pandoc.brew](#brew-to-pandoc).
* globally adjustable options through [panderOptions/evalsOptions](#general-options).

## Rendering R objects

The core functionality of `pander` is centered around rendering `R` objects into `Pandoc`'s markdown. Let's dive in to a demo of the most common usage of `pander`:

```{r}
pander(head(iris))
pander(head(mtcars[1:5]))
pander(tabular( (Species + 1) ~ (n=1) + Format(digits=2)*
         (Sepal.Length + Sepal.Width)*(mean + sd), data=iris ))
```

As you have probably guessed, this is achieved via the generic `pander` `S3` method. Out of the box, `pander` supports a variety of classes:

```{r}
methods(pander)
```

If you think that pander lacks support for any other R class(es), please feel free to open a [ticket](https://github.com/Rapporter/pander/pulls) suggesting a new feature or submit [pull request](https://github.com/Rapporter/pander/issues) and we will be happy to extend the package.

Under the hood, `pander` S3 methods rely on different `pandoc.*` methods, where most of functionality is implemented in `pandoc.table` which is used for rendering tables. `pandoc.table` provides functionality similar to `knitr::kable` in rendering markdown, but also adds a truly rich functionality with a variety of rendering options inherited from `pander`. For more usage/implementation details and examples, please refer to the `pandoc.table` vignette, which can be accessed by `vignette('pandoc_table')` (and is also available [online](https://rapporter.github.io/pander/pandoc_table.html)).

## Evals

The `pander` package was originally developed in conjunction with [rapport](https://github.com/Rapporter/rapport) package, when a need arose for a function that could evaluate `R` expressions while also capturing errors and warnings. So `evals` emerged and soon some further feature requests arose, like identifying if an R expression results in a plot, etc.

But probably it's easier to explain what `evals` can do with a simple example:

```{r}
evals('1:10')
```

`evals` is aimed at collecting as much information as possible while evaluating R code. It can evaluate a character vector of R expressions, and it returns a list of information captured while running them:

* `src` holds the R expression,
* `result` contains the raw R object as-is,
* `output` represents how the R object is printed to the standard output,
* `type` is the class of the returned R object,
* `msg` is a list of possible messages captured while evaluating the R expression. Among other messages, warnings/errors will appear here.
* `stdout` contains what, if anything, was written to the standard output.

For more usage/implementation details and examples, please refer to the `evals` vignette, which can be accessed by `vignette('evals')` (also available [online](https://rapporter.github.io/pander/evals.html)).

## Brew to Pandoc

The [brew](https://cran.r-project.org/package=brew) package, a templating framework for report generation, has not been updated since 2011, but it's still some of R projects based on its simple design and useful literate programming features. For a quick overview, please see the following documents if you are not familiar with brew:

* [slides on "Building a reporting sytem with BREW"](https://www.slideshare.net/xavierguardiola/building-a-reporting-sytem-with-brew)
* [learnr blogpost on brew](https://learnr.wordpress.com/2009/09/09/brew-creating-repetitive-reports/)

A brew document is a simple text file with some special tags. `Pandoc.brew` uses only two of them (as building on a personalized version of Jeff's really great brew function):

* `<% ... %>` stands for running inline R commands as usual,
* `<%= ... %>` does pretty much the same but applies pander to the returning R object (instead of `cat` like the original brew function does). So inserting any R object into the tag would return it in Pandoc markdown format, with all possible error/warning messages, etc.

The latter tries to be smart in some ways:

* A code chunk block (`R` commands between the tags) can return any number of values at any part of the block.
* Plots and images are grabbed in the document, rendered to a png file and pander method would result in a Pandoc markdown formatted image link. This means that the image would be rendered/shown/included in the exported document.
* All warnings/messages and errors are recorded in the blocks and returned in the document as footnotes or inline messages.
* All heavy `R` commands (e.g. those taking more then 0.1 sec to evaluate) are cached so rebrewing a report would not result in a coffee break.

Besides this, the custom brew function can do more and also less compared to the original brew package. First of all, the internal caching mechanism of brew has been rewritten for benefits besides improved caching. Quick example:

```{r}
str(Pandoc.brew(text ='Pi equals to `<%= pi %>`. And here are some random data: `<%= runif(10)%>`'))
```

The package bundles some examples for `Pandoc.brew` to let you quickly check its features.
To brew these examples on your machine, run the following commands:

```
Pandoc.brew(system.file('examples/minimal.brew', package='pander'))
Pandoc.brew(system.file('examples/minimal.brew', package='pander'),
            output = tempfile(), convert = 'html')

Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'))
Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'),
                        output = tempfile(), convert = 'html')

Pandoc.brew(system.file('examples/graphs.brew', package='pander'))
Pandoc.brew(system.file('examples/graphs.brew', package='pander'),
                        output = tempfile(), convert = 'html')
```

## General Options

The package comes with a variety of globally adjustable options that have an effect on the result of your reports. A full list of options can be viewed by calling `?panderOptions` or in the online [readme](https://github.com/Rapporter/pander#general-options).

You can query and update these options with the `panderOptions` function:

```{r}
pots <- panderOptions("table.style")
panderOptions("table.style", "simple")
pander(mtcars[1:3, 1:4])
pander(head(iris))
panderOptions("table.style", "grid")
pander(head(iris))
panderOptions("table.style", pots)
```

---
title: "Using pander with knitr"
author: "Roman Tsegelskyi, Gergely Daróczi"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using pander with knitr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(collapse = T, comment = "#>")
library(pander)
library(knitr)
```

While the report generation functionality of `pander` and `knitr` do overlap, we feel that the most powerful way to use `R/knitr/pander` for report generation is to utilize them together. This short vignette aims to explain how to embed `pander` output in reports generated by `knitr`. If you are not aware of `knitr`, be sure to check out the project's [homepage](https://yihui.org/knitr/) for extensive documentation and examples.

One of `knitr`'s most useful features is the ability to convert tables to output format on the fly. For example:

```{r}
head(iris)
knitr::kable(head(iris))
```

However, `kable` table generator is simple by design, and does not capture all the variety of classes that `R` has to offer. For example, `CrossTable` and `tabular` are not supported:

```{r, error = TRUE}
library(descr, quietly = TRUE)
ct <- CrossTable(mtcars$gear, mtcars$cyl)
knitr::kable(ct)
library(tables, quietly = TRUE)
tab <- tabular( (Species + 1) ~ (n=1) + Format(digits=2)*
         (Sepal.Length + Sepal.Width)*(mean + sd), data=iris )
knitr::kable(tab)
```

This is where `pander` comes in handy, as `pander` supports rendering for many popular classes:

```{r}
methods(pander)
```

Also, `pander` is integrated with `knitr` by default. `pander` simply identifies if `knitr` is running in the background, and if so, it uses `capture.output` to return the resulting string as an `knit_asis` object, meaning that you do not need to specify the `results='asis'` option in your knitr chunk:

```{r, error = TRUE}
library(descr, quietly = TRUE)
pander(CrossTable(mtcars$gear, mtcars$cyl))
library(tables, quietly = TRUE)
tab <- tabular( (Species + 1) ~ (n=1) + Format(digits=2)*
         (Sepal.Length + Sepal.Width)*(mean + sd), data=iris )
pander(tab)
```

In a nutshell, this is achieved by modification that whenever you call `pander` inside of a `knitr` document, instead of returning the markdown text to the standard output (the default behavior), `pander` returns a `knit_asis` class object, which renders correctly in the resulting document — without the double comment chars, thus properly rendering the tables in HTML, PDF, or other document formats.

If you don't want the results of `pander` to be converted automatically, just set `knitr.auto.asis` to `FALSE`  using `panderOptions`:

```{r}
panderOptions('knitr.auto.asis', FALSE)
pander(head(iris))
```
```{r}
panderOptions('knitr.auto.asis', TRUE)
```

#### Rendering markdown inside loop/vectorized function

One frequenly asked question is how to use `pander` with `knitr` in a loop or vectorized function. For example, we have 3 tables that we want to render using `lapply`:

```{r}
dfs <- list(mtcars[1:3, 1:4], mtcars[4:6, 1:4], mtcars[7:9, 1:4])
lapply(dfs, pander)
```

As you can see, this doesn't work correctly because `pander` tries to return a `knit_asis` class object when run inside `knitr`,  but for loops/vectorized functions this results in incorrect output. The recommended way to solve this is to disable this behavior by setting `knitr.auto.asis` to `FALSE` using `panderOptions`. However, we also need to tell `knitr` to convert the table on the fly by specifying `results='asis'` in the chunk options:

```{r results='asis'}
panderOptions('knitr.auto.asis', FALSE)
dfs <- list(mtcars[1:3, 1:4], mtcars[4:6, 1:4], mtcars[7:9, 1:4])
invisible(lapply(dfs, pander))
```

---
title: "Rendering tables with pandoc.table"
author: "Roman Tsegelskyi, Gergely Daróczi"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Rendering tables with pandoc.table}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE, message = FALSE}
knitr::opts_chunk$set(collapse = T, comment = "#>")
library(pander)
```

Core functionality of `pander` is centered around `pandoc.table`, which is aimed at rendering tables in `markdown`. In case of 2D tables, `pander` calls `pandoc.table` internally, thus in such cases `pander` and `pandoc.table` support the same argument and in this vignette will be used iterchangingly. `pandoc.table` has a wide variety of options ([highlighting](#highlighting-cells), [styles](#table-styles), etc.) and this vignette aims to give a more detailed overview of the most common options. `pander` comes with a variety of globally adjustable options, which have an effect on the result of your reports. You can query and update these options with the `panderOptions` function.

## Table styles

Since `pander` aims at rendering R objects into [Pandoc](https://pandoc.org/)'s markdown all four (`multiline`, `simple`, `grid`, `rmarkdown`) of  [Pandoc](https://johnmacfarlane.net/pandoc/README.html#tables)'s formats are supported. Users are advised to stick with the default `multiline` style, but if there is a need to change it either specify `style` argument when calling `pander/pandoc.table` or change the default `style` using `panderOptions`.

### multiline tables

`multiline` tables allow headers and table rows to span multiple lines of text (but cells that span multiple columns or rows of the table are not supported). Also note that, for simplicity, line breaks are removed from cells by default, so multiline cells are typically the result of [splitting large cells](#table-and-cell-width) or setting `keep.line.breaks` to `TRUE`:

```{r}
m <- data.frame('Value\n1', 'Value\n2')
colnames(m) <- c('Multiline\nCol1', 'Multiline\nCol2')
pandoc.table(m, keep.line.breaks = TRUE)
m <- mtcars[1:3, 1:4]
pandoc.table(m)
```

### simple tables

`simple` tables are have more compact syntax that all other styles, but don't they support multiline cells:

```{r, error=TRUE}
m <- mtcars[1:3, 1:4]
pandoc.table(m, style = 'simple')
m <- data.frame('Value\n1', 'Value\n2')
colnames(m) <- c('Multiline\nCol1', 'Multiline\nCol2')
pandoc.table(m, keep.line.breaks = TRUE, style='simple')
```

### grid tables

`grid` format is really handy for `emacs` users ([Emacs table mod](https://table.sourceforge.net/)) and it does support block elements (multiple paragraphs, code blocks, lists, etc.) inside cells, but cells can't span multiple columns or rows. Alignments are not supported for grid tables by most parsers, meaning that even though `pander` will produce a table with alignment, it will be lost during conversion from `markdown` to `HTML/PDF/DOCX`.

```{r, error=TRUE}
m <- mtcars[1:3, 1:4]
pandoc.table(m, style = 'grid')
m <- data.frame('Value\n1', 'Value\n2')
colnames(m) <- c('Multiline\nCol1', 'Multiline\nCol2')
pandoc.table(m, keep.line.breaks = TRUE, style='grid')
```

### rmarkdown tables

`rmarkdown` or pipe table format, is often used directly with `knitr`, since it was supported by the first versions of the `markdown` package. It is similar to `simple` table in that multiline cells are also not supported. The beginning and ending pipe characters are optional, but pipes are required between all columns:

```{r, error=TRUE}
m <- mtcars[1:3, 1:4]
pandoc.table(m, style = 'rmarkdown')
m <- data.frame('Value\n1', 'Value\n2')
colnames(m) <- c('Multiline\nCol1', 'Multiline\nCol2')
pandoc.table(m, keep.line.breaks = TRUE, style='rmarkdown')
```

## Cell alignment

`pander` allows users to control cell alignment (`left`, `right` or `center/centre`) in a table directly by setting the `justify` parameter when calling `pander/pandoc.table`. Note that it is possible to specify alignment for each column separately by supplying a vector to justify:

```{r}
pandoc.table(head(iris[,1:3], 2), justify = 'right')
pandoc.table(head(iris[,1:3], 2), justify = c('right', 'center', 'left'))
```

Another  way to define alignment is by using a permanent option `table.alignment.default/table.alignment.rownames` in `panderOptions` (preferred way) or by using `set.alignment` function (legacy way of setting alignment for next table or permanently) which support setting alignment separately for cells and rownames:

```{r}
set.alignment('left', row.names = 'right') # set only for next table since permanent parameter is falce
pandoc.table(mtcars[1:2,  1:5])
```

Interesting application for this functionality is specifying a function that takes the R object as its argument to compute some unique alignment for your table based on e.g. column values or variable types:

```{r}
panderOptions('table.alignment.default',
    function(df)
        ifelse(sapply(df, mean) > 2, 'left', 'right'))
pandoc.table(head(iris[,1:3], 2))
panderOptions('table.alignment.default', 'center')
```
## Highlighting cells

One of the great features of `pander` is the ease of highlighting rows, columns and cells in a table. This is a native `markdown` feature without custom `HTML` or `LaTeX`-only tweaks, so all `HTML/PDF/MS Word/OpenOffice` etc. formats are supported.

This can be achieved by specifying one of the arguments below when calling `pander`/`pandoc.table` or change default style using `panderOptions`:

* emphasize.italics.rows
* emphasize.italics.cols
* emphasize.italics.cells
* emphasize.strong.rows
* emphasize.strong.cols
* emphasize.strong.cells
* emphasize.verbatim.rows
* emphasize.verbatim.cols
* emphasize.verbatim.cells

The `emphasize.italics` helpers would turn the affected cells to *italic*, `emphasize.strong` would apply a **bold** style to the cell and `emphasize.verbatim` would apply a `verbatim` style to the cell. A cell can be also *italic*,  **bold** and `verbatim` at the same time.

Those functions and arguments ending in rows or cols take a vector (like which columns or rows to emphasize in a table), while the cells argument take either a vector (for one dimensional "tables") or an array-like data structure with two columns holding row and column indexes of cells to be emphasized -- just like what `which(..., arr.ind = TRUE)` returns:

```{r}
t <- mtcars[1:3, 1:5]
emphasize.italics.cols(1)
emphasize.italics.rows(1)
emphasize.strong.cells(which(t > 20, arr.ind = TRUE))
pandoc.table(t)
pandoc.table(t, emphasize.verbatim.rows = 1, emphasize.strong.cells = which(t > 20, arr.ind = TRUE))
```

For more elaborative examples, please see our blog post - [Highlight cells in markdown tables](http://blog.rapporter.net/2013/04/hihglight-cells-in-markdown-tables.html).

## Table and Cell width

`pander/pandoc.table` is able to deal with wide tables. Ever had an issue in `LaTeX` or `MS Word` when trying to print a correlation matrix of 40 variables? This problem is carefully addressed with `split.table` parameter:

```{r}
pandoc.table(mtcars[1:2, ], style = "grid", caption = "Wide table to be split!")
```

`split.table` defaults to 80 characters and to turn it off, set `split.table` to `Inf`:

```{r}
pandoc.table(mtcars[1:2, ], style = "grid",
             caption = "Wide table to be split!", split.table = Inf)
```

Also, `pander` tries to split too wide cells into multiline cells. The maximum number of characters in a cell is specified by the `split.cells` parameter (defaults to 30), which can be a single value, vector (values for each column separately) and relative vector (percentages of `split.tables` parameter). Please not that this only works for `multiline` and `grid` tables:

```{r, error=TRUE}
df <- data.frame(a = 'Lorem ipsum', b = 'dolor sit', c = 'amet')
pandoc.table(df, split.cells = 5)
pandoc.table(df, split.cells = c(5, 20, 5))
pandoc.table(df, split.cells = c("80%", "10%", "10%"))
pandoc.table(df, split.cells = 5, style = 'simple')
```

In some cases it is also useful to split too long words with hyphens, and `pander` uses `sylly` functionality for that. Just specify `use.hyphening` argument and have `sylly` installed:

```{r}
pandoc.table(data.frame(baz = 'foobar', foo='accoutrements'),
             use.hyphening = TRUE, split.cells = 3)
```

## Rounding and number formatting

`pander/pandoc.table` deals with formatting numbers by having 4 parameters:

* `round` to the number of decimal places.
* `digits` to specify how many significant digits are to be used for numeric
* `decimal.mark/big.mark` to specify character for decimal point/orders of magnitude

`round` and `digits` parameter can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.

Now let's get to some examples:
```{r}
r <- matrix(c(283764.97430, 29.12345678901, -7.1234, -100.1), ncol = 2)
pandoc.table(r, round = 2)
pandoc.table(r, round = c(4,2)) # vector for each column
pandoc.table(r, digits = 2)
pandoc.table(r, digits = c(1, 5)) # vector for each column
pandoc.table(r, big.mark = ',')
pandoc.table(r, decimal.mark = ',')
```
## Other options

Functionality described in other sections is most notable, but `pander/pandoc.table` also has smaller nifty features that are worth mentioning:

* `plain.ascii` - allows to have the output without `markdown` markup:
```{r}
pandoc.table(mtcars[1:3, 1:4])
pandoc.table(mtcars[1:3, 1:4], plain.ascii = TRUE)
```

* `caption` - set caption (string) to be shown under the table:
```{r}
pandoc.table(mtcars[1:3, 1:5], style = "grid", caption = "My caption!")
```

* `missing` - set a string to replace missing values:
```{r}
m <- mtcars[1:3, 1:5]
m$mpg <- NA
pandoc.table(m, missing = '?')
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.lm}
\alias{pander.lm}
\title{Pander method for summary.lm class}
\usage{
\method{pander}{lm}(x, caption = attr(x, "caption"), covariate.labels,
  omit, ...)
}
\arguments{
\item{x}{a summary.glm object}

\item{caption}{caption (string) to be shown under the table}

\item{covariate.labels}{vector to replace covariate lables in the table}

\item{omit}{vector of variable to omit for priting in resulting table}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints a summary.lm object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{multitable}
\alias{multitable}
\title{Create a multitable used for rendering objects from rms package}
\usage{
multitable(v)
}
\arguments{
\item{v}{list of vectors/lists to be merge in the table}
}
\value{
data.frame in specified format
}
\description{
When ols/lrm/orm from rms package get rendered, main statistics are group in table of tables.
Since pandoc doesn't support row or col-span,
we chose to group those statistics in a column each.
This function takes care of that
}
\examples{
pander:::multitable(list(list(a=1, b=2),list(c=3, d=4)))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.gtable}
\alias{pander.gtable}
\title{Pander method for gtable class}
\usage{
\method{pander}{gtable}(x, zsort = FALSE, ...)
}
\arguments{
\item{x}{an gtable object}

\item{zsort}{Sort by z values? Default \code{FALSE}}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Renders an gtable object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evals.R
\name{eval.msgs}
\alias{eval.msgs}
\title{Evaluate with messages}
\usage{
eval.msgs(src, env = NULL, showInvisible = FALSE,
  graph.unify = evalsOptions("graph.unify"))
}
\arguments{
\item{src}{character values containing R code}

\item{env}{environment where evaluation takes place. If not set (by default), a new temporary environment is created.}

\item{showInvisible}{return \code{invisible} results?}

\item{graph.unify}{should \code{eval.msgs} try to unify the style of (\code{lattice} and \code{ggplot2}) plots? If set to \code{TRUE} (by default), some \code{panderOptions()} would apply. Please note that this argument has no effect on \code{base} plots, use \code{evals} instead.}
}
\value{
a list of parsed elements each containing: \code{src} (the command run), \code{result} (R object: \code{NULL} if nothing returned), \code{print}ed \code{output}, \code{type} (class of returned object if any), informative/wawrning and error messages (if any returned by the command run, otherwise set to \code{NULL}) and possible \code{stdout}t value. See Details above.
}
\description{
This function takes text(s) of R code and \code{eval}s all at one run - returning a list with four elements. See \code{Details}.
}
\details{
\code{eval.msgs} returns a detailed list of the result of evaluation:
\itemize{
    \item \emph{src} - character vector of specified R code.
    \item \emph{result} - result of evaluation. \code{NULL} if nothing is returned. If any R code returned an R object while evaluating then the \emph{last} R object will be returned as a raw R object. If a graph is plotted in the end of the given R code (remember: \emph{last} R object), it would be automatically printed (see e.g. \code{lattice} and \code{ggplot2}).
     \item \emph{output} - character vector of printed version (\code{capture.output}) of \code{result}
    \item \emph{type} - class of generated output. 'NULL' if nothing is returned, 'error' if some error occurred.
    \item \emph{msg} - possible messages grabbed while evaluating specified R code with the following structure:
    \itemize{
        \item \emph{messages} - character vector of possible diagnostic message(s)
        \item \emph{warnings} - character vector of possible warning message(s)
        \item \emph{errors} - character vector of possible error message(s)
    }
    \item \emph{stdout} - character vector of possibly printed texts to standard output (console)
}
}
\examples{
\dontrun{
eval.msgs('1:5')
eval.msgs('x <- 1:5')
eval.msgs('lm(mtcars$hp ~ mtcars$wt)')

## plots
eval.msgs('plot(runif(100))')
eval.msgs('histogram(runif(100))')

## error handling
eval.msgs('runiff(23)')
eval.msgs('runif is a nice function')
eval.msgs('no.R.object.like.that')

## messages
eval.msgs(c('message("FOO")', '1:2'))
eval.msgs(c('warning("FOO")', '1:2'))
eval.msgs(c('message("FOO");message("FOO");warning("FOO")', '1:2'))
eval.msgs('warning("d");warning("f");1')

## stdout
eval.msgs('cat("writing to console")')
eval.msgs('cat("writing to console");1:4')
}
}
\seealso{
\code{\link{evals}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evals.R
\name{redrawPlot}
\alias{redrawPlot}
\title{Redraw a recordedplot, grid, trellis, or ggplot2 plot.}
\usage{
redrawPlot(rec_plot)
}
\arguments{
\item{rec_plot}{the plot object to redraw}
}
\description{
This function redraws the plot represented by \code{rec_plot}. It can redraw grid/trellis/ggplot2/etc plots, as well as \code{recordedplot} objects. For \code{recordedplot} objects it acts as a wrapper around \code{replayPlot} with memory tweaks to fix native symbol address errors when the recordedplot was loaded from an rda/rds file.
}
\references{
Thanks to Jeroen Ooms \url{https://stat.ethz.ch/pipermail/r-devel/2012-January/062973.html}, JJ Allaire \url{https://github.com/rstudio/rstudio/commit/eb5f6f1db4717132c2ff111f068ffa6e8b2a5f0b}, and Gabriel Becker.
}
\seealso{
\code{\link{redraw.recordedplot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{cache.off}
\alias{cache.off}
\alias{cache.on}
\title{Toggle cache}
\usage{
cache.on()

cache.off()
}
\description{
This function is just a wrapper around \code{\link{evalsOptions}} to switch pander's cache on or off easily, which might be handy in some brew documents to prevent repetitive strain injury :)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.formula.return}
\alias{pandoc.formula.return}
\alias{pandoc.formula}
\title{Formulas}
\usage{
pandoc.formula.return(x, text = NULL, max.width = 80, caption,
  add.line.breaks = FALSE, ...)
}
\arguments{
\item{x}{formula}

\item{text}{text to be written before result in the same line. Typically used by calls from other functions in the package}

\item{max.width}{maximum width in characters per line}

\item{caption}{caption (string) to be shown under the formula}

\item{add.line.breaks}{if to add 2 line breaks after formula}

\item{...}{extra arguments passed by from parent call, disregarded}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Pandoc's mardown formula.
}
\examples{
pandoc.formula(y ~ x)
pandoc.formula(formula(paste('y ~ ', paste0('x', 1:12, collapse = ' + '))))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.summary.glm}
\alias{pander.summary.glm}
\title{Pander method for summary.glm class}
\usage{
\method{pander}{summary.glm}(x, caption = attr(x, "caption"),
  covariate.labels, omit, summary = TRUE, ...)
}
\arguments{
\item{x}{an summary.glm object}

\item{caption}{caption (string) to be shown under the table}

\item{covariate.labels}{vector to replace covariate lables in the table}

\item{omit}{vector of variable to omit for priting in resulting table}

\item{summary}{(defaut:\code{TRUE}) if used for summary.lm or lm}

\item{...}{optional parameters passed to special methods and/or raw \code{pandoc.*} functions}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Prints a summary.glm object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander_return}
\alias{pander_return}
\title{Pander and capture output}
\usage{
pander_return(...)
}
\arguments{
\item{...}{everything passed to \code{pander}}
}
\description{
This is a wrapper function around \code{pander} but instead of printing to \code{stdout}, this function returns a character vector of the captured lines.
}
\seealso{
pander
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evals.R
\name{redraw.recordedplot}
\alias{redraw.recordedplot}
\title{Redraws plot saved in file}
\usage{
redraw.recordedplot(file)
}
\arguments{
\item{file}{path and name of an rds file containing a plot object to be redrawn}
}
\description{
This function is a wrapper around \code{redrawPlot}.
}
\references{
Thanks to Jeroen Ooms \url{https://stat.ethz.ch/pipermail/r-devel/2012-January/062973.html}, JJ Allaire \url{https://github.com/rstudio/rstudio/commit/eb5f6f1db4717132c2ff111f068ffa6e8b2a5f0b}, and Gabriel Becker.
}
\seealso{
\code{\link{evals}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.describe}
\alias{pander.describe}
\title{Pander method for describe class}
\usage{
\method{pander}{describe}(x, caption = attr(x, "caption"),
  digits = panderOptions("digits"), ...)
}
\arguments{
\item{x}{an describe object}

\item{caption}{caption (string) to be shown under the table}

\item{digits}{number of digits of precision}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints a describe object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{check_caption}
\alias{check_caption}
\title{Check if caption is valid}
\usage{
check_caption(caption)
}
\arguments{
\item{caption}{R object to check}
}
\value{
boolean
}
\description{
Check if caption is valid
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.table.return}
\alias{pandoc.table.return}
\alias{pandoc.table}
\title{Create a table}
\usage{
pandoc.table.return(t, caption, digits = panderOptions("digits"),
  decimal.mark = panderOptions("decimal.mark"),
  big.mark = panderOptions("big.mark"), round = panderOptions("round"),
  missing = panderOptions("missing"), justify, style = c("multiline",
  "grid", "simple", "rmarkdown", "jira"),
  split.tables = panderOptions("table.split.table"),
  split.cells = panderOptions("table.split.cells"),
  keep.trailing.zeros = panderOptions("keep.trailing.zeros"),
  keep.line.breaks = panderOptions("keep.line.breaks"),
  plain.ascii = panderOptions("plain.ascii"),
  use.hyphening = panderOptions("use.hyphening"), row.names, col.names,
  emphasize.rownames = panderOptions("table.emphasize.rownames"),
  emphasize.rows, emphasize.cols, emphasize.cells, emphasize.strong.rows,
  emphasize.strong.cols, emphasize.strong.cells, emphasize.italics.rows,
  emphasize.italics.cols, emphasize.italics.cells, emphasize.verbatim.rows,
  emphasize.verbatim.cols, emphasize.verbatim.cells, ...)
}
\arguments{
\item{t}{data frame, matrix or table}

\item{caption}{caption (string) to be shown under the table}

\item{digits}{passed to \code{format}. Can be a vector specifying values for each column (has to be the same length as number of columns).}

\item{decimal.mark}{passed to \code{format}}

\item{big.mark}{passed to \code{format}}

\item{round}{passed to \code{round}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.}

\item{missing}{string to replace missing values}

\item{justify}{defines alignment in cells passed to \code{format}. Can be \code{left}, \code{right} or \code{centre}, which latter can be also spelled as \code{center}. Defaults to \code{centre}. Can be abbreviated to a string consisting of the letters \code{l}, \code{c} and \code{r} (e.g. 'lcr' instead of c('left', 'centre', 'right').}

\item{style}{which Pandoc style to use: \code{simple}, \code{multiline}, \code{grid} or \code{rmarkdown}}

\item{split.tables}{where to split wide tables to separate tables. The default value (\code{80}) suggests the conventional number of characters used in a line, feel free to change (e.g. to \code{Inf} to disable this feature) if you are not using a VT100 terminal any more :)}

\item{split.cells}{where to split cells' text with line breaks. Default to \code{30}, to disable set to \code{Inf}. Can be also supplied as a vector, for each cell separately (if length(split.cells) == number of columns + 1, then first value in split.cells if for row names, and others are for columns). Supports relative (percentage) parameters in combination with split.tables.}

\item{keep.trailing.zeros}{to show or remove trailing zeros in numbers on a column basis width}

\item{keep.line.breaks}{(default: \code{FALSE}) if to keep or remove line breaks from cells in a table}

\item{plain.ascii}{(default: \code{FALSE}) if output should be in plain ascii (without markdown markup) or not}

\item{use.hyphening}{boolean (default: \code{FALSE}) if try to use hyphening when splitting large cells according to table.split.cells. Requires \pkg{sylly}.}

\item{row.names}{if \code{FALSE}, row names are suppressed. A character vector of row names can also be specified here. By default, row names are included if \code{rownames(t)} is neither
\code{NULL} nor identical to \code{1:nrow(x)}}

\item{col.names}{a character vector of column names to be used in the table}

\item{emphasize.rownames}{boolean (default: \code{TRUE}) if row names should be highlighted}

\item{emphasize.rows}{deprecated for \code{emphasize.italics.rows} argument}

\item{emphasize.cols}{deprecated for \code{emphasize.italics.cols} argument}

\item{emphasize.cells}{deprecated for \code{emphasize.italics.cells} argument}

\item{emphasize.strong.rows}{see \code{emphasize.italics.rows} but in bold}

\item{emphasize.strong.cols}{see \code{emphasize.italics.cols} but in bold}

\item{emphasize.strong.cells}{see \code{emphasize.italics.cells} but in bold}

\item{emphasize.italics.rows}{a vector for a two dimensional table specifying which rows to emphasize}

\item{emphasize.italics.cols}{a vector for a two dimensional table specifying which cols to emphasize}

\item{emphasize.italics.cells}{a vector for one-dimensional tables or a matrix like structure with two columns for row and column indexes to be emphasized in two dimensional tables. See e.g. \code{which(..., arr.ind = TRUE)}}

\item{emphasize.verbatim.rows}{see \code{emphasize.italics.rows} but in verbatim}

\item{emphasize.verbatim.cols}{see \code{emphasize.italics.cols} but in verbatim}

\item{emphasize.verbatim.cells}{see \code{emphasize.italics.cells} but in verbatim}

\item{...}{unsupported extra arguments directly placed into \code{/dev/null}}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call \code{pandoc.table.return} instead.
}
\description{
Creates a Pandoc's markdown style table with optional caption and some other tweaks. See 'Details' below.
}
\details{
This function takes any tabular data as its first argument and will try to make it pretty like: rounding and applying \code{digits} and custom \code{decimal.mark} to numbers, auto-recognizing if row names should be included, setting alignment of cells and dropping trailing zeros by default.

\code{pandoc.table} also tries to split large cells with line breaks or even the whole table to separate parts on demand. Other arguments lets the use to highlight some rows/cells/cells in the table with italic or bold text style.

For more details please see the parameters above and passed arguments of \code{\link{panderOptions}}.
}
\note{
If \code{caption} is missing, then the value is first checked in \code{t} object's \code{caption} attribute and if not found in an internal buffer set by \code{link{set.caption}}. \code{justify} parameter works similarly, see \code{\link{set.alignment}} for details.
}
\examples{
pandoc.table(mtcars)

# caption
pandoc.table(mtcars, 'Motor Trend Car Road Tests')

# other input/output formats
pandoc.table(mtcars[, 1:3], decimal.mark = ',')
pandoc.table(mtcars[, 1:3], decimal.mark = ',', justify = 'right')
pandoc.table(matrix(sample(1:1000, 25), 5, 5))
pandoc.table(matrix(runif(25), 5, 5))
pandoc.table(matrix(runif(25), 5, 5), digits = 5)
pandoc.table(matrix(runif(25),5,5), round = 1)
pandoc.table(table(mtcars$am))
pandoc.table(table(mtcars$am, mtcars$gear))
pandoc.table(table(state.division, state.region))
pandoc.table(table(state.division, state.region), justify = 'centre')

m <- data.frame(a = c(1, -500, 10320, 23, 77),
  b = runif(5),
  c = c('a', 'bb', 'ccc', 'dddd', 'eeeee'))
pandoc.table(m)
pandoc.table(m, justify = c('right', 'left', 'centre'))
pandoc.table(m, justify = 'rlc') # Same as upper statement

## splitting up too wide tables
pandoc.table(mtcars)
pandoc.table(mtcars, caption = 'Only once after the first part!')

## tables with line breaks in cells
## NOTE: line breaks are removed from table content in case keep.line.breaks is set to FALSE
## and added automatically based on "split.cells" parameter!
t <- data.frame(a = c('hundreds\\nof\\nmouses', '3 cats'), b=c('FOO is nice', 'BAR\\nBAR2'))
pandoc.table(t)
pandoc.table(t, split.cells = 5)

## exporting tables in other Pandoc styles
pandoc.table(m)
pandoc.table(m, style = "grid")
pandoc.table(m, style = "simple")
pandoc.table(t, style = "grid")
pandoc.table(t, style = "grid", split.cells = 5)
tryCatch(pandoc.table(t, style = "simple", split.cells = 5),
  error = function(e) 'Yeah, no newline support in simple tables')

## highlight cells
t <- mtcars[1:3, 1:5]
pandoc.table(t$mpg, emphasize.italics.cells = 1)
pandoc.table(t$mpg, emphasize.strong.cells = 1)
pandoc.table(t$mpg, emphasize.italics.cells = 1, emphasize.strong.cells = 1)
pandoc.table(t$mpg, emphasize.italics.cells = 1:2)
pandoc.table(t$mpg, emphasize.strong.cells = 1:2)
pandoc.table(t, emphasize.italics.cells = which(t > 20, arr.ind = TRUE))
pandoc.table(t, emphasize.italics.cells = which(t == 6, arr.ind = TRUE))
pandoc.table(t, emphasize.verbatim.cells = which(t == 6, arr.ind = TRUE))
pandoc.table(t, emphasize.verbatim.cells = which(t == 6, arr.ind = TRUE),
 emphasize.italics.rows = 1)
## with helpers
emphasize.cols(1)
emphasize.rows(1)
pandoc.table(t)

emphasize.strong.cells(which(t > 20, arr.ind = TRUE))
pandoc.table(t)

### plain.ascii
pandoc.table(mtcars[1:3, 1:3], plain.ascii = TRUE)

### keep.line.breaks
x <- data.frame(a="Pandoc\\nPackage")
pandoc.table(x)
pandoc.table(x, keep.line.breaks = TRUE)

## split.cells
x <- data.frame(a = "foo bar", b = "foo bar")
pandoc.table(x, split.cells = 4)
pandoc.table(x, split.cells = 7)
pandoc.table(x, split.cells = c(4, 7))
pandoc.table(x, split.cells = c("20\%", "80\%"), split.tables = 30)

y <- c("aa aa aa", "aaa aaa", "a a a a a", "aaaaa", "bbbb bbbb bbbb", "bb bbb bbbb")
y <- matrix(y, ncol = 3, nrow = 2)
rownames(y) <- c("rowname one", "rowname two")
colnames(y) <- c("colname one", "colname two", "colname three")
pandoc.table(y, split.cells = 2)
pandoc.table(y, split.cells = 6)
pandoc.table(y, split.cells = c(2, 6, 10))
pandoc.table(y, split.cells = c(2, Inf, Inf))

## first value used for rownames
pander(y, split.cells = c(5, 2, Inf, Inf))
pandoc.table(y, split.cells = c(5, 2, Inf, 5, 3, 10))

## when not enough reverting to default values
pandoc.table(y, split.cells = c(5, 2))

## split.cells with hyphenation
x <- data.frame(a = "Can be also supplied as a vector, for each cell separately",
       b = "Can be also supplied as a vector, for each cell separately")
pandoc.table(x, split.cells = 10, use.hyphening = TRUE)
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}
\seealso{
\code{\link{set.caption}}, \code{\link{set.alignment}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.date.return}
\alias{pandoc.date.return}
\alias{pandoc.date}
\title{Dates}
\usage{
pandoc.date.return(x, inline = TRUE, simplified = FALSE, ...)
}
\arguments{
\item{x}{date or vector of dates}

\item{inline}{if to render vector of dates as inline paragraph or not (as list)}

\item{simplified}{if just add date formatting to vector of dates}

\item{...}{extra arguments passed by from parent call, disregarded}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Pandoc's mardown date.
}
\examples{
pandoc.date(Sys.Date())
pandoc.date(Sys.Date() - 1:10)
pandoc.date(Sys.Date() - 1:10, inline = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{add.blank.lines}
\alias{add.blank.lines}
\title{Add trailing and leading blank line}
\usage{
add.blank.lines(x)
}
\arguments{
\item{x}{character vector}
}
\description{
Adds a line break before \emph{and} after the character string(s).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{remove.extra.newlines}
\alias{remove.extra.newlines}
\title{Remove more then two joined newlines}
\usage{
remove.extra.newlines(x)
}
\arguments{
\item{x}{character vector}
}
\description{
Remove more then two joined newlines
}
\examples{
remove.extra.newlines(c('\\n\\n\\n', '\\n\\n', '\\n'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.R
\name{openFileInOS}
\alias{openFileInOS}
\title{Open file}
\usage{
openFileInOS(f)
}
\arguments{
\item{f}{file (with full path)}
}
\description{
Tries to open a file with operating system's default program.
}
\references{
This function is a fork of David Hajage's \code{convert} function: \url{https://github.com/eusebe/ascii/blob/master/R/export.r}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.data.table}
\alias{pander.data.table}
\title{Pander method for data.table class}
\usage{
\method{pander}{data.table}(x, caption = attr(x, "caption"),
  keys.as.row.names = TRUE, ...)
}
\arguments{
\item{x}{a data.table object}

\item{caption}{caption (string) to be shown under the table}

\item{keys.as.row.names}{controls whether to use data.table key as row names when calling pandoc.table}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints a data.table object in Pandoc's markdown. Data.tables drop attributes (like row names) when called.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{set.alignment}
\alias{set.alignment}
\title{Sets alignment for tables}
\usage{
set.alignment(default = panderOptions("table.alignment.default"),
  row.names = panderOptions("table.alignment.rownames"),
  permanent = FALSE)
}
\arguments{
\item{default}{character vector which length equals to one (would be repeated \code{n} times) ot \code{n} - where \code{n} equals to the number of columns in the following table}

\item{row.names}{string holding the alignment of the (optional) row names}

\item{permanent}{(default \code{FALSE}) if alignment is permanent (for all future tables) or not. It's cleaner to use \code{panderOptions} instead.}
}
\description{
This is a helper function to update the alignment (\code{justify} parameter in \code{pandoc.table}) of the next returning table. Possible values are: \code{centre} or \code{center}, \code{right}, \code{left}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get.caption}
\alias{get.caption}
\title{Get caption}
\usage{
get.caption()
}
\value{
stored caption as string
}
\description{
Get caption from temporary environment and truncates that
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.anova}
\alias{pander.anova}
\title{Pander method for anova class}
\usage{
\method{pander}{anova}(x, caption = attr(x, "caption"),
  add.significance.stars = FALSE, ...)
}
\arguments{
\item{x}{an anova object}

\item{caption}{caption (string) to be shown under the table}

\item{add.significance.stars}{if significance stars should be shown for P value}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an anova object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{panderOptions}
\alias{panderOptions}
\title{Querying/setting pander option}
\usage{
panderOptions(o, value)
}
\arguments{
\item{o}{option name (string). See below.}

\item{value}{value to assign (optional)}
}
\description{
To list all \code{pander} options, just run this function without any parameters provided. To query only one value, pass the first parameter. To set that, use the \code{value} parameter too.
}
\details{
The following \code{pander} options are available:

\itemize{
     \item \code{digits}: numeric (default: \code{2}) passed to \code{format}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
     \item \code{decimal.mark}: string (default: \code{.}) passed to \code{format}
     \item \code{formula.caption.prefix}: string (default: \code{'Formula: '}) passed to \code{\link{pandoc.formula}} to be used as caption prefix. Be sure about what you are doing if changing to other than \code{'Formula: '} or \code{':'}.
     \item \code{big.mark}: string (default: '') passed to \code{format}.
     \item \code{round}: numeric (default: \code{Inf}) passed to \code{round}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.
     \item \code{keep.trailing.zeros}: boolean (default: \code{FALSE}) to show or remove trailing zeros in numbers
     \item \code{keep.line.breaks}: boolean (default: \code{FALSE}) to keep or remove line breaks from cells in a table
     \item \code{missing}: string (default: \code{NA}) to replace missing values in vectors, tables etc.
     \item \code{date}: string (default: \code{'\%Y/\%m/\%d \%X'}) passed to \code{format} when printing dates (\code{POSIXct} or \code{POSIXt})
     \item \code{header.style}: \code{'atx'} or \code{'setext'} passed to \code{\link{pandoc.header}}
     \item \code{list.style}: \code{'bullet'}, \code{'ordered'} or \code{'roman'} passed to \code{\link{pandoc.list}}. Please not that this has no effect on \code{pander} methods.
     \item \code{table.style}: \code{'multiline'}, \code{'grid'}, \code{'simple'} or \code{'rmarkdown'} passed to \code{\link{pandoc.table}}
     \item \code{table.emphasize.rownames}: boolean (default: \code{TRUE}) if row names should be highlighted
     \item \code{table.split.table}: numeric passed to \code{\link{pandoc.table}} and also affects \code{pander} methods. This option tells \code{pander} where to split too wide tables. The default value (\code{80}) suggests the conventional number of characters used in a line, feel free to change (e.g. to \code{Inf} to disable this feature) if you are not using a VT100 terminal any more :)
     \item \code{table.split.cells}: numeric or numeric vector (default: \code{30}) passed to \code{\link{pandoc.table}} and also affects \code{pander} methods. This option tells \code{pander} where to split too wide cells with line breaks. Numeric vector specifies values for cells separately. Set \code{Inf} to disable.
     \item \code{table.caption.prefix}: string (default: \code{'Table: '}) passed to \code{\link{pandoc.table}} to be used as caption prefix. Be sure about what you are doing if changing to other than \code{'Table: '} or \code{':'}.
     \item \code{table.continues}: string (default: \code{'Table continues below'}) passed to \code{\link{pandoc.table}} to be used as caption for long (split) without a use defined caption
     \item \code{table.continues.affix}: string (default: \code{'(continued below)'}) passed to \code{\link{pandoc.table}} to be used as an affix concatenated to the user defined caption for long (split) tables
     \item \code{table.alignment.default}: string (default: \code{centre}) that defines the default alignment of cells. Can be \code{left}, \code{right} or \code{centre} that latter can be also spelled as \code{center}.
     \item \code{table.alignment.rownames}: string (default: \code{centre}) that defines the alignment of rownames in tables. Can be \code{left}, \code{right} or \code{centre} that latter can be also spelled as \code{center}.
     \item \code{use.hyphening}: boolean (default: \code{FALSE}) if try to use hyphening when splitting large cells according to table.split.cells. Requires \pkg{sylly}.
     \item \code{evals.messages}: boolean (default: \code{TRUE}) passed to \code{evals}' \code{pander} method specifying if messages should be rendered
     \item \code{p.wrap}: a string (default: \code{'_'}) to wrap vector elements passed to \code{p} function
     \item \code{p.sep}: a string (default: \code{', '}) with the main separator passed to \code{p} function
     \item \code{p.copula}: a string (default: \code{' and '}) with ending separator passed to \code{p} function
     \item \code{plain.ascii}: boolean (default: \code{FALSE}) to define if output should be in plain ascii or not
     \item \code{graph.nomargin}: boolean (default: \code{TRUE}) if trying to keep plots' margins at minimal
     \item \code{graph.fontfamily}: string (default: \code{'sans'}) specifying the font family to be used in images. Please note, that using a custom font on Windows requires \code{grDevices:::windowsFonts} first.
     \item \code{graph.fontcolor}: string (default: \code{'black'}) specifying the default font color
     \item \code{graph.fontsize}: numeric (default: \code{12}) specifying the \emph{base} font size in pixels. Main title is rendered with \code{1.2} and labels with \code{0.8} multiplier.
     \item \code{graph.grid}: boolean (default: \code{TRUE}) if a grid should be added to the plot
     \item \code{graph.grid.minor}: boolean (default: \code{TRUE}) if a miner grid should be also rendered
     \item \code{graph.grid.color}: string (default: \code{'grey'}) specifying the color of the rendered grid
     \item \code{graph.grid.lty}: string (default: \code{'dashed'}) specifying the line type of grid
     \item \code{graph.boxes}: boolean (default: \code{FALSE}) if to render a border around of plot (and e.g. around strip)
     \item \code{graph.legend.position}: string (default: \code{'right'}) specifying the position of the legend: 'top', 'right', 'bottom' or 'left'
     \item \code{graph.background}: string (default: \code{'white'}) specifying the plots main background's color
     \item \code{graph.panel.background}: string (default: \code{'transparent'}) specifying the plot's main panel background. Please \emph{note}, that this option is not supported with \code{base} graphics.
     \item \code{graph.colors}: character vector of default color palette (defaults to a colorblind theme: \url{http://jfly.iam.u-tokyo.ac.jp/color/}). Please \emph{note} that this update work with \code{base} plots by appending the \code{col} argument to the call if not set.
     \item \code{graph.color.rnd}: boolean (default: \code{FALSE}) specifying if the palette should be reordered randomly before rendering each plot to get colorful images
     \item \code{graph.axis.angle}: numeric (default: \code{1}) specifying the angle of axes' labels. The available options are based on \code{par(les)} and sets if the labels should be:
     \itemize{
             \item \code{1}: parallel to the axis,
             \item \code{2}: horizontal,
             \item \code{3}: perpendicular to the axis or
             \item \code{4}: vertical.
     }
     \item \code{graph.symbol}: numeric (default: \code{1}) specifying a symbol (see the \code{pch} parameter of \code{par})
     \item \code{knitr.auto.asis}: boolean (default: \code{TRUE}) if the results of \code{pander} should be considered as \code{'asis'} in \code{knitr}. Equals to specifying \code{results='asis'} in the R chunk, so thus there is no need to do so if set to \code{TRUE}.
     \item \code{pandoc.binary}: full path of \code{pandoc}'s binary. By default, \code{pandoc} is in the path.
}
}
\examples{
\dontrun{
panderOptions()
panderOptions('digits')
panderOptions('digits', 5)
}
}
\seealso{
\code{\link{evalsOptions}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.orm}
\alias{pander.orm}
\title{Prints an orm object from rms package in Pandoc's markdown.}
\usage{
\method{pander}{orm}(x, coefs = TRUE, intercepts = x$non.slopes < 10,
  ...)
}
\arguments{
\item{x}{an orm object}

\item{coefs}{if to the table of model coefficients, standard errors, etc. default(\code{TRUE})}

\item{intercepts}{if to print intercepts, by default, intercepts are only printed if there are fewer than 10 of them}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an orm object from rms package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{p}
\alias{p}
\title{Inline Printing}
\usage{
p(x, wrap = panderOptions("p.wrap"), sep = panderOptions("p.sep"),
  copula = panderOptions("p.copula"), limit = Inf,
  keep.trailing.zeros = panderOptions("keep.trailing.zeros"),
  missing = panderOptions("missing"), digits = panderOptions("digits"),
  round = panderOptions("round"))
}
\arguments{
\item{x}{an atomic vector to get merged for inline printing}

\item{wrap}{a string to wrap vector elements (uses value set in \code{p.wrap} option: \code{'_'} by default, which is a markdown-friendly wrapper and it puts the string in \emph{italic})}

\item{sep}{a string with the main separator, i.e. the one that separates all vector elements but the last two (uses the value set in \code{p.sep} option - \code{','} by default)}

\item{copula}{a string with ending separator - the one that separates the last two vector elements (uses the value set in \code{p.copula} option, \code{'and'} by default)}

\item{limit}{maximum character length (defaults to \code{Inf}initive  elements)}

\item{keep.trailing.zeros}{to show or remove trailing zeros in numbers}

\item{missing}{string to replace missing values}

\item{digits}{numeric (default: 2) passed to format}

\item{round}{numeric (default: Inf) passed to round}
}
\value{
a string with concatenated vector contents
}
\description{
\code{\link{p}} merges elements of a vector in one string for the sake of pretty inline printing. Default parameters are read from appropriate \code{option} values (see argument description for details). This function allows you to put the results of an expression that yields a variable \emph{inline}, by wrapping the vector elements with the string provided in \code{wrap}, and separating elements by main and ending separator (\code{sep} and \code{copula}). In case of a two-length vector, value specified in \code{copula} will be used as a separator. You can also control the length of provided vector by altering an integer value specified in \code{limit} argument (defaults to \code{Inf}).
}
\examples{
p(c('fee', 'fi', 'foo', 'fam'))
# [1] '_fee_, _fi_, _foo_ and _fam_'
p(1:3, wrap = '')
# [1] '1, 2 and 3'
p(LETTERS[1:5], copula = 'and the letter')
# [1] '_A_, _B_, _C_, _D_ and the letter _E_'
p(c('Thelma', 'Louise'), wrap = '', copula = '&')
# [1] 'Thelma & Louise'
}
\references{
This function was moved from \code{rapport} package: \url{http://rapport-package.info}.
}
\author{
Aleksandar Blagotic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.R
\name{Pandoc.convert}
\alias{Pandoc.convert}
\title{Converts Pandoc to other format}
\usage{
Pandoc.convert(f, text, format = "html", open = TRUE, options = "",
  footer = FALSE, proc.time, portable.html = TRUE,
  pandoc.binary = panderOptions("pandoc.binary"))
}
\arguments{
\item{f}{Pandoc's markdown format file path. If URL is provided then the generated file's path is \code{tempfile()} but please bear in mind that this way only images with absolute path would shown up in the document.}

\item{text}{Pandoc's markdown format character vector. Treated as the content of \code{f} file - so the \code{f} parameter is ignored. The generated file's path is \code{tempfile()}.}

\item{format}{required output format. For all possible values here check out Pandoc homepage: \url{http://johnmacfarlane.net/pandoc/}}

\item{open}{try to open converted document with operating system's default program}

\item{options}{optionally passed arguments to Pandoc (instead of \code{pander}'s default)}

\item{footer}{add footer to document with meta-information}

\item{proc.time}{optionally passed number in seconds which would be shown in the generated document's footer}

\item{portable.html}{instead of using local files, rather linking JS/CSS files to an online CDN for portability and including base64-encoded images if converting to \code{HTML} without custom \code{options}}

\item{pandoc.binary}{path to \code{pandoc}'s binary if not found in the path}
}
\value{
Converted file's path.
}
\description{
Calling John MacFarlane's great program to convert specified file (see \code{f} parameter below) or character vector {see \code{text} paramater} to other formats like \code{HTML}, \code{pdf}, \code{docx}, \code{odt} etc.
}
\note{
This function depends on \code{Pandoc} which should be pre-installed on user's machine. See the \code{INSTALL} file of the package.
}
\examples{
\dontrun{
Pandoc.convert(text = c('# Demo', 'with a paragraph'))
Pandoc.convert('http://rapporter.github.io/pander/minimal.md')
# Note: the generated HTML is not showing images with relative path from the above file.
# Based on that `pdf`, `docx` etc. formats would not work! If you want to convert an
# online markdown file to other formats with this function, please pre-process the file
# to have absolute paths instead.
}
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.summary.nls}
\alias{pander.summary.nls}
\title{Prints an summary.nls object from stats package in Pandoc's markdown.}
\usage{
\method{pander}{summary.nls}(
  x,
  summary = TRUE,
  add.significance.stars = FALSE,
  digits = panderOptions("digits"),
  show.convergence = FALSE,
  ...
)
}
\arguments{
\item{x}{an summary.nls object}

\item{summary}{(defaut:\code{TRUE}) if used for summary.lm or lm}

\item{add.significance.stars}{if significance stars should be shown for P value}

\item{digits}{number of digits of precision}

\item{show.convergence}{(defaut:\code{FALSE}) if to print convergence info}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an summary.nls object from stats package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.polr}
\alias{pander.polr}
\title{Prints an polr object from MASS package in Pandoc's markdown.}
\usage{
\method{pander}{polr}(x, ...)
}
\arguments{
\item{x}{an polr object}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an polr object from MASS package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.nls}
\alias{pander.nls}
\title{Prints an nls object from stats package in Pandoc's markdown.}
\usage{
\method{pander}{nls}(x, digits = panderOptions("digits"),
  show.convergence = FALSE, ...)
}
\arguments{
\item{x}{an nls object}

\item{digits}{number of digits of precision}

\item{show.convergence}{(defaut:\code{FALSE}) if to print convergence info}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an nls object from stats package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph.R
\name{add.lattice.grid}
\alias{add.lattice.grid}
\title{Add grids}
\usage{
add.lattice.grid(side = c("top", "bottom", "left", "right"), ...,
  ticks = c("default", "yes", "no"), scales, components, line.col)
}
\arguments{
\item{side}{see \code{latticeExtra::axis.grid}}

\item{...}{see \code{latticeExtra::axis.grid}}

\item{ticks}{see \code{latticeExtra::axis.grid}}

\item{scales}{see \code{latticeExtra::axis.grid}}

\item{components}{see \code{latticeExtra::axis.grid}}

\item{line.col}{see \code{latticeExtra::axis.grid}}
}
\description{
Exactly the same fn as  see \code{latticeExtra::axis.grid} but using \code{grid} colors instead.
}
\references{
This function is a forked version of \code{latticeExtra::axis.grid}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{add.significance.stars}
\alias{add.significance.stars}
\title{Add significance stars}
\usage{
add.significance.stars(p, cutoffs = c(0.05, 0.01, 0.001))
}
\arguments{
\item{p}{numeric vector or tabular data}

\item{cutoffs}{the cutoffs for the 1/2/3 significance stars}
}
\value{
character vector
}
\description{
This function adds significance stars to passed \code{p} value(s) as: one star for value below \code{0.05}, two for \code{0.01} and three for \code{0.001}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{coef_mat}
\alias{coef_mat}
\title{Calculate coef matrix for models from rms package
Forked from prModFit from rms}
\usage{
coef_mat(obj, coefs)
}
\arguments{
\item{obj}{object list}

\item{coefs}{numeric value if to print only the first n regression coefficients in the model.}
}
\value{
coeficients matrix
}
\description{
Calculate coef matrix for models from rms package
Forked from prModFit from rms
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/R5.R
\docType{class}
\name{Pandoc-class}
\alias{Pandoc-class}
\alias{Pandoc}
\title{Reporting with Pandoc}
\arguments{
\item{...}{this is an R5 object without any direct params but it should be documented, right?}
}
\description{
This \code{R5} reference class can hold bunch of elements (text or R objects) from which it tries to create a Pandoc's markdown text file. Exporting the report to several formats (like: PDF, docx, odt etc. - see Pandoc's documentation) is also possible, see examples below.
}
\section{Methods}{

\describe{
\item{\code{export(Class)}}{Returns the result of coercing the object to
Class.  No effect on the object itself.}
}}

\examples{
\dontrun{
## Initialize a new Pandoc object
myReport <- Pandoc$new()

## Add author, title and date of document
myReport$author <- 'Anonymous'
myReport$title  <- 'Demo'

## Or it could be done while initializing
myReport <- Pandoc$new('Anonymous', 'Demo')

## Add some free text
myReport$add.paragraph('Hello there, this is a really short tutorial!')

## Add maybe a header for later stuff
myReport$add.paragraph('# Showing some raw R objects below')

## Adding a short matrix
myReport$add(matrix(5,5,5))

## Or a table with even # TODO: caption
myReport$add.paragraph('Hello table:')
myReport$add(table(mtcars$am, mtcars$gear))

## Or a "large" data frame which barely fits on a page
myReport$add(mtcars)

## And a simple linear model with Anova tables
ml <- with(lm(mpg ~ hp + wt), data = mtcars)
myReport$add(ml)
myReport$add(anova(ml))
myReport$add(aov(ml))

## And do some principal component analysis at last
myReport$add(prcomp(USArrests))

## Sorry, I did not show how Pandoc deals with plots:
myReport$add(plot(1:10)) # TODO: caption

## Want to see the report? Just print it:
myReport

## Exporting to PDF (default)
myReport$export()

## Or to docx in tempdir:
myReport$format <- 'docx'
myReport$export(tempfile())

## You do not want to see the generated report after generation?
myReport$export(open = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.survfit}
\alias{pander.survfit}
\title{Pander method for survfit class}
\usage{
\method{pander}{survfit}(x, caption = attr(x, "caption"), scale = 1,
  print.rmean = getOption("survfit.print.rmean"),
  rmean = getOption("survfit.rmean"), ...)
}
\arguments{
\item{x}{the result of a call to the survfit function.}

\item{caption}{caption (string) to be shown under the table}

\item{scale}{a numeric value to rescale the survival time, e.g., if the input data to survfit were in days, scale=365 would scale the printout to years.}

\item{print.rmean, rmean}{options for computation and display of the restricted mean}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an survfit object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.CrossTable}
\alias{pander.CrossTable}
\title{Pander method for CrossTable class}
\usage{
\method{pander}{CrossTable}(x, caption = attr(x, "caption"),
  digits = panderOptions("digits"), total.r = x$total.r,
  total.c = x$total.c, ...)
}
\arguments{
\item{x}{a CrossTable object}

\item{caption}{caption (string) to be shown under the table}

\item{digits}{number of digits of precision}

\item{total.r}{if to print row totals. Default values is taken from CrossTable object}

\item{total.c}{if to print column totals. Default values is taken from CrossTable object}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints a CrossTable object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{repChar}
\alias{repChar}
\title{Repeating chars}
\usage{
repChar(x, n, sep = "")
}
\arguments{
\item{x}{string to repeat}

\item{n}{integer}

\item{sep}{separator between repetitions}
}
\value{
character vector
}
\description{
Repeating a string \code{n} times and returning a concatenated character vector.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{set.caption}
\alias{set.caption}
\title{Adds caption in current block}
\usage{
set.caption(x, permanent = FALSE)
}
\arguments{
\item{x}{string}

\item{permanent}{(default \code{FALSE}) if caption is permanent (for all future tables) or not}
}
\description{
This is a helper function to add a caption to the returning image/table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{has.rownames}
\alias{has.rownames}
\title{Check if rownames are available}
\usage{
has.rownames(x)
}
\arguments{
\item{x}{a tabular-like R object}
}
\value{
\code{TRUE} OR \code{FALSE}
}
\description{
Dummy helper to check if the R object has real rownames or not.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.list.return}
\alias{pandoc.list.return}
\alias{pandoc.list}
\title{Create a list}
\usage{
pandoc.list.return(elements, style = c("bullet", "ordered", "roman"),
  loose = FALSE, add.line.breaks = TRUE, add.end.of.list = TRUE,
  indent.level = 0, missing = panderOptions("missing"), ...)
}
\arguments{
\item{elements}{character vector of strings}

\item{style}{the required style of the list}

\item{loose}{adding a newline between elements}

\item{add.line.breaks}{adding a leading and trailing newline before/after the list}

\item{add.end.of.list}{adding a separator comment after the list}

\item{indent.level}{the level of indent}

\item{missing}{string to replace missing values}

\item{...}{extra arguments passed by from parent call, disregarded}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Creates a Pandoc's markdown format list from provided character vector/list.
}
\examples{
## basic lists
pandoc.list(letters[1:5])
pandoc.list(letters[1:5])
pandoc.list(letters[1:5], 'ordered')
pandoc.list(letters[1:5], 'roman')
pandoc.list(letters[1:5], loose = TRUE)

## nested lists
l <- list("First list element",
  rep.int('sub element', 5),
  "Second element",
  list('F', 'B', 'I', c('phone', 'pad', 'talics')))
pandoc.list(l)
pandoc.list(l, loose = TRUE)
pandoc.list(l, 'roman')

## complex nested lists
pandoc.list(list('one', as.list(2)))
pandoc.list(list('one', list('two')))
pandoc.list(list('one', list(2:3)))
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.summary.lm}
\alias{pander.summary.lm}
\title{Pander method for summary.lm class}
\usage{
\method{pander}{summary.lm}(
  x,
  caption = attr(x, "caption"),
  covariate.labels,
  omit,
  summary = TRUE,
  add.significance.stars = FALSE,
  move.intercept = FALSE,
  ...
)
}
\arguments{
\item{x}{an summary.lm object}

\item{caption}{caption (string) to be shown under the table}

\item{covariate.labels}{vector to replace covariate lables in the table}

\item{omit}{vector of variable to omit for priting in resulting table}

\item{summary}{(defaut:\code{TRUE}) if used for summary.lm or lm}

\item{add.significance.stars}{if significance stars should be shown for P value}

\item{move.intercept}{by default, the Intercept is the first coefficient in the table, which can be moved to the bottom of the table}

\item{...}{optional parameters passed to special methods and/or raw \code{pandoc.*} functions}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Prints a summary.lm object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.tabular}
\alias{pander.tabular}
\title{Pander method for tabular class}
\usage{
\method{pander}{tabular}(x, caption = attr(x, "caption"),
  emphasize.rownames = TRUE, digits = panderOptions("digits"), ...)
}
\arguments{
\item{x}{an function object}

\item{caption}{caption (string) to be shown under the table}

\item{emphasize.rownames}{(defaut:\code{TRUE}) if rownames should be highlighted}

\item{digits}{number of digits of precision}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Renders an tabular object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.horizontal.rule.return}
\alias{pandoc.horizontal.rule.return}
\alias{pandoc.horizontal.rule}
\title{Create horizontal rule}
\usage{
pandoc.horizontal.rule.return()
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Creates a Pandoc's markdown format horizontal line with trailing and leading newlines.
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.strikeout.return}
\alias{pandoc.strikeout.return}
\alias{pandoc.strikeout}
\title{Add strikeout}
\usage{
pandoc.strikeout.return(x)
}
\arguments{
\item{x}{character vector}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Pandoc's markdown strikeout format (e.g. \code{~~FOO~~}) is added to character string.
}
\examples{
pandoc.strikeout('FOO')
pandoc.strikeout(c('FOO', '~~FOO~~'))
pandoc.strikeout.return('FOO')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}
\seealso{
\code{\link{pandoc.emphasis}} \code{\link{pandoc.strong}} \code{\link{pandoc.verbatim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.factor}
\alias{pander.factor}
\title{Pander method for factor class}
\usage{
\method{pander}{factor}(x, ...)
}
\arguments{
\item{x}{a factor object}

\item{...}{igroned parameters}
}
\description{
Prints a factor object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get.storage}
\alias{get.storage}
\title{Get a value from internal buffer}
\usage{
get.storage(what)
}
\arguments{
\item{what}{string}
}
\description{
And truncate content.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander}
\alias{pander}
\title{Generic pander method}
\usage{
pander(x = NULL, ...)
}
\arguments{
\item{x}{an R object}

\item{...}{optional parameters passed to special methods and/or raw \code{pandoc.*} functions}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Prints an R object in Pandoc's markdown.
}
\note{
This function can be called by \code{pander} and \code{pandoc} too.
}
\examples{

## Vectors
pander(1:10)
pander(letters)
pander(mtcars$am)
pander(factor(mtcars$am))

## Lists
pander(list(1, 2, 3, c(1, 2)))
pander(list(a = 1, b = 2, c = table(mtcars$am)))
pander(list(1, 2, 3, list(1, 2)))
pander(list(a = 1, 2, 3, list(1, 2)))
pander(list('FOO', letters[1:3], list(1:5), table(mtcars$gear), list('FOOBAR', list('a', 'b'))))
pander(list(a = 1, b = 2, c = table(mtcars$am), x = list(myname = 1, 2), 56))
pander(unclass(chisq.test(table(mtcars$am, mtcars$gear))))

## Arrays
pander(mtcars)
pander(table(mtcars$am))
pander(table(mtcars$am, mtcars$gear))

## Tests
pander(ks.test(runif(50), runif(50)))
pander(chisq.test(table(mtcars$am, mtcars$gear)))
pander(t.test(extra ~ group, data = sleep))

## Models
ml <- with(lm(mpg ~ hp + wt), data = mtcars)
pander(ml)
pander(anova(ml))
pander(aov(ml))
## Dobson (1990) Page 93: Randomized Controlled Trial (examples from: ?glm)
counts <- c(18, 17, 15, 20, 10, 20, 25, 13, 12)
outcome <- gl(3, 1, 9)
treatment <- gl(3, 3)
m <- glm(counts ~ outcome + treatment, family = poisson())
pander(m)
pander(anova(m))
pander(aov(m))
## overwriting labels
pander(lm(Sepal.Width ~ Species, data = iris), covariate.labels = c('Versicolor', 'Virginica'))

## Prcomp
pander(prcomp(USArrests))

## Others
pander(density(runif(10)))
pander(density(mtcars$hp))

## default method
x <- chisq.test(table(mtcars$am, mtcars$gear))
class(x) <- 'I heave never heard of!'
pander(x)
}
\references{
\itemize{
  \item John MacFarlane (2013): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
  \item David Hajage (2011): _ascii. Export R objects to several markup languages._ \url{https://cran.r-project.org/package=ascii}
  \item Hlavac, Marek (2013): _stargazer: LaTeX code for well-formatted regression and summary statistics tables._ \url{https://cran.r-project.org/package=stargazer}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{path_to_pandoc}
\alias{path_to_pandoc}
\title{Find path to the pandoc binary by checking the \code{PATH} and the \code{RSTUDIO_PANDOC} env vars}
\usage{
path_to_pandoc()
}
\value{
file path
}
\description{
Find path to the pandoc binary by checking the \code{PATH} and the \code{RSTUDIO_PANDOC} env vars
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{check_digits}
\alias{check_digits}
\title{Check if vector parameter for round/digits and adjust accodingly}
\usage{
check_digits(param, name, n)
}
\arguments{
\item{param}{vector to be checked}

\item{name}{parameter name}

\item{n}{needed size of vector}
}
\value{
original vector if size is good, vector of default values otherwise
}
\description{
Check if vector parameter for round/digits and adjust accodingly
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{wrap}
\alias{wrap}
\title{Wrap Vector Elements}
\usage{
wrap(x, wrap = "\\"")
}
\arguments{
\item{x}{a vector to wrap}

\item{wrap}{a string to wrap around vector elements}
}
\value{
a string with wrapped elements
}
\description{
Wraps vector elements with string provided in \code{wrap} argument.
}
\examples{
\dontrun{
wrap('foobar')
wrap(c('fee', 'fi', 'foo', 'fam'), '_')
}
}
\references{
This function was moved from \code{rapport} package: \url{http://rapport-package.info}.
}
\author{
Aleksandar Blagotic
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.function}
\alias{pander.function}
\title{Pander method for function class}
\usage{
\method{pander}{function}(x, add.name = FALSE, verbatim = TRUE,
  syntax.highlighting = FALSE, ...)
}
\arguments{
\item{x}{an function object}

\item{add.name}{(defaut:\code{FALSE}) if to add function name to output or just to print a body}

\item{verbatim}{(defaut:\code{TRUE}) if to add tabulation, so pandoc conversion will rander it properly}

\item{syntax.highlighting}{(defaut:\code{FALSE}) if to add hyghlighting tag for R syntax}

\item{...}{ignored parameters}
}
\description{
Prints an function object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.call}
\alias{pander.call}
\title{Pander method for call class}
\usage{
\method{pander}{call}(x, ...)
}
\arguments{
\item{x}{a call object}

\item{...}{optional parameters passed to raw \code{pandoc.formula} function}
}
\description{
Prints a call object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.link.return}
\alias{pandoc.link.return}
\alias{pandoc.link}
\title{Create pandoc link
Pandoc's markdown format link.}
\usage{
pandoc.link.return(url, text = url)
}
\arguments{
\item{url}{hyperlink}

\item{text}{link text}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Create pandoc link
Pandoc's markdown format link.
}
\examples{
pandoc.link('http://r-project.org')
pandoc.link('http://r-project.org', 'R')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph.R
\name{add.minor.ticks}
\alias{add.minor.ticks}
\title{Minor tick marks}
\usage{
add.minor.ticks(nx = 4, ny = 4, grid = TRUE)
}
\arguments{
\item{nx}{intervals to split on x axis}

\item{ny}{intervals to split on x axis}

\item{grid}{show grid lines at minor ticks?}
}
\description{
Adds minor tick marks to an existing \code{base} plot.
}
\references{
This function is a forked version of \code{Hmisc::minor.tick}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.verbatim.return}
\alias{pandoc.verbatim.return}
\alias{pandoc.verbatim}
\title{Add verbatim}
\usage{
pandoc.verbatim.return(x, style = c("inline", "indent", "delim"),
  attrs = "")
}
\arguments{
\item{x}{character vector}

\item{style}{show code \code{inline} or in a separate (\code{indent}ed or \code{delim}ited) block}

\item{attrs}{(optionally) pass ID, classes and any attribute to the \code{delimited} block}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Pandoc's markdown verbatim format (e.g. \code{`FOO`}) is added to character string.
}
\examples{
# different styles/formats
pandoc.verbatim('FOO')

src <- c('FOO', 'indent', 'BAR' )
pandoc.verbatim(src)
pandoc.verbatim.return(src)
pandoc.verbatim(c('FOOO\\nBAR  ', ' I do R'), 'indent')
pandoc.verbatim(c('FOOO\\nBAR  ', ' I do R'), 'delim')

# add highlighting and HTML/LaTeX ID and classes (even custom attribute)
pandoc.verbatim(c('cat("FOO")', 'mean(bar)'), 'delim', '.R #MyCode custom_var="10"')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}
\seealso{
\code{\link{pandoc.emphasis}} \code{\link{pandoc.strikeout}} \code{\link{pandoc.strong}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get.alignment}
\alias{get.alignment}
\title{Get alignment}
\usage{
get.alignment(df, remove.obious.rownames = TRUE)
}
\arguments{
\item{df}{data.frame}

\item{remove.obious.rownames}{if this rule should be applied or not}
}
\value{
vector of alignment parameters
}
\description{
Get alignment from temporary environment, truncating that and applying rownames and other columns alignment to passed \code{df}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.vector}
\alias{pander.vector}
\title{Helper function to deal with atomic vectors}
\usage{
\method{pander}{vector}(x, ...)
}
\arguments{
\item{x}{vector}

\item{...}{ignored parameters}
}
\description{
Helper function to deal with atomic vectors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brew.R
\name{brew}
\alias{brew}
\title{Patched brew}
\usage{
brew(text = NULL, envir = parent.frame())
}
\arguments{
\item{text}{character vector}

\item{envir}{environment}
}
\value{
\code{brew}ed document to \code{stdout} and raw results while evaluating the \code{text} in a structured list.
}
\description{
This is a forked/patched version of `brew` package made by Jeffrey Horner (c) 2007. See: \code{References} about the original version.
}
\details{
This custom function can do more and also less compared to the original \code{brew} package. First of all the internal caching mechanism (and other, from \code{pander} package POV needless features) of `brew` is removed for some extra profits:
\itemize{
     \item multiple R expressions can be passed between \code{<\%= ... \%>} tags,
     \item the text of the file and also the evaluated R objects are (invisibly) returned in a structured list, which can be really useful while post-processing the results of `brew`.
}
}
\note{
This function should be never called directly (use \code{brew::brew} instead) as being a helper function of \code{Pandoc.brew}.
}
\references{
Jeffrey Horner (2011). _brew: Templating Framework for Report Generation._ \url{https://cran.r-project.org/package=brew}
}
\seealso{
\code{\link{Pandoc.brew}}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.summary.polr}
\alias{pander.summary.polr}
\title{Prints an summary.polr object from MASS package in Pandoc's markdown.}
\usage{
\method{pander}{summary.polr}(x, digits = panderOptions("digits"),
  round = panderOptions("round"),
  keep.trailing.zeros = panderOptions("keep.trailing.zeros"), ...)
}
\arguments{
\item{x}{an summary.polr object}

\item{digits}{number of digits of precision passed to format}

\item{round}{number of rounding digits passed to round}

\item{keep.trailing.zeros}{to show or remove trailing zeros in numbers on a column basis width}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an summary.polr object from MASS package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.survreg}
\alias{pander.survreg}
\title{Prints an survreg object from survival package in Pandoc's markdown.}
\usage{
\method{pander}{survreg}(x, ...)
}
\arguments{
\item{x}{an survreg object}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an survreg object from survival package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.POSIXct}
\alias{pander.POSIXct}
\title{Pander method for POSIXct class}
\usage{
\method{pander}{POSIXct}(x, ...)
}
\arguments{
\item{x}{a POSIXct object}

\item{...}{optional parameters passed to raw \code{pandoc.date} function}
}
\description{
Prints a POSIXct object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.footnote.return}
\alias{pandoc.footnote.return}
\alias{pandoc.footnote}
\title{Footnote}
\usage{
pandoc.footnote.return(x)
}
\arguments{
\item{x}{character vector}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Creates a Pandoc's markdown format footnote.
}
\examples{
pandoc.footnote('Automatically numbered footnote, right?')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.ols}
\alias{pander.ols}
\title{Prints an ols object from rms package in Pandoc's markdown.}
\usage{
\method{pander}{ols}(x, long = FALSE, coefs = TRUE,
  digits = panderOptions("digits"), round = panderOptions("round"),
  ...)
}
\arguments{
\item{x}{an ols object}

\item{long}{if to print the correlation matrix of parameter estimates. default(\code{FALSE})}

\item{coefs}{if to the table of model coefficients, standard errors, etc. default(\code{TRUE})}

\item{digits}{passed to \code{format}. Can be a vector specifying values for each column (has to be the same length as number of columns).}

\item{round}{passed to \code{round}. Can be a vector specifying values for each column (has to be the same length as number of columns). Values for non-numeric columns will be disregarded.}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an ols object from rms package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{splitLine}
\alias{splitLine}
\title{Split line with line breaks depending on max.width}
\usage{
splitLine(x, max.width = panderOptions("table.split.cells"),
  use.hyphening = FALSE)
}
\arguments{
\item{x}{string to be split. Works only with one string. Non-string arguments and multi-dimensional arguments are returned unchaged}

\item{max.width}{default integer value specyfing max number of characters between line breaks}

\item{use.hyphening}{(default: \code{FALSE}) if try to use hyphening when splitting large cells according to table.split.cells. Requires \pkg{sylly}.}
}
\value{
character string with line breaks
}
\description{
This is a helper function to insert line breaks depending on (\code{split.cells} parameter of \code{pandoc.table}) of the returning table.
}
\examples{
splitLine('foo bar', 6)
splitLine('foo bar', 7)
splitLine('Pandoc Package', 3, TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brew.R
\name{Pandoc.brew}
\alias{Pandoc.brew}
\title{Brew in pandoc format}
\usage{
Pandoc.brew(file = stdin(), output = stdout(), convert = FALSE,
  open = TRUE, graph.name, graph.dir, graph.hi.res = FALSE,
  text = NULL, envir = parent.frame(), append = FALSE, ...)
}
\arguments{
\item{file}{file path of the brew template. As this is passed to \code{readLines}, \code{file} could be an URL too, but not over SSL (for that latter \code{RCurl} would be needed).}

\item{output}{(optional) file path of the output file}

\item{convert}{string: format of required output document (besides Pandoc's markdown). Pandoc is called if set via \code{Pandoc.convert} and the converted document could be also opened automatically (see below).}

\item{open}{try to open converted document with operating system's default program}

\item{graph.name}{character string (default to \code{\%t} when \code{output} is set to \code{stdout} and \code{paste0(basename(output), '-\%n')} otherwise) passed to \code{\link{evals}}.  Besides \code{\link{evals}}'s possible tags \code{\%i} is also available which would be replaced by the chunk number (and optionally an integer which would handle nested \code{brew} calls) and \code{\%I} with the order of the current expression.}

\item{graph.dir}{character string (default to \code{tempdir()} when \code{output} is set to \code{stdout} and \code{dirname(graph.name)} otherwise) passed to \code{\link{evals}}}

\item{graph.hi.res}{render high resolution images of plots? Default is \code{FALSE} except for HTML output.}

\item{text}{character vector (treated as the content of the \code{file}}

\item{envir}{environment where to \code{brew} the template}

\item{append}{should append or rather overwrite (default) the \code{output} markdown text file? Please note that this option only affects the markdown file and not the optionally created other formats.}

\item{...}{additional parameters passed to \code{\link{Pandoc.convert}}}
}
\value{
converted file name with full path if \code{convert} is set, none otherwise
}
\description{
This function behaves just like \code{brew} except for the \code{<\%=...\%>} tags, where \code{Pandoc.brew} first translate the R object found between the tags to Pandoc's markdown before passing to the \code{cat} function.
}
\details{
This parser tries to be smart in some ways:

\itemize{
     \item a block (R commands between the tags) could return any value at any part of the block and there are no restrictions about the number of returned R objects
     \item plots and images are grabbed in the document, rendered to a png file and \code{pander} method would result in a Pandoc's markdown formatted image link (so the image would be shown/included in the exported document). The images are put in \code{plots} directory in current \code{getwd()} or to the specified \code{output} file's directory.
     \item all warnings/messages and errors are recorded in the blocks and returned in the document as a footnote
}

Please see my Github page for details (\url{http://rapporter.github.com/pander/#brew-to-pandoc}) and examples (\url{http://rapporter.github.com/pander/#examples}).
}
\note{
Only one of the input parameters (\code{file} or \code{text}) is to be used at once!
}
\examples{
\dontrun{
text <- paste('# Header', '',
  'What a lovely list:\\n<\%=as.list(runif(10))\%>',
  'A wide table:\\n<\%=mtcars[1:3, ]\%>',
  'And a nice chart:\\n\\n<\%=plot(1:10)\%>', sep = '\\n')
Pandoc.brew(text = text)
Pandoc.brew(text = text, output = tempfile(), convert = 'html')
Pandoc.brew(text = text, output = tempfile(), convert = 'pdf')

## pi is awesome
Pandoc.brew(text='<\%for (i in 1:5) {\%>\\n Pi has a lot (<\%=i\%>) of power: <\%=pi^i\%><\%}\%>')

## package bundled examples
Pandoc.brew(system.file('examples/minimal.brew', package='pander'))
Pandoc.brew(system.file('examples/minimal.brew', package='pander'),
  output = tempfile(), convert = 'html')
Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'))
Pandoc.brew(system.file('examples/short-code-long-report.brew', package='pander'),
  output = tempfile(), convert = 'html')

## brew returning R objects
str(Pandoc.brew(text='Pi equals to <\%=pi\%>.
And here are some random data:\\n<\%=runif(10)\%>'))

str(Pandoc.brew(text='# Header <\%=1\%>\\nPi is <\%=pi\%> which is smaller then <\%=2\%>.
foo\\nbar\\n <\%=3\%>\\n<\%=mtcars[1:2,]\%>'))

str(Pandoc.brew(text='<\%for (i in 1:5) {\%>
Pi has a lot (<\%=i\%>) of power: <\%=pi^i\%><\%}\%>'))
}
}
\references{
\itemize{
     \item Jeffrey Horner (2011). _brew: Templating Framework for Report Generation._ \url{https://cran.r-project.org/package=brew}
     \item John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.cph}
\alias{pander.cph}
\title{Prints an cph object from rms package in Pandoc's markdown.}
\usage{
\method{pander}{cph}(x, table = TRUE, conf.int = FALSE, coefs = TRUE,
  ...)
}
\arguments{
\item{x}{an cph object}

\item{table}{if to print event frequency statistics. default(\code{TRUE})}

\item{conf.int}{set to e.g. .95 to print 0.95 confidence intervals on simple hazard ratios (which are usually meaningless as one-unit changes are seldom relevant and most models contain multiple terms per predictor)}

\item{coefs}{if to the table of model coefficients, standard errors, etc. default(\code{TRUE})}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an cph object from rms package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.evals}
\alias{pander.evals}
\title{Pander method for evals class}
\usage{
\method{pander}{evals}(x, ...)
}
\arguments{
\item{x}{a evals object}

\item{...}{ignored parameters}
}
\description{
Prints a evals object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.title.return}
\alias{pandoc.title.return}
\alias{pandoc.title}
\title{Create title block}
\usage{
pandoc.title.return(author = "", title = "", date = "")
}
\arguments{
\item{author}{character vector or semicolon delimited list of authors without line break}

\item{title}{character vector of lines of title or multiline string with \code{\\n} separators}

\item{date}{any string fit in one line}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Creates a Pandoc's markdown style title block with optional author, title and date fields.
}
\examples{
pandoc.title('Tom', 'Render pandoc in R', '2012-05-16')
pandoc.title(c('Tom', 'Jerry'), 'Render pandoc in R', '2012-05-16')
pandoc.title('Tom; Jerry', 'Render pandoc in R', '2012-05-16')
pandoc.title('Tom; Jerry', c('Render', 'pandoc', 'in R'), '2012-05-16')
pandoc.title('Tom; Jerry', 'Render\\n    pandoc \\n    in R', '2012-05-16')

## missing fields

pandoc.title('Tom; Jerry', 'Render pandoc in R')
pandoc.title('Tom; Jerry')
pandoc.title(title = 'Render pandoc in R', date= '2012-05-16')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.smooth.spline}
\alias{pander.smooth.spline}
\title{Pander method for smooth.spline class}
\usage{
\method{pander}{smooth.spline}(x, ...)
}
\arguments{
\item{x}{an smooth.spline object}

\item{...}{igroned parameters}
}
\description{
Prints an smooth.spline object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{emphasize.rows}
\alias{emphasize.rows}
\alias{emphasize.cols}
\alias{emphasize.cells}
\alias{emphasize.strong.rows}
\alias{emphasize.strong.cols}
\alias{emphasize.strong.cells}
\alias{emphasize.italics.rows}
\alias{emphasize.italics.cols}
\alias{emphasize.italics.cells}
\alias{emphasize.verbatim.rows}
\alias{emphasize.verbatim.cols}
\alias{emphasize.verbatim.cells}
\title{Emphasize rows/columns/cells}
\usage{
emphasize.rows(x)

emphasize.cols(x)

emphasize.cells(x)

emphasize.strong.rows(x)

emphasize.strong.cols(x)

emphasize.strong.cells(x)

emphasize.italics.rows(x)

emphasize.italics.cols(x)

emphasize.italics.cells(x)

emphasize.verbatim.rows(x)

emphasize.verbatim.cols(x)

emphasize.verbatim.cells(x)
}
\arguments{
\item{x}{vector of row/columns indexes or an array like returned by \code{which(..., arr.ind = TRUE)}}
}
\description{
Storing indexes of cells to be (strong) emphasized of a tabular data in an internal buffer that can be released and applied by \code{\link{pandoc.table}}, \code{\link{pander}} or \code{\link{evals}} later.
}
\examples{
\dontrun{
n <- data.frame(x = c(1,1,1,1,1), y = c(0,1,0,1,0))
emphasize.cols(1)
emphasize.rows(1)
pandoc.table(n)

emphasize.strong.cells(which(n == 1, arr.ind = TRUE))
pander(n)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.irts}
\alias{pander.irts}
\title{Prints an irts object from tseries package in Pandoc's markdown.}
\usage{
\method{pander}{irts}(x, caption = attr(x, "caption"),
  format = panderOptions("date"), ...)
}
\arguments{
\item{x}{an irts object}

\item{caption}{caption (string) to be shown under the table}

\item{format}{string passed to format when printing dates (POSIXct or POSIXt)}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an irts object from tseries package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.sessionInfo}
\alias{pander.sessionInfo}
\title{Pander method for sessionInfo class}
\usage{
\method{pander}{sessionInfo}(x, locale = TRUE, compact = TRUE, ...)
}
\arguments{
\item{x}{an sessionInfo object}

\item{locale}{(defaut:\code{TRUE}) if to print locale output}

\item{compact}{(defaut:\code{TRUE}) if output shoud be compact (ommiting extra line breaks and spaces, inline printing of lists)}

\item{...}{ignored parameters}
}
\description{
Prints an sessionInfo object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.Arima}
\alias{pander.Arima}
\title{Prints an arima object from stats package in Pandoc's markdown.}
\usage{
\method{pander}{Arima}(x, digits = panderOptions("digits"), se = TRUE,
  ...)
}
\arguments{
\item{x}{an arima object}

\item{digits}{number of digits of precision}

\item{se}{if to include standard error in coefficients table (default \code{TRUE})}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an arima object from stats package in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{evalsOptions}
\alias{evalsOptions}
\title{Querying/setting evals option}
\usage{
evalsOptions(o, value)
}
\arguments{
\item{o}{option name (string). See below.}

\item{value}{value to assign (optional)}
}
\description{
To list all \code{evals} options, just run this function without any parameters provided. To query only one value, pass the first parameter. To set that, use the \code{value} parameter too.
}
\details{
The following \code{evals} options are available:

\itemize{
     \item \code{parse}: if \code{TRUE} the provided \code{txt} elements would be merged into one string and parsed to logical chunks. This is useful if you would want to get separate results of your code parts - not just the last returned value, but you are passing the whole script in one string. To manually lock lines to each other (e.g. calling a \code{plot} and on next line adding an \code{abline} or \code{text} to it), use a plus char (\code{+}) at the beginning of each line which should be evaluated with the previous one(s). If set to \code{FALSE}, \code{evals} would not try to parse R code, it would get evaluated in separate runs - as provided. Please see examples of \code{\link{evals}}.
     \item \code{cache}: caching the result of R calls if set to \code{TRUE}
     \item \code{cache.mode}: cached results could be stored in an \code{environment} in \emph{current} R session or let it be permanent on \code{disk}.
     \item \code{cache.dir}: path to a directory holding cache files if \code{cache.mode} set to \code{disk}. Default to \code{.cache} in current working directory.
     \item \code{cache.time}: number of seconds to limit caching based on \code{proc.time}. If set to \code{0}, all R commands, if set to \code{Inf}, none is cached (despite the \code{cache} parameter).
     \item \code{cache.copy.images}: copy images to new files if an image is returned from cache? If set to \code{FALSE} (default) the "old" path would be returned.
     \item \code{classes}: a vector or list of classes which should be returned. If set to \code{NULL} (by default) all R objects will be returned.
     \item \code{hooks}: list of hooks to be run for given classes in the form of \code{list(class = fn)}. If you would also specify some parameters of the function, a list should be provided in the form of \code{list(fn, param1, param2=NULL)} etc. So the hooks would become \code{list(class1=list(fn, param1, param2=NULL), ...)}. See examples of \code{\link{evals}}. A default hook can be specified too by setting the class to \code{'default'}. This can be handy if you do not want to define separate methods/functions to each possible class, but automatically apply the default hook to all classes not mentioned in the list. You may also specify only one element in the list like: \code{hooks=list('default' = pander_return)}. Please note, that nor error/warning messages, nor stdout is captured (so: updated) while running hooks!
     \item \code{length}: any R object exceeding the specified length will not be returned. The default value (\code{Inf}) does not filter out any R objects.
     \item \code{output}: a character vector of required returned values. This might be useful if you are only interested in the \code{result}, and do not want to save/see e.g. \code{messages} or \code{print}ed \code{output}. See examples of \code{\link{evals}}.
     \item \code{graph.unify}: boolean (default: \code{FALSE}) that determines if \code{evals} should try to unify the style of (\code{base}, \code{lattice} and \code{ggplot2}) plots? If set to \code{TRUE}, some \code{panderOptions()} would apply.
     \item \code{graph.name}: set the file name of saved plots which is \code{\link{tempfile}} by default. A simple character string might be provided where \code{\%d} would be replaced by the index of the generating \code{txt} source, \code{\%n} with an incremented integer in \code{graph.dir} with similar file names and \code{\%t} by some random characters. A function's name to be \code{eval}uated can be passed here too.
     \item \code{graph.dir}: path to a directory where to place generated images. If the directory does not exist, \code{\link{evals}} try to create that. Default set to \code{plots} in current working directory.
     \item \code{graph.output}: set the required file format of saved plots. Currently it could be any of  \code{grDevices}: \code{png}, \code{bmp}, \code{jpeg}, \code{jpg}, \code{tiff}, \code{svg} or \code{pdf}. Set to \code{NA} not to save plots at all and tweak that setting with \code{capture.plot()} on demand.
     \item \code{width}: width of generated plot in pixels for even vector formats
     \item \code{height}: height of generated plot in pixels for even vector formats
     \item \code{res}: nominal resolution in \code{ppi}. The height and width of vector images will be calculated based in this.
     \item \code{hi.res}: generate high resolution plots also? If set to \code{TRUE}, each R code parts resulting an image would be run twice.
     \item \code{hi.res.width}: width of generated high resolution plot in pixels for even vector formats. The \code{height} and \code{res} of high resolution image is automatically computed based on the above options to preserve original plot aspect ratio.
     \item \code{graph.env}: save the environments in which plots were generated to distinct files (based on \code{graph.name}) with \code{env} extension?
     \item \code{graph.recordplot}: save the plot via \code{recordPlot} to distinct files (based on \code{graph.name}) with \code{recodplot} extension?
     \item \code{graph.RDS}: save the raw R object returned (usually with \code{lattice} or \code{ggplot2}) while generating the plots to distinct files (based on \code{graph.name}) with \code{RDS} extension?
     \item \code{log}: \code{NULL} or  an optionally passed \emph{logger name} from \pkg{futile.logger} to record all info, trace, debug and error messages.
}
}
\examples{
evalsOptions()
evalsOptions('cache')
evalsOptions('cache', FALSE)
}
\seealso{
\code{\link{evals}} \code{\link{panderOptions}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.formula}
\alias{pander.formula}
\title{Pander method for formula class}
\usage{
\method{pander}{formula}(x, max.width = 80, caption = attr(x,
  "caption"), ...)
}
\arguments{
\item{x}{a formula object}

\item{max.width}{maximum width in characters per line}

\item{caption}{caption (string) to be shown under the formula}

\item{...}{optional parameters passed to raw \code{pandoc.formula} function}
}
\description{
Prints a formula object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.list}
\alias{pander.list}
\title{Pander method for list class}
\usage{
\method{pander}{list}(x, ...)
}
\arguments{
\item{x}{a list object}

\item{...}{ignored parameters}
}
\description{
Prints a list object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph.R
\name{add.lattice.xsubticks}
\alias{add.lattice.xsubticks}
\alias{add.lattice.ysubticks}
\title{Add subticks}
\usage{
add.lattice.xsubticks(lim, ..., n = 2)
}
\arguments{
\item{lim}{see \code{latticeExtra::xscale.components.subticks}}

\item{...}{see \code{latticeExtra::xscale.components.subticks}}

\item{n}{see \code{latticeExtra::xscale.components.subticks}}
}
\description{
See \code{latticeExtra::xscale.components.subticks}.
}
\references{
This is a simplified verwion of \code{latticeExtra::xscale.components.subticks}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{get.emphasize}
\alias{get.emphasize}
\title{Get emphasize params from internal buffer}
\usage{
get.emphasize(df)
}
\arguments{
\item{df}{tabular data}
}
\value{
R object passed as \code{df} with possibly added \code{attr}s captured from internal buffer
}
\description{
And truncate content.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.microbenchmark}
\alias{pander.microbenchmark}
\title{Pander method for microbenchmark class}
\usage{
\method{pander}{microbenchmark}(x, caption = attr(x, "caption"),
  expr.labels, unit, ...)
}
\arguments{
\item{x}{an microbenchmark object}

\item{caption}{caption (string) to be shown under the table}

\item{expr.labels}{expression labels that will replace default ones (similar to rownames, which microbenchmark class table does not have)}

\item{unit}{units in which values should be printed (for example second, microseconds, etc.). Should be one of ns, us, ms, s, t, hz, khz, mhz, eps, f}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints an microbenchmark object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.character}
\alias{pander.character}
\title{Pander method for character class}
\usage{
\method{pander}{character}(x, ...)
}
\arguments{
\item{x}{a character object}

\item{...}{igroned parameters}
}
\description{
Prints a character class in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.image}
\alias{pander.image}
\title{Pander method for image class}
\usage{
\method{pander}{image}(x, caption = attr(x, "caption"), href = attr(x,
  "href"), ...)
}
\arguments{
\item{x}{a image object}

\item{caption}{caption (string) to be shown under the table}

\item{href}{link that image should be linked with}

\item{...}{ignored parameters}
}
\description{
Prints a image object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers.R
\name{trim.spaces}
\alias{trim.spaces}
\title{Trim leading and trailing spaces}
\usage{
trim.spaces(x)
}
\arguments{
\item{x}{character vector}
}
\value{
character vector
}
\description{
Trim leading and trailing spaces
}
\seealso{
\code{trim.space} in \code{rapport} package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.image.return}
\alias{pandoc.image.return}
\alias{pandoc.image}
\title{Create pandoc image tags}
\usage{
pandoc.image.return(img, caption = storage$caption)
}
\arguments{
\item{img}{image path}

\item{caption}{text}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Creates a Pandoc's markdown format image hyperlink.
}
\note{
The \code{caption} text is read from an internal buffer which defaults to \code{NULL}. To update that, call \code{link{set.caption}} before.
}
\examples{
pandoc.image('foo.png')
pandoc.image('foo.png', 'Nice image, huh?')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}
\seealso{
\code{\link{set.caption}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.NULL}
\alias{pander.NULL}
\title{Pander method for a NULL object}
\usage{
\method{pander}{NULL}(x, ...)
}
\arguments{
\item{x}{a NULL object}

\item{...}{ignored parameters}
}
\description{
Prints a NULL object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.randomForest}
\alias{pander.randomForest}
\title{Pander method for randomForest class}
\usage{
\method{pander}{randomForest}(x, digits = panderOptions("digits"), ...)
}
\arguments{
\item{x}{an randomForest object}

\item{digits}{number of digits of precision}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Renders an randomForest object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.ftable}
\alias{pander.ftable}
\title{Pander method for ftable class}
\usage{
\method{pander}{ftable}(x, ...)
}
\arguments{
\item{x}{a ftable object}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints a ftable object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.header.return}
\alias{pandoc.header.return}
\alias{pandoc.header}
\title{Create header}
\usage{
pandoc.header.return(x, level = 1, style = c("atx", "setext"))
}
\arguments{
\item{x}{character vector}

\item{level}{integer}

\item{style}{atx or setext type of heading}
}
\value{
By default this function outputs (see: \code{cat}) the result. If you would want to catch the result instead, then call the function ending in \code{.return}.
}
\description{
Creates a (Pandoc's) markdown style header with given level.
}
\examples{
pandoc.header('Foo!', 4)
pandoc.header('Foo!', 2, 'setext')
pandoc.header('Foo **bar**!', 1, 'setext')
}
\references{
John MacFarlane (2012): _Pandoc User's Guide_. \url{http://johnmacfarlane.net/pandoc/README.html}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.default}
\alias{pander.default}
\title{Default Pander method}
\usage{
\method{pander}{default}(x, ...)
}
\arguments{
\item{x}{an object}

\item{...}{optional parameters passed to raw \code{pandoc.list} function}
}
\description{
Method to be used, when no exact S3 method for given object is found. Tries to render object as a list
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.R
\name{pander.clogit}
\alias{pander.clogit}
\title{Pander method for clogit class}
\usage{
\method{pander}{clogit}(x, caption = attr(x, "caption"), ...)
}
\arguments{
\item{x}{an clogit object}

\item{caption}{caption (string) to be shown under the table}

\item{...}{optional parameters passed to raw \code{pandoc.table} function}
}
\description{
Prints a clogit object in Pandoc's markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pandoc.R
\name{pandoc.indent}
\alias{pandoc.indent}
\title{Indent text}
\usage{
pandoc.indent(x, level = 0)
}
\arguments{
\item{x}{character vector}

\item{level}{integer}
}
\description{
Indent all (optionally concatenated) lines of provided text with given level.
}
\examples{
pandoc.indent('FOO', 1)
pandoc.indent(pandoc.table.return(table(mtcars$gear)), 2)
cat(pandoc.indent(pandoc.table.return(table(mtcars$gear)), 3))
}
